/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	
	if (!gadgets || !gadgets.io || !gadgets.oauth) 
		return; // Sanity check: the core and oauthpopup features is a requirement and must be present

	/* Redefine the makeRequest function to intercept OAuth requests */
	var originalMakeRequest = gadgets.io.makeRequest;
	function makeRequestListener(url, callback, opt_params) {
		originalMakeRequest(url, function(result) {
			/* Redefine the callback so that we can examine the results before
			 * passing them on to the actually callback. */
			if (!opt_params.AUTHORIZATION || opt_params.AUTHORIZATION != "OAUTH") { 
				callback(result);
				return;
			}
			
		    if (result.oauthApprovalUrl) {
		    	// We have detected an OAuth failure for the current domain
		    	gadgets.rpc.call(null, "oauth_failure", null, window.name, url);
		    } else {
		    	gadgets.rpc.call(null, "oauth_success", null, window.name, url);
		    }
			callback(result);
		}, opt_params);
	};
	gadgets.io.makeRequest = makeRequestListener;

	
	/* Redefine the gadgets.oauth.Popup. Popup is defined as a prototyped class - to ensure
	 * the API remains compatible, form a new class (with custom constructor) using the 
	 * existing Popup prototype. */
	var originalOauthPopup = gadgets.oauth.Popup;
	function newOauthPopup(destination, windowOptions, openCallback,
		    closeCallback) {
		gadgets.rpc.register("execute-oauth-callback",
			function _ExecuteOAuthCallback() {
				closeCallback&&closeCallback();
			});
		
		// Notify the container that this gadget has created an oauthpopup and has a valid closeCallback 
		if (closeCallback) 
			gadgets.rpc.call(null, "register_oauth_callback", null, window.name, true);
		// Call the original constructor
		originalOauthPopup.call(this, destination, windowOptions, openCallback,
		    closeCallback);
	}
	
	function dummyObj() {};
	// Assign the old gadget.oauth.Popup prototype to a dummy object so that we don't have to call
	// the original constructor when assigning the prototype
	dummyObj.prototype = gadgets.oauth.Popup.prototype;
	newOauthPopup.prototype = new dummyObj();
	newOauthPopup.prototype.constructor = newOauthPopup;
	gadgets.oauth.Popup = newOauthPopup;
	
})();

