/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {

dojo.provide("jazz.jsp.Parser");

dojo.require("dijit._TemplatedMixin");

var _Templated = dijit._TemplatedMixin;

var buildDijitRendering = _Templated.prototype.buildRendering;

function _attachPoint(values, widget, toAttach) {
	if (values) {
		values = values.split(/\s*,\s*/);
		for (i = 0; i < values.length; i++) {
			var tokens = values[i].split(":");
			var point = tokens[0];
			if (tokens[1])
				widget = dijit.byId(tokens[1]);
			if (dojo.isArray(widget[point]))
				widget[point].push(toAttach);
			else
				widget[point] = toAttach;
			widget._attachPoints.push(point);
		}
	}
}

function _attachEvent(values, widget, eventSource) {
	if (values) {
		values = values.split(/\s*,\s*/);
		for(i = 0; i < values.length; i++) {
			var tokens = values[i].split(":");
			var event = tokens[0];
			var functionName = tokens[1] || tokens[0];
			if (tokens[2])
				widget = dijit.byId(tokens[2]);
			widget._attachEvents.push(
					widget.connect(eventSource, event, functionName));
		}
	}	
}

function buildServerRendering(widget) {
	var node = widget.domNode = widget.srcNodeRef;
	delete widget.srcNodeRef;

	/* dijit.getEnclosingWidget is broken without this because
	 * even though the widget has registered itself by its "id" attribute,
	 * the lookup requires the "widgetId" attribute to be set.
	 */
	node.setAttribute("widgetId", widget.id);
	widget._startupWidgets = widget._supportingWidgets = [];
	recurse(widget, node, true);
}

/**
 * Recurse from the given node, check for attach points and event attachments.
 * If a nested widget is found, stop recursing and just parse that widget.
 */
function recurse(widget, node, isSupporting) {
	_attachPoint(node.getAttribute("dojoAttachPoint"), widget, node);
	_attachEvent(node.getAttribute("dojoAttachEvent"), widget, node);
	if (node == widget.containerNode)
		isSupporting = false;
	if (node.children) {
		for (var i = 0; i < node.children.length; i++) {
			var child = node.children[i];
			if (child.getAttribute("data-dojo-type") || child.getAttribute("data-jazz-type")) {
				dojo.forEach(parse(child), function(childWidget) {
					_attachPoint(childWidget.dojoAttachPoint, widget, childWidget);
					_attachEvent(childWidget.jazzAttachEvent, widget, childWidget);
					if (isSupporting)
						widget._supportingWidgets.push(childWidget);
				});
			} else
				recurse(widget, child, isSupporting);
		}
	}
}

/**
 * Finds and constructs all widgets searching the given DOM element, inclusively.
 * Returns an array of the constructed widgets.
 */
function parse(root) {
	var stack = [root], result = [], operations = [], node, type, i;
	
	while (node = stack.pop()) {
		if ((type = node.getAttribute("data-jazz-type")) && !node.getAttribute("widgetId")) {
			operations.push([type, node]);
			continue;
		}
		if ((type = node.getAttribute("data-dojo-type")) && !node.getAttribute("widgetId"))
			operations.push([type, node]);
		if (node.children)
			for (i = node.children.length - 1; i >= 0; i--)
				stack.push(node.children[i]);
	}
	
	for (i = 0; i < operations.length; i++)
		result.push(create.apply(null, operations[i]));
	return result;
}

function create(type, node) {
	var props, args = {}, constructor = dojo.getObject(type);
	if (props = node.getAttribute("data-dojo-props"))
		args = dojo.fromJson('{' + props + '}');
	if (node.getAttribute("data-jazz-type"))
		args.serverTemplated = true;
	return new constructor(args, node);
}

if (dojo.config.serverTemplated)
	dojo.ready(function(){
		dojo.forEach(
				parse(dojo.body()),
				function(widget) {widget.startup();});
	});

jazz.jsp.Parser.parse = parse;

dojo.extend(_Templated, {
	buildRendering: function () {
		if (this.serverTemplated)
			buildServerRendering(this);
		else
			buildDijitRendering.apply(this);
	}
});

})();
