/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.common.internal.PracticeHelper");//$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
	dojo.require("dojox.uuid.generateTimeBasedUuid"); //$NON-NLS-1$
	dojo.require("dojox.uuid.Uuid"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	 
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	
	com.ibm.team.process.authoring.web.common.common.internal.PracticeHelper = {
	
		messages: dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"),//$NON-NLS-1$ //$NON-NLS-2$
		result: {
			success: true,
			lastModified: null,
			headerMessage: "", //$NON-NLS-1$
			message: "", //$NON-NLS-1$
			errorData: null
		},
		
		resetResult: function() {
			this.result = {
				success: true,
				headerMessage: "", //$NON-NLS-1$
				message: "", //$NON-NLS-1$
				errorData: null
			};
		},
		
		handleError: function(err, msg, ioArgs) {
			if (ioArgs) {
				var licenseWarning = UIHelper.getLicenseWarning(ioArgs);
				if (licenseWarning) {
					msg = licenseWarning;
				}
			}
			this.result = {
				success: false,
				headerMessage: msg,
				message: err.responseText ? err.responseText : err.message,
				errorData: err
			};
		},
		
		updateProcessDescription: function(resource, language, isDefault, ETag, title, description, owner, practiceList, isPLMShared, callback, useLocalServices, reload, archived) {
			this.resetResult();
			var args = {
				ETag: ETag,
				title: title,
				description: description,
				owner: owner,
				language: language,
				isDefault: isDefault,
				isPLMShared: isPLMShared,
				archived: archived,
				practiceList: practiceList,
				load: dojo.hitch(this, function(data, ioArgs) {
					this.result.type = "updateProcessDescription"; //$NON-NLS-1$
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					var returnedETag = ioArgs.xhr.getResponseHeader("Etag"); //$NON-NLS-1$
					callback(returnedETag);
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.ProcessDescriptionError_1, ioArgs);
					this.result.type = "updateProcessDescription"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			};
			var updateProcessDescription = function() {
				if(useLocalServices) {
					args['itemId'] = resource.substring(resource.lastIndexOf("/") + 1, resource.length); //$NON-NLS-1$ //$NON-NLS-2$
					ProcessAuthoringClient.updateProcessDescription(args);
				} else {
					args['url'] = resource; //$NON-NLS-1$
					ProcessAuthoringClient.updateProcessDescriptionWithURL(args);
				}		
			};
			if (reload === true) {
				ProcessAuthoringClient.getProcessDescription({
					resourceURL: resource + "?lang=" + language, //$NON-NLS-1$
					load: function(data, ioArgs) { //$NON-NLS-1$
						args.ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
						updateProcessDescription();
					},
					error: dojo.hitch(this, function(err, ioArgs) {
						this.handleError(err, this.messages.ProcessDescriptionError_1, ioArgs);
						this.result.type = "updateProcessDescription"; //$NON-NLS-1$
						dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					})
				});
			} else {
				updateProcessDescription();
			}
		},
		
		updatePractice: function(practice, putData, language, ETag, scanCallback, useLocalServices, reload) {
			this.resetResult();
			var args = {
				ETag: ETag,
				putData: putData,
				practiceId: practice.practiceId,
				language: language,
				load: dojo.hitch(this, function(data, ioArgs) {
					this.result.type = "updatePractice"; //$NON-NLS-1$
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					scanCallback(data, ioArgs.xhr.getResponseHeader("Etag")); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.PracticeError_1, ioArgs);
					this.result.type = "updatePractice"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			};
			
			var updatePractice = function() {
				if (useLocalServices) {
					args['practiceId'] = practice.practiceId; //$NON-NLS-1$
					ProcessAuthoringClient.updatePractice(args);
				} else {
					args['url'] = practice.resource; //$NON-NLS-1$
					ProcessAuthoringClient.updatePracticeWithURL(args);
				}	
			};
			
			if (reload === true) {
				ProcessAuthoringClient.getPractice({
					url: practice.resource + "?lang=" + language, //$NON-NLS-1$
					load: function(data, ioArgs) { //$NON-NLS-1$
						args.ETag = ioArgs.xhr.getResponseHeader("ETag"); //$NON-NLS-1$
						updatePractice();
					},
					error: dojo.hitch(this, function(err, ioArgs) {
						this.handleError(err, this.messages.PracticeError_1, ioArgs);
						this.result.type = "updatePractice"; //$NON-NLS-1$
						dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					})
				});
			} else {
				updatePractice();
			}
		},
		
		deletePractice: function(id, processDescriptionETag, scanCallback) {
			this.resetResult();
			ProcessAuthoringClient.deletePractice({
				practiceId: id,
				ETag: processDescriptionETag,
				load: dojo.hitch(this, function(data, ioArgs) {
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					this.result.type = "deletePractice"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					scanCallback(ioArgs.xhr.getResponseHeader("Etag")); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.PracticeError_2, ioArgs);
					this.result.type = "deletePractice"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			});
		},
		
		createPracticeElement: function(practiceResource, practiceETag, parentId, data, callback) {
			this.resetResult();
			var postUrl = null;
			if (parentId) {
				postUrl = practiceResource + "?g=" + parentId; //$NON-NLS-1$
			} else {
				postUrl = practiceResource;
			}
			ProcessAuthoringClient.postPracticeElement({
				url: postUrl,
				postData: data,
				ETag: practiceETag,
				load: dojo.hitch(this, function(data, ioArgs) {
					var location = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					var ETag = ioArgs.xhr.getResponseHeader("Etag"); //$NON-NLS-1$
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					this.result.type = "createPracticeElement"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					callback(data, location, ETag); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.PracticeError_1, ioArgs);
					this.result.type = "createPracticeElement"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			});
			
		},
		
		updatePracticeElement: function(practiceElementResource, data, ETag, language, etagCallback) {
			this.resetResult();
			var putUrl = practiceElementResource + "?updateType=partial&lang=" + language; //$NON-NLS-1$
			ProcessAuthoringClient.updatePracticeElement({
				url: putUrl,
				ETag: ETag,
				putData: data,
				load: dojo.hitch(this, function(data, ioArgs) {
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					this.result.type = "updatePracticeElement"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					etagCallback(data, ioArgs.xhr.getResponseHeader("Etag")); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.PracticeError_1, ioArgs);
					this.result.type = "updatePracticeElement"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			});
		},
		
		createPractice: function(practice, processDescriptionETag, data, callback) {
			this.resetResult();
			var postUrl = ProcessAuthoringClient.getJtsProcessDescriptionUrl() + "/" + practice.processDescriptionId; //$NON-NLS-1$
			ProcessAuthoringClient.postPracticeElement({
				url: postUrl,
				postData: data,
				ETag: processDescriptionETag,
				load: dojo.hitch(this, function(data, ioArgs) {
					var location = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					var ETag = ioArgs.xhr.getResponseHeader("Etag"); //$NON-NLS-1$
					this.result.lastModified = ioArgs.xhr.getResponseHeader(ProcessAuthoringConstants.HEADER_LASTMODIFIED);
					this.result.type = "createPractice"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
					callback(data, location, ETag); //$NON-NLS-1$
				}),
				error: dojo.hitch(this, function(err, ioArgs) {
					this.handleError(err, this.messages.PracticeError_1, ioArgs);
					this.result.type = "createPractice"; //$NON-NLS-1$
					dojo.publish(ProcessAuthoringConstants.PUBLISHTYPE_RESULTNOTICE, [this.result]);
				})
			});
			
		}
	};
	
})();
