/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

function IFrameWidgetLoader(jazzID) {
	
this.instance = {

	_id: jazzID,
		
	// TODO: Move to a different subfolder? Not util
	
	_frontsideURL: null,
	_iframeDomainURL: null,
	_widgetState: null,
	_height: 0,
	_width: 0,
	_iframe: null,
	_handler: null,
	
	_containerNode: null,
	_autoHeight: false,
	
	_debugMode: false,
	
	_static: function() {}, 

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_initStatics: function() {
		
		this._static.COMMAND_STATUS = "1";
		this._static.COMMAND_RESIZE = "2";
		this._static.COMMAND_SETAUTOHEIGHT = "3";
		this._static.COMMAND_SETWIDGETSTATE = "4";
		this._static.COMMAND_START = "5";
		this._static.COMMAND_NAVIGATE = "6";
		
		this._static.STATUS_READY = "1";
		this._static.STATUS_OK = "2";
		this._static.STATUS_ERROR = "3";
		
		this._static.NO_DATA = ".";
		this._static.DELIMETER = "~";

		this._static.MIN_WIDGET_HEIGHT = 34; // Pixels
	},	
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_addPostMessageListener: function() {
		
		var context = this;
		this._handler = function(event) {
			context._receivePostMessage(event);
		};
		
		if (window.attachEvent) {
			// IE8
			window.attachEvent("onmessage", this._handler);
		} else if (window.addEventListener) {
			// IE9, FF
			window.addEventListener("message", this._handler, false);
		} else {
			this.logError(new Error("Failed to add the post message listener"), this._name);
			// TODO: Abort loading?
		}
				
	},			
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_removePostMessageListener: function() {
		
		if (window.attachEvent) {
			// IE8
			window.detachEvent("onmessage", this._handler);
		} else if (window.addEventListener) {
			// IE9, FF
			window.removeEventListener("message", this._handler, false);
		} else {
			this.logError(new Error("Failed to remove the post message listener"), this._name);
		}
				
	},		
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_receivePostMessage: function(event) {
		
		if (!event) {
			// Error
			this.logError(new Error("Command failed. The post message event is missing."), this._name);
			return false;
		} else if (!event.source) {
			this.logError(new Error("Command failed. The post message event source cannot be determined."), this._name);
			return false;
		} else if (event.origin !== this._iframeDomainURL) {
			this.logError(new Error("Command failed. The post message event origin does not match the server origin."), this._name);
			return false;
		}
		
		// TODO: Yield UI thread with timeout?
		
		var result = event.data.split(this._static.DELIMETER);
		// TODO: Validate result
		var id = result[0];
		var command = result[1];
		var data = result[2];
		
		// TODO: Figure out a way not to broadcast message to other loaders scripts in the same context
		if (id !== this._id) {
			this.logInfo(new Error("Received an unrecognized post message ID"), this._name);
			return false;
		}
		
		if (command === this._static.COMMAND_STATUS) {
			this.logInfo(new Error("COMMAND:STATUS"), this._name);
			if (data === this._static.STATUS_READY) {
				this.logInfo(new Error("DATA:READY"), this._name);
				this._setupContainer();
			} else if (data === this._static.STATUS_OK) {
				// TODO: Handle OK ?
				this.logInfo(new Error("DATA:OK"), this._name);
			} else if (data === this._static.STATUS_ERROR) {
				// TODO: Handle ERROR
				this.logInfo(new Error("DATA:ERROR"), this._name);
			} else {
				this.logError(new Error("Unrecognized command status received: " + data), this._name);
				return false;
			}
		} else if (command === this._static.COMMAND_RESIZE) {
			this.logInfo(new Error("COMMAND:RESIZE"), this._name);
			this._resizeIFrameHeight(data);
		} else if (command === this._static.COMMAND_NAVIGATE) {
			this.logInfo(new Error("COMMAND:NAVIGATE"), this._name);
			this._navigateToURL(data);
		} else {
			this.logError(new Error("Unrecognized command received: " + command), this._name);
			return false;
		}
	 
		return true;
	},		

	/*
 	 * @param String url
 	 * 
 	 * @return void
 	 */		
	_navigateToURL: function(url) {
		location.href = url;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_createIFrame: function() {
		
		var iframeSRC = this._frontsideURL + 
						"/_ajax-modules/jazz.ensemble.iframe.viewlet.container";
		if (this._debugMode) {
			this.logInfo(new Error("DEBUG MODE"), this._name);
			iframeSRC = iframeSRC + "?debug=true";
		}
		iframeSRC = iframeSRC + "#" + encodeURIComponent(this._getDomainURL());
		
		this._iframe = document.createElement("iframe");

		// Styling
		this._iframe.style.border = "1px solid #DCDCDC";
		this._iframe.style.borderRadius = "4pt 4pt 4pt 4pt";		
		// Hide horizontal scrollbar
		this._iframe.style.overflowX = "hidden";
		
		if (this._autoHeight) {
			this._height = this._static.MIN_WIDGET_HEIGHT;
			// Hide both vertical and horizontal scrollbar
			this._iframe.style.overflow = "hidden";
			// Hide both vertical and horizontal scrollbar (redundant)
			this._iframe.scrolling = "no";			
		}
		
		this._iframe.title = "Ajax Module";
		// TODO: Decode url ?
		this._iframe.src = iframeSRC;
		this._iframe.height = this._height;
		this._iframe.width = this._width;
		this._iframe.name = this._id;
		// Hide 3-D borders on IE8
		this._iframe.frameBorder = 0;
		
		this._containerNode.appendChild(this._iframe);
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return String
 	 */		
	_getDomainURL: function() {

		var location = window.location;
		
		var url = location.protocol + 
								'//' + 
								location.hostname;
		if (location.port) {
			url = url + ":" + location.port;
		}
	
		return url;
	},		
	
	/*
 	 * @param String id
 	 * 
 	 * @return String
 	 */		
	_getStringData: function(id) {
		
		var container = document.getElementById(id);
		if (!container) {
			this.logError(new Error("Failed to locate the target DOM element by ID: " + id), this._name);
			return null;
		}
		
		var data = container.innerHTML; 
		
		this._containerNode.removeChild(container);
		
		return data;
		
	},	
	
	/*
 	 * @param String id
 	 * 
 	 * @return Number
 	 */		
	_getNumberData: function(id) {
		
		var stringData = this._getStringData(id);
		var numberData = parseInt(stringData, 10);
		
		if (isNaN(numberData) ||
				(numberData < 0)) {
			return 0;
		}
		return numberData;		
		
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return Boolean
 	 */		
	_initContainerNode: function() {
		this._containerNode = document.getElementById(this._id);
		if (!this._containerNode) {
			return false;
		}
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_loadVariables: function() {
		
		// Note: For future use to address N-1 compatibility issues
		// HTML snippet version
		// this._containerNode.getAttribute("data-version"));
		// Version of the application server used to generate the HTML snippet
		// this._containerNode.getAttribute("data-server-version"));
		
		this._frontsideURL = this._getStringData(this._id + "_frontside-url");
		// TODO: Verify front side url defined
		this._widgetState = this._getStringData(this._id + "_widget-state");
		// TODO: Verify widget state is defined
		this._width = this._getNumberData(this._id + "_width");

		this._height = this._getNumberData(this._id + "_height");
		if (this._isAutoHeight(this._height)) {
			// Auto
			this._autoHeight = true;
		}
		
		this._loadIframeDomainURL();
	},		
	
	/*
 	 * @param Number height
 	 * 
 	 * @return Boolean
 	 */		
	_isAutoHeight: function(height) {
		if (height === 0) {
			return true;
		}
		return false;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_loadIframeDomainURL: function() {
		if (!this._frontsideURL) {
			this.logError(new Error("Undefined front-side URL encountered"), this._name);
			return;
		}
		
		// e.g. Front-side URL
		// https://localhost:9443/jazz
		// e.g. IFrame domain URL
		// https://localhost:9443
		
		var result = this._frontsideURL.match(/^(http[s]*:\/\/[\w\.\-]+[:\d]*)/);
		if ((result instanceof Array) &&
				(result.length === 2)) {
			this._iframeDomainURL = result[1];
		} else {
			this.logError(new Error("Failed to parse out the iframe domain URL from the front-side URL"), this._name);
		}
	},		
	
	/*
 	 * @param String command
 	 * @param String data
 	 * 
 	 * @return void
 	 */		
	_sendPostMessageData: function(command, data) {
		if (!data) {
			data = this._static.NO_DATA;
		}
		// Note: No need to specify ID. We have a direct reference to the iframe's window.
		var iframeWindow = this._iframe.contentWindow;
		var result = command + this._static.DELIMETER + data;
		iframeWindow.postMessage(result, this._iframeDomainURL);
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_setupContainer: function() {

		// Send auto height
		if (this._autoHeight) {
			this._sendPostMessageData(this._static.COMMAND_SETAUTOHEIGHT);
			delete this._autoHeight;
		}
		
		// Send widget state
		this._sendPostMessageData(this._static.COMMAND_SETWIDGETSTATE, this._widgetState);
		delete this._widgetState;

		// Start container
		this._sendPostMessageData(this._static.COMMAND_START);
	
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */	
	_isDebugMode: function() {
		var re = /debug=true/i;
		return re.test(window.location.search);
	},		
	
	/*
 	 * @param Number height
 	 * 
 	 * @return void
 	 */		
	_resizeIFrameHeight: function(height) {

		if (height < this._static.MIN_WIDGET_HEIGHT) {
			this.logWarning(new Error("Invalid computed height encountered: " + height), this._name);
			height = this._static.MIN_WIDGET_HEIGHT;
		} 
		
		this.logInfo(new Error("RESIZE: Height: " + height), this._name);
		this._iframe.height = height;
	},	
	
	/*
 	 * @param Error errorObject
 	 * @param String id
 	 * 
 	 * @return void
 	 */
	logInfo: function(errorObject, id) {
		// Log info only in debug mode
		if (console && 
				this._debugMode) {
			console.log(this.getErrorMessage("INFO", errorObject, id));
		}
	},	
	
	/*
 	 * @param Error errorObject
 	 * @param String id
 	 * 
 	 * @return void
 	 */
	logError: function(errorObject, id) {
		if (console) {
			console.error(this.getErrorMessage("ERROR", errorObject, id), errorObject);
		}
	},	
	
	/*
 	 * @param Error errorObject
 	 * @param String id
 	 * 
 	 * @return void
 	 */
	logWarning: function(errorObject, id) {
		if (console) {
			console.warn(this.getErrorMessage("WARNING", errorObject, id));
		}
	},		
	
	/*
 	 * @param String prefix
 	 * @param Error errorObject
 	 * @param String id
 	 * 
 	 * @return String
 	 */
	getErrorMessage: function(prefix, errorObject, id) {
		
		var classID = "";
		var lineNumber = "";
		var message = "";

		// Class name
		if (id) {
			classID = id + ": ";
		} 

		if (errorObject) {
			// Line number
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			// Error message
			if (errorObject.message) {
				message = errorObject.message;
			} else {
				message = "Message unavailable";
			}
		} else {
			// Error message
			message = "Undefined exception";			
		}
		
		var errorMessage = "[" + prefix + "]: " + 
							classID +
							lineNumber +
							message;
							
		return errorMessage;
	},
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	run: function() {
		
		this._name = "[loader]: " + this._id;
		this._debugMode = this._isDebugMode();
		
		if (!this._id) {
			this.logError(new Error("The Jazz widget ID is missing"), this._name);
			return;
		}
		
		this._initStatics();
		
		var status = this._initContainerNode(); 
		if (!status) {
			this.logError(new Error("Failed to locate the root Jazz widget DOM element by ID: " + this._id), this._name);
			return;
		}
		this._addPostMessageListener();
		this._loadVariables();
		this._createIFrame();
		
	}
	
};

}


