/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2009, 2012. All
 * Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("jazz.ui.internal.ScrollableView");

dojo.require("dijit._Widget");
dojo.require("dijit._Container");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.underlay");

(function(){
	
var FADE_OUT_DELAY = 600;
	
dojo.declare("jazz.ui.internal.ScrollableView", [dijit._Widget, dijit._Templated, dijit._Container], {
	height: "100%",
	templatePath: dojo.moduleUrl("jazz.ui", "internal/templates/ScrollableView.html"),
	widgetsInTemplate: true,
	overNode: null,
	scrollTop: 0,
	_scrollMin: 0,
	postCreate: function() {
		// handle scroll wheel
		this.connect(this.domNode, (!dojo.isMozilla ? "onmousewheel" : "DOMMouseScroll"), this._onMouseWheel);
	},
	checkHeight: function() {
		if (this._events || !this.domNode) return;

		var containerWrapper = this._containerWrapper,
			scrollHeight = this._scrollHeight = containerWrapper.scrollHeight,
			height = this._height = containerWrapper.clientHeight,
			scrollMin = this._scrollMin = this._header ? this._header.clientHeight : 0;
		
		dojo.toggleClass(this.domNode, "no-scroll", height >= scrollHeight);
		
		if (height < scrollHeight) {
			var scrollerHeight = this._scrollerHeight = Math.max(Math.round(((height - scrollMin) / scrollHeight) * (height - scrollMin)), 25);
			this._scrollMax = height - scrollerHeight;
			this._scroller.style.height = scrollerHeight - 2 + "px";
			this.scrollTo(containerWrapper.scrollTop / (scrollHeight - height) * (this._scrollMax - scrollMin) + scrollMin);
		} else {
			this.scrollTop = 0;
		}
		this.containerNode.style.paddingTop = scrollMin ? (scrollMin + "px") : "";
	},
	scrollTo: function(scrollTop) {
		if (!dojo.hasClass(this.domNode, "no-scroll")) {
			var scrollMin = this._scrollMin, scrollMax = this._scrollMax;
			scrollTop = this.scrollTop = Math.min(scrollMax, Math.max(scrollMin, scrollTop));
			this._scrollPos.style.top = scrollTop + "px";
			this._containerWrapper.scrollTop = (this._scrollHeight - this._height) * (scrollTop - scrollMin) / (scrollMax - scrollMin);
			dojo.toggleClass(this.domNode, "scrolled", scrollTop > scrollMin);
			dojo.toggleClass(this.domNode, "scrolled-down", scrollTop == scrollMax);
		}
	},
	_cancelSelect: function(e) {
		dojo.stopEvent(e);
		return false;
	},
	_setOverNodeAttr: function(overnode){
		if (!this._userEvents) 
			this._userEvents = [];
		else {
			while (this._userEvents.length) {
				this.disconnect(this._userEvents.pop());
			}
		}
		this._userEvents.push(this.connect(this.overNode||this.domNode, "onmouseover", this._mouseOver));
		this._userEvents.push(this.connect(this.overNode||this.domNode, "onmouseout", this._mouseOut));
		this._userEvents.push(this.connect(this.overNode||this.domNode, "onkeypress", this._keyPress));
	},
	startup: function() {
		this._header = dojo.query(".jazz-ui-ScrollableViewHeader", this.domNode)[0];	
		this.checkHeight();
		this.inherited(arguments);
	},
	_onMouseWheel: function(e) {
		dojo.stopEvent(e);
		if (e[(dojo.isMozilla ? "detail" : "wheelDelta")] * (dojo.isMozilla ? -1 : 1) < 0) {
	   		this.scrollTo(this.scrollTop + 30);
		} else {
			this.scrollTo(this.scrollTop - 30);
		}
	},
	_dragStart: function(e) {
		if (!this._events) this._events = [];
		this._events.push(this.connect(document.body, "onmouseup", this._mouseUp));
		this._events.push(this.connect(document.body, "onmousemove", this._drag));
		// Prevent text selection while dragging the scrollbar in IE9-.
		// For IE10, FF and WebKit, this is done in CSS with user-select rule.
		if (dojo.isIE < 10) {
			this._events.push(this.connect(document, "onselectstart", this._cancelSelect));
			this._events.push(this.connect(document, "ondragstart", this._cancelSelect));
		}
		this.checkHeight();
		dojo.addClass(document.body, "jazz-ui-ScrollableView-body-scrolling");
		jazz.ui.underlay.show({opacity: 1, color: "transparent", zIndex: 10000});
		this._initY = e.pageY;
		this._initScrollTop = this.scrollTop;
		this._drag(e);
		dojo.addClass(this.domNode, "scrolling");
	},
	_onClick: function(e) {
		var delta = this._height / this._scrollHeight * (this._height - this._scrollerHeight - this._scrollMin);
		var down = e.pageY - dojo.position(e.currentTarget, true).y > this.scrollTop;
		this.scrollTo(this.scrollTop + (down ? delta : -delta));
		this.checkHeight();
		return false;
	},
	_drag: function(e) {
		var delta = e.pageY - this._initY;
		this.scrollTo(this._initScrollTop + delta);
		dojo.stopEvent(e);
	},
	_keyPress: function(e) {
		if (e.charOrCode == dojo.keys.DOWN_ARROW) {
			this.scrollTo(this.scrollTop + 30);
		} else if (e.charOrCode == dojo.keys.UP_ARROW) {
			this.scrollTo(this.scrollTop - 30);
		} else if (e.charOrCode == dojo.keys.TAB){
			this.checkHeight();
			return;
		} else {
			return;
		}
		this._mouseOver();
		this._mouseOut();
		e.preventDefault();
	},
	_mouseUp: function(e) {
		if (this._events) {
			while(this._events.length){
				this.disconnect(this._events.pop());
			}
			delete this._events;
		}
		dojo.removeClass(document.body, "jazz-ui-ScrollableView-body-scrolling");
		dojo.removeClass(this.domNode, "scrolling");
		jazz.ui.underlay.hide();
		if (this._deferMouseOut) {
			delete this._deferMouseOut;
			this._mouseOut();
		}
		this.checkHeight();
	},
	_mouseOver: function(event) {
		if (this._deferMouseOut) {
			delete this._deferMouseOut;
		}
		if (this._fadeAnim)
			this._fadeAnim.stop();
		dojo.addClass(this.domNode, 'mouseOver');
		this.checkHeight();
	},
	_mouseOut: function(event) {
		var animationCleanup = dojo.hitch(this, function () {
			var scroller = this._scroller;
			dojo.removeClass(this.domNode, 'mouseOver');
			dojo.style(scroller, "opacity", "");
			if (scroller.style.removeAttribute) {
				// Work-around: IE fadeout bug
				scroller.style.removeAttribute("filter");
			}
			delete this._fadeAnim;
		});
		if (this._events) {
			this._deferMouseOut = true;
			return;
		}
		this._fadeAnim = dojo.animateProperty({
			node: this._scroller,
			duration: 500,
			delay: FADE_OUT_DELAY,
			properties: { opacity: { start: 0.17, end: 0}},
			onEnd: animationCleanup,
			onStop: animationCleanup
		}).play();
	},
	_onScroll:function(e) {
		this.bustOutHandler(false);
		this.bustOutHandler(true);
	},
	bustOutHandler: function(bustOut) {
		var containerWrapper = this._containerWrapper;
		if (bustOut) {
			// only when scrolling
			if (dojo.hasClass(this.domNode, "no-scroll"))
				return false;
			if (!this._onScrollHandle) this._onScrollHandle = this.connect(window, "onscroll", this._onScroll);
			var vp = dijit.getViewport();
			var scrollTop = this._absScrollTop = containerWrapper.scrollTop;
			containerWrapper.style.marginTop = - scrollTop + "px";
			containerWrapper.style.paddingBottom = scrollTop + "px";
			this._scroller.style.display = "none";
			var pos = dojo.position(this.domNode);
			if (this._header) {
				this._header.style.width = pos.w + "px";
			}
			dojo.style(this.domNode, {
				position: "fixed",
				overflow: "hidden",
				width: pos.w + "px",
				top: pos.y + "px",
				left: 0,
				right: 0,
				bottom: 0,
				padding: "0 " + (vp.w - pos.w - pos.x) + "px 0 " + pos.x + "px"
			});
			return true;
		} else {
			this.disconnect(this._onScrollHandle);
			delete this._onScrollHandle; 
			this._scroller.style.display = "";
			containerWrapper.style.marginTop = "";
			containerWrapper.style.paddingBottom = "";
			if (this._header) {
				this._header.style.width = "";
			}
			dojo.style(this.domNode, {
				position: "",
				overflow: "",
				width: "",
				top: "",
				left: "",
				right: "",
				bottom: "",
				padding: ""
			});
			containerWrapper.scrollTop = this._absScrollTop;
			this.checkHeight();
		}
	}
});

dojo.provide("jazz.ui.internal.ScrollableViewHeader");

dojo.declare("jazz.ui.internal.ScrollableViewHeader", [dijit._Widget, dijit._Templated, dijit._Container], {
	templateString: "<div class=\"jazz-ui-ScrollableViewHeader\" dojoattachpoint=\"containerNode\"></div>"
});

})();