/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.app.internal.util.Rootservices");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.client.xhr");
dojo.require("jazz.app.internal.util.XhrUtil");
dojo.require("jazz.app.proxy");
dojo.require("jazz.app.auth");
dojo.require("jazz.util.binding");

(function() {
var binding = jazz.util.binding;
var XhrUtil = jazz.app.internal.util.XhrUtil;

var documentCache={};
var bindingCache={};

dojo.declare("jazz.app.internal.util.Rootservices", null, {
	
	url          : null,
	_document    : null,
	_binding     : null,
	_friendsList : null,
	_title       : null,
	
	constructor: function(url) {
		this.url = url;
		this._document = documentCache[url];
		this._binding = bindingCache[url];
	},

	load : function(passive){
		if (this.isLoaded()) return XhrUtil.immediateCallback(this);
		
		var _this = this;
		
		var deferred = XhrUtil.getXml(this.url, {passive: passive});
		
		deferred.addCallback(function(doc){
			/* handle a passive call */
			if (doc) {
			   _this._document = doc;
			   _this._binding = binding.read(doc.documentElement, ROOTSERVICES);
			   documentCache[_this.url] = doc;
			   bindingCache[_this.url] = _this._binding;
			}
			
			return _this;
		});
		
		return deferred;
	},
	
	getTitle: function() {
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getTitle : not loaded");
			return null;
		}
		return this._binding.title;
	},
	
	getPublisher : function(){
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getPublisher : not loaded");
			return null;
		}
		return this._binding.publisher;
	},
	
	getAbout: function() {
		if (!this.isLoaded()){
			log.error("jazz.app.internal.util.Rootservices.getAbout : not loaded");
			return null;
		}
		return this._binding.about;
	},
	getServiceUrl : function (ns, name){
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getServiceUrl : not loaded");
			return null;
		}
		var service = binding.read(this._document.documentElement, {
			namespaces: {
				ns: ns,
				rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
			},
			value: ["string", "ns:" + name + "/@rdf:resource", "1"]
		}).value;
		if (!service) {
			console.warn("jazz.app.internal.util.Rootservices.getServiceUrl: no service for: " + ns + name);
			return null;
		}
		return service;
	},
	
	/*
	 * must be called after this is loaded
	 * 
	 * returns in a deferred
	 * [
	 *    {
	 *    	title: "title",
	 *    	rootServices : "http://someservice.com"
	 *    }
	 *  ]
	 */
	loadFriends : function(passive)	{
		if (!this.isLoaded()) return XhrUtil.immediateErrback(new Error("jazz.app.Rootservices.loadFriends rootservice not loaded"));
				
		if (this._friendsList) return XhrUtil.immediateCallback(this._friendsList);

		var deferred = XhrUtil.getXml(this._binding.friends, {schema: FRIENDS_LIST, passive:passive}); 

		var _this = this;
		deferred.addCallback(function (result){ 
			if (!result) return null;
			dojo.forEach(result.friends, function(f){
				f.title = decodeURI(f.title);
			});
			_this._friendsList 	= result.friends;
			return _this._friendsList;
		});
		
		return deferred;
	},
	
	isLoaded : function(){
		return (this._document&&true);
	},
	
	getCurrentUser : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getCurrentUser : not loaded");
			return null;
		}
		
		return this._binding.currentUser;
	},

	getProjectAreas : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getProjectAreas : not loaded");
			return null;
		}
		
		return this._binding.projectAreas;
	},
	
	getViewletServiceRoot : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getViewletServiceRoot : not loaded");
			return null;
		}
		
		return this._binding.viewletServiceRoot;
	},
	
	getViewletWebUIRoot : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getViewletWebUIRoot : not loaded");
			return null;
		}
		
		return this._binding.viewletWebUIRoot;
	},
		
	getHomeMenuIcon : function()
	{
		if (!this.isLoaded()){
			console.error("jazz.app.internal.util.Rootservices.getHomeMenuIcon : not loaded");
			return null;
		}
		
		return this._binding.homeMenuIcon;
	}	
});

var localRoot = null;

jazz.app.internal.util.Rootservices.loadLocal = function() 
{
	if (localRoot) return XhrUtil.immediateCallback(localRoot);
	
	var deferred = jazz.app.internal.util.Rootservices.load (net.jazz.ajax.contextRoot() + "/rootservices");

	deferred.addCallback(function (root) {
		if (root) localRoot = root;	
	});

	return deferred;
}

jazz.app.internal.util.Rootservices.load = function(url, passive)
{
	return (new jazz.app.internal.util.Rootservices(url)).load(passive);
}

var NAMESPACES = {
	dc: "http://purl.org/dc/terms/",
	jd: "http://jazz.net/xmlns/prod/jazz/discovery/1.0/",
	jdb: "http://jazz.net/xmlns/prod/jazz/dashboard/1.0/",
	jfs: "http://jazz.net/xmlns/prod/jazz/jfs/1.0/",
	jp06: "http://jazz.net/xmlns/prod/jazz/process/0.6/",
	oslc: "http://open-services.net/ns/core#",
	rdf: "http://www.w3.org/1999/02/22-rdf-syntax-ns#"
};

var ROOTSERVICES = {
	namespaces: NAMESPACES,
	about: ["string", "@rdf:about", "1"],
	currentUser: ["string", "jfs:currentUser/@rdf:resource", "1"],
	friends: ["string", "jd:friends/@rdf:resource", "1"],
	homeMenuIcon: ["string", "jdb:homeMenuIcon/@rdf:resource", "1"],
	publisher: ["string", "oslc:publisher/@rdf:resource", "1"],
	projectAreas: ["string", "jp06:projectAreas/@rdf:resource", "1"],
	title: ["string", "dc:title", "1"],
	viewletServiceRoot: ["string", "jd:viewletServiceRoot/@rdf:resource", "1"],
	viewletWebUIRoot: ["string", "jd:viewletWebUIRoot/@rdf:resource", "1"]
};

var FRIEND = {
	namespaces: NAMESPACES,
	rootServices: ["string", "jd:rootServices/@rdf:resource", "1"],
	title: ["string", "dc:title", "1"]
};

var FRIENDS_LIST = {
	namespaces: NAMESPACES,
	friends: [FRIEND, "jd:Friend", "*"]
};

})();
