/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("jazz.ensemble.internal.wizards.ExportWidgetWizardStep1");

/*
 * Imports 
 */
dojo.require("dojo.i18n");

dojo.require("dijit.form.TextBox");

dojo.require("jazz.ui.wizard.WizardStep");

dojo.requireLocalization("jazz.ensemble.internal", "Messages");

(function() {

/*
* Aliases 
*/
var WizardStep = jazz.ui.wizard.WizardStep;
	

/*
 * Constants 
 */
var DEFAULT_WIDTH = 400;


/*
 * Class
 */
dojo.declare("jazz.ensemble.internal.wizards.ExportWidgetWizardStep1",
		[dijit._Widget, dijit._Templated, WizardStep], {

	// Properties //----------------------------------------------------------->

	context: null,
	
	// ------------------------------------------------------------------------>

	/*
	 * Templates
	 */	
	templatePath: dojo.moduleUrl("jazz.ensemble", "internal/wizards/templates/ExportWidgetWizardStep1.html"),
	widgetsInTemplate: true,
	
	/*
 	 * @param Object args
 	 * 
 	 * @return ExportWidgetWizardStep1
 	 */
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("jazz.ensemble.internal", "Messages");
		
		this.stepTitle = this.messages.wizardsConfigureDimensions; 
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */	
	postCreate: function() {

		this.inherited(arguments);

		// Instance Variables //----------------------------------------------->

	
		// Initialization //--------------------------------------------------->
		
		this._initFields();
		this._initHandlers();
		
	},

	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initFields: function() {
		
		// Width
		if (this._isValidWidth(this.context.width)) {
			this._textBoxWidth.set('value', this.context.width);
		} else {
			// Default
			this._textBoxWidth.set('value', DEFAULT_WIDTH);
		}
		
		// Height
		if (this._isValidHeight(this.context.height)) {
			this._textBoxHeight.set('value', this.context.height);
		} else {
			// Auto
			this._textBoxHeight.set('value', this.messages.wizardsAuto);
		}
	},
	
	/*
 	 * @param Number
 	 * 
 	 * @return Boolean
 	 */		
	_isValidWidth: function(width) {		
		if (isNaN(width) ||
				(width < 100) ||
				(width > 9999)) {
			return false;
		} 
		return true;
	},		
	
	/*
 	 * @param Number
 	 * 
 	 * @return Boolean
 	 */		
	_isValidHeight: function(height) {
		if (isNaN(height) ||
				(height < 50) ||
				(height > 9999)) {
			return false;
		}
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 */		
	_initHandlers: function() {
		// Width
		this.connect(this._textBoxWidth.textbox, "onchange", this._handleEventOnChangeWidth);
		// Height
		this.connect(this._textBoxHeight.textbox, "onchange", this._handleEventOnChangeHeight);
	},	

	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnChangeWidth: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		
		var newWidth = parseInt(this._textBoxWidth.get('value'), 10);
		
		if (this._isValidWidth(newWidth)) {
			this.context.width = newWidth;
		} else {
			// Invalid number
			this.context.width = DEFAULT_WIDTH;
			this._textBoxWidth.set('value', DEFAULT_WIDTH);
		}
		
		return true;
	},			
	
	/*
 	 * @param Event event
 	 * 
 	 * @return Boolean
 	 */		
	_handleEventOnChangeHeight: function(event) {		
		if (event) {
			dojo.stopEvent(event);
		}
		
		var newHeight = parseInt(this._textBoxHeight.get('value'), 10);
		
		if (this._isValidHeight(newHeight)) {
			this.context.height = newHeight;
		} else {
			// Invalid number
			this.context.height = 0; // Auto
			this._textBoxHeight.set('value', this.messages.wizardsAuto);
		}
		
		return true;
	},		
	
	/*
 	 * @param void
 	 * 
 	 * @return void
 	 * @overrides super
 	 */		
	destroy: function() {
		
		this.inherited(arguments);
	}

});

})();
