/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.ProcessAreaDashboard");

dojo.require("com.ibm.team.dashboard.web.ui.Dashboard");
dojo.require("com.ibm.team.dashboard.web.ui.internal.ProjectInfo");

dojo.require("jazz.core.loader");

var Dashboard = com.ibm.team.dashboard.web.ui.Dashboard;
var ProjectInfo = com.ibm.team.dashboard.web.ui.internal.ProjectInfo;

/**
 * An abstract base class for a process area (e.g. team, project) dashboard. Provides
 * API and behavior common to these.
 * 
 * @since 4.0.1
 */
dojo.declare("com.ibm.team.dashboard.web.ui.ProcessAreaDashboard",
		[Dashboard], {
	
	/**
	 * (Optional)
	 * An array of widget class names to display in the details panel of the dashboard.
	 * 
	 * @type string[]
	 */
	contextInfoWidgets: [],
	
	// internal
	showBreadcrumbs: false,
	
	postCreate: function() {
		// create contextInfoWidget only if there are contributions
		if (this.contextInfoWidgets && this.contextInfoWidgets.length > 0) {
			this.contextInfoWidget = new ProjectInfo({scopeInfo: {scope: this.scope}});
		}
		this.inherited(arguments);
		if (this.contextInfoWidget) {
			// load the contributions
			var contextInfoWidgets = this.contextInfoWidgets;
			jazz.core.loader.batch_load_async(dojo.clone(contextInfoWidgets), dojo.hitch(this, function() {
				if (this.domNode) {
					for (var i=0;i<contextInfoWidgets.length;++i) {
						try {
							// instantiate and add each contribution
							var clazz = dojo.getObject(contextInfoWidgets[i]);
							var widget = new clazz({dashboard: this, projectInfo: this.contextInfoWidget});
							this.contextInfoWidget.addChild(widget);
						}
						catch (e) {
							console.error(e);
						}
					}
				}
			}));
		}
	},
	
	destroy: function() {
		this.inherited(arguments);
		if (this.contextInfoWidget) {
			this.contextInfoWidget.destroyRecursive();
			delete this.contextInfoWidget;
		}
	}
});

})();
