/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.authoring.web.common.ui.ProcessGuidance"); //$NON-NLS-1$

dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$

dojo.require("jazz.ui.HoverPopup"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.RestApis"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.dialogs.GuidanceSelectionDialog"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

/*
 * com.ibm.team.process.authoring.web.common.ui.ProcessGuidance
 * 
 * This widget provides a hover-over icon that is meant to provide context sensitive process guidance.
 * 
 * parameters:
 *   uri: (String) the URI of the process guidance, this can be null if a new guidance should be selected for the widget
 *   scope: (String) the URI of the scope (project area, process template...) to select the process guidance for this widget, this parameter is not used if uri is not null
 * callbacks:
 *   onGuidanceChanged: this function will get called when the guidance (uri) of this widget has been changed. The new uri will be passed to this function in the first argument, the second argument is the guidance title. 
 * usage:
 *   var guidanceHook = document.createElement("span");
 *   // create a view-only guidance widget
 *   var guidanceWidget = new com.ibm.team.process.authoring.web.common.ui.ProcessGuidance({uri: "aGuidanceURI", viewOnly: true);
 *   // create a guidance widget that allows selecting guidance
 *   var guidanceWidget = new com.ibm.team.process.authoring.web.common.ui.ProcessGuidance({
 *   	uri: "aGuidanceURI", 
 *   	scope: "aProjectAreaURI",
 *   	onGuidanceChanged: function(newGuidanceURI, newGuidanceTitle) {
 *   		...
 *   	}
 *   );
 *   // hook the guidance widget
 *   guidanceHook.appendChild(guidanceWidget);
 *   contextDomNode.appendChild(guidanceHook);
 * 
 * @since 4.0.1
 */

var RestApis = com.ibm.team.process.authoring.web.common.client.internal.RestApis;
var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
var GuidanceSelectionDialog = com.ibm.team.process.authoring.web.common.ui.dialogs.GuidanceSelectionDialog;

dojo.declare("com.ibm.team.process.authoring.web.common.ui.ProcessGuidance", jazz.ui.ResourceLink, { //$NON-NLS-1$
	
	/**
	 * The URI of the scope (project area, process template...) to select the process guidance for this widget
	 */
	scope: null,
	
	/**
	 * If true then this widget is only for viewing the process guidance. The default value is false that allows 
	 * selecting different process guidance for the widget. 
	 */
	viewOnly: false,
	
	/**
	 * A function that will be called when the guidance (uri) of this widget has been changed. The new uri will be
	 * passed to this function in the first argument, the second argument is the guidance title. 
	 */
	onGuidanceChanged: null,
	
	_title: null,
	
	constructor: function(args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	},
	
	postCreate: function() {
		if (this.uri) {
			this.lazyFetch = true;
			this.label = this.messages.ProcessGuidance_title;
			this.inherited(arguments);
		} else {
			this.popup = new jazz.ui.HoverPopup({
				around: this.domNode,
				createContents: dojo.hitch(this, this._createContents)
			});
		}
	},
	
	refresh: function() {
		this._title = null;
		this.inherited(arguments);
	},
	
	_createContents: function() {
		var div = dojo.create("div", {innerHTML: "No process guidance available. "}); //$NON-NLS-1$ //$NON-NLS-2$
		if (!this.viewOnly) {
			var anchor = dojo.create("a", { //$NON-NLS-1$
				href: "javascript:void(0);", //$NON-NLS-1$
				innerHTML: this.messages.ProcessGuidance_selectGuidance,
				title: this.messages.ProcessGuidance_selectGuidance
			}, div);
			this.connect(anchor, "onclick", this._onSelectGuidance); //$NON-NLS-1$
		}
		return div;
	},
	
	buildRendering: function() {
		this.domNode = dojo.create("img"); //$NON-NLS-1$
		this.domNode.src = this._blankGif;
		this.domNode.alt = this.messages.ProcessGuidance_title;
		dojo.addClass(this.domNode, "com-ibm-team-process-authoring-web-common-ui-ProcessGuidance"); //$NON-NLS-1$
	},
	
	_onSelectGuidance: function() {
		if (this.scope) {
			this.selectionDialog = new GuidanceSelectionDialog({
				scope: this.scope,
				selectedUrl: this.uri, // url of practice or practice element
				onOk: dojo.hitch(this, this._dialogOk)
			});
		}
	},
	
	_dialogOk: function() {
		// set label and id in the settings dialog for the selected field, section, practice, practice element
		var practice = this.selectionDialog.getSelectedPracticeOrPracticeElement();
		this.onGuidanceChanged(practice.url, practice.label); //$NON-NLS-1$
		this.selectionDialog.close();
		
		if (this.uri) {
			this.uri = practice.url;
			this.refresh();
		} else {
			this.destroyRecursive();
		}
	},
	
	fetchTitle: function(load, error) {
		if (this.uri) {
			if (this.popup && this.popup.compactRendering) {
				load(this.popup.compactRendering.title);
			} else {
				if (this._title == null) {
					// TODO: fetch only the title, not the entire resource
					// 227327: Provide REST API to fetch only a specified set of properties of a process authoring resource
					ProcessAuthoringClient.getResource({
						url: this.uri,
						load: dojo.hitch(this, function(data, ioArgs) {
							this._title = UIHelper.getTitle(data);
							if (this._title == null) {
								this._title = ""; //$NON-NLS-1$
							}
							load(this._title);
						}),
						error: error
					});
				} else {
					load(this._title);
				}
			}
		} else {
			load(""); //$NON-NLS-1$
		}
	}
		
});

})();
