/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.details.TimelineInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("net.jazz.ajax.contextRoot");
dojo.require("jazz.app.context");
dojo.require("jazz.client.xhr");
dojo.require("jazz.util.binding");
dojo.require("jazz.util.i18n");
dojo.require("jazz.app.i18n");
dojo.require("dojo.date.stamp");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "TimelineInfoMessages");

(function() {

var DashboardConstants = com.ibm.team.dashboard.web.ui.DashboardConstants;

var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var messages;

var DEFAULT_VISIBLE = 3;

dojo.declare("com.ibm.team.dashboard.web.ui.details.TimelineInfo", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/TimelineInfo.html"),
	
	constructor: function(args) {
		if (!messages) messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "TimelineInfoMessages");
		this.messages = messages;
	},
	
	postCreate: function() {
		this._projectEditorLink.href = this._titleLink.href = net.jazz.ajax.contextRoot() +
				"/admin#action=com.ibm.team.process.editProjectArea&itemId=" + jazz.app.context.get().itemId + "&tabId=" + com.ibm.team.process.web.common.internal.ProcessConstants.TabEnum.TIMELINE;
		if (this.dashboard.scope === DashboardConstants.SCOPE_PROJECT_AREA) {
			this.projectInit();
		} else if (this.dashboard.getBundle()) {
			this.teamInit();
		} else {
			this.dashboard.addListener(dojo.hitch(this,this.teamInit), "onload");
		}
	},
	
	projectInit: function() {
		var dfd = jazz.client.xhrGet({
			url: jazz.app.context.get().uri + "/timelines",
			headers: {
				accept: "application/rdf+xml"
			},
			handleAs: "xml"
		});
		dfd.addCallback(dojo.hitch(this,function(result){
			this._timelines = jazz.util.binding.bind(result,root).getTimelines();
			if (!this._timelines.length) {
				this._handleUAMessage(messages.noTimelines);
			} else {
				for (var i = 0; i < this._timelines.length; i++) {
					if (this._timelines[i].isProjectTimeline()) {
						var timeline = this._timelines[i];
						break;
					}
				}
				if (!timeline)
					timeline = this._timelines[0];
				var url = timeline.getUrl();
				this._timelineItemId = url.substring(url.lastIndexOf("/") + 1);
				this._subHeading.innerHTML = timeline.getLabel();
				this.fetchMainIterations();
			}
		}));
	},
	
	teamInit: function() {
		var bundle = this.dashboard.getBundle();
		var sr = new ServiceRequest("com.ibm.team.process.internal.service.web.IProcessWebUIService","getTeamAreaByUUID", {uuid: bundle.scopeItem.itemId});
		var serviceObj = {
			success: dojo.hitch(this, function(team) {
				if (!team.developmentLine) {
					this._handleUAMessage(messages.noTimelines);
				} else {
					this._subHeading.innerHTML = team.developmentLine.label;
					this._timelineItemId = team.developmentLine.itemId;
					this.fetchMainIterations();
				}
			}),
			failure: dojo.hitch(this, function(error) {
				console.error(error);
			})
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		TeamServerClient.invokeService(sr, srh);
	},
	
	fetchMainIterations: function() {
		// TODO: We should be using the process rest APIs for this, but they don't expose "current" iterations, see 193554
		var sr = new ServiceRequest("com.ibm.team.process.internal.service.web.IProcessWebUIService","getIterations", {uuid: this._timelineItemId});
		var serviceObj = {
			success: dojo.hitch(this, function(iterations) {
				this.parseMainIterations(iterations);
			}),
			failure: dojo.hitch(this, function(error) {
				console.error(error);
			})
		};
		var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
		TeamServerClient.invokeService(sr, srh);
	},
	
	parseMainIterations: function(iterations) {
		var current = this.findCurrentIteration(iterations);
		if (typeof current === "undefined") {
			this._handleUAMessage(messages.noCurrentIteration);
		} else {
			while (!iterations[current].leaf && iterations[current].iterations) {
				iterations = iterations[current].iterations;
				var next = this.findCurrentIteration(iterations);
				if (typeof next === "undefined") {
					break;
				}
				current = next;
			}
			this._mainIterations = iterations;
			this._currentIteration = current;
			this.populateMainIterations();
		}
	},
	
	_handleUAMessage: function(message) {
		var permissions = dojo.getObject("com.ibm.team.dashboard.internal.permissions");
		if (!permissions) {
			this.dashboard.addListener(dojo.hitch(this,this._handleUAMessage, message), "onload");
			return;
		}
		if (!permissions.modify) {
			this.projectInfo.removeChild(this);
			this.destroy();
			return;
		}
		dojo.style(this._bottomArea, {display: "none"});
		dojo.addClass(this._content, "ua-message");
		this._content.innerHTML = "";
		var link = jazz.util.i18n.addMessage(this._content, message)[0];
		link.href = this._titleLink.href;
	},
	
	populateMainIterations: function(showAll) {
		this._content.innerHTML = "";
		var l = this._mainIterations.length;
		if (showAll) {
			var start = 0;
		} else {
			var c = start = this._currentIteration;
			if (l <= DEFAULT_VISIBLE) {
				start = 0;
				dojo.addClass(this._showAllWrapper, "hidden");
			} else if (c + DEFAULT_VISIBLE > l) {
				start = l - 3;
			}
		}
		for (var i = start; i < l && (showAll || i < start + DEFAULT_VISIBLE); i++) {
			var iteration = this._mainIterations[i];
			var className = "iteration";
			if (iteration.current) className += " current";
			if (iteration.completed) className += " completed";
			var html = 	"<div class='icon'></div>" +
						"<div class='label-wrapper'>" +
							"<span class='label'>" + iteration.label + "</span>" +
							"<div class='end-date'>" + this._getEndDateString(iteration) + "</div>" +
						"</div>";
			dojo.create("div", {className: className, innerHTML: html}, this._content);
		}
	},
	
	findCurrentIteration: function(iterations) {
		if (!iterations) return;
		for (var i = 0; i < iterations.length; i++)
			if (iterations[i].current)
				return i;
	},
	
	_getEndDateString: function(iteration) {
		if (!iteration.endDate) return "";
		var date = jazz.app.i18n.date.locale.format(new jazz.app.i18n.Date(dojo.date.stamp.fromISOString(iteration.endDate)), {formatLength: "medium", selector: "date"});
		return dojo.string.substitute(messages.endDate, [date]);
	},
	
	_toggleShowAll: function(e) {
		if (e) e.preventDefault();
		dojo.toggleClass(this._showAllLink, "show-less");
		var showAll = dojo.hasClass(this._showAllLink, "show-less");
		this._showAllLabel.innerHTML = showAll ? messages.showLess : messages.showMore;
		this.populateMainIterations(showAll);
	}
});

var NAMESPACES = {
	jp: "http://jazz.net/xmlns/prod/jazz/process/1.0/"
};
var Iteration = {
	namespaces: NAMESPACES
};
var Timeline = {
	namespaces: NAMESPACES,
	getLabel: ["string", "jp:label", "1"],
	getUrl: ["string", "jp:url", "1"],
	getIterationsUrl: ["string", "jp:iterations-url", "1"],
	isProjectTimeline: ["boolean", "jp:project-timeline", "1"]
};
var root = {
	namespaces: NAMESPACES,
	getTimelines: [Timeline, "jp:timelines/jp:timeline", "*"]
};

})();