/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.jfs.web.ui.internal.ApplicationAdmin"); 

	dojo.require("jazz.app.internal.applicationAbout");
	dojo.require("jazz.app.internal.util.Rootservices");
	dojo.require("com.ibm.team.app.web.internal.AdminApplication");
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
	dojo.require("com.ibm.team.repository.web.client.session.Session");
	dojo.require("com.ibm.team.repository.web.ui.internal.SystemAlertsWidget");
	dojo.require("dojo.i18n");
	dojo.require("jazz.app.context");
	dojo.requireLocalization("com.ibm.team.jfs.web.ui.internal", "Messages");
	dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages");

	var Rootservices = jazz.app.internal.util.Rootservices;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var Session = com.ibm.team.repository.web.client.session;
	var AdminApplication = com.ibm.team.app.web.internal.AdminApplication;
	var OWNING_APP_KEY = "owningApplicationKey";
	var OSLC_NS = "http://open-services.net/ns/core#";
	var OSLC_PUBLISHER = "publisher";
	var messages, adminAppMessages;

	dojo.declare("com.ibm.team.jfs.web.ui.internal.ApplicationAdmin", AdminApplication, { 

		iconLarge: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/images/admin.png"),

		constructor: function(){
			this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.jfs.web.ui.internal", "Messages");
			this.adminAppMessages = adminAppMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "AdminApplicationMessages");
			var queryParams = dojo.queryToObject(window.location.search.slice(1));	
			this._owningApplicationKey = queryParams.owningApplicationKey;
			this._bannerTitle = this.messages.defaultTitle;
			this._pageTitle = this.adminAppMessages.applicationAdmin;
			var context = jazz.app.context.set({
				applicationAdmin: true
			});
		},
		
		//Overriding from AdminApplication.js
		_handleInitData: function(initData) {
			Session.init(initData);	
			var contrib = Session.getAuthenticatedContributor();
			if (contrib) {
				this.config.user = {
					name: contrib.name,
					userId: contrib.userId
				};	
			} else {
				this._handleNoContributor();
			}
				
			if (this._userIsAdmin == null || !this._userIsAdmin) {
				this._suppressAdminOnlyPages();
			} 
			
			this.config.buttons.push({
				label: adminAppMessages.admin,
				className: "admin-menu",
				align: "right",
				menuProvider: dojo.hitch(this,this._adminMenuProvider)
			});
			
			this.config.buttons.push({
				label: adminAppMessages.help,
				className: "help-menu j-ar-mirror",
				align: "right",
				menuProvider: dojo.hitch(this,this._helpMenuProvider)
			});
			
			this._aboutStr = adminAppMessages.aboutServer;
			var node = dojo.create("a", { href: "#", innerHTML: this.ui.get("title"), title: this._aboutStr });
			dojo.connect(node, "onclick", this, this._openAboutBox);
			this.ui.set("title", node);
			
			var serverMessages = initData["com.ibm.team.repository.service.compatibility.util.serverMessages"];
			var alertsArgs = {messages: []};
			if(serverMessages && serverMessages.items && serverMessages.items.length) {
				alertsArgs.messages = serverMessages.items;
			}
			this.ui.set("info", new com.ibm.team.repository.web.ui.internal.SystemAlertsWidget(alertsArgs));
			
			if (com.ibm.team.repository.web.logouturi) {
				this.config.logout = { href: com.ibm.team.repository.web.logouturi };
			}
			
			if (!this._owningApplicationKey) {
				var mode = dojo.getObject("com.ibm.team.repository.server.mode");
				if(mode == "STANDALONE") {
					this._startApplication();
				} else {
					this._handleInvalidKey();
				}
			} else {
				// Verify the owningApplicationKey matches an application
				var srh = new ServiceResponseHandler(this, "_handleFriend", "_handleError");
				ProcessClient.getApplication({
					owningApplicationKey: this._owningApplicationKey
				}, srh);
			}
			
		},
		
		_handleFriend: function(response) {
			var friend = dojo.fromJson(response);
			if(friend && friend.rootServices) {
				this._getPublisher(friend.rootServices);
			} else {
				this._handleInvalidKey();
			}
		},

		_handleInvalidKey: function() {
			var summary = dojo.string.substitute(this.messages.applicationKeyNotValid, [this._owningApplicationKey || OWNING_APP_KEY]);
			this.ui.set("bannerTitle", this._bannerTitle); 
			this.onFailure(new Error(summary));
		},
		
		_handleError: function(error) {
			this.onFailure(error, this.messages.errorFetchingFriends);
		},
		
		_startApplication: function() {
			this.ui.set("bannerTitle", this._bannerTitle);
			this.workbench._setDefaultTitle(this._pageTitle + " - Rational Jazz Team Server"); //$NON-NLS-1$		
			this._disableHiding();
			this.start();	
		},
		
		_disableHiding: function () {
			var hideProcessPages = dojo.getObject("com.ibm.team.process.service.hidePagesWebUIInitializationDataService");
			if (hideProcessPages) {
				dojo.setObject("com.ibm.team.process.service.hidePagesWebUIInitializationDataService", null);
			}
		},
		
		_getPublisher: function(rootServicesURL) {
			var rootDeferred = Rootservices.load(rootServicesURL, /* passive */ true);
			rootDeferred.addCallback(dojo.hitch(this, function(root) {
				var publisher = root.getPublisher();
				if(publisher) {
					var appAbout = jazz.app.internal.applicationAbout.get(publisher) || {};
					var appTitle = "";
					if(appAbout.title) {
						appTitle = " - " + appAbout.title;
						this._pageTitle += appTitle;
						if(appAbout.instanceName) {
							appTitle += " (" + appAbout.instanceName + ")";
						}
					}
					this._bannerTitle = this.messages.defaultTitle + appTitle;			
				}
				this._startApplication();
			}));		
			rootDeferred.addErrback(dojo.hitch(this, function(error){
				this.onFailure(error);
			}));	
		}		
	});
})();
