/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.viewlets.web.ui.internal.ConnectionStatusViewlet");

dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("dojo.date.locale");
dojo.require("dojox.gfx");

dojo.require("jazz.client.xhr");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.ui.toolbar.Button");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet");
dojo.require("com.ibm.team.dashboard.web.util.internal.Util");
dojo.require("com.ibm.team.repository.web.transport.ServiceRequest");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.transport.TeamServerClient");
dojo.require("com.ibm.team.repository.web.client.session.Session");

dojo.requireLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ConnectionStatusMessages");

(function() {

/*
 * Aliases
 */
var Util = com.ibm.team.dashboard.web.util.internal.Util;
var ServiceRequest = com.ibm.team.repository.web.transport.ServiceRequest;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var TeamServerClient = com.ibm.team.repository.web.transport.TeamServerClient;
var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var CONNECTION_SERVICE_URI = "com.ibm.team.repository.service.internal.IServerConnectionStatusRestService";
var CONTENT_SERVICE_URI = "com.ibm.team.repository.service.ITeamServerStatusContentService";
var PING_TEST = "getPingTime";
var DB_PING_TEST = "getDatabasePingTime";
var DOWNLOAD_TEST = "downloadTest";
var UPLOAD_TEST = "uploadTest";
var NUM_OF_PRE_PINGS = 10;
var NUM_OF_PINGS_DEFAULT = 20;
var NUM_OF_PINGS_MAX = 40;
var NUM_OF_PINGS = NUM_OF_PINGS_DEFAULT;
var NUM_OF_DB_PINGS = 10;
var THROUGHPUT_TEST_SIZE = 4 * 1024 * 1024; //4MB
var TEST_INTERVAL = 500; //ms 
var PING_TEST_TIMEOUT = 5*1000; //ms, 5 sec
var DB_PING_TEST_TIMEOUT = 5*1000; //ms, 5 sec
var TRANSFER_TEST_TIMEOUT = 7*60*1000; //7 minutes

//graphic constants
var PING_BAR_MAX_VALUE = 600;
var PING_BAR_NUM_MINOR_DIVIDERS = 1;
var PING_BAR_LABEL_OFFSET = -2;
var DB_PING_BAR_MAX_VALUE = 6;
var DB_PING_BAR_NUM_MINOR_DIVIDERS = 0;
var THROUGHPUT_BAR_MAX_VALUE = 600;
var THROUGHPUT_BAR_NUM_MINOR_DIVIDERS = 1;
var THROUGHPUT_BAR_LABEL_OFFSET = -2;

var SURFACE_WIDTH = 260;
var SURFACE_HEIGHT = 45;
var BAR_ORIGIN_X = 11;
var BAR_ORIGIN_Y = 15;
var BAR_WIDTH = 200;
var BAR_HEIGHT = 15;
var MARK_LENGTH_S = 6;
var MARK_LENGTH_L = 10;
var LABEL_ORIGIN_X = 8;
var LABEL_ORIGIN_Y = 40;
var ARROW_WIDTH = 10;
var ARROW_ORIGIN_Y = 12;
var ARROW_LABEL_OFFSET_X = -3;
var ARROW_LABEL_OFFSET_Y = -2;

var BAR_COLOR_A = "#77B359"; // "green";
var BAR_COLOR_B = "#F3F465";  //"yellow";
var BAR_COLOR_C = "#CC6666"; //"red"; 
var MARK_COLOR = "#333333"; //gray
var LABEL_COLOR = "#333333";
var LABEL_FONT = {family:"Arial", size:"7pt"};
var ARROW_COLOR = "#333333";


/*
 * A viewlet that runs and summarizes the client server connection status.
 */
dojo.declare("com.ibm.team.dashboard.viewlets.web.ui.internal.ConnectionStatusViewlet",
	Viewlet, {
	
	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.viewlets.web", "ui/internal/templates/ConnectionStatusViewlet.html"),

	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.viewlets.web.ui.internal", "ConnectionStatusMessages");
		

		this.PING_BAR_LABELS = ["0", "100", "200", "300", "400", "500", "600"];
		this.DB_PING_BAR_LABELS = ["0", "1", "2", "3", "4", "5", "6"];
		this.THROUGHPUT_BAR_LABELS = ["600+", "500", "400", "300", "200", "100", "0"];
		
		this.PING_BAR_LABELS[this.PING_BAR_LABELS.length - 1] = dojo.string.substitute(this.messages.plus_ms, [this.PING_BAR_LABELS[this.PING_BAR_LABELS.length - 1]]);
		this.DB_PING_BAR_LABELS[this.DB_PING_BAR_LABELS.length - 1] = dojo.string.substitute(this.messages.plus_ms, [this.DB_PING_BAR_LABELS[this.DB_PING_BAR_LABELS.length - 1]]);
		this.THROUGHPUT_BAR_LABELS[this.THROUGHPUT_BAR_LABELS.length - 1] = dojo.string.substitute(this.messages.brackets_kb_s, [this.THROUGHPUT_BAR_LABELS[this.THROUGHPUT_BAR_LABELS.length - 1]]);
		
		this.firstRefreshFromLoad = true;	
		this.firstRun = true;
	},
	
	init: function() {
		var site = this.getSite();
		if (!site.isRemote || site.isRemote()) {
			this._mb = new jazz.ui.MessageBox();
			dojo.place(this._mb.domNode, this._messageNode,"first");
			this._mb.showMessage({
				type: jazz.ui.MessageBox.STATUS_TYPES.WARNING,
				summary: dojo.create("div",{innerHTML: this.messages.remoteViewletWarning}),
				closable: false
			});
		}
		
		this._serverNode.innerHTML = this.getWebURL();
		this._hoverPopups = [];
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._serverLabelNode, html: this.messages.serverDescription, openDelay: 300}) );
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._refreshLabelNode, html: this.messages.refreshDescription, openDelay: 300}) );
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._pingLabelNode, html: this.messages.pingDescription, openDelay: 300}) );
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._dbPingLabelNode, html: this.messages.dbPingDescription, openDelay: 300}) );
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._downloadLabelNode, html: this.messages.downloadDescription, openDelay: 300}) );	
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: this._uploadLabelNode, html: this.messages.uploadDescription, openDelay: 300}) );	
		
		this._runButton = new jazz.ui.toolbar.Button({
			label: this.messages.runTestButton,
			onClick: dojo.hitch(this, this._runTests)
		});
		this._runButtonNode.appendChild(this._runButton.domNode);
	},	
	
	destroy: function() {
		if(this._hoverPopups) {
			for(var p in this._hoverPopups) {
				this._hoverPopups[p].destroy();
				this._hoverPopups[p] = null;
			}
			this._hoverPopups = null;
		}
		if(this._runButton) {
			this._runButton.destroy();
			this._runButton = null;
		}
		if(this._mb) {
			this._mb.destroy();
			delete this._mb;
		}
		this.inherited(arguments);
	},

	/*
	 * Returns an appropriate URL for the viewlet's title link.
	 */
	/* string */ getTitleHyperlinkURL: function() {
		return this.getWebURL() + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.server";
	},
	
	refresh: function() {		
		// workaround viewlet is always refreshed right after loading.
		if(this.firstRefreshFromLoad === true) {
			this.firstRefreshFromLoad = false;
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();
		}
		else
			this._runTests();
	},
	
	_runTests: function() {
		
		if(this.firstRun === true) {
			if(this._runButton) {
				this._runButton._setLabelAttr(this.messages.rerunTestButton);
			}
		}
		
		var serverPingTimes = this.getPreference("serverPingTimes");
		if (serverPingTimes && serverPingTimes != NUM_OF_PINGS) {
			serverPingTimes = parseInt(serverPingTimes, 10);
			if(serverPingTimes >= NUM_OF_PINGS_DEFAULT && serverPingTimes <= NUM_OF_PINGS_MAX)
				NUM_OF_PINGS = serverPingTimes;
			else
				NUM_OF_PINGS = NUM_OF_PINGS_DEFAULT;
		}
		this.getSite().setLoading(true);
		delete this._testId;
		delete this._serverPingResponses; //[] of Response Object - testId, sequenceId, startTime, endTime, error, isTimedout
		delete this._dbPingResponses; //[] of Response Object - testId, sequenceId time, error, isTimedout
		delete this._downloadTestResponse; //in KB/s //Response Object - testId, startTime, endTime, error, isTimedout
		delete this._uploadTestResponse; //in KB/s //Response Object - testId, startTime, endTime, error, isTimedout
		this._serverPingResponses = [];		
		this._dbPingResponses = [];
		this._stopAllTimers();	
		this._pingTestStarted = false;
		this._dbPingTestStarted = false;
		this._downloadTestStarted = false;
		
		try {
			var authenticatedContributor = getAuthenticatedContributor();
		} catch (errorObject) {
			var errorMessage = "[ERROR]: " + this.id + ": " + errorObject.message;
			console.error(errorMessage);
		}
		var userName = null;
		if(authenticatedContributor && authenticatedContributor.name)
			userName = authenticatedContributor.name;
		
		//use the timestamp string as an unique ID for test from the same client.
		var dateTime = new Date();
		this._testId = dateTime.getTime(); 
		
		if(userName) {
			this._serverRefreshNode.innerHTML = dojo.string.substitute(this.messages.refreshTimeUser, 
					[dojo.date.locale.format(dateTime, {formatLength: "medium"}), userName]);
		}
		else {
			this._serverRefreshNode.innerHTML = dojo.date.locale.format(dateTime, {formatLength: "medium"});
		}
				
		this._resetTestLabels();		
		
		//The first couple of server ping tests(requests of the same URL) usually take time to create a TCP connection first. 
		//after that, the rest of the ping tests tend to stablize and don't have that overhead.
		//send a couple "pre-runs" to reduce noise caused by connection creation
		this._preServerPingTest(this._testId);

		//start ping test if _preServerPingTest() timed out
		this._testTimeOutTimer = setTimeout(dojo.hitch(this, this._serverPingTest, 1, this._testId), PING_TEST_TIMEOUT);
		
		// after _preServerPingTest() all responded or timed out, run tests in this order: 
		// #1 server latency test, after finished, run 
		// #2 DB latency test, then 
		// #3 download speed, lastly 
		// #4 upload speed 
	},
	
	_resetTestLabels: function() {
		dojo.empty(this._pingLabelNode);
		dojo.create("span", {innerHTML: this.messages.pingTimeLabel}, this._pingLabelNode);
		dojo.empty(this._dbPingLabelNode);
		dojo.create("span", {innerHTML: this.messages.dbPingTimeLabel}, this._dbPingLabelNode);
		dojo.empty(this._downloadLabelNode);
		dojo.create("span", {innerHTML: this.messages.downloadSpeedLabel}, this._downloadLabelNode);
		dojo.empty(this._uploadLabelNode);
		dojo.create("span", {innerHTML: this.messages.uploadSpeedLabel}, this._uploadLabelNode);
		
		dojo.empty(this._pingTimeNode);
		this._pingTimeNode.innerHTML = this.messages.notStarted;
		dojo.empty(this._uploadSpeedNode);
		this._uploadSpeedNode.innerHTML = this.messages.notStarted;
		dojo.empty(this._downloadSpeedNode);
		this._downloadSpeedNode.innerHTML = this.messages.notStarted;
		dojo.empty(this._dbPingTimeNode);
		this._dbPingTimeNode.innerHTML = this.messages.notStarted;
	},
	
	_stopAllTimers: function() {
		if(this._testIntervalTimer) {
			clearTimeout(this._testIntervalTimer);
			delete this._testIntervalTimer;
		}
		if(this._testTimeOutTimer) {
			clearTimeout(this._testTimeOutTimer);
			delete this._testTimeOutTimer;
		}	
	},
	
	_preServerPingTest: function(testId) {			
		
		this.prePingResponses = 0;
		for(var p = 1; p <= NUM_OF_PRE_PINGS; p ++) {
			var srh = new ServiceResponseHandler({
				success: dojo.hitch(this, this._preServerPingTestCallback), 
				failure: dojo.hitch(this, this._preServerPingTestCallback)
				}, 
				"success", 
				"failure");
			var param = {testId: testId, prerunId: p};
			var req = new ServiceRequest(CONNECTION_SERVICE_URI, PING_TEST, param);
			TeamServerClient.invokeService(req, srh);
		}
	},
	
	_preServerPingTestCallback: function() {
		this.prePingResponses ++;
		
		if(NUM_OF_PRE_PINGS == this.prePingResponses && !this._pingTestStarted)
			this._serverPingTest(1, this._testId);
	},
	
	_serverPingTest: function(pingCounter, testId) {
		
		//already running a different test, ignore runs from obsoleted test.
		if(!this._testId || !testId || testId != this._testId)
			return;

		this._stopAllTimers();
		this._pingTestStarted = true;
		
		if(pingCounter <= NUM_OF_PINGS) {
			
			if(pingCounter === 1) {
				var samplesLabel = dojo.string.substitute(this.messages.numOfSamples, [NUM_OF_PINGS]);
				dojo.create("br", null, this._pingLabelNode);
				var samplesNode = dojo.create("span", {innerHTML: samplesLabel}, this._pingLabelNode);
				dojo.addClass(samplesNode, "secondaryLabel");
				dojo.empty(this._pingTimeNode);
				this._pingTimeNode.innerHTML = this.messages.running;
			}
			
			//make sure the same test hasn't started yet
			//do nothing if there's already one running
			for(var r in this._serverPingResponses) {
				if(this._serverPingResponses[r].sequenceId === pingCounter) {
					return;
				}
			}
			
			//invoke test
			var serviceObj = {
				// pass the testId with the callback so that the callback can validate if it's still for the same test.
				success: dojo.hitch(this, this._serverPingTestCallBack, testId, pingCounter, true),
				failure: dojo.hitch(this, this._serverPingTestCallBack, testId, pingCounter, false)
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			var param = {testId: testId, sequenceId: pingCounter};
			var req = new ServiceRequest(CONNECTION_SERVICE_URI, PING_TEST, param);
			
			var start = new Date();
			TeamServerClient.invokeService(req, srh);
			
			this._serverPingResponses.push({
				testId: testId,
				sequenceId: pingCounter, 
				startTime: start.getTime()
			});
			
			//start the next one if no response before the timeout.
			this._testTimeOutTimer = setTimeout(dojo.hitch(this, this._serverPingTestTimeout, pingCounter, testId), PING_TEST_TIMEOUT);
		}
		else if(this._dbPingTestStarted === false){			
			// all server ping tests finished or timed out, display result 
			this._displayServerPingResult();
			// run next test #2 DB latency test
			this._dbPingTest(1, testId);			
		}
		//else, callback after timed out, do nothing
	},
	
	_serverPingTestCallBack: function(testId, sequenceId, isSuccess, response) {
		//record the result
		var end = new Date();
		
		if(this.isDestroyed())
			return response;
		
		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return response;
		
		var wasTimedout = false;
		if(this._serverPingResponses && sequenceId <= this._serverPingResponses.length) { //sanity check
			
			//find the right response object based on sequenceId
			for(var i = (this._serverPingResponses.length - 1); i >=0; i--) {
				if(sequenceId == this._serverPingResponses[i].sequenceId) {
					if(isSuccess) {
						this._serverPingResponses[i].endTime = end.getTime();
					}
					else {
						if(response)
							this._serverPingResponses[i].error = response;
					}
					if(this._serverPingResponses[i].isTimedout === true)
						wasTimedout = true;
					
					break;
				}						
			}
		}
		
		//call for the next one after an interval, only if this is not a time out
		if(!wasTimedout)
			this._testIntervalTimer = setTimeout(dojo.hitch(this, this._serverPingTest, sequenceId+1, testId), TEST_INTERVAL);
		
		return response;
	},
	
	_serverPingTestTimeout: function(sequenceId, testId) {
		if(this.isDestroyed())
			return;

		this._stopAllTimers();
		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return;
		
		if(this._serverPingResponses && sequenceId <= this._serverPingResponses.length) { //sanity check
			
			//find the right response object based on sequenceId
			for(var i = (this._serverPingResponses.length - 1); i >=0; i--) {
				if(sequenceId == this._serverPingResponses[i].sequenceId) {
					this._serverPingResponses[i].isTimedout = true;
					break;
				}						
			}
		}
		
		//call for the next one right away
		this._serverPingTest(sequenceId+1, testId);
	},

	_dbPingTest: function(pingCounter, testId) {
		if(!testId || !this._testId || testId != this._testId)
			return;
		
		this._dbPingTestStarted = true;
		this._stopAllTimers();
		
		if(pingCounter <= NUM_OF_DB_PINGS) {
			if(pingCounter === 1) {
				var samplesLabel = dojo.string.substitute(this.messages.numOfSamples, [NUM_OF_DB_PINGS]);
				dojo.create("br", null, this._dbPingLabelNode);
				var samplesNode = dojo.create("span", {innerHTML: samplesLabel}, this._dbPingLabelNode);
				dojo.addClass(samplesNode, "secondaryLabel");
				dojo.empty(this._dbPingTimeNode);
				this._dbPingTimeNode.innerHTML = this.messages.running;				
			}
			
			//make sure the same test hasn't started yet
			//do nothing if there's already one running
			for(var r in this._dbPingResponses) {
				if(this._dbPingResponses[r].sequenceId === pingCounter)
					return;
			}
			
			this._dbPingResponses.push( {
				sequenceId: pingCounter,
				testId: testId
			});
			
			var serviceObj = {
				success: dojo.hitch(this, this._dbPingTestCallBack, pingCounter, testId, true),
				failure: dojo.hitch(this, this._dbPingTestCallBack, pingCounter, testId, false)
			};
			var srh = new ServiceResponseHandler(serviceObj, "success", "failure");
			var param = {testId: testId, sequenceId: pingCounter};
			var req = new ServiceRequest(CONNECTION_SERVICE_URI, DB_PING_TEST, param);
			TeamServerClient.invokeService(req, srh);
	
			//start the next one if no response before the timeout.
			this._testTimeOutTimer = setTimeout(dojo.hitch(this, this._dbPingTestTimedout, pingCounter, testId), DB_PING_TEST_TIMEOUT);
		}
		else if(this._downloadTestStarted === false){
			this._displayDBPingResult();
			//sequential test run: #3 download speed test
			this._downloadSpeedTest(testId);			
		}
		//else, callback came after timed out, do nothing.
	},
	
	_dbPingTestCallBack: function(sequenceId, testId, isSuccess, response) {
		if(this.isDestroyed())
			return response;
		
		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return response;
		
		for(var r in this._dbPingResponses) {
			if(sequenceId === this._dbPingResponses[r].sequenceId) {
				if(isSuccess === true) {
					this._dbPingResponses[r].time = Number(response); // ms
				}
				else {
					this._dbPingResponses[r].error = response;
				}
				break;
			}				
		}

		//call for the next one after an interval 
		this._testIntervalTimer = setTimeout(dojo.hitch(this, this._dbPingTest, sequenceId+1, testId), TEST_INTERVAL);
		
		return response;
	},
	
	_dbPingTestTimedout: function(sequenceId, testId) {
		if(this.isDestroyed())
			return;

		this._stopAllTimers();
		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return;		
		
		for(var r in this._dbPingResponses) {
			if(sequenceId === this._dbPingResponses[r].sequenceId) {
				this._dbPingResponses[r].isTimedout = true;
				break;
			}				
		}

		//call for the next one right away
		this._dbPingTest(sequenceId+1, testId);
	},
 	
	_displayDBPingResult: function() {		
		this._stopAllTimers();
		
		var resultDisplay = null;
		if(!this._dbPingResponses || !dojo.isArray(this._dbPingResponses) || this._dbPingResponses.length < 1)
			resultDisplay = this._createErrorElement();
		else {
			var successResults = [];
			var sum = 0;
			for(var r in this._dbPingResponses) {
				if(this._dbPingResponses[r].time >= 0 && !this._dbPingResponses[r].isTimedout) {
					successResults.push(this._dbPingResponses[r].time);
					sum += this._dbPingResponses[r].time;
				}
			}

			if(successResults.length > 0) {

				if (djConfig.isDebug) {
					console.log("Success db ping test results (" + successResults.length + "): " + successResults.join(", "));
				} 
				
				var average = Math.round(sum / successResults.length);
				var max = Math.max.apply( Math, successResults );
				var min = Math.min.apply( Math, successResults );
	
				var tooltip = dojo.string.substitute(this.messages.dbServerResultTooltip, [average, max, min]); 
					
				var div = dojo.create("span");	
				dojo.addClass(div, "graphicsResult");
				dojo.empty(this._dbPingTimeNode);
				this._dbPingTimeNode.appendChild(div);
				this._hoverPopups.push(new jazz.ui.HoverPopup({ around: div, html: tooltip, openDelay: 300}));
				this._generateDBPingTimeNode(average, div);	
			}
			else {
				resultDisplay = this._createErrorElement();
			}
		}

		if(resultDisplay)
			this._updateTestResult(this._dbPingTimeNode, resultDisplay);		
	},
	
	_downloadSpeedTest: function(testId) {

		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return;
		
		this._downloadTestStarted = true;
		this._stopAllTimers();
		
		var transferSizeLabel = dojo.string.substitute(this.messages.sizeOfTransfer, [THROUGHPUT_TEST_SIZE / (1024*1024)]);
		dojo.create("br", null, this._downloadLabelNode);
		var samplesNode = dojo.create("span", {innerHTML: transferSizeLabel}, this._downloadLabelNode);
		dojo.addClass(samplesNode, "secondaryLabel");
		dojo.empty(this._downloadSpeedNode);
		this._downloadSpeedNode.innerHTML = this.messages.running;
		
		var args= {};
		args.url= Util.getServiceURL(CONTENT_SERVICE_URI, DOWNLOAD_TEST + "&testId=" + testId);
		args.headers= {
				"accept": "text/plain",
				"content-type": "charset=utf-8",
				"X-com-ibm-team-configuration-versions": "LATEST",
				"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
		};

		var start = new Date();		
		var deferred = jazz.client.xhrGet(args);
		
		this._downloadTestResponse = {
			testId: testId,
			startTime: start.getTime()
		};
		
		deferred.addCallback(dojo.hitch(this, this._transferSpeedTestCallBack, true, testId, true));
		deferred.addErrback(dojo.hitch(this, this._transferSpeedTestCallBack, true, testId, false));
		
		// if no response after certain time, declare it's done
		this._testTimeOutTimer = setTimeout(dojo.hitch(this, this._transferSpeedTestTimeout, true, testId), TRANSFER_TEST_TIMEOUT);
	},
	
	_transferSpeedTestCallBack: function(isDownload, testId, isSuccess, response) {
		if(this.isDestroyed())
			return response;
		
		// the current test is already done or is a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId) {
			return response;
		}
		
		var end = new Date();		
		var responseObj = (isDownload === true) ? this._downloadTestResponse : this._uploadTestResponse;
		if(!responseObj)
			responseObj = {testId: testId};
		
		if(isSuccess) {
			responseObj.endTime = end.getTime();
		}
		else {
			responseObj.error = response;
		}

		// response received after timed out, do nothing
		if(responseObj.isTimedout === true)
			return response;

		this._displayTransferSpeedResult(isDownload);
		if(isDownload === true) {
			//sequential test run: #4 upload speed test
			this._testIntervalTimer = setTimeout(dojo.hitch(this, this._uploadSpeedTest, testId), TEST_INTERVAL);
		}
		
		return response;
	},
	
	_transferSpeedTestTimeout: function(isDownload, testId) {
		if(this.isDestroyed())
			return;

		this._stopAllTimers();
		// the current test is already done or is a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return;

		var responseObj = (isDownload === true) ? this._downloadTestResponse : this._uploadTestResponse;
		if(!responseObj)
			responseObj = {testId: testId};
		responseObj.isTimedout = true;
		responseObj.error = this.messages.timedOut;

		this._displayTransferSpeedResult(isDownload);
		if(isDownload === true) {
			//sequential test run: #4 upload speed test
			this._uploadSpeedTest(testId);
		}
	},
	
	_displayTransferSpeedResult:function(/*boolean*/ isDownload) {
		
		this._stopAllTimers();
		
		if(isDownload === false) {	
			this._testId = null; //upload test finished, conclude the current test run.	
			this.getSite().setLoading(false);
			this.getSite().doneRefresh();	
			
			if (djConfig.isDebug) {
				var downloadTime = (this._downloadTestResponse.endTime && this._downloadTestResponse.startTime) ? 
						(this._downloadTestResponse.endTime - this._downloadTestResponse.startTime)+"ms. " : "no result. ";
				downloadTime += this._downloadTestResponse.isTimedout ? "Timed out. " : "";
				downloadTime += this._downloadTestResponse.error ? this._downloadTestResponse.error : ""; 
				console.log("Download test time: " + downloadTime);
				
				var uploadTime = (this._uploadTestResponse.endTime && this._uploadTestResponse.startTime) ? 
						(this._uploadTestResponse.endTime - this._uploadTestResponse.startTime)+"ms. " : "no result. ";
				uploadTime += this._uploadTestResponse.isTimedout ? "Timed out. " : "";
				uploadTime += this._uploadTestResponse.error ? this._uploadTestResponse.error : ""; 				
				console.log("Upload test time: " + uploadTime);
			} 
		}

		var resultDisplay = null;
		var responseObj = (isDownload === true) ? this._downloadTestResponse : this._uploadTestResponse;
		if(!responseObj)
			resultDisplay = this._createErrorElement();
		else {
			if(responseObj.endTime && responseObj.startTime) {
				var transferTime = responseObj.endTime - responseObj.startTime;
				
				var parentNode = isDownload === true ? this._downloadSpeedNode : this._uploadSpeedNode;
				resultDisplay = this._getThroughputSpeedValueNode(transferTime, parentNode);
			}
			else if(responseObj.isTimedout === true) {
				resultDisplay = this._createErrorElement(
						this.messages.timedOut, 
						dojo.string.substitute(this.messages.transferTimedOutTooltip, [Math.round((THROUGHPUT_TEST_SIZE/1024)/(TRANSFER_TEST_TIMEOUT/1000))]));
			}
			else if(responseObj.error)
				resultDisplay = this._createErrorElement(responseObj.error);
			else
				resultDisplay = this._createErrorElement();				
		}

		if(resultDisplay)
			this._updateTestResult((isDownload === true ? this._downloadSpeedNode : this._uploadSpeedNode), resultDisplay);	
	},
	
	_uploadSpeedTest: function(testId) {
		// the current test is already a different one, ignore this callback
		if(!this._testId || !testId || testId != this._testId)
			return;

		var transferSizeLabel = dojo.string.substitute(this.messages.sizeOfTransfer, [THROUGHPUT_TEST_SIZE / (1024*1024)]);
		dojo.create("br", null, this._uploadLabelNode);
		var samplesNode = dojo.create("span", {innerHTML: transferSizeLabel}, this._uploadLabelNode);
		dojo.addClass(samplesNode, "secondaryLabel");
		dojo.empty(this._uploadSpeedNode);
		this._uploadSpeedNode.innerHTML = this.messages.running;
		
		this._stopAllTimers();
		
		//generate a string for upload of size THROUGHPUT_TEST_SIZE
		var contentToUpload = "";
		var str = "ABCDEFGHIJKLMNOPQRSTUVWXTZabcdefghiklmnopqrstuvwxyz0123456789";
		var repeat = Math.floor(THROUGHPUT_TEST_SIZE / str.length);		
		for(var i = 0; i < repeat; i++) {
			contentToUpload = contentToUpload.concat(str);
		}
		var remaining = THROUGHPUT_TEST_SIZE % str.length;
		contentToUpload = contentToUpload.concat(str.substring(0, remaining));
		
		var args= {};
		args.url= Util.getServiceURL(CONTENT_SERVICE_URI, UPLOAD_TEST + "&testId=" + testId);
		args.headers= {
				"accept": "text/plain",
				"content-type": "charset=utf-8",
				"X-com-ibm-team-configuration-versions": "LATEST",
				"X-Jazz-CSRF-Prevent": dojo.cookie("JSESSIONID")
		};
		args.postData = contentToUpload;		

		var start = new Date();		
		var deferred = jazz.client.xhrPut(args);		

		this._uploadTestResponse = {
			testId: testId,
			startTime: start.getTime()
		};
		
		//_transferSpeedTestCallBack(isDownload, testId, isSuccess, response)
		deferred.addCallback(dojo.hitch(this, this._transferSpeedTestCallBack, false, testId, true)); 
		deferred.addErrback(dojo.hitch(this, this._transferSpeedTestCallBack, false, testId, false));
		
		// if no response after certain time, declare it's done
		this._testTimeOutTimer = setTimeout(dojo.hitch(this, this._transferSpeedTestTimeout, false, testId), TRANSFER_TEST_TIMEOUT);
	},
	
	
	_updateTestResult: function(/*node*/ resultNode, /*node or string*/ result) {
		dojo.empty(resultNode);
		if (typeof(result) == "string") {
			resultNode.innerHTML = result;
		}
		else {
			resultNode.appendChild(result);
		}
	},
	
	_updateTestErrorResult: function(/*node*/ resultNode, error) {		
		dojo.empty(resultNode);
		resultNode.appendChild(this._createErrorElement(error));
	},

	_displayServerPingResult: function() {
		
		this._stopAllTimers();
		
		//shouldn't happen, but in case
		if(!this._serverPingResponses || !dojo.isArray(this._serverPingResponses) || this._serverPingResponses.length < 1)
			this._updateTestResult(this._pingTimeNode, this._createErrorElement());
			
		var successPingResults = [];
		for(var o in this._serverPingResponses) {
			if(this._serverPingResponses[o].endTime && this._serverPingResponses[o].startTime && !this._serverPingResponses[o].isTimedout) {
				successPingResults.push(this._serverPingResponses[o].endTime - this._serverPingResponses[o].startTime);
			}
		}
		
		var count = successPingResults.length;
		if(count > 0) {
			if (djConfig.isDebug) {
				console.log("Success server ping test results (" + successPingResults.length + "): " + successPingResults.join(", "));
			} 
			
			var max = Math.max.apply( Math, successPingResults );
			var min = Math.min.apply( Math, successPingResults );
			var sum = 0;
			var sumSquare = 0;
			for(var t in successPingResults) {
				sum += successPingResults[t];
	            sumSquare += successPingResults[t] * successPingResults[t];
			}
			
			var average = Math.round(sum / count);
			var stddev = Math.round(Math.sqrt((sumSquare / count) - (average * average)));
//			var lostPackage = this._serverPingResponses.length - count;
			
			var tooltip = dojo.string.substitute(this.messages.serverResultTooltip, [average, max, min, stddev]); 
				
			var div = dojo.create("span");	
			dojo.addClass(div, "graphicsResult");
			dojo.empty(this._pingTimeNode);
			this._pingTimeNode.appendChild(div);
			this._hoverPopups.push(new jazz.ui.HoverPopup({ around: div, html: tooltip, openDelay: 300}));
			this._renderPingTimeValueGraphic(div, average);
		}
		else {
			this._updateTestResult(this._pingTimeNode, this._createErrorElement());
		}
	},
	
	_renderPingTimeValueGraphic: function(parentNode, /*number, in ms*/ pingAverage) {
		
		var surface = dojox.gfx.createSurface(parentNode, SURFACE_WIDTH, SURFACE_HEIGHT);
		
		surface.whenLoaded(dojo.hitch(this, function(surface, pingValue) {
			this._renderScaler(surface, this.PING_BAR_LABELS, PING_BAR_LABEL_OFFSET, PING_BAR_NUM_MINOR_DIVIDERS);		
			var valueLabel = dojo.string.substitute(this.messages.ms, [pingValue]);
			this._renderArrow(surface, valueLabel, pingValue, PING_BAR_MAX_VALUE);
			
		}, surface, pingAverage));
	},
	
	_getThroughputSpeedValueNode: function(/*number, in ms*/ time, parentNode) {
		if(typeof(time) == "number") {
			var speed = (THROUGHPUT_TEST_SIZE / 1024) / (time / 1000); // KB/s
			speed = Math.floor(speed);
			var div = dojo.create("span");
			dojo.addClass(div, "graphicsResult");
			if(parentNode) {
				dojo.empty(parentNode);
				parentNode.appendChild(div);
			}
			var surface = dojox.gfx.createSurface(div, SURFACE_WIDTH, SURFACE_HEIGHT);				
			surface.whenLoaded(dojo.hitch(this, function(surface, speed) {
				this._renderScaler(surface, this.THROUGHPUT_BAR_LABELS, THROUGHPUT_BAR_LABEL_OFFSET, THROUGHPUT_BAR_NUM_MINOR_DIVIDERS, true);			
				this._renderArrow(surface, dojo.string.substitute(this.messages.speed_kb_s, [speed]), speed, THROUGHPUT_BAR_MAX_VALUE, true);
				
			}, surface, speed));
			
			return null;
		}
		else
			return time;
	},
	
	_renderScaler: function(/*dojox.gfx.Surface or dojox.gfx.Group*/ surface, /*[] of strings*/ labels, /*number*/ labelOffset, /*number*/ minorDividers, /*boolean*/ isMaxOnLeft){
		var rtlShift = SURFACE_WIDTH - BAR_WIDTH - (2 * BAR_ORIGIN_X) + 1;
		var rectangle = surface.createRect({x: this.isLeftToRight() ? (BAR_ORIGIN_X-1) : rtlShift + (BAR_ORIGIN_X-1), y: BAR_ORIGIN_Y, width: (BAR_WIDTH+2), height: BAR_HEIGHT });
		rectangle.setFill({ type:"linear",
		    x1: this.isLeftToRight() ? 0 : rtlShift, y1: 0,   
		    x2: (BAR_WIDTH+2), y2: 0,
		    colors: [{
		        offset: 0,
		        color: this.isLeftToRight() ? BAR_COLOR_A : BAR_COLOR_C
		    }, {
		        offset: 0.55,
		        color: BAR_COLOR_B
		    }, {
		    	offset: 1,
		    	color: this.isLeftToRight() ? BAR_COLOR_C : BAR_COLOR_A
		    }]
		});
		
		if(!labels || labels.length === 0)
			return;
		
		var c = labels.length; // number of long markers
		var markerWidth = (c == 1) ? BAR_WIDTH : BAR_WIDTH / (c - 1);
		var x = BAR_ORIGIN_X;
		var y_l1 = BAR_ORIGIN_Y + BAR_HEIGHT - MARK_LENGTH_L;  //long marker
		var y_s1 = BAR_ORIGIN_Y + BAR_HEIGHT - MARK_LENGTH_S;  //short marker
		var y2 = BAR_ORIGIN_Y + BAR_HEIGHT;
		var label_x = LABEL_ORIGIN_X;
		var label_y = LABEL_ORIGIN_Y;
		var xNew;
		var labelAlign = this.isLeftToRight() ? "start" : "end";
		
		for(var i in labels) {
			var y1 = y_l1; //draw a long marker for each label 
			xNew = this.isLeftToRight() ? x : SURFACE_WIDTH  - x;
			surface.createLine({ x1:  xNew, y1: y1, x2: xNew, y2: y2 }).setStroke(MARK_COLOR);
			surface.createText({ x: this.isLeftToRight() ? label_x :SURFACE_WIDTH - label_x, y: label_y, text: labels[i], align:labelAlign}).setFill(LABEL_COLOR).setFont(LABEL_FONT);
							
			// draw short markers in between long markers
			if(i < (c - 1) && minorDividers) {
				y1 = y_s1;
				var x_s = x;
				for(var m = 0; m < minorDividers; m++) {
					x_s += markerWidth / (minorDividers + 1);
					xNew = this.isLeftToRight() ? x_s :SURFACE_WIDTH - x_s;
					surface.createLine({ x1: xNew, y1: y1, x2: xNew, y2: y2 }).setStroke(MARK_COLOR);
				}
			} 
			
			x += markerWidth;
			label_x += markerWidth;
			if(labelOffset)
				label_x += labelOffset;
		}
	},
	
	_renderArrow: function(surface, arrowLabel, arrowValue, maxValue, isMaxOnLeft) {
		var l2_x;
		if(isMaxOnLeft && isMaxOnLeft === true) {
			if(arrowValue < maxValue)
				l2_x = (BAR_ORIGIN_X + BAR_WIDTH) - BAR_WIDTH / maxValue * arrowValue;
			else
				l2_x = BAR_ORIGIN_X;
		}
		else {
			if(arrowValue < maxValue)
				l2_x = BAR_ORIGIN_X + BAR_WIDTH / maxValue * arrowValue;
			else
				l2_x = BAR_ORIGIN_X + BAR_WIDTH;
		}
		if(!this.isLeftToRight()){
			l2_x = SURFACE_WIDTH - l2_x;
		}

		var l2_y = BAR_ORIGIN_Y + BAR_HEIGHT;
		var m_x = l2_x - ARROW_WIDTH / 2;
		var m_y = l1_y = ARROW_ORIGIN_Y;
		var l1_x = l2_x + ARROW_WIDTH / 2;

		var path = surface.createPath("M" + m_x + " " + m_y + "L" + l1_x + " " + l1_y + "L" + l2_x + " " + l2_y + "Z");
		path.setFill({ type:"linear",
		    x1: 0,
		    y1: 0,   
		    x2: ARROW_WIDTH, 
		    y2: BAR_HEIGHT,
		    colors: [{
		        offset: 0,
		        color: "skyblue"
		    }, {
		        offset: 1,
		        color: ARROW_COLOR
		    }]
		});
		
		surface.createText({x: (m_x + ARROW_LABEL_OFFSET_X), y: (m_y + ARROW_LABEL_OFFSET_Y), text: arrowLabel, align: this.isLeftToRight()?"start" : "end"}).setFill(LABEL_COLOR).setFont(LABEL_FONT);
	},
	
	_generateDBPingTimeNode: function(/*number, in ms*/ pingTime, parentNode) {
		
		if(typeof(pingTime) == "number") {
			var surface = dojox.gfx.createSurface(parentNode, SURFACE_WIDTH, SURFACE_HEIGHT);				
			surface.whenLoaded(dojo.hitch(this, function(surface, pingValue) {
							
				this._renderScaler(surface, this.DB_PING_BAR_LABELS, 0, DB_PING_BAR_NUM_MINOR_DIVIDERS);		
				var valueLabel = dojo.string.substitute(this.messages.ms, [pingValue]);
				this._renderArrow(surface, valueLabel, pingValue, DB_PING_BAR_MAX_VALUE);
				
			}, surface, pingTime));
			
			return null;
		}
		else
			return pingTime;
	},
	
	_createErrorElement: function(error, tooltip) {
		//TODO create standard error element instead.
		var anchor = document.createElement("a");
		dojo.addClass(anchor, "error-status");
		anchor.href = this.getWebURL() + "/admin#action=jazz.viewPage&id=com.ibm.team.repository.server";
		var errMsg = (error && dojo.isString(error)) ? error : this.messages.unavailable;
		anchor.appendChild(document.createTextNode(errMsg));
		var tooltipMsg = tooltip ? tooltip : this.messages.testFailedTooltip;
		this._hoverPopups.push( new jazz.ui.HoverPopup({ around: anchor, html: tooltipMsg, openDelay: 300}) );
		
		return anchor;
	}
});

})();
