/*******************************************************************************
 * Licensed Materials - Property of IBM (c) Copyright IBM Corporation 2007, 2011,
 * 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.DataWarehouseDiagnostics");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");


dojo.require("jazz.util.html");
dojo.require("jazz.ui.StyledTable");

dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal",
		"ReportsManagementMessages");

(function() { // module container
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	dojo
			.declare(
					"com.ibm.team.reports.web.ui.internal.DataWarehouseDiagnostics",
					[ dijit._Widget, dijit._Templated ],
					{
						templatePath : dojo.moduleUrl(
								"com.ibm.team.reports.web",
								"ui/internal/templates/Diagnostics.html"),

						constructor : function() {
							this.messages = dojo.i18n.getLocalization(
									"com.ibm.team.reports.web.ui.internal",
									"ReportsManagementMessages");

						},

						postCreate : function() {
							com.ibm.team.reports.web.ui.internal.DataWarehouseDiagnostics.superclass.postCreate
									.apply(this);
							this._checkIsJTS();
						},


						_checkIsJTS : function() {
							var srh = new ServiceResponseHandler(this,
									"_handleJTSCheck", "_handleJTSCheckFailure");
							ServerStatusClient.getIsJazzTeamServer(srh);
						},

						_handleJTSCheck : function(response) {
							this.isJTS = response == "true";
							if (!this.isJTS) {
								this._checkIsStandalone();
							}else{
								this._performDiagnostics();
							}
						},

						_checkIsStandalone : function() {
							var srh = new ServiceResponseHandler(this,
									"_handleStandaloneCheck",
									"_handleJTSCheckFailure");
							ServerStatusClient.getServerModeString(srh);
						},

						_handleStandaloneCheck : function(response) {
							this.isJTS = response.toLowerCase() === "standalone";
							if(this.isJTS){
								this._performDiagnostics();
							}else{
								this._displayMessage(this.messages.nonJtsDiagnostics);
							}
						},

						_handleJTSCheckFailure : function(errorObj) {
							this.isJTS = false;
							this._displayMessage(this.messages.nonJtsDiagnostics);
						},
						
						_displayMessage : function(message) {
							this._contentDiv.appendChild(document.createTextNode(message));
						},
						
						_performDiagnostics : function() {
							var diagnosticsHandler = new ServiceResponseHandler(
									this, "_performDiagnosticsHandler",
									"_remoteError");
							ReportsClient
									.getPerformDataWarehouseDiagnostics(diagnosticsHandler);
						},

						_performDiagnosticsHandler : function(results) {
							var table = new jazz.ui.StyledTable( {
								columns : [ {
									label : this.messages.application,
									sortable : false,
									width : "250px"
								}, {
									label : this.messages.location,
									sortable : false,
									width : "300px"
								}, {
									label : this.messages.valid,
									sortable : false,
									width : "10%"
								} ],
								sortBy : 2,
								sortByAscending : false,
								handleSort : function() {
								},
								trim : jazz.ui.styledTable.TrimStyles.BLUE
							});
							var allValid = true;
							for(var i = 0; i < results.length; i++){
								var result = results[i];
								var name = document.createElement("td");
								var location = document.createElement("td");
								var valid = document.createElement("td");
								var row = document.createElement("tr");
								name.appendChild(document.createTextNode(result["Name"]));
								location.appendChild(document.createTextNode(result["Location"]));
								valid.appendChild(document.createTextNode(result["Valid"]));
								allValid = allValid && result["Valid"];
								row.appendChild(name);
								row.appendChild(location);
								row.appendChild(valid);
								table.tbody.appendChild(row);
							}
							this._clearContent();
							this._contentDiv.appendChild(table.domNode);
							if(!allValid){
								this._contentDiv.appendChild(document.createTextNode(this.messages.diagnosticHelp));
							}
						},

						_clearContent : function() {
							while(this._contentDiv.hasChildNodes()){
								this._contentDiv.removeChild(this._contentDive.firstChild);
							}
						},
						
						_remoteError : function() {
							this._displayMessage(this.messages.errorPerformingDiagnostics);
						}
					});

})();
