/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.dashboard.web.ui.internal.ProjectInfo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit._Container");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.Menu");
dojo.require("jazz.ui.MenuPopup");

dojo.require("com.ibm.team.dashboard.web.ui.DashboardConstants");

dojo.requireLocalization("com.ibm.team.dashboard.web.ui.internal", "ProjectInfoMessages");

(function() {

var messages;

dojo.declare("com.ibm.team.dashboard.web.ui.internal.ProjectInfo", [dijit._Widget, dijit._Templated, dijit._Container], {

	templatePath: dojo.moduleUrl("com.ibm.team.dashboard.web", "ui/internal/templates/ProjectInfo.html"),
	
	constructor: function(args) {
		if (!messages) messages = dojo.i18n.getLocalization("com.ibm.team.dashboard.web.ui.internal", "ProjectInfoMessages");
		this.headerMessage = args.scopeInfo.scope === com.ibm.team.dashboard.web.ui.DashboardConstants.SCOPE_PROJECT_AREA ?
				messages.projectDetails : messages.teamDetails;
	},
	
	getMenu: function() {
		if (!this._menu) {
			this._menu = new jazz.ui.Menu();
			var toolbar = new jazz.ui.SimpleToolbar({hoverStyle: jazz.ui.SimpleToolbar.HoverStyles.NONE}).placeAt(this._toolbar);
			var button  = toolbar.addButton(null, null, {
				label: messages.menuTitle,
				className: "menu",
				index: 0
			});
			this._menuPopup = new jazz.ui.MenuPopup({
				align: "right",
				menuProvider: dojo.hitch(this,function() {
					return this._menu;
				})
			}, button.domNode);
		}
		return this._menu;
	},
	
	addChild: function(child, insertIndex) {
		if (this.getChildren().length > 0) {
			var refNode = this.containerNode;
			if(insertIndex && typeof insertIndex == "number"){
				var children = this.getChildren();
				var separatorAfter = children.length === insertIndex;
				if(children && children.length >= insertIndex){
					refNode = children[insertIndex-1].domNode;
					insertIndex = "after";
				}
			}
			// if we need to add the separator after the new child, add the child first and return early
			if (separatorAfter) this.inherited(arguments);
			dojo.create("div", {className: "separator"}, this.containerNode, insertIndex);
			if (separatorAfter) return;
		}
		this.inherited(arguments);
	},
	
	removeChild: function(/*Widget or int*/ widget){
		if(typeof widget == "number"){
			widget = this.getChildren()[widget];
		}

		if(widget && widget.domNode){
			var node = this._findSeparator(widget.domNode);
			if(node && node.parentNode){
				node.parentNode.removeChild(node); // detach but don't destroy
			}
		}
		this.inherited(arguments);
	},
	
	_findSeparator: function(widgetNode) {
		if (widgetNode.nextSibling && dojo.hasClass(widgetNode.nextSibling, "separator"))
			return widgetNode.nextSibling;
		if (widgetNode.previousSibling && dojo.hasClass(widgetNode.previousSibling, "separator"))
			return widgetNode.previousSibling;
	}
});

})();