/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportEditor");

dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");

dojo.require("com.ibm.team.repository.web.client.session.Session");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.Tree");
dojo.require("jazz.ui.tree.IconItem");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
var session = com.ibm.team.repository.web.client.session;
var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var SHARED = 1 << 0;
var PRIVATE = 1 << 2;
var SYSTEM = 1 << 3;

/**
 * Widget to create a new report query
 */
var ReportEditor = dojo.declare("com.ibm.team.reports.web.ui.internal.ReportEditor", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportEditor.html"),
		_csPageHelpWidgets: null,
		csHelpNode: null,
		
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			this._csPageHelpWidgets= [];
		},
				
		postCreate: function() {
			this.dirty = false;
			this._responseCount = 0;
			this.typeIcon.src = ImagePool.REPORT_OBJ;
			this._allFolders = [];
			// A map mapping folder nodes to the underlying folder
			this._folderMap = {};
			
			this._createFolderTree();
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			var args = {
				ownerUUID: session.getAuthenticatedContributor().itemId,
				flags: PRIVATE
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			args = {
				flags: (SHARED | SYSTEM)
			};
			responseHandler = new ServiceResponseHandler(
					this,
					"_addFolders",
					"_remoteError");
			ReportsClient.getFolderDescriptors(responseHandler, args);
			
			this._createContextSensitiveHelp();
		},
		
		_handleQueryDescriptor: function(query)
		{
			this.query = query;
			this._nameInput.value = query.name;
			PlatformUI.getWorkbench().setTitle(query.name);
			this._descriptionText.value = query.description;
			this._selectFolder(query.folderUUID);
			
			
			//Get the report template for the report so we can show it's name and link
			var args = {};
			args.reportUUID = query.reportUUID;
			args.queryUUID = query.queryUUID;
			args.projectAreaUUID = jazz.app.context.get().itemId;
			var srh = new ServiceResponseHandler(this, "_handleTemplateDescriptor", "_remoteError");
			ReportsClient.getReportDescriptor(srh, args);
		},
		
		_handleTemplateDescriptor: function(template)
		{
			//Save the ID of the template
			this.templateUUID = template.reportUUID;
			
			//Set the name of the template into the HTML
			this._templateAnchor.innerHTML = template.name;
			
			//Set the href of the template to be the url to the template editor
			var location = this.editTemplateAction;
			
			//A bit of a hack.
			//if run from RQM, the subaction is in variable called editTemplateSubAction
			//It could not be passed as the usual subAction because the Edit Report URL already contains a subAction
			if (this.editTemplateSubAction != null)
				location += "&subAction=" + this.editTemplateSubAction;
				
			location += "&reportUUID=" + this.templateUUID;
			this._templateAnchor.href = location;						
			
		},
		
		_selectFolder: function(folderUUID)
		{
			//Get the itemID from the folder map based on the folder UUID
			//the select that item in the tree
			for (var key in this._folderMap)
			{
				//If the folder in the map is the same as the supplied folder UUID then get the item
				if (this._folderMap[key].folderUUID == folderUUID)
				{
					//Get the item from the tree
					var folderToSelect = this._folderTreeWidget.getItem(this._folderMap[key].treeItemID);
					//Expand the tree down to the folder to select
					var parentItem = folderToSelect.getParent();
					while (parentItem != null) 
					{
						parentItem.setExpanded(true);
						parentItem = parentItem.getParent();
					}
					//select it
					this._folderTreeWidget.setSelection(folderToSelect);
					break;
				}
			}
			
			this._folderTreeWidget.onSelectionChanged = dojo.hitch(this,this._handleFolderSelectionChange);
		},

		_createContextSensitiveHelp: function() {
			dojo.empty(this.csHelpNode);
			this._createCSHWidget(ReportEditor.CSH_NEW_REPORT_EDITOR);
		},
	
		_createCSHWidget: function(uaKey) {
			var parms = {};
			parms.uaKey = uaKey;
			var pageHelp = new PageHoverHelp(parms);
			this.csHelpNode.appendChild(pageHelp.domNode);
			this._csPageHelpWidgets.push(pageHelp);
		},
		
		_handleSave: function() {
			this.messageBox.hideMessage();
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			var args = {
		        name: this._getName(),
		        queryUUID: this.queryUUID,
				ignoreParameters: true  //Leave the parameters as they are
	        }
			var description = this._getDescription();
			args.description = (description && description.length > 0) ? description : " ";
	        
	        var currentFolderNode = this._folderTreeWidget.getSelection();
	        var currentFolder = this._folderMap[currentFolderNode];
	        if (!currentFolder || currentFolder == null) {
	        	this._showErrorMessage(this.messages.pleaseSelectFolder);
	        	return;
	        }
	        
	        // If this is a special folder, just set the sharing flag. Otherwise
	        // set the parent folder.
	        if (currentFolder.processAreaUUID == null) {
	        	args.sharedIfNullFolder = currentFolder.shared;
	        } else {
	        	args.folderUUID = currentFolder.folderUUID;
	        }
	        var responseHandler = new ServiceResponseHandler(
					this,
					"_saveComplete",
					"_remoteError");
			ReportsClient.postSaveQueryDescriptor(responseHandler, args);
		},
		
		_handleCancel: function() {
			history.go(-1);
		},
		
		_getName: function() {
			return this._nameInput.value;
		},
		
		_getDescription: function() {
			return this._descriptionText.value;
		},
		
		_saveComplete: function(args) {
			document.location.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + args;
		},
		
		_remoteError: function(error) {
			// Check if the content is still displaying
			if (!dojo.isDescendant(this._saveButton, document.body)) {
				// Not still displaying, ignore the error
				return;
			}
			this._showErrorMessage("Error invoking service; message: " + error.message);
		},

		_addFolders: function(folders) {
			this._responseCount++;
			if (!folders) folders = [];
			for (var i = 0; i < folders.length; i++) {
				this._allFolders.push(folders[i]);
			}
			if (this._responseCount == 1) {
				return;
			}
			this._addFoldersToTree();
			
			//Now that the folders are loaded, load the query
			//Load the supplied report
			args = {
				queryUUID: this.queryUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleQueryDescriptor", "_remoteError");
			ReportsClient.getQueryDescriptor(srh, args);			
		},
		
		_addFoldersToTree: function() {
			// First, find the two root folders
			var sharedRoot = null;
			var privateRoot = null;
			for (var i = 0; i < this._allFolders.length; i++) {
				var folder = this._allFolders[i];
				if (folder.system) {
					if (folder.shared) {
						sharedRoot = folder;
					} else {
						privateRoot = folder;
					}
				}
			}

			if (privateRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: privateRoot.name, iconClass: "iconFolder"});
				privateRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = privateRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, privateRoot, this._allFolders, true, false);
			}
			if (sharedRoot) {
				var rootItem = new jazz.ui.tree.IconItem({label: sharedRoot.name, iconClass: "iconFolder"});
				sharedRoot.treeItemID = rootItem.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[rootItem] = sharedRoot;
				this._folderTreeWidget.addChild(rootItem);
				this._addChildrenForFolder(rootItem, sharedRoot, this._allFolders, true, true);
			}
		},
		
		/**
		 * From the list of folders, find any that are children of parentFolder, and
		 * add them to the tree. Then recurse for each of the children.
		 */
		_addChildrenForFolder: function(parentFolderNode, parentFolder, folders, parentIsRoot, parentIsShared) {
			var foldersToAdd = [];
			for (var i = 0; i < folders.length; i++) {
				// Special case where the parent is a root system folder
				if (parentIsRoot) {
					if (!folders[i].parentUUID && (folders[i].shared == parentIsShared) && (folders[i].name != parentFolder.name)) {
						foldersToAdd.push(folders[i]);
					}
				} else {
					if (folders[i].parentUUID == parentFolder.folderUUID) {
						foldersToAdd.push(folders[i]);
					}
				}
			}
			foldersToAdd = foldersToAdd.sort(function(a, b) {return a.name > b.name;});
			for (var i = 0; i < foldersToAdd.length; i++) {
				var childNode = new jazz.ui.tree.IconItem({label: foldersToAdd[i].name, iconClass: "iconFolder"});
				foldersToAdd[i].treeItemID = childNode.getId();  //Save the item ID in the folders array in case we need to make tree calls based on the id
				this._folderMap[childNode] = foldersToAdd[i];
				parentFolderNode.addChild(childNode);
				this._addChildrenForFolder(childNode, foldersToAdd[i], folders, false, parentIsShared);
			}
		},

		_createFolderTree: function() {
			this._folderTreeWidget = new jazz.ui.Tree();
			this._folderTree.appendChild(this._folderTreeWidget.domNode);
		},
		
		_handleNameChange: function() {
			this._setDirty(true);
			this._updateEnablement();
		},

		_handleDescriptionChange: function() {
			this._setDirty(true);
			this._updateEnablement();
		},
		
		_handleFolderSelectionChange: function(selection)
		{
			this._setDirty(true);
			this._updateEnablement();			
		},
		
		_updateEnablement: function() {
			var name = this._getName();
			var hasName = (name && name.length > 0);
			this._saveButton.disabled = !hasName || !this.dirty;
		},
		
		_showErrorMessage: function(message) {
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		},
		
		_setDirty: function(isDirty) {
			if (isDirty === this.dirty) {
				this._updateEnablement();
				return;
			}
		
			if (isDirty) {
				this.reportTitle.innerHTML = this.messages.report + "*";
			} else {
				this.reportTitle.innerHTML = this.messages.report;
			}
	
			this.dirty = isDirty;
			this._updateEnablement();
		}
			
	}
);

ReportEditor.CSH_NEW_REPORT_EDITOR = "com.ibm.jazz.reports.web.cshelp.new_report_editor";

})();