/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:  Use,
 * duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.ReportViewer");

dojo.require("dojo.i18n");
dojo.require("dijit.Dialog");
dojo.require("dijit.InlineEditBox");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dijit.form.FilteringSelect");

dojo.require("dojox.uuid.generateRandomUuid");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");

dojo.require("com.ibm.team.reports.web.ui.internal.MetaNav");
dojo.require("com.ibm.team.reports.web.ui.ReportsConstants");

dojo.require("com.ibm.team.reports.web.ui.internal.SaveAsDialog");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamFactory");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParamGroup");
dojo.require("com.ibm.team.reports.web.ui.internal.utils.StringUtils");
dojo.require("com.ibm.team.reports.web.ui.internal.ReportParametersPanel");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");
dojo.require("com.ibm.team.reports.web.ui.internal.ImagePool");
dojo.require("com.ibm.team.reports.web.util.ReportsUtil");


dojo.require("jazz.app.context");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.ui.SimpleToolbar");
dojo.require("jazz.ui.StyledBox");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");

(function() {
	var PlatformUI = net.jazz.ajax.ui.PlatformUI;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var ReportParamFactory = com.ibm.team.reports.web.ui.internal.ReportParamFactory;
	var ReportsClient = com.ibm.team.reports.web.client.internal.ReportsClient;
	var StringUtils = com.ibm.team.reports.web.ui.internal.utils.StringUtils;
	var MAX_MESSAGE_LENGTH = 500;
	var ImagePool = com.ibm.team.reports.web.ui.internal.ImagePool;
	var ReportsUtil = com.ibm.team.reports.web.util.ReportsUtil;

	var ReportsConstants = com.ibm.team.reports.web.ui.ReportsConstants;
	
	dojo.declare("com.ibm.team.reports.web.ui.internal.ReportViewer", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/ReportViewer.html"),
		
		widgetsInTemplate: true,
		projectAreaUUID: "", /* Required: If not operating within the web ui */
		queryUUID: "", /* Required: If no reportUUID is set */
		reportUUID: "", /* Required: If no query UUID is set */
		showArchived: false, /* Optional: Set if you wish to display archived results */
		getCached: false, /* Optional: Set if you wish to display cached results */
		controller: null, /* Optional: Set by web ui */
		urlPrefix: net.jazz.ajax._contextRoot + "/", /* Optional */
		customParams: false, /*True if this report provides its own custom parameters*/
		supportsArchived: false, /*True if this report supports archived data*/
		_customParamQueryUUID: null, /*If this report uses custom params, the UUID of the temp query descriptor from which param values can be fetched*/
		
		constructor: function(props) {
			dojo.mixin(this, props);
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "ReportMessages");
			if (!this.parameterName || !this.parameterValue) {
				/*
				 * If we have not been passed in any parameter names or values, we must check if they
				 * are provided in the url.  Since we do not know beforehand the names of all potential
				 * parameters, we map each of the url params to 'this' to be matched with the parameters
				 * retrieved from the server for this query (or report).
				 */
				this.parameterName = [];
				this.parameterValue = [];
				window.location.search.replace(new RegExp("([^?=&]+)(=([^&]*))?", "g"), dojo.hitch(this, function($0, $1, $2, $3) {
					var s = decodeURIComponent($3);
					this[$1] = s;
				}));
			}
			this.reportParams = [];
			this.concealedReportParams = [];
			this.inEditMode = false;
			this.enableParams = false;
			this.enableExport = false;
			this.titleHidden = false;
			this.reportTitle = "";
			this.rcpCall = false;
			//Note isDirtie is deliberately spelled incorrectly.
			//QM's infrastructure calls the isDirty( ) method on the view in a tab to determine if a tab can be closed.
			//The variable with the same name was breaking that code.
			this.isDirtie = false;
			this.isObsolete = false;
		},
		
		postCreate: function() {
			this.reportParamsPanel.setDirtyListener(this);
			if (location.href.match(/_ajax-modules/)) {
				// Widget embedded in the RCP context
				this._loadRCPParameters();
				this.rcpCall = true;
			}
			
			dojo.connect(this.metanav._exportButton, "onClick", this, this._handleExport);
			
			if (this.displayBreadcrumbs){
				this.metanav.populateReportsBreadcrumbs(this.shared);
			} else {
				this.metanav.handle.style.height = "42px";
				this.metanav.handle.style.minHeight = "42px";
			}
			// Set the project area uuid if it was not provided in the constructor
			if (!this.rcpCall && this.projectAreaUUID == "") {
				this.projectAreaUUID = jazz.app.context.get().itemId;
			}
			
			// Set loading message
			if (this.controller) {
				this.controller.startOperation(this.messages.loadingReport);				
			}
			if (!this.rcpCall) {
				if (!this.navigateAwayHandle) {
					this.navigateAwayHandle = PlatformUI.getWorkbench().registerBodyClickHandler(dojo.hitch(this, this._handleNavigateAway), /*priority*/ 10000);
				}
				if (!this.onBeforeUnloadHandle){
					this.onBeforeUnloadHandle = PlatformUI.getWorkbench().registerOnBeforeUnloadHandler(dojo.hitch(this, this._handleOnBeforeUnload), /*priority*/ 10000);	
				}				
			}

			var deferred = ReportParamFactory.init();
			deferred.addCallback(dojo.hitch(this, function(){
				if (this.reportUUID.length > 0) {
					// Disable saving of report templates
					this._saveButton.disabled = true;
					this._initReport();
				}
				else {
					this._initQuery();
				}
			}));

			if (!this.rcpCall && this.projectAreaUUID != jazz.app.context.get().itemId) {
				var responseHandler = new ServiceResponseHandler(this, "_setProjectAreaMismatchMessage", "_remoteError");
				com.ibm.team.process.web.client.internal.ProcessClient.getProjectAreaByUUID({uuid: this.projectAreaUUID}, responseHandler);
			}
			
			var editDiv = this._editDiv.parentNode.removeChild(this._editDiv);
			this._styledBox = new jazz.ui.StyledBox({
				closable: false, 
				contentNode: editDiv,
				trim: jazz.ui.StyledBox.TrimStyles.EDITOR_FOCUS,
				headerless: true,
				headerDefaultToHoverState: true
			});
			this._styledBoxContainer.appendChild(this._styledBox.domNode);
			
			this.messageBox = new jazz.ui.MessageBox();
			this._messageBoxContainer.appendChild(this.messageBox.domNode);
			
			this.metanav.heading.style.top = "8px";
			
			this.typeIcon = document.createElement("img");
			this.typeIcon.className = "typeIcon";
			this.typeIcon.src = ImagePool.REPORT_OBJ_32;
			this.metanav.insertTypeIcon(this.typeIcon);
		},
		
		_initToolbar: function() {
			if (this.metanav._toolbarDiv.hasChildNodes()) {
				while(this.metanav._toolbarDiv.childNodes.length >= 1 ) {
					this.metanav._toolbarDiv.removeChild(this.metanav._toolbarDiv.firstChild);
				}
			}
 			this._toolbar = new jazz.ui.SimpleToolbar({});
			this.metanav._toolbarDiv.appendChild(this._toolbar.domNode);		

			if (this.controller) {
				this._archivedDiv = document.createElement("div");
				this._archivedDiv.className = "jazz-ui-SimpleToolbar-button";
				this._archivedDiv.style.paddingTop = "2px"
				this._archivedDiv.title = this.messages.includeArchivedTooltip;		
				
				this._archivedCheckbox = document.createElement("input");
				this._archivedCheckbox.type = "checkbox";
				this._archivedCheckbox.onclick = dojo.hitch(this, this._onCheckBoxClick);
				this._archivedCheckbox.style.verticalAlign = "middle";
				this._archivedCheckbox.style.marginLeft = "3px";
				this._archivedCheckbox.style.marginRight = "7px";
				this._archivedCheckbox.checked = this._showArchived();
				
				this._archivedDiv.appendChild(this._archivedCheckbox);
				
				var span = document.createElement("span");
				span.className = "com-ibm-team-reports-archivedCheckbox";
				span.style.marginRight = "3px";
				span.appendChild(document.createTextNode(this.messages.includeArchived));
				
				this._archivedDiv.appendChild(span);
				
				if ((!this.supportsArchived) || (!this.displayBreadcrumbs)) {
					this._archivedDiv.style.visibility = "hidden";
				}
				this._toolbar.addWidget({domNode: this._archivedDiv});
			}
			
			this._toolbar.addButton(this, this._showExportDialog, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/export.gif"), label: this.messages.exportTitle});
		
			if (this.enableParams == true) {
				this._button = document.createElement("div");
				this._button.className = "jazz-ui-SimpleToolbar-button";
				this._button.title = this.messages.parametersTooltip;		
				
				var img = document.createElement("img");
				img.src = dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/edit.gif");
				img.alt = this.messages.parametersTooltip;
				this._button.appendChild(img);
				dojo.connect(this._button, "onclick", this, this._handleEdit);
				this._toolbar.addWidget({domNode: this._button});
			}
				
			if (location.href.match(/_ajax-modules/)) {
				// Widget embedded in the RCP context
				this._loadRCPParameters();
				this.rcpCall = true;
			}
			if (!this.rcpCall) {
				this._toolbar.addSeparator();
				this._toolbar.addButton(this, this._viewInPrintableFormat, {imgSrc: dojo.moduleUrl("com.ibm.team.reports.web","ui/internal/images/print_co.gif"), label: this.messages.viewInPrintableFormatTitle});
			}
 		},
 
 		_onCheckBoxClick: function() {
 			dojo.publish("/jazz/reports/includeArchived", [this._archivedCheckbox.checked]);
 		},
 		
		_showExportDialog: function() {
			this._clearErrorMessage();
			if (!this.enableExport) {
				this._showErrorMessage(this.messages.exportOperationNotSupported);
				return;
			}
			this.metanav._exportDialog.show();
		},
		
		_viewInPrintableFormat: function() {
			var win = window.open('');
			win.document.write('<html xmlns="http://www.w3.org/1999/xhtml">\n');
			win.document.write('<head>\n');
			win.document
					.write('<meta http-equiv="content-type" content="text/html; charset=UTF-8"/>\n');
			win.document.write('<style type="text/css">\n');
			win.document
					.write('a { text-decoration: none; }\n');
			win.document.write('img { border: 0 none; }\n');
			win.document.write('</style>\n');
			win.document.write('</head>\n');
			win.document.write('<body>\n')
			win.document
					.write(this.renderedReportDiv.innerHTML);
			win.document.write('\n');
			win.document.write('</body>\n');
			win.document.write('</html>\n');
			win.document.close();
		},
		
		_loadRCPParameters: function() {
			// Widget parameters are passed via URL path segments after the Ajax 
			// module ID
			// e.g.
			// https://localhost:9443/jazz/_ajax-modules/com.ibm.team.reports.web.reportViewer/_fa8m8B_GEd6-xKd-yus_YQ/_j4YOsCOvEd6HYJh3DaxtXw/undef/false/false

			var segments = location.href.split(/\//);
			var ajaxModuleID = "com.ibm.team.reports.web.reportViewer";
			var index = 0;
			var notSpecified = "undef";
			var expectedParamCount = 5;
			
			// Find first parameter index (after Ajax module ID)
			for (index = 0; index < segments.length; index++) {
				if (segments[index] === ajaxModuleID) {
					index++;
					break;
				}
			}
			
			// Validate parameters
			var actualParamCount = segments.length - index;
			if (actualParamCount < expectedParamCount) {
				// Serious error
				var errorMessage = "Failed to load RCP parameters. Number of expected (" + 
									expectedParamCount + 
									") versus actual (" + 
									actualParamCount + 
									") parameters did not match.";
				this._logError(new Error(errorMessage));
				this._showErrorMessage(errorMessage);
			}
			
			// Parse parameters
				
            // 0: projectAreaUUID
			this.projectAreaUUID = decodeURIComponent(segments[index]); 
			index++;

            // 1: queryUUID
			var myQueryUUID = decodeURIComponent(segments[index]);
			if (myQueryUUID !== notSpecified) {
				this.queryUUID = myQueryUUID;
			}
			index++;

            // 2: reportUUID
			var myReportUUID = decodeURIComponent(segments[index]);
			if (myReportUUID !== notSpecified) {
            	this.reportUUID = myReportUUID;
			}
			index++;

           	// 3: showArchived
           	this.showArchived = segments[index];
			index++;

           	// 4: getCached
			this.getCached = segments[index];
		},
		
		_logError: function(errorObject) {
			
			var className = "";
			if (this.declaredClass) {
				className = this.declaredClass + ": ";
			} 
			
			var lineNumber = "";
			if (errorObject.lineNumber) {
				lineNumber = "(line " + errorObject.lineNumber + "): ";
			} 
			
			var errorMessage = "[ERROR]: " +
								className +
								lineNumber +
								errorObject.message;
								
			console.error(errorMessage);
		},		
		
		_initReport: function() {
			// Get the report descriptor that is used to select parameters
			var args = {
				projectAreaUUID: this.projectAreaUUID,
				reportUUID: this.reportUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleReportDescriptor", "_remoteError");
			ReportsClient.getReportDescriptor(srh, args);
		},
		
		_initQuery: function() {
			this._isQuery = true;
			// Get the query descriptor that is used to select parameters
			var args = {
				queryUUID: this.queryUUID
			};
			var srh = new ServiceResponseHandler(this, "_handleQueryDescriptor", "_remoteError");
			ReportsClient.getQueryDescriptor(srh, args);
		},
		
		_handleReportDescriptor: function(reportDescriptor) {
			this.result = reportDescriptor;
			this.reportUUID = reportDescriptor.reportUUID;
			this._initReportEngineParameters(reportDescriptor.engine);
			if (this.queryUUID.length > 0) {
				var args = {
					queryUUID: this.queryUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleReportQueryDescriptor", "_remoteError");
				ReportsClient.getQueryDescriptor(srh, args);
			}
			else {
				if (!this.titleHidden) {
					this._setReportTitle(reportDescriptor.name);
				}
				
				if (this.customParams) {
					var args = {
						reportUUID: this.reportUUID,
						projectAreaUUID: this.projectAreaUUID,
					};
					 var srh = new ServiceResponseHandler(this, "_handleCustomParameters", "_remoteError");
					 ReportsClient.getCustomParams(srh, args);
				} else {
					var args = {
							projectAreaUUID: this.projectAreaUUID,
							reportUUID: this.reportUUID
					};
					var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
					ReportsClient.getParameters(srh, args);
				}
			}
			this._flushPendingRequests();
		},
		
		_handleReportQueryDescriptor: function(queryDescriptor) {
			this.result = queryDescriptor;
			if (!this.titleHidden) {
				this._setReportTitle(queryDescriptor.name);
			}
			
			if (this.customParams) {
				var args = {
						reportUUID: this.reportUUID,
						projectAreaUUID: this.projectAreaUUID,
						queryUUID: this.queryUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleCustomParameters", "_remoteError");
				ReportsClient.getCustomParams(srh, args);
			} else {
				// Fetch the parameters
				if (queryDescriptor.parameterNames) {
					this.parameterName = queryDescriptor.parameterNames;
					this.parameterValue = queryDescriptor.parameterValues;
				}
				this.queryUUID = "";
				var args = {
						projectAreaUUID: this.projectAreaUUID,
						reportUUID: this.reportUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
				ReportsClient.getParameters(srh, args);
			}
			this._flushPendingRequests();
		},
		
		_handleQueryDescriptor: function(queryDescriptor) {
			this.result = queryDescriptor;
			this.reportUUID = queryDescriptor.reportUUID;
			this._initReportEngineParameters(queryDescriptor.engine);
			if (!this.titleHidden) {
				this._setReportTitle(queryDescriptor.name);
			}
			
			if (this.customParams) {
				var args = {
					reportUUID: this.reportUUID,
					projectAreaUUID: this.projectAreaUUID,
					queryUUID: this.queryUUID
				};
				 var srh = new ServiceResponseHandler(this, "_handleCustomParameters", "_remoteError");
				 ReportsClient.getCustomParams(srh, args);
			} else {
				// Fetch the parameters
				if (queryDescriptor.parameterNames) {
					this.parameterName = queryDescriptor.parameterNames;
					this.parameterValue = queryDescriptor.parameterValues;
				}
				var args = {
						queryUUID: this.queryUUID
				};
				var srh = new ServiceResponseHandler(this, "_handleParameters", "_remoteError");
				ReportsClient.getParameters(srh, args);
			}
			this._flushPendingRequests();
		},
		
		_initReportEngineParameters: function(descriptor) {
			this.titleHidden = descriptor.titleHidden;			
			this.customParams = descriptor.hasCustomParams;
			this.supportsArchived = descriptor.supportsArchived;
			if (this._archivedDiv) {
				if (descriptor.supportsArchived) {
					this._archivedDiv.style.visibility = "visible";
				} else {
					this._archivedDiv.style.visibility = "hidden";
				}
			}
			for (var i = 0; i < this.metanav._exportSelect.options.length; i++) {
  				this.metanav._exportSelect.options[i] = null;
			}
			if (!descriptor.exportValues || descriptor.exportValues.length == 0) {
				// Do nothing
			}
			else {
				this.enableExport = true;
				var index = 0;
				for (var i = 0; i < descriptor.exportValues.length; i++) {
					var value = descriptor.exportValues[i];
					var label = value;
					if (descriptor.exportLabels != null && descriptor.exportLabels.length > i) {
						label = descriptor.exportLabels[i];
					}
					if (value != null) {
						var option = document.createElement("option");
						option.value = value;
						option.text = label;
						this.metanav._exportSelect.options[index] = option;
						index++;
					}
				}
			}
		},
		
		destroy: function() {
			if (!this.rcpCall) {
				if (this.navigateAwayHandle) {
					PlatformUI.getWorkbench().unregisterBodyClickHandler(this.navigateAwayHandle);
				}
				if (this.onBeforeUnloadHandle){
					PlatformUI.getWorkbench().unregisterOnBeforeUnloadHandler(this.onBeforeUnloadHandle);	
				}
			}
			var params = this.concealedReportParams;
			if (params) {
				var length=params.length;
				for (var i = 0; i<length; ++i){
					params[i].destroy();
				}	
			}
			
			//TODO if there was a temp query UUID, delete it
			if (this._customParamQueryUUID) {
				var obj = {
					success: function() {
						if (djConfig.isDebug) {
							console.log("[INFO]: ReportViewer.js: Temp report query deleted");
						}
					},
					
					error: function() {
						console.error("[ERROR]: ReportViewer.js: Failed to delete the temp report query");
					}
				};
				
				var args = {
					queryUUID: this._customParamQueryUUID,
					onlyIfInTempStorage: true		
				};
				
				this._callRestService("postDeleteQueryDescriptor", args, obj, "success", "error");
				delete this._customParamQueryUUID;
			}
				
			this.metanav._exportDialog.destroyRecursive();
			
			if (this._styledBox) {
				this._styledBox.destroy();
			}
		},
		
		startup: function() {
			this.inherited(arguments);
			this._styledBox.startup();
		},			
		
		setDirty: function(dirty) {
			if (dirty == this.isDirtie) return;
			
			if (this._saveButton.disabled === true) {
				// if save is disabled, don't show the dirty indicator either
				return;
			}
			
			this.isDirtie = dirty;
			this._setReportTitle(this.reportTitle);
		},
		
		_handleRun: function(event) {
			this._clearErrorMessage();
			dojo.stopEvent(event);
			this.renderedReportDiv.className = "loading";
			this.renderedReportDiv.innerHTML = this.messages.loading;
			this._updateReport();
		},
		
		_handleRefresh: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (this.inEditMode) {
				this._handleEdit();
			}
			for (i in this.reportParams) {
				var reportParam = this.reportParams[i];
				reportParam.destroy();
			}
			this.reportParams.length = 0;
			this._removeChildren(this.reportParamsPanel.domNode);
			if (this.reportUUID.length > 0) {
				this._initReport();
			}
			else {
				this._initQuery();
			}
			this._flushPendingRequests();
		},
		
		_flushPendingRequests: function() {
			if (this._pendingInfoRequests && this.displayBreadcrumbs) {
				var info = this._getInfo();
				for (var i = 0; i < this._pendingInfoRequests.length; i++) {
					this._pendingInfoRequests[i].doCallback(info);
				}
			}
			this._pendingInfoRequests = null;
		},
		
		_handleCancel: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			
			if (this.isDirtie) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					for (var i = 0; i < this.reportParams.length; i++) {
						var reportParam = this.reportParams[i];
						var vals = reportParam.getSelectedValues();
						var noSelection = (vals == null || vals.length == 0 || vals[0] == undefined);
						if (!noSelection) { reportParam.resetValues(); }
					}
					this.inEditMode = false;
					this._button.className = "jazz-ui-SimpleToolbar-button";
					this._styledBoxContainer.className = "display-none";
					this.setDirty(false);
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.inEditMode = false;
				this._button.className = "jazz-ui-SimpleToolbar-button";
				this._styledBoxContainer.className = "display-none";
				return false;
			}
		},
		
		_handleEdit: function(event) {
			if (event) {
				dojo.stopEvent(event);
			}
			if (!this.enableParams) {
				return;
			}
			if (this.inEditMode) {
				this.inEditMode = false;
				this._button.className = "jazz-ui-SimpleToolbar-button";
				this._styledBoxContainer.className = "display-none";
				if (this._editableTitle) {
					this._editableTitle.set('disabled', true);
				}
			} else {
				this.inEditMode = true;
				if (this._button) {
					this._button.className = "jazz-ui-SimpleToolbar-button-selected";
				}
				this._styledBoxContainer.className = "editPanel";
				var self = this;
				if (!this._editableTitle) {
			        this._editableTitle = new dijit.InlineEditBox({title: this.messages.clickToEdit, autoSave: true, 
			        	width: "400px", noValueIndicator: "?", 
			        	onChange: dojo.hitch(this, function(newName) {
							self._handleResourceNameChange(newName);
						})}, this.metanav.heading);
					this._editableTitle.startup();
				} else {
					this._editableTitle.set('disabled', false);
				}
			}
		},
		
		_handleResourceNameChange: function(newName) {
			if (this.inEditMode === false && this.isDirtie === false && newName === this.reportTitle) { return; }
			this._setReportTitle(newName);
			this.setDirty(true);
		},
		
		_handleCustomParameters: function(customParams) {
			if (!customParams || !customParams.html || customParams.html === "") {
				//this report has no parameters; go ahead and display it directly
				this._setEnableParams(false);
				this._updateReport()
			} else {
				this._setEnableParams(true);
				this.reportParamsPanel.insertParamHTML(customParams.html);
				this._customParamQueryUUID = customParams.tempQueryUUID;
				
				if (customParams.missingRequired) {
					this._removeChildren(this.renderedReportDiv);
					this._handleEdit();
				} else if (this.parameterName) {
					//TODO Cognos params do not load properly if edit panel is closed - can we work around it another way?
					this._removeChildren(this.renderedReportDiv);
					this._handleEdit();
				}
			}			
			
			if (this.controller) {
				this.controller.finishOperation();
			}			
		},		
				
		_handleParameters: function(baseParamsArray) {
			if (!baseParamsArray || baseParamsArray.length == 0) {
				baseParamsArray = [];
				this._setEnableParams(false);
			}
			else {
				this._setEnableParams(true);
			}
			
			if (!this._isQuery) {
				dojo.forEach(baseParamsArray, dojo.hitch(this, function(param) {
					if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
						for (var i = 0; i < param.parameters.length; i++) {
							var parameterDTO = param.parameters[i];
							if (this[parameterDTO.name]) {
								this.parameterName[this.parameterName.length] = parameterDTO.name;
								this.parameterValue[this.parameterValue.length] = this[parameterDTO.name];
							}
						}
					}
					else if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
						var parameterDTO = param;
						if (this[parameterDTO.name]) {
							this.parameterName[this.parameterName.length] = parameterDTO.name;
							this.parameterValue[this.parameterValue.length] = this[parameterDTO.name];
						}
					}
				}));
			}
			
			var missingQueryParams = false;
			
			dojo.forEach(baseParamsArray, dojo.hitch(this, function(param) {
				var args = {
					projectAreaUUID: this.projectAreaUUID,
					queryUUID: this.queryUUID,
					reportUUID: this.reportUUID,
					showArchived: this._showArchived()
				};
				if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterGroupDTO") {
					var reportParamGroup = new com.ibm.team.reports.web.ui.internal.ReportParamGroup({
						parameterGroupDTO: param,
						urlPrefix: this.urlPrefix,
						parameterName: this.parameterName,
						parameterValue: this.parameterValue,
						dirtyListener: this
					});
					this.reportParamsPanel.appendParamGroup(reportParamGroup);
					reportParamGroup.appendReportParameters(this.reportParams);
					var self = this;
					var obj = {
						success: function(reportParameterDTO){
							reportParamGroup.initGroupValues(reportParameterDTO);
							reportParamGroup.startup();
						},
						error: function(error){
							self._showErrorMessage(error.message);
						}
					}
					var srh = new ServiceResponseHandler(obj, "success", "error");
					args.groupName = param.name;
					ReportsClient.getParameterGroupValues(srh, args);
					
					dojo.forEach(param.parameters, dojo.hitch(this, function(singleParam) {
						if (singleParam.required && 
						    (!singleParam.defaultValue || 
						     singleParam.defaultValue.length == 0 || 
						     singleParam.defaultValue == "''")) {
							var found = -1;
							if (this.parameterName.length == 0) {
								missingQueryParams = true;
							} else {
								for (var i = 0; i < this.parameterName.length; i++) {
									if (singleParam.name == this.parameterName[i]) {
										found = i;
										break;
									}
								}
								if (found != -1 && 
									   (found >= this.parameterValue.length || 
									    !this.parameterValue[found] || 
									    this.parameterValue[found].length == 0 ||
									    this.parameterValue[found] == "''")) {
									missingQueryParams = true;
								}
							}
						}
					}));
				} else if (param._eQualifiedClassName == "com.ibm.team.reports.dto:ReportParameterDTO") {
					var reportParam = ReportParamFactory.create(param, this.urlPrefix);
					if (!param.concealed) {
						reportParam.isRoot = true;
						this.reportParamsPanel.appendParam(reportParam);
					} else{
						this.concealedReportParams.push(reportParam);
					}					 
					
					this.reportParams.push(reportParam);
					
					if (reportParam.isDeferred) {												
						//provide call back to the parameter object so it can fetch its own values when ready
						reportParam.getValuesDeferred = dojo.hitch(this, function(additionalArgs) {
							
							var obj = {
									success: function(reportParameterDTO) {
										reportParam.init(reportParameterDTO);
									},
									error: dojo.hitch(this, function(error) {
										this._showErrorMessage(error.message);
									})
									
							};
							var srh = new ServiceResponseHandler(obj, "success", "error");						
														
							args.parameter = param.name;						
							
							var deferred = this._getParameterSelections(false);
							deferred.addCallback(dojo.hitch(this, function(selections){
								args.existingParameterNames = selections.parameterNames;
								args.existingParameterValues = selections.parameterValues;
								
								if (additionalArgs) {
									dojo.mixin(args, additionalArgs);
								}
								
								ReportsClient.postComputeParameterValues(srh, args);
							}));
							deferred.addErrback(function(err) {
								obj.error(err);
							});
						});
						
						//if there is an existing value for this parameter, set it
						if (this.parameterName.length > 0) {
							for (var i = 0; i < this.parameterName.length; i++) {
								if (this.parameterName[i] == reportParam.parameterDTO.name) {
									reportParam.setValues(this.parameterValue[i]);
									break;
								}
							}
						}
					} else {
						var self = this;
						var obj = {
							success: function(reportParameterDTO) {
								reportParam.init(reportParameterDTO);
								if (self.parameterName.length > 0) {
									for (var i = 0; i < self.parameterName.length; i++) {
										if (self.parameterName[i] == reportParam.parameterDTO.name) {
											reportParam.setValues(self.parameterValue[i]);
											break;
										}
									}
								}
								reportParam.postInit();
							},
							error: function(error) {
								self._showErrorMessage(error.message);
							}
						}
						
						var srh = new ServiceResponseHandler(obj, "success", "error");
						args.parameter = param.name;
						ReportsClient.postComputeParameterValues(srh, args);
					}
					
					if (param.required && 
					    (!param.defaultValue || 
					     param.defaultValue.length == 0 || 
					     param.defaultValue == "''")) {
						var found = -1;
						if (this.parameterName.length == 0) {
							missingQueryParams = true;
						}
						else {
						for (var i = 0; i < this.parameterName.length; i++) {
							if (param.name == this.parameterName[i]) {
								found = i;
								break;
							}
						}
							if (found != -1 && 
							    (found >= this.parameterValue.length || 
								!this.parameterValue[found] || 
								this.parameterValue[found].length == 0 ||
							     this.parameterValue[found] == "''")) {
							missingQueryParams = true;
						}
					}
				}
				}
			}));
			if (this.controller) {
				this.controller.finishOperation();
			}
			if (missingQueryParams) {
				this._removeChildren(this.renderedReportDiv);
				this._handleEdit();
			} else if (this.parameterName) {
				this._updateReport(this.parameterName, this.parameterValue);
			}
		},
		
		_displayChart: function(chartHTML) {
			this.renderedReportDiv.className = "renderedReport";
			
			// Check the chart for any <link> elements in the head. If they exist,
			// add the styles to the document.
			var startIndex = chartHTML.indexOf("<link rel=\"stylesheet\" type=\"text/css\"");
			if (startIndex != -1) {
				startIndex = chartHTML.indexOf("href=");
				var endIndex = chartHTML.indexOf("</link>");
				var cssHref = chartHTML.substring(startIndex + 6, endIndex - 2);
				
				var linkNode = document.createElement("link");
				linkNode.setAttribute("rel", "stylesheet");
				linkNode.setAttribute("type", "text/css");
				linkNode.setAttribute("href", cssHref);
				document.getElementsByTagName("head")[0].appendChild(linkNode);
			}
						
			if (dojo.isIE) {
				// Insert innerHTML directly. That, coupled with defer tags
				// on the scripts, causes new Javascript to be executed properly.
				this.renderedReportDiv.innerHTML = chartHTML;
			} else {
				// Create a new node and appendChild so javascript gets executed.
				var newNode = document.createElement('div');
				newNode.innerHTML = chartHTML;
				this._removeChildren(this.renderedReportDiv);
				this.renderedReportDiv.appendChild(newNode);
			}
		},
		
		_callRestService: function(methodName, args, scopeObj, successMethodName, failureMethodName) {
			var srh = new com.ibm.team.repository.web.transport.ServiceResponseHandler(scopeObj, successMethodName, failureMethodName);
			var serviceRequest = new com.ibm.team.repository.web.transport.ServiceRequest("com.ibm.team.reports.common.internal.service.IReportRestService", methodName, args);
			com.ibm.team.repository.web.transport.TeamServerClient.invokeService(serviceRequest, srh);
		},
		
		_updateReport: function(parameterNames, parameterValues) {
			var deferred = null;
			if (!parameterNames && !parameterValues) {
				deferred = this._getParameterSelections(true);
			}
			else {
				selections = {};
				selections.parameterNames = parameterNames;
				selections.parameterValues = parameterValues;
				//make a "dummy" deferred object that will just return the passed selections
				var deferred = new dojo.Deferred();
				deferred.callback(selections);
			}
			
			deferred.addCallback(dojo.hitch(this, function(selections){
				var args = {
						projectAreaUUID: this.projectAreaUUID,
						embeddable: true,
						parameterName: selections.parameterNames,
						parameterValue: selections.parameterValues,
						urlPrefix: this.urlPrefix,
						showArchived: this._showArchived(),
						getCached: this.getCached,
						rcpCall: this.rcpCall,
						requestId: dojox.uuid.generateRandomUuid()
					};
					var restServiceName;
					if (this.reportUUID.length > 0) {
						restServiceName = "postRenderReport"
						args.reportUUID = this.reportUUID;
						args.queryUUID = this.queryUUID;
					}
					else if (this.queryUUID.length > 0) {
						restServiceName = "postRenderQuery"
						args.queryUUID = this.queryUUID;
					}
					else {
						return;
					}
					var callback = {
						success: dojo.hitch(this, this._displayChart),
						error: dojo.hitch(this, this._remoteError)
					}
					this._callRestService(restServiceName, args, callback, "success", "error");
					
					if (this.inEditMode) {
						// collapse the parameters section			
						this._handleEdit();
					}
			}));
			deferred.addErrback(dojo.hitch(this, function(err){
				this._remoteError({
					message: err
				});
			}));			
		},
		
		_postSave: function() {
			this.setDirty(false);
			if (this.controller) {
				// do nothing
			}
			else {
				// Assume we are in the RCP ui
				window.location = "http://com.ibm.team.reports/new_query/";
			}
		},
		
	    _handleNavigateAway: function(url) {
	    	var currentLocation = document.location.href;
			var badLink = currentLocation.substring(0, currentLocation.indexOf("#")+1);
			if (url == badLink) {
				return true;
			}
			
			if (this.isDirtie) {
				if (confirm(this.messages.navigateAway + "\n\n" + this.messages.unsavedChanges + "\n\n" + this.messages.okOrCancel)) {
					this.destroy();
					return false;
				}
				else {
					return true;
				}
			}
			else {
				this.destroy();
				return false;
			}
        },
        
        _handleOnBeforeUnload: function() {
        	return this.isDirtie ? this.messages.unsavedChanges + "\n\n" : false;
        },
		
        /**
		 * Returns the currently selected values for all parameters.
		 * 
		 * param reportMissing (true/false) - if true, then missing values for
		 * any required parameters will be treated as a fatal error and
		 * reported. If false, then parameters with missing values are simply
		 * not included in the returned result.
		 */
		_getParameterSelections: function(reportMissing) {
			if (this.customParams && this._customParamQueryUUID) {
				//call the report service to get the query descriptor where params are being stored and act accordingly
				var args = {
						queryUUID: this._customParamQueryUUID
				};
				var deferred = new dojo.Deferred();
				var self = this;
				var serviceObject = {
						success: function(queryDescriptor) {							
							var result = { 
									parameterNames : queryDescriptor.parameterNames,
									parameterValues : queryDescriptor.parameterValues
							};
							
							if (!result.parameterNames) {
								result.parameterNames = [];
							}
							
							if (!result.parameterValues) {
								result.parameterValues = [];
							}

							deferred.callback(result);							
						},
						error: function(err) {
							self._remoteError(err);
						}
				};

				var srh = new ServiceResponseHandler(serviceObject, "success", "error");
				ReportsClient.getQueryDescriptor(srh, args);						
				return deferred;
			} else {
				var deferred = new dojo.Deferred();
				var result = {
						parameterNames: [],
						parameterValues: []
				};
				var parameterNames = [];
				var parameterValues = [];
				for (var i = 0, len = this.reportParams.length; i < len; i++) {
					var reportParam = this.reportParams[i];				
					var vals = reportParam.getSelectedValues();
					var noSelection = (vals == null || vals.length == 0 || vals[0] == undefined || vals == "''");
					if (noSelection) {
						if (reportParam.parameterDTO.required) {
							if (reportMissing) {
								var label = reportParam.parameterDTO.name;
								if (reportParam.parameterDTO.displayName && reportParam.parameterDTO.displayName.length > 0) {
									label = reportParam.parameterDTO.displayName;
								}
								var message = dojo.string.substitute(this.messages.missingReqParam, [label]);
								deferred.errback(message);
								return deferred;
							}
						} else {
							result.parameterNames.push(reportParam.parameterDTO.name);
							result.parameterValues.push("");
						}
					} else {
						result.parameterNames.push(reportParam.parameterDTO.name);
						result.parameterValues.push(vals.join(",")); 
					}
				}
				deferred.callback(result);
				return deferred;
			}			
		},
		
		_handleExport: function() {
			if (!this.enableExport) {
				return;
			}
			var deferred = this._getParameterSelections(true);
			deferred.addCallback(dojo.hitch(this, function(selections){
				if (selections.parameterNames.length > 0) {
					var args = {
						queryUUID: this.queryUUID,
						reportUUID: this.reportUUID,
						parameterName: selections.parameterNames,
						parameterValue: selections.parameterValues
					};
					var srh = new ServiceResponseHandler(this, "_handleSaveQueryParameters", "_remoteError");
					ReportsClient.postSaveQueryParameters(srh, args);
				}
				else {
					var fmt = this.metanav._exportSelect.options[this.metanav._exportSelect.selectedIndex].value;
					var url = this.urlPrefix + "service/com.ibm.team.reports.service.internal.IReportViewerService";
					url += "?__format=" + fmt;
					url += "&__reportUUID=" + this.reportUUID;
					url += "&__queryUUID=" + this.queryUUID;
					if (this.controller){
						url += "&__showArchived=" + this._showArchived();
					}
					this.metanav._exportDialog.hide();
					document.location.href = url;
				}
			}));
			deferred.addErrback(dojo.hitch(this, function(err){
				this._remoteError({
					message: err
				});
			}));			
		},
		
		_handleSaveQueryParameters: function(result) {
			var fmt = this.metanav._exportSelect.options[this.metanav._exportSelect.selectedIndex].value;
			var url = this.urlPrefix + "service/com.ibm.team.reports.service.internal.IReportViewerService";
			url += "?__format=" + fmt;
			url += "&__queryUUID=" + result;
			if (this.controller){
				url += "&__showArchived=" + this._showArchived();
			}
			this.metanav._exportDialog.hide();
			document.location.href = url;
		},
		
		_handleSave: function() {
			//TODO if save successful, and we have a temp query UUID, make sure to delete it
			this._clearErrorMessage();
			var deferredGetSelections = this._getParameterSelections(true);
			deferredGetSelections.addCallback(dojo.hitch(this, function(selections){
				if (!selections) {
					return;
				}
				var obj = {
					success: dojo.hitch(this, function(args) {
						this._postSave();
					}),
					error: dojo.hitch(this, "_remoteError")
				};
				var args = {
					projectAreaUUID: this.projectAreaUUID,
					queryUUID: this.queryUUID,
					reportUUID: this.reportUUID,
					folderUUID: "" /* Not moving the report */,
					name: this.reportTitle,
					parameterName: selections.parameterNames,
					parameterValue: selections.parameterValues
				};
				this._callRestService("postSaveQueryDescriptor", args, obj, "success", "error");
			}));
			deferredGetSelections.addErrback(dojo.hitch(this, function(err){
				this._remoteError({
					message: err
				});
			}));
		},
		
		_handleSaveAs: function() {
			var onSubmit = dojo.hitch(this, function(saveArgs) {
				var deferred = this._getParameterSelections(true);
				deferred.addCallback(dojo.hitch(this, function(selections){
					saveArgs.parameterName = selections.parameterNames;
					saveArgs.parameterValue = selections.parameterValues;
					var obj = {
						success: dojo.hitch(this, function(args) {
							document.location.href = "#action=com.ibm.team.reports.viewQuery&queryUUID=" + args;
							this._postSave();
						}),
						error: dojo.hitch(this, "_remoteError")
					};
					this._callRestService("postSaveQueryDescriptor", saveArgs, obj, "success", "error");
				}));
				deferred.addErrback(dojo.hitch(this, function(err){
					this._remoteError({
						message: err
					});
				}));
			});
			var saveAsDialog = new com.ibm.team.reports.web.ui.internal.SaveAsDialog({
				onSubmit: onSubmit,
				reportUUID: this.reportUUID,
				queryUUID: this.queryUUID,
				projectAreaUUID: this.projectAreaUUID
				
			});
			saveAsDialog.open();
		},
		
		_setReportTitle: function(title) {
			if (this.isDirtie) {
				this.metanav.heading.innerHTML = StringUtils.escapeAndCheckForNull(title + "*");
				this.metanav.heading.style.color = "#A83300";
			} else {
				this.metanav.heading.innerHTML = StringUtils.escapeAndCheckForNull(title);
				this.metanav.heading.style.color = "#000000";
			}
			this.reportTitle = title;
		},
		
		_remoteError: function(error) {
			if (this.isObsolete) {
				return;
			}
			this._removeChildren(this.renderedReportDiv);
			this._showErrorMessage(error.message);
		},
		
		_removeChildren: function(element) {
			while (element.firstChild) {
				element.removeChild(element.firstChild);
			}
		},
		
		_setProjectAreaMismatchMessage: function(projectArea) {
			this.setHeaderMessage({
				message: dojo.string.substitute(this.messages.msgProjectAreaIsDifferent, [projectArea.name, jazz.app.context.get().name]),
				details: this.messages.msgProjectAreaIsDifferentDetails,
				showOnlyMessage: true
			});		
		},
		
		setHeaderMessage: function(message) {
			this.clearValidationMessage();
			if (!message) {
				return;
			}
			
            if (!message.details) {
                message.details = "";
            }
            
            var messageText;
            if (message.message.length > MAX_MESSAGE_LENGTH) {
                messageText = message.message.substring(0, MAX_MESSAGE_LENGTH) + "...";
            } else {
            	if (message.showOnlyMessage == true) {
            		messageText = message.message
            	} else {
                	messageText = message.message + " " + message.details;            		
            	}
                if (messageText.length > MAX_MESSAGE_LENGTH) {
                    messageText = messageText.substring(0, MAX_MESSAGE_LENGTH) + "...";
                }
            }
            messageText = messageText.replace(/\.\.\.\./, "...");
            
            this.metanav.validationMessage.__jazzMessage = message;
            
			// We only support Info
			this.metanav.validationMessageSpan.className = "validationMessageInfo";

            this.metanav.validationMessage.title = message.details || message.message;
            dojox.data.dom.replaceChildren(this.metanav.validationMessage, document.createTextNode(messageText));
		},
		
		_handleHeaderMessageClick: function(e) {
			e.preventDefault();
			if (!e || !e.target || !e.target.__jazzMessage) return;

			var message = e.target.__jazzMessage;
			var text = message.message;
			if (message.details) {
				text += "\n\n";
				text += message.details;
			}
			alert(text);
		},
		
		clearValidationMessage: function(force) {
			this.metanav.validationMessageSpan.className = "validationMessageEmpty";
		},
		
		_setEnableParams: function(enabled) {
			this.enableParams = enabled;
			this._initToolbar();
		},
		
		_showErrorMessage: function(message) {
			this._clearErrorMessage();
			var status = {
				type : jazz.ui.MessageBox.STATUS_TYPES.ERROR,
				summary : message,
				closable : false
			};
			dojo.addClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.showMessage(status);
		},
		
		_clearErrorMessage: function() {
			dojo.removeClass(this._messageBoxContainer, "messageBoxVisible");
			this.messageBox.hideMessage();
		},
		
		_showArchived: function() {
			if (this.controller) {
				return this.controller.getIncludeArchived();	
			}
			return this.showArchived;
		},
		
		getArtifactInfo: function(srh) {
			if (!this.result || !this.displayBreadcrumbs) {
				if (!this._pendingInfoRequests) {
					this._pendingInfoRequests = [];
				}
				this._pendingInfoRequests.push(srh);
			}
			else {
				srh.doCallback(this._getInfo());
			}
		},
		
		_getInfo: function() {
			if (!this.result || !this.displayBreadcrumbs) {
				return null;
			}
			var shared = this.shared == null ? "" : this.shared.toString();
			var itemUUID = this._isQuery ? this.queryUUID : this.reportUUID;
			return {
				type: ReportsConstants.VIEWER_TYPE_ID,
				itemId: itemUUID,
				title: this.reportTitle,
				iconUrl: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/images/report-obj.gif"),
				href: ReportsUtil.getReportHref(this.reportTitle,itemUUID,shared),
				url: ReportsUtil.getReportUri(ReportsConstants.VIEWER_TYPE_ID,this.reportTitle,itemUUID,shared)
			};
			
		},
		
	});
	
})();
