/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.web.ui.internal.SelectionDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.ui.Dialog");

dojo.require("com.ibm.team.reports.web.ui.internal.ListBox");
dojo.require("com.ibm.team.reports.web.client.internal.ReportsClient");

dojo.requireLocalization("com.ibm.team.reports.web.ui.internal", "SelectionDialogMessages");

(function(){

/**
 * A dialog box that allows the user to select from a list of values.  The values (along with their labels) can be provided
 * in the constructor, or clients may call populateSelectionList to initialize the list after instantiation. 
 */
dojo.declare("com.ibm.team.reports.web.ui.internal.SelectionDialog", [dijit._Widget, dijit._Templated], {
	
	/**
	 * Template elements:
	 * contentNode - the wrapping widget div
	 * listDiv - div where the selection list goes
	 * okButton - onclick: _onOk
	 * cancelButton -onclick: _onCancel
	 */
	templatePath: dojo.moduleUrl("com.ibm.team.reports.web", "ui/internal/templates/SelectionDialog.html"),
	
	//mixin fields
	onOK: null, //function; called when a selection is made
	values: null, //[]; optional.  If null, indicates that the values are still being loaded.
	labels: null, //[]; required if values are provided; otherwise optional
	
	_listBox: null,
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.web.ui.internal", "SelectionDialogMessages");
	},

	postCreate: function() {	 	
		var listArgs = {
			allowMultipleSelections: false, //only allow single select for now
			width: "", //unset the width to have it fill available space
			callbackSingleClick: dojo.hitch(this, this._itemSelected),
			sortDirection: "asc"
		};
		this._listBox = new com.ibm.team.reports.web.ui.internal.ListBox(listArgs);
		this.listDiv.appendChild(this._listBox.domNode);
		
		if (this.values) {
			this.populateSelectionList(this.values, this.labels);
		} else {
			this._listBox.showLoading(true);
		}
	
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.dialogTitle,
			width: "300px",
			isClosable: true
		});
		
	},
	
	populateSelectionList: function(values, labels) {
		this.values = values;
		this.labels = labels;	
		this._resetList();
		this._populateList(values, labels);
	},
	
	/**
	 * Clears the contents of the list box.
	 */
	_resetList: function() {
		this._listBox.showLoading(false);
		this._listBox.removeAllEntries();
		this.okButton.disabled = true;
	},
	
	/**
	 * Called when the OK button is selected.
	 */
	_onOk: function() {
		var selected = this._listBox.getSelectedEntries()[0]; //we only allow single selection
		this._closeDialog();
		this.onOk(selected); //pass the entry directly, as it has the required label and value fields
	},
	
	/**
	 * Called when the cancel button is selected.
	 */
	_onCancel: function() {
		this._closeDialog();
	},
	
	/**
	 * Closes this dialog.
	 */
	_closeDialog: function() {
		if (this.dialog) {
			this.dialog.close();
		}		
	},
	
	/**
	 * Callback when an entry in the list box is clicked.
	 */
	_itemSelected: function() {
		//as soon as something gets selected, we can enable the OK button - don't have to worry about deselection
		this.okButton.disabled = false;
	},
	
	/**
	 * Updates the list box with the given values and labels.
	 */
	_populateList: function(values, labels) {
		//TODO remove all existing entries first?
		var entries = [];
		for (var i = 0; i < labels.length; i++) {
			var id = values[i]; //TODO?
			if (this._listBox.getEntry(id) == null) {
				var label = labels[i];
				var value = values[i];
				entries.push({
					label: label,
					value: value,
					id: id,
					type: 0
				});
			}
		}
		this._listBox.addEntries(entries);
	},
	
});	
	
})();
