/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2011. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.reports.cognos.web.ui.internal.UploadCognosReport");

dojo.require("com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient");
dojo.require("com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.Tree");
dojo.require("dojo.data.ItemFileReadStore");

dojo.require("jazz.app.context");

dojo.requireLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");

(function() { // module container

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var CognosReportsClient = com.ibm.team.reports.cognos.web.client.internal.CognosReportsClient;
var NewTemplateDialog = com.ibm.team.reports.cognos.web.ui.internal.NewTemplateDialog;

/**
 * Widget to create dialog for Uploading cognos report.
 */
dojo.declare("com.ibm.team.reports.cognos.web.ui.internal.UploadCognosReport", [dijit._Widget, dijit._Templated], {
		templatePath: dojo.moduleUrl("com.ibm.team.reports.cognos.web", "ui/internal/templates/UploadCognosReport.html"),
				
		constructor: function() {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.reports.cognos.web.ui.internal", "messages");
		},
		
		postCreate: function() {
			var self = this;		
			this.newTemplateDialog = new NewTemplateDialog();
			this.connect(this.newTemplateDialog._okButton, "onclick", this._onOk);			
			this.connect(this.newTemplateDialog._cancelButton, "onclick", this.destroy);			
		},
		
		uninitialize: function(){
			this.newTemplateDialog.destroy();
		},
		
		_onOk: function(){
			var currentProjectArea = jazz.app.context.get();
			var args={
				reportPath: this.newTemplateDialog.reportPath,
				name: this.newTemplateDialog._nameInput.value,
				projectArea: currentProjectArea.itemId,
				//processId: processDefinitionId.itemId,
				description: this.newTemplateDialog._descriptionInput.value,
				sharedIfNullFolder: false,  //This might be set again later on if the folder is null (meaning create at the root).
				id: this.newTemplateDialog._IDInput.value,
				fileName: this.newTemplateDialog._reportInput.value,
				isShared: true
			};
			//If the folder is not null then use the folder ID to create the report
			//Otherwise, if the folder is null then set the shardIfNullFolder attribute so the report
			//can be created in either the MyReports or SharedReports root.
			var folderUUID = this.newTemplateDialog._folderTreeWidget.getSelection()[0].folderUUID;
			if (folderUUID != null)
			{
				args.folderUUID = folderUUID;
			}
			else
				args.sharedIfNullFolder = this.newTemplateDialog._folderTreeWidget.getSelection()[0].isSharedRoot;
				
			var responseHandler = new ServiceResponseHandler(
				this,
				"_uploadComplete",
				"_Error");
			CognosReportsClient.postUploadCognosReportTemplate(responseHandler, args);
			
		},
		
		_Error: function(error){
			if (error.errorClass && error.errorClass.indexOf("ReportDescriptorConflictException") >= 0) 
				alert(this.messages.errorDuplicateTemplate);
			else
				alert(error.message);
			this.destroy();
		},
			
		_uploadComplete: function(message){
			if ((message != null) && (message.indexOf("_") != -1)) {
				console.debug("Template uploaded");
				alert(this.messages.msgSuccess);
			} else if (message=="ReportDescriptorConflictException") {
				alert(this.messages.errorDuplicateTemplate);
			}
			this.reportsView.refreshCallback();
			
			this.destroy();
		}
	}
);

})();