/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorContainer"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ErrorTab"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
	dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.underlay"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.require("net.jazz.ajax.contextRoot");//$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorContainer", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ConfigurationDataEditorContainer.html"), //$NON-NLS-1$ //$NON-NLS-2$
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._processAreaEditor = args.processAreaEditor; //hold the outer project area editor
			this._projectAreaItemId = args.projectAreaItemId;
			this._configurationDataEditorId = args.configurationEditorId;
			this._configurationDataEditorMgr = args.configurationDataEditorMgr;
			this._configurationDataEditorName = this._configurationDataEditorMgr.getConfigurationDataEditorExtension(this._configurationDataEditorId).name;
			this._configurationDataEditor = null;
			this._isDeleteConfirmed = false;
			this._isFinalStatusChanged = false;
			this._isCustomizeConfirmed = false;
			this._isFinalChecked = false;
			this._isDataInherited = false;
			this._isInheritedDataFinal = false;
		},

		postCreate: function(){
			this.connect(this.finalCheckBox, "onclick", "_onFinalClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.customizeAnchor, "onclick", "_onCustomizeClick"); //$NON-NLS-1$ //$NON-NLS-2$
			this._asynCreateConfigurationDataEditor();
		},
		
		/**
		 * Process framework call this method to refresh the editor.
		 */
		refresh: function() {
			this._clearState();
			this._asynCreateConfigurationDataEditor();
		},
		
		deactivateTab: function() {
			if (this.changesPending()) {
				//we don't want to hinder tab deactivation because of an error
				//in a contributed widget
				UIHelper.runSafe(dojo.hitch(this, function() {
					this._configurationDataEditorMgr.doSyncConfigurationDataFromEditor(this);
				}));
			}
			return true;
		},
		
		/**
		 * Clear the status of this editor container
		 */
		_clearState: function() {
			this._configurationDataEditor = null;
			this._isDeleteConfirmed = false;
			this._isFinalStatusChanged = false;
			this._isCustomizeConfirmed = false;
			this._isFinalChecked = false;
			this._isDataInherited = false;
			this._isInheritedDataFinal = false;
			// enable "Delete" tool button
			this.deleteToolButton.set("enabled", true); //$NON-NLS-1$
 			// enable "Final" check box
			this.finalCheckBox.disabled = false;
			dojo.toggleClass(this.finalCheckBoxLabel, "disabled", false); //$NON-NLS-1$
			
		},
		
		/**
		 *  Create new configuration data editor in this container. 
		 *  Firstly, this will get the configuration data elements from the model.
		 *  Then the new editor will be created in the callback method.
		 */
		_asynCreateConfigurationDataEditor: function() {
			this._processAreaEditor.startLoading(this.messages.OsgiBasedConfigurationData_LoadingConfigurationData);
			this._configurationDataEditorMgr.getConfigurationDataModel().getConfigurationDataElements(this._projectAreaItemId, this._configurationDataEditorMgr.getConfigurationDataIdsForEditor(this._configurationDataEditorId), this, "_getConfigurationDataContentSuccess", "_getConfigurationDataContentError"); //$NON-NLS-1$  //$NON-NLS-2$ 
		},
		
		/**
		 * Callback when fetching configuration data content successfully from server
		 * @param {Object} configDataElementList
		 */
		_getConfigurationDataContentSuccess: function(configDataElementList) {
			var configurationDataContentList = [];
			// We just only need to pass the content of configuration data to editor
			// Because there is no need to let the editor knows "isFinal" or "isInherited".
			// We also need store the editor's status, such as isFinalChecked, isDataInherited
			for (var i = 0; i < configDataElementList.length; i++) {
				var content = {};
				content.configurationDataId = configDataElementList[i].configurationDataId;
				content.configurationDataContent = configDataElementList[i].savedContent;
				configurationDataContentList.push(content);
				if (configDataElementList[i].isFinal) {
					this._isFinalChecked = true;
				}
				if (configDataElementList[i].isInherited) {
					this._isDataInherited = true;
				}
				if (configDataElementList[i].isFinal && configDataElementList[i].isInherited) {
					// in this case, customization will NOT be allowed
					this._isInheritedDataFinal = true;
				}
			}
			this._loadConfigurationDataEditor(configurationDataContentList);
		},
		
		/**
		 * Callback when fetching configuration data content unsuccessfully from server
		 * @param {Object} error
		 */
		_getConfigurationDataContentError: function(error) {
			var currentEditorId = this._configurationDataEditorId; 
			var extension = this._configurationDataEditorMgr.getConfigurationDataEditorExtension(currentEditorId);
			var message = dojo.string.substitute(this.messages.OsgiBasedConfigurationData_ErrorFetchConfigurationDataContent,[extension.name]);
			this._configurationDataEditorMgr.displayErrorMessage(message, error, true);
		},
		
		/**
		 * Load the configuration data editor with the given configuration data elements.
		 * @param {Object} configurationDataContentList
		 */
		_loadConfigurationDataEditor: function(configurationDataContentList) {
			var extension = this._configurationDataEditorMgr.getConfigurationDataEditorExtension(this._configurationDataEditorId);
			
			if(extension && extension.widget) {
				var args = {};
				args.configurationDataContentList = configurationDataContentList;
				args.processAreaEditor = this._processAreaEditor;
				args.editorId = this._configurationDataEditorId;
				args.editorContainer = this;
				jazz.core.loader.load_async(extension.widget, dojo.hitch(this, function() {
					try {
						extension.widgetConstructor = dojo.getObject(extension.widget);
						if (!extension.widgetConstructor) {
							var message = dojo.string.substitute(this.messages.ConfigurationDataEditorContainer_EditorUndefined, [extension.widget]);
							this._displayTabWidgetLoadingError(extension.widget, message);
							return;
						}
						this._configurationDataEditor = new extension.widgetConstructor(args);
						this._showConfigurationDataEditor(this._configurationDataEditor);
					} catch (error){
						var message = dojo.string.substitute(this.messages.ConfigurationDataEditorContainer_ErrorLoadingEditor, [extension.widget]);
						message = error.message || message;
						this._displayTabWidgetLoadingError(extension.widget, message);
					} finally {
						this._processAreaEditor.stopLoading();
					}
				}));
			} else {
				var message = dojo.string.substitute(this.messages.ConfigurationDataEditorContainer_ErrorFindingEditorExtension, [this._configurationDataEditorId]);
				this._configurationDataEditorMgr.displayErrorMessage(message,null,true);
			}
		},
		
		/**
		 * Display tab widget loading error 
		 * @param {String} tabWidget the tabWidget that can't be loaded
		 * @param {String} message error message
		 */
		_displayTabWidgetLoadingError: function(tabWidget,message) {
			var tabContainer = {};
			var errorTab = new com.ibm.team.process.web.ui.internal.admin.projects.ErrorTab({
				itemUUID: this._projectAreaItemId,
				tabContainer: tabContainer,
				tabID: this._configurationDataEditorId,
				tabTitle: this._configurationDataEditorName,
				tabWidget: tabWidget,
				tabError: message
			});
			dojox.xml.parser.removeChildren(this.configurationDataEditorDiv);
			this.configurationDataEditorDiv.appendChild(errorTab.domNode);	
			this._configurationDataEditorMgr.removeConfigDataContainer(this._configurationDataEditorId);
		},
		
		/**
		 * Show the configuration data editor with given configuration data editor.
		 * @param {Object} editor
		 */
		_showConfigurationDataEditor: function(editor) {
			if (editor && editor.domNode) {
				if (this._isInheritedDataFinal) {
					this._showInheritedWarning(this.messages.ConfigurationDataEditorContainer_InheritedFinalWarning, false);
				} else if (this._isDataInherited) {
					this._showInheritedWarning(this.messages.ConfigurationDataEditorContainer_InheritedWarning, true);
				} else  {
					this._doShowConfigurationDataEditor(editor);
				}
			}
		},
		
		/**
		 * Show the configuration data editor
		 * @param {Object} editor
		 */
		_doShowConfigurationDataEditor: function(editor) {
			if (editor && editor.domNode) {
				dojox.xml.parser.removeChildren(this.configurationDataEditorDiv);
				// hide inherited warning
				dojo.toggleClass(this.inheritedWarningDiv, "hidden", true); //$NON-NLS-1$
				// show final checkbox
				dojo.toggleClass(this.configdataEdtiorOuterHeaderDiv, "hidden", false); //$NON-NLS-1$
				// set clear override button visibility
				var processProvider = this._processAreaEditor.getExistingProcessProvider();
				if (processProvider) {
					// this is a consumer project area and show clear override button
					dojo.toggleClass(this.deleteToolButtonSpan, "hidden", false); //$NON-NLS-1$
				} else {
					// hide clear override button if this is not a consumer project
					dojo.toggleClass(this.deleteToolButtonSpan, "hidden", true); //$NON-NLS-1$
				}
				// show editor
				dojo.toggleClass(this.configurationDataEditorDiv, "hidden", false); //$NON-NLS-1$
				// set final checkbox status
				this.finalCheckBox.checked = this._isFinalChecked;
				// invoke the method aboutToShowEditor in the editor
				var conextArguments = this._configurationDataEditorMgr.getCurrentContextArguments();
				editor.aboutToShowEditor(conextArguments);
				this._configurationDataEditorMgr.setCurrentContextArguments(null);
				// show the editor
				this.configurationDataEditorDiv.appendChild(editor.domNode);
			}
		},

		/**
		 * An event handler when the finalCheckBox is clicked
		 * @param {Object} event
		 */
		_onFinalClick: function(event) {
			// set editor status
			this._isFinalChecked = this.finalCheckBox.checked;
			this._isFinalStatusChanged = !this._isFinalStatusChanged;
			// update dirty status
			this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());
			// update "isFinal" attribute for the configuration data elements handled by current editor
			var ids = this._configurationDataEditorMgr.getConfigurationDataIdsForEditor(this._configurationDataEditorId);
			if (ids && ids.length > 0) {
				var elements = this._configurationDataEditorMgr.getConfigurationDataModel().getConfigurationDataElementsDirectlyFromCache(ids);
				if (elements && elements.length >0) {
					for (var i = 0; i < elements.length; i++) {
						elements[i].isFinal = this.finalCheckBox.checked;
						elements[i].isFinalStatusChanged = !elements[i].isFinalStatusChanged;
						if (!elements[i].unsavedContent) {
							elements[i].unsavedContent = elements[i].savedContent;
						}
					}
				}
			}
		},
		
		/**
		 * An event handler when the customizeAnchor is clicked
		 * @param {Object} event
		 */
		_onCustomizeClick: function(event) {
			// set dirty
			this._processAreaEditor.setDirty(true);
			// update editor status
			this._isCustomizeConfirmed = true;
			// update configuration data
			// we need to copy the data from process provider to consumer
			var ids = this._configurationDataEditorMgr.getConfigurationDataIdsForEditor(this._configurationDataEditorId);
			if (ids && ids.length > 0) {
				var elements = this._configurationDataEditorMgr.getConfigurationDataModel().getConfigurationDataElementsDirectlyFromCache(ids);
				if (elements && elements.length >0) {
					for (var i = 0; i < elements.length; i++) {
						elements[i].unsavedContent = elements[i].savedContent;
					}
				}
			}
			
			this._doShowConfigurationDataEditor(this._configurationDataEditor);
		},
		
		/**
		 * Click event handler of Delete tool button.
		 * 
		 * @param {Object} event
		 */
		_onDeleteClick: function(event) {
			if (confirm(this.messages.ConfigurationDataEditorContainer_DeleteWarning)) {
				// update model
				var ids = this._configurationDataEditorMgr.getConfigurationDataIdsForEditor(this._configurationDataEditorId);
				if (ids && ids.length > 0) {
					var elements = this._configurationDataEditorMgr.getConfigurationDataModel().getConfigurationDataElementsDirectlyFromCache(ids);
					if (elements && elements.length >0) {
						for (var i = 0; i < elements.length; i++) {
							// if isDeleted is true, we will ignore all the other attributes
							elements[i].isDeleted = true;
						}
					}
				}
				// update editor status
				this._isDeleteConfirmed = true;
				// disable current editor
				this._disableConfigurationDataEditor();
				// hide current editor and show "to be deleted" warning 
				this._showToBeDeletedOnSaveWarning();
				// clear error status
				// it dose NOT make sense to keep the error status if the configuration data is to be deleted
				this.clearErrorStatus(this._configurationDataEditorId);
				// set dirty
				this._processAreaEditor.setDirty(true);
			}
		},
		
		/**
		 * Click event handler of Undo Changes tool button.
		 * 
		 * @param {Object} event
		 */
		_onUndoChangesClick: function(event) {
			if (confirm(this.messages.ConfigurationDataEditorContainer_UndoChangesWarning)) {
				// restore the configuration data elements in model
				var ids = this._configurationDataEditorMgr.getConfigurationDataIdsForEditor(this._configurationDataEditorId);
				if (ids && ids.length > 0) {
					this._configurationDataEditorMgr.getConfigurationDataModel().restoreConfigurationDataElements(ids);
				}
				// clear editor status
				this._clearConfigurationDataEditorStatus();
				// recreate the configuration data editor
				this._enableConfigurationDataEditor();
				this._hideToBeDeletedOnSaveWarning();
				this._configurationDataEditor = null;
				dojox.xml.parser.removeChildren(this.configurationDataEditorDiv);
				this._asynCreateConfigurationDataEditor();
				// clear error status
				// it dose NOT make sense to keep the error status if the changes have been discarded
				this.clearErrorStatus(this._configurationDataEditorId);
				// update dirty status of project area editor
				this._processAreaEditor.setDirty(this._processAreaEditor.changesPending());
			}
		},

		/**
		 * Disable the current configuration data editor.
		 */
		_disableConfigurationDataEditor: function() {
			if (this._configurationDataEditor) {
				jazz.ui.underlay.show({
					appendTo: this._configurationDataEditor.domNode,
					zIndex: 999,
					animate: false,
					stopEvents: true 
				});
				
				// disable "Delete" tool button
				this.deleteToolButton.set("enabled", false); //$NON-NLS-1$
	 			// disable "Final" check box
				this.finalCheckBox.disabled = true;
				dojo.toggleClass(this.finalCheckBoxLabel, "disabled", true); //$NON-NLS-1$
			}
		},
		
		/**
		 * Hide the current configuration data editor and show "to be deleted" warning. 
		 */
		_showToBeDeletedOnSaveWarning: function() {
			dojo.toggleClass(this.configurationDataEditorDiv, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.toBeDeteledWarning, "hidden", false); //$NON-NLS-1$
		},
		
		/**
		 * Show the current configuration data editor and hide "to be deleted" warning. 
		 */
		_hideToBeDeletedOnSaveWarning: function() {
			dojo.toggleClass(this.configurationDataEditorDiv, "hidden", false); //$NON-NLS-1$
			dojo.toggleClass(this.toBeDeteledWarning, "hidden", true); //$NON-NLS-1$
		},

		/**
		 * Enable the current configuration data editor.
		 */
		_enableConfigurationDataEditor: function() {
			if (this._configurationDataEditor) {
				jazz.ui.underlay.hide();
				// enable "Delete" tool button
				this.deleteToolButton.set("enabled", true); //$NON-NLS-1$
	 			// enable "Final" check box
				this.finalCheckBox.disabled = false;
				dojo.toggleClass(this.finalCheckBoxLabel, "disabled", false); //$NON-NLS-1$
			}
		},
		
		/**
		 * Show the warning message about customizing the configuration data inherited from process sharing.
		 * 
		 * @param {String} warningMessage
		 * @param {Boolean} isAllowedToCustomize
		 */
		_showInheritedWarning: function(warningMessage, isAllowedToCustomize) {
			var processProvider = this._processAreaEditor.getExistingProcessProvider();
			if (!processProvider) {
				return;
			}
			dojo.toggleClass(this.configdataEdtiorOuterHeaderDiv, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.configurationDataEditorDiv, "hidden", true); //$NON-NLS-1$
			dojo.toggleClass(this.inheritedWarningDiv, "hidden", false); //$NON-NLS-1$
			dojo.toggleClass(this.customizeAnchor, "hidden", !isAllowedToCustomize); //$NON-NLS-1$
			dojox.xml.parser.removeChildren(this.inheritedWarningLabel);
			var processProviderUrl = processProvider.value;
			if (processProvider.isLocal) {
				processProviderUrl = this._processAreaEditor._getServerUrl() + net.jazz.ajax.contextRoot() + "/process/project-areas/" + processProviderUrl //$NON-NLS-1$
			}
			var providerLink = "<a href=\"" + processProviderUrl + "\">" + processProvider.name + "</a>"; //$NON-NLS-1$  //$NON-NLS-2$  //$NON-NLS-3$
			var inheritedWarning = dojo.string.substitute(warningMessage, [providerLink]);
			this.inheritedWarningLabel.innerHTML = inheritedWarning;
		},

		/**
		 *  Clear the status of current configuration data editor.
		 */
		_clearConfigurationDataEditorStatus: function() {
			if (this._isFinalStatusChanged) {
				// rollback the final status
				this._isFinalChecked = !this._isFinalChecked;
			}
			this._isFinalStatusChanged = false;
			this._isCustomizeConfirmed = false;
			this._isDeleteConfirmed = false;
		},
		
		/**
		 * Process framework call this method to check whether configuration data editor has data changes
		 */
		changesPending: function() {
			if (this._configurationDataEditor) {
				if (this._isDeleteConfirmed || 
						this._isFinalStatusChanged || 
						this._isCustomizeConfirmed) {
					// we need to consider the following cases:
					// 1. use choose to delete the configuration data
					// 2. final status is changed by user
					// 3. user choose to customize configuration data if they come from process provider, 
					return true;
				}
				if (this._isFinalStatusChanged) {
					return true;
				}
				if (this._configurationDataEditor.changesPending) {
					if(this._configurationDataEditor.changesPending()) {
						return true;
					}
				}
			}
			return false;
		},
		
		/**
		 * Process framework call this method to give each method after the project area editor is saved 
		 * @param {Object} saveResponse
		 */
		saved: function(saveResponse) {
			// Due to a loading issue roles are control in the editor so nothing to do here.
			// We just need to call the saved method of the inner editor.
			if (this._configurationDataEditor && this._configurationDataEditor.saved) {
				this._configurationDataEditor.saved(saveResponse);
			}
		},
		
		/**
		 * Process framework will call this method to determine if the project area editor can be set to dirty. 
		 */
		canSave: function() {
			if (this._isDeleteConfirmed) {
				// there is no need to check the status if this configuration data is to be deleted
				return true;
			}
			if (this._configurationDataEditor && this._configurationDataEditor.canSave) {
				return this._configurationDataEditor.canSave();
			}
			return true;
		},
		
		/**
		 * Process framework will call this method to validate each editor page. 
		 * 
		 * @return {String} the message if any error found
		 */
		validate: function() {
			if (this._configurationDataEditor && this._configurationDataEditor.validate) {
				return this._configurationDataEditor.validate();
			}
		},
		
		/**
		 * Process framework will call this method to activate a cached tab widget. 
		 */
		activateTab: function() {
			if (this._configurationDataEditor) {
				this.aboutToShowEditor();
			}
		},

		/**
		 * Return an array of configuration data that has latest modified configuration data content
		 * Each element in the array is a JSON object, in the format:
		 * { "configurationDataId": <configurationDataId>, 
		 *   "configurationDataContent": <configurationDataContent>
		 *  }
		 */
		getUnsavedConfigurationDataList: function() {
			if (this._configurationDataEditor && this._configurationDataEditor.getUnsavedConfigurationDataList) {
				try {
					return this._configurationDataEditor.getUnsavedConfigurationDataList();
				} catch (error) {
					//See reference to field customEditorId above and in
					//AbstractProcessAreaEditor
					error.customWidgetId = this._configurationDataEditorId;
					throw error;
				}
			}
		},
		
		/**
		 * This is a callback method that will be called by the Process framework before the editor is shown
		 * It gives the editor a chance to populate the data in the editor
		 * @param {Object} contextArguments context object that is passed into the editor to be shown.
		 */
		aboutToShowEditor: function() {
			if (this._configurationDataEditor) {
				var contextArguments = this._configurationDataEditorMgr.getCurrentContextArguments();

				// disable or enable the editor based on the delete status
				if (this._isDeleteConfirmed) {
					this._disableConfigurationDataEditor();
				} else {
					this._enableConfigurationDataEditor();
				}
				
				if (this._configurationDataEditor.aboutToShowEditor) {
					this._configurationDataEditor.aboutToShowEditor(contextArguments);
				}

				// update error status
				if (this._configurationDataEditor.changesPending()) {
					// there is no need to check the error status if there is no changes pending.
					var msg = this._configurationDataEditor.validate();
					if (msg) {
						this._configurationDataEditorMgr.setErrorStatus(msg, this._configurationDataEditorId);
					} 
				} 

				this._configurationDataEditorMgr.setCurrentContextArguments(null);
			}
		},

		/**
		 * Set the current configuration data editor in error status. This will
		 * show the given error message in current project area editor.
		 * 
		 * @param {String} message the message that is to be shown
		 * @param {String} configurationDataEditorId the id of the configuration data editor that the error status is associated with.
		 */
		setErrorStatus: function(message, configurationDataEditorId) {
			this._configurationDataEditorMgr.setErrorStatus(message, configurationDataEditorId);
		},
		
		/**
		 * Clear the error status for current configuration data editor. This will
		 * clear the message of current project area editor.
		 * 
		 * @param {String} configurationDataEditorId the id of the configuration data editor that the error status is associated with.
		 */
		clearErrorStatus: function(configurationDataEditorId) {
			this._configurationDataEditorMgr.clearErrorStatus(configurationDataEditorId);
		},
		
		/**
		 * Get an array of the configuration data for the given configurationDataIds.
		 * Each element is a JSON object,the format is:
		 * 	{ "configurationDataId":<configurationDataId>, 
		 *    "configurationDataContent": <changedConfigurationDataContent>
		 *  }
		 * @param {Object} configurationDataIds
		 * @param {Object} callbackContext
		 * @param {String} successCallbackName
		 * @param {String} errorCallbackName
		 */
		getConfigurationDataContentList: function(configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			this._configurationDataEditorMgr.getConfigurationDataContentList(configurationDataIds, callbackContext, successCallbackName, errorCallbackName);
		},
		
		/**
		 * Show the configuration data editor whose id is configurationDataEditorId
		 * @param {String} configurationDataEditorId
		 * @param {Object} contextArguments
		 */
		showConfigurationDataEditor: function(configurationDataEditorId, contextArguments) {
			this._configurationDataEditorMgr.showConfigurationDataEditor(configurationDataEditorId, contextArguments);
		},
		
		uninitialize: function() {
			if (this._configurationDataEditor) {
				delete this._configurationDataEditor;
			}
			
			if (this._processAreaEditor) {
				delete this._processAreaEditor;
			}

			this.inherited(arguments);
		}
		
	});
})();
