/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorManager"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorContainer"); //$NON-NLS-1$
	
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("jazz.core.RegistryFactory"); //$NON-NLS-1$
	dojo.require("jazz.ui.underlay"); //$NON-NLS-1$
	
	dojo.require("net.jazz.ajax.contextRoot");//$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var OsgiBasedConfigurationDataClient = com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataClient;
	var CONFIGURATION_DATA_EDITOR_EXTENSION_POINT_ID = "com.ibm.team.process.web.configurationDataEditor"; //$NON-NLS-1$
	var RegistryFactory = jazz.core.RegistryFactory;
	var ConfigurationDataEditorContainer = com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorContainer;
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ConfigurationDataEditorManager", null, { //$NON-NLS-1$
		constructor: function(args){
			this._processAreaEditor = args.editor; //hold the outer project area editor
			this.projectAreaItemId = args.projectAreaItemId;
			this._configurationDataModel = args.configurationDataModel;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.configurationDataEditorExtensions = [];
			this.configurationDataElementsExtensions = [];
			this.configurationDataCategoriesExtensions = [];
			this._configurationDataEditorContainerCache = {};
			this._currentContextArguments = null;
			if (this._processAreaEditor) {
				this._processAreaEditor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataEditorManager"); //$NON-NLS-1$
			}
		},
	
		/**
		 * load the extensions to the web extension point <configurationDataEditor>
		 */
		loadConfigurationDataEditorExtensions: function() {
			var extPt = RegistryFactory.getRegistry().getExtensionPoint(CONFIGURATION_DATA_EDITOR_EXTENSION_POINT_ID);
			dojo.forEach(extPt.getExtensions(), dojo.hitch(this, function(extension) {
				var pluginNamespace = extension.getNamespace();
				dojo.forEach(extension.getConfigurationElements(), dojo.hitch(this, function(cfgElement) {
					var configDataElements = [];
					var dataElements = cfgElement.getChildren();
					var description = ""; //$NON-NLS-1$
					if (dataElements) {
						dojo.forEach(dataElements, dojo.hitch(this, function(dataElement) {
							if(dataElement.getName() === "description") {  //$NON-NLS-1$
								description = dataElement.getValue();
							} else {
								configDataElements.push(dataElement.getAttribute("id")); //$NON-NLS-1$	
							}
						}));
					}
					var extData = {
						nameSpace: pluginNamespace,
						id: cfgElement.getAttribute("id"), //$NON-NLS-1$						
						widget: cfgElement.getAttribute("widget"), //$NON-NLS-1$
						name: cfgElement.getAttribute("label"), //$NON-NLS-1$
						uaKey: cfgElement.getAttribute("uaKey"), //$NON-NLS-1$
						categoryId: cfgElement.getAttribute("categoryId"), //$NON-NLS-1$
						description: description,
						configurationDataElements: configDataElements
					};
					this.configurationDataEditorExtensions.push(extData);
				}));
			}));
		},
		
		/**
		 * Get the cached configuration data editor extensions.
		 */
		getConfigurationDataEditorExtensions: function() {
			return this.configurationDataEditorExtensions;
		},
		
		
		/**
		 * load the extensions to the extension point(configurationPoints/staticConfigurationData)
		 */
		loadOsgiBasedConfigurationDataDeclarations: function(contextObject, successCallback, errorCallback) {
			var srh = new ServiceResponseHandler(contextObject, successCallback, errorCallback);
			OsgiBasedConfigurationDataClient.getConfigurationDataDeclarations(srh);
		},
		
		/**
		 * Show the configuration data editor whose id is configurationDataEditorId
		 * @param {String} configurationDataEditorId
		 * @param {Object} contextArguments
		 */
		showConfigurationDataEditor: function(configurationDataEditorId, contextArguments) {
			this._currentContextArguments = contextArguments;
			this._processAreaEditor._switchTab(configurationDataEditorId);
		},
			
		/**
		 * Display error message in the project area message area
		 * @param {String} message
		 * @param {Object} error
		 * @param {Boolean} stopLoading
		 */
		displayErrorMessage: function(message, error,  stopLoading) {
			if (error) {
				this._processAreaEditor.handleServerResponseError(error, message);
			} else {
				this._processAreaEditor._displayLongErrorMessage(message);
			}
			if (stopLoading) {
				this._processAreaEditor.stopLoading();
			}
		},

		/**
		 * Process framework call this method to check whether configuration data editor has data changes
		 */
		changesPending: function() {
			for (var editorId in this._configurationDataEditorContainerCache) {
				var container = this._configurationDataEditorContainerCache[editorId];
				if(container.changesPending()) {
					return true;
				}
			}
			return false;
		},
		
		/**
		 * Process framework call this method to collect the changes from all the editors
		 * @param {Object} saveObj
		 */		
		aboutToBeSaved: function(saveObj) {
			if(saveObj.jsonOsgiBasedConfigurationData) {
				delete saveObj.jsonOsgiBasedConfigurationData;
			}
			
			this._collectUnsavedConfigurationDataFromAllEditors();
			
			var unsavedConfigurationDataList = this._configurationDataModel.getUnsavedConfigurationDataList();
			
			if (unsavedConfigurationDataList && unsavedConfigurationDataList.length > 0) {
				saveObj.jsonOsgiBasedConfigurationData = unsavedConfigurationDataList;
			}
		},
		
		/**
		 * Process framework call this method to give each method after the project area editor is saved 
		 * @param {Object} saveResponse
		 */
		saved: function(saveResponse) {
			for (var editorId in this._configurationDataEditorContainerCache) {
				var container = this._configurationDataEditorContainerCache[editorId];
				if(container && container.changesPending()) {
					container.saved();
				} 
			}
		},
		
		/**
		 * Process framework call this method to refresh all the configuration data editor. 
		 */
		refresh: function() {
			this._clearState();
			
			this._processAreaEditor.addSaveParticipant(this, "com.ibm.team.process.web.ui.internal.admin.projects.com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataEditorManager"); //$NON-NLS-1$

			for (var editorId in this._configurationDataEditorContainerCache) {
				var container = this._configurationDataEditorContainerCache[editorId];
				if(container && container.refresh()) {
					container.refresh();
				} 
			}
		},

		/**
		 * Get all the configuration data that have latest modified content from all the editors
		 */
		_collectUnsavedConfigurationDataFromAllEditors: function() {
			for (var editorId in this._configurationDataEditorContainerCache) {
				var container = this._configurationDataEditorContainerCache[editorId];
				if(container && container.changesPending()) {
					this.doSyncConfigurationDataFromEditor(container);
				} 
			}
		},
		
		/**
		 * sync the unsaved configuration data of the given editor to the model
		 * @param {Object} editor
		 */
		doSyncConfigurationDataFromEditor: function(editor) {
			var unsavedContentList = editor.getUnsavedConfigurationDataList();
			if (unsavedContentList) {
				this._configurationDataModel.updateUnsavedConfigurationData(unsavedContentList);
			}
		},
		
		/**
		 * Get all the configurationDataIs which belong to the given editorId
		 * @param {String} editorId
		 */
		getConfigurationDataIdsForEditor: function(editorId) {
			for (var i = 0; i < this.configurationDataEditorExtensions.length; i++) {
				if(this.configurationDataEditorExtensions[i].id === editorId) {
					return this.configurationDataEditorExtensions[i].configurationDataElements;
				}
			}
			return null;
		},
		
		/**
		 * Get the configuration data editor extension by the given editorId
		 * @param {String} editorId
		 */
		getConfigurationDataEditorExtension: function(editorId) {
			for (var i = 0; i < this.configurationDataEditorExtensions.length; i++) {
				if(this.configurationDataEditorExtensions[i].id === editorId) {
					return this.configurationDataEditorExtensions[i];
				}
			}
			return null;
		},

		/**
		 * Get the editorId whose editor owns the configuration data
		 * @param {String} configurationDataId
		 */
		_getEditorIdForConfigurationData: function(configurationDataId) {
			for (var i = 0; i < this.configurationDataEditorExtensions.length; i++) {
				var elementIds = this.configurationDataEditorExtensions[i].configurationDataElements;
				for (var j = 0; j < elementIds.length; j++) {
					if (elementIds[j] === configurationDataId) {
						return this.configurationDataEditorExtensions[i].id;
					}
				}
			}
			return null;
		},
		
		/**
		 * Get an array of the configuration data for the given configurationDataIds.
		 * Each element is a JSON object,the format is:
		 * 	{ "configurationDataId":<configurationDataId>, 
		 *    "configurationDataContent": <changedConfigurationDataContent>
		 *  }
		 * @param {Object} configurationDataIds
		 * @param {Object} callbackContext
		 * @param {String} successCallbackName
		 * @param {String} errorCallbackName
		 */
		getConfigurationDataContentList: function(configurationDataIds, callbackContext, successCallbackName, errorCallbackName) {
			this._configurationDataModel.getConfigurationDataContent(this.projectAreaItemId, configurationDataIds, callbackContext, successCallbackName, errorCallbackName);
		},
		
		/**
		 * clear the state of configuration data
		 */
		_clearState: function(){
			this._processAreaEditor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataEditorManager"); //$NON-NLS-1$
			
			if (this._configurationDataModel) {
				this._configurationDataModel.clear();
			}

			this._currentContentArguments = null;
		},
		
		/**
		 * Set the current configuration data editor in error status. This will
		 * show the given error message in current project area editor 
		 * 
		 * @param {String} message the message that is to be shown
		 * @param {String} configurationDataEditorId the id of the configuration data editor that the error status is associated with.
		 */
		setErrorStatus: function(message, configurationDataEditorId) {
			if (message && this._getCurrentTabId() === configurationDataEditorId) {
				this._processAreaEditor._displayLongErrorMessage(message);
			}
		},
		
		/**
		 * Clear the error status for current configuration data editor. This will
		 * clear the message of current project area editor
		 * 
		 * @param {String} configurationDataEditorId the id of the configuration data editor that the error status is associated with.
		 */
		clearErrorStatus: function(configurationDataEditorId) {
			if (this._getCurrentTabId() === configurationDataEditorId) {
				this._processAreaEditor._clearMessage();
			}
		},
		
		/**
		 * Process framework will call this method to determine if the project area editor can be set to dirty. 
		 */
		canSave: function() {
			for (var editorId in this._configurationDataEditorContainerCache) {
				var container = this._configurationDataEditorContainerCache[editorId];
				var canBeSaved = container.canSave();
				if (!canBeSaved) {
					return false;
				}
			}
			return true;
		},
		
		/**
		 * Process framework will call this method to validate each editor page. 
		 * 
		 * @return {String} the message if any error found
		 */
		validateRequired: function() {
			// validate the activate editor first
			var currentTabId = this._processAreaEditor.getSelectedTabId();
			if (this._configurationDataEditorContainerCache[currentTabId]) {
				var msg = this._validateConfigurationDataEditor(this._configurationDataEditorContainerCache[currentTabId]);
				if (msg) {
					return msg;
				}
			}
			
			// validate other editors
			for (var editorId in this._configurationDataEditorContainerCache) {
				if (editorId !== currentTabId) {
					var container = this._configurationDataEditorContainerCache[editorId];
					var msg = this._validateConfigurationDataEditor(container);
					if (msg) {
						return msg;
					}
				}
			}
		},
		
		/**
		 * Validate the given configuration data editor(container).
		 * 
		 * @param {Object} container
		 * @return {String} the message if any error found

		 */
		_validateConfigurationDataEditor: function(container) {
			if (container) {
				var msg = container.validate();
				return msg;
			}
			return null;
		},
		
		/**
		 * Return the cached configuration data model.
		 */
		getConfigurationDataModel: function() {
			return this._configurationDataModel;
		},
		
		/**
		 * Add the configuration data editor (container) to the cache.
		 * 
		 * @param {Object} editorId
		 * @param {Object} configDataContainer
		 */
		addConfigDataContainer: function(editorId, configDataContainer) {
			this._configurationDataEditorContainerCache[editorId] = configDataContainer;
		},
		
		/**
		 * Remove the cached configuration data container by the given editorId
		 * @param {String} editorId
		 */
		removeConfigDataContainer: function(editorId) {
			delete this._configurationDataEditorContainerCache[editorId];
		},
		
		/**
		 * Return the cached configuration data editor container with the given id.
		 * 
		 * @param {Object} editorId
		 */
		getConfigDataContainer: function(editorId) {
			return this._configurationDataEditorContainerCache[editorId];
		},
		
		/**
		 * Store the given context arguments.
		 * 
		 * @param {Object} currentContextArguments
		 */
		setCurrentContextArguments: function(currentContextArguments) {
			this._currentContextArguments = currentContextArguments;
		},

		/**
		 * Return the cached context arguments.
		 */
		getCurrentContextArguments: function() {
			return this._currentContextArguments;
		},
		
		/**
		 * Get the tab id of the activate tab.
		 * NOTES: the tab id of a configuration data editor is the same as the id of this editor.
		 */
		_getCurrentTabId: function() {
			return this._processAreaEditor.getSelectedTabId();
		},
	
		uninitialize: function() {
			this._processAreaEditor.removeSaveParticipant("com.ibm.team.process.web.ui.internal.admin.projects.com.ibm.team.process.web.ui.internal.admin.projects.OsgiBasedConfigurationDataEditorManager"); //$NON-NLS-1$
			
			for (var editorId in this._configurationDataEditorContainerCache) {
				if (this._configurationDataEditorContainerCache[editorId]) {
					this._configurationDataEditorContainerCache[editorId].destroyRecursive();
					this._configurationDataEditorContainerCache[editorId] = null;
				}
			}
			delete this._configurationDataEditorContainerCache;
			
			if(this._processAreaEditor) {
				delete this._processAreaEditor;
			}
			
			this.inherited(arguments);
		}
		
	});
})();
