/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialog"); //$NON-NLS-1$

	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
	dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("dijit.form.TimeTextBox");  //$NON-NLS-1$
	
	dojo.require("jazz.app.ua.TextHoverHelp"); //$NON-NLS-1$

	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureTable"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.DateTimeTextBox"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;

	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		showArchived: false,
		
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/CreateIterationsDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			if (args.storedState) {
				this._scheduledDays = args.storedState.scheduledDays;
				this._startTime = args.storedState.startTime;
				this._endTime = args.storedState.endTime;
			} else {
				this._scheduledDays = [false, true, true, true, true, true, false];
				this._startTime = new Date(0, 0, 0, 0, 0, 0, 0);
				this._endTime = new Date(0, 0, 0, 23, 59, 0, 0);
			}
			
			this._scheduledDaysValid = true;
		},
		
		postCreate: function() {
			//Move the warning icon for the start date inside of the start date
			//input field. This is insane, but looks cool and a unit test will fail
			//if the structure of Dojo widget's changes.
			dojo.place(this._startDateWarningIcon, this.startDate.domNode.childNodes[this.startDate.domNode.childNodes.length-1], "last"); //$NON-NLS-1$

			this._messageBox = new jazz.ui.MessageBox();
			this.errorMessageBoxContainer.appendChild(this._messageBox.domNode);
		},
		
		startup: function() {
			this.inherited(arguments);
			
			UIHelper.resizeI18NButtons(this.dialogFooter, [this.finishButton, this.cancelButton]);
			
			this._scheduledDaysUA = new jazz.app.ua.TextHoverHelp({
				contentNode: this.scheduledDaysLabel,
				helpContentString: this.messages.CreateIterationsDialog_4
			});
			
			this.summaryTextNode.appendChild(document.createTextNode(this.summaryText));
			
			this._createIterationStructureModel();
			
			this._createEditorSection();
			
			if (false === this.isNewTimeline) {
				this._createInitialIteration();
			}
			
			this._initializeStartDate();
			
			this._adjustDateTimeHeaderLayout();
			
			this._onResizeEvent = jazz.util.ResizeNotifier.register({
				context: this,
				callback: dojo.hitch(this, function() {
					this._iterationStructureTable.resized();
					
					if (this._adjustWidthTimeout) {
						clearTimeout(this._adjustWidthTimeout);
					}
					
					this._adjustWidthTimeout = setTimeout(dojo.hitch(this, function() {
						this._adjustDateTimeHeaderLayout();
						delete this._adjustWidthTimeout;
					}), 50);
				})
			});
		},
		
		_reportError: function(message) {
			if (message === undefined) {
				this._messageBox.hideMessage();
			} else {
				this._messageBox.showMessage({
					type: jazz.ui.MessageBox.STATUS_TYPES.ERROR,
					summary: message
				});
			}
		},
		
		/**
		 * Should be called after the containing dialog has been displayed and
		 * centered on the screen, (the widget may take actions that result
		 * in ugly thrashing if they're performed while the dialog is opening
		 * and adjusting its size and position).
		 */
		init: function() {
			this._iterationStructureTable.init();
			
			this._previewTree.commonStartDateChanged(new Date(this._startDate.getFullYear(), this._startDate.getMonth(), this._startDate.getDate()));
			
			this.startDate.watch("displayedValue", dojo.hitch(this, function() { //$NON-NLS-1$
				if (this.startDate.isValid()) {
					this._startDate = this.startDate.get('value'); //$NON-NLS-1$
					this._previewTree.commonStartDateChanged(this._startDate === null ? null : new Date(this._startDate.getFullYear(), this._startDate.getMonth(), this._startDate.getDate()));
					this._startDateShifted = this._shiftDateToScheduledDay(this._startDate);
				}
				this._validate();
			}));
			
			this.scheduledDays.setScheduledDays(this._scheduledDays);
			
			this.connect(this.scheduledDays, "scheduledDaysChanged", dojo.hitch(this, function(scheduledDays) { //$NON-NLS-1$
				if (dojo.every(scheduledDays, function(scheduledDay) {
					return !scheduledDay;
				})) {
					dojo.toggleClass(this.scheduledDaysLabel, "errorText", true); //$NON-NLS-1$
					dojo.toggleClass(this._scheduledDaysErrorIcon, "hidden", false); //$NON-NLS-1$
					dojo.attr(this._scheduledDaysErrorIcon, "title", this.messages.CreateIterationsDialog_5); //$NON-NLS-1$
					
					this._scheduledDaysValid = false;
				} else {
					if (dojo.hasClass(this.scheduledDaysLabel, "errorText")) { //$NON-NLS-1$
						dojo.toggleClass(this.scheduledDaysLabel, "errorText", false); //$NON-NLS-1$
						dojo.toggleClass(this._scheduledDaysErrorIcon, "hidden", true); //$NON-NLS-1$
					}
					
					this._scheduledDays = scheduledDays;
					this._previewTree.scheduledDaysChanged(this._scheduledDays);
					
					this._scheduledDaysValid = true;
				}
				
				this._startDate = this.startDate.get('value'); //$NON-NLS-1$
				this._startDateShifted = this._shiftDateToScheduledDay(this._startDate);
				
				this._validate();
			}));
			
			this.startTime.set('required', true); //$NON-NLS-1$
			this.startTime.set('value', this._startTime); //$NON-NLS-1$
			
			this.startTime.watch("displayedValue", dojo.hitch(this, function() { //$NON-NLS-1$
				if (this.startTime.isValid()) {
					this._startTime = this.startTime.get('value'); //$NON-NLS-1$
				}
				this._validate();
			}));
			
			this.endTime.set('required', true); //$NON-NLS-1$
			this.endTime.set('value', this._endTime); //$NON-NLS-1$
			
			this.endTime.watch("displayedValue", dojo.hitch(this, function() { //$NON-NLS-1$
				if (this.endTime.isValid()) {
					this._endTime = this.endTime.get('value'); //$NON-NLS-1$
				}
				this._validate();
			}));
		},
		
		/**
		 * Returns an object that can be passed to another instance of this
		 * widget as parameter 'storedState'. Certain values, such as the
		 * number of scheduled days and start and end times, may be remembered
		 * across sessions with the dialog.
		 */
		getStoredState: function() {
			return {
				scheduledDays: this._scheduledDays,
				startTime: this._startTime,
				endTime: this._endTime
			};
		},
		
		_initializeStartDate: function() {
			if (false === this.isNewTimeline) {
				this.timelineModel.getChildren(this.rootItem, dojo.hitch(this, function(childItems) {
					if (childItems.length === 0) {
						//If the root item has no children, the start date
						//is the start date of the root item...
						var startDate = this.timelineModel.store.getValue(this.rootItem, "startDate"); //$NON-NLS-1$
						if (startDate) {
							this._startDate = new Date(startDate.valueOf());
						}
					} else {
						//...otherwise the start date is the day after the last
						//child's end date, (or start date if it has no end
						//date)
						var endDate = this.timelineModel.store.getValue(childItems[childItems.length-1], "endDate"); //$NON-NLS-1$
						if (endDate) {
							this._startDate = new Date(endDate.valueOf());
						} else {
							var startDate = this.timelineModel.store.getValue(childItems[childItems.length-1], "startDate"); //$NON-NLS-1$
							if (startDate) {
								this._startDate = new Date(startDate.valueOf());
							}
						}
						if (this._startDate) {
							this._startDate.setDate(this._startDate.getDate() + 1);
						}
					}
				}), function(error) {
					// This should only be the result of a programming error;
					// log to the console w/ static message
					console.error(this, "error in _initializeStartDate(): ", error); //$NON-NLS-1$
				});
			}
			
			if (!this._startDate) {
				//If we're creating a timeline or the date couldn't be set for
				//any reason, use the current date
				this._startDate = new Date();
				this._startDate.setHours(0);
				this._startDate.setMinutes(0);
				this._startDate.setSeconds(0);
			}
			
			this._shiftDateToScheduledDay(this._startDate);
			
			this.startDate.set('value', this._startDate); //$NON-NLS-1$
		},
		
		_shiftDateToScheduledDay: function(date) {
			var daysShifted = 0;
			if (date) {
				var currentDay = date.getDay();
				for (var i = 0; i < 7; i++) {
					if (this._scheduledDays[currentDay] === true) {
						break;
					}
					currentDay = (currentDay + 1) % this._scheduledDays.length;
					daysShifted += 1;
				}
				if (daysShifted !== 0) {
					date.setDate(date.getDate() + daysShifted);
				}
			}
			return daysShifted !== 0;
		},
		
		/**
		 * Initialize the tree store model that will back the Iteration
		 * Structure table/tree. The Preview tree will be backed by a different
		 * model, but changes to the iteration structure model will drive
		 * updates to the Preview model.
		 */
		_createIterationStructureModel: function() {
			var store = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessStore({
				data: {
					label: "name", //$NON-NLS-1$
					items: []
				}
			});
			
			var rootItem;
			
			if (true === this.isNewTimeline) {
				rootItem = store.newItem({
					name: this.initialName,
					originalName: this.initialName,
					isTimeline: this._isSelectedRootItemATimeline(),
					projectTimeline: false,
					quantity: 1
				});
			} else {
				rootItem = store.newItem({
					name: this.timelineModel.store.getValue(this.rootItem, "name"), //$NON-NLS-1$
					isTimeline: this._isSelectedRootItemATimeline()
				});
			}
			
			this._iterationStructureModel = new com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureModel({
				store: store,
				root: rootItem
			});
		},
		
		/**
		 * Create the editor section of the dialog, which contains two sub-sections:
		 * IterationStructure and the PreviewSection
		 * 
		 * The content is a dijit.layout.BorderContainer with sections for
		 * each of the editor's first (left) and second (center) columns. 
		 */
		_createEditorSection: function() {
			this._timelineLeft = document.createElement("div"); //$NON-NLS-1$
			this._timelineRight = document.createElement("div"); //$NON-NLS-1$
			
			this._createIterationStructureSection(this._timelineLeft);
			this._createPreviewSection(this._timelineRight);
			
			dojo.addClass(this._timelineLeft, "iterationStructureSection"); //$NON-NLS-1$
			dojo.addClass(this._timelineRight, "previewSection"); //$NON-NLS-1$
			
			var contentContainer = document.createElement("div"); //$NON-NLS-1$
			this.editorContainer.appendChild(contentContainer);
			
			// Create a border container to hold the two columns
			this._editorContainer = new dijit.layout.BorderContainer({
				design : "sidebar", //$NON-NLS-1$
				liveSplitters:true,
				style: "height: 32em; width: 100%;" //$NON-NLS-1$
			}, contentContainer);
			
			var createSection = dojo.hitch(this, function(region, contentWidget, width) {
				var section = new dijit.layout.ContentPane({
					splitter : true,
					style: "height: 100%; width: " + width + "; padding: 3px; overflow: hidden; background: none", //$NON-NLS-1$ //$NON-NLS-2$
					region : region
				});
				section.domNode.appendChild(contentWidget);
				dojo.addClass(section.domNode, "no-border"); //$NON-NLS-1$
				this._editorContainer.addChild(section);
				return section;
			});
			
			var leftSection = createSection(this.isLeftToRight() ? "left" : "right", this._timelineLeft, "58%"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			createSection("center", this._timelineRight, "37%"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this.connect(leftSection, "resize", function(event) { //$NON-NLS-1$
				this._iterationStructureTable.resized();
			});
			
			this._editorContainer.startup();
		},
		
		_createIterationStructureSection: function(container) {
			var titleDiv = dojo.create("div", {'class': "j-h2"}, container); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			titleDiv.appendChild(document.createTextNode(this.messages.CreateIterationsDialog_1));
			
			this._iterationStructureTable = com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureTable({
				model: this._iterationStructureModel,
				rootIsEditable: true === this.isNewTimeline,
				changesPendingCallback: dojo.hitch(this, this._validate),
				timelineStore: this.timelineModel.store,
				reportError: dojo.hitch(this, this._reportError)
			});
			
			var tableDiv = dojo.create("div", {'class': "iterationStructureContainer"}, container); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			tableDiv.appendChild(this._iterationStructureTable.domNode);
		},
		
		_validate: function() {
			var isValid = this._iterationStructureTable.validate();
			isValid = isValid && this.startDate.isValid();
			isValid = isValid && this._scheduledDaysValid;
			isValid = isValid && this.startTime.isValid();
			isValid = isValid && this.endTime.isValid();
			
			//Disable OK if we're editing an existing timeline/iteration and
			//no iterations have been added
			isValid = isValid && (this.isNewTimeline || this._iterationStructureModel.store.getValues(this._iterationStructureModel.root, "children").length !== 0); //$NON-NLS-1$
			
			this.finishButton.disabled = !isValid;
			
			if (this._startDateShifted) {
				dojo.toggleClass(this._startDateWarningIcon, "hidden", false); //$NON-NLS-1$
				dojo.attr(this._startDateWarningIcon, "title", this.isNewTimeline //$NON-NLS-1$
					? this.messages.CreateIterationsDialog_6
					: this.messages.CreateIterationsDialog_7
				);
			} else {
				dojo.toggleClass(this._startDateWarningIcon, "hidden", true); //$NON-NLS-1$
			}
		},
		
		_createPreviewSection: function(container) {
			var titleDiv = dojo.create("div", {'class': "j-h2"}, container); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			titleDiv.appendChild(document.createTextNode(this.messages.CreateIterationsDialog_2));
			
			this.previewStore = new com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewStore({
				data: {
					label: "name", //$NON-NLS-1$,
					items: []
				}
			});
			
			var existingRootItemChildren = this._addExistingPreviewChildren(this.rootItem);
			
			// CREATE ROOT PREVIEW IEM
			var rootItemObject = {
				name: this._iterationStructureModel.store.getValue(this._iterationStructureModel.root, "name"), //$NON-NLS-1$
				isTimeline: this._isSelectedRootItemATimeline(),
				iterationStructureId: this._iterationStructureModel.getIdentity(this._iterationStructureModel.root),
				enabledContext: this._getSelectedRootItemContext(),
				items: existingRootItemChildren,
				existing: (typeof this.rootItem != "undefined"),//$NON-NLS-1$
				startDate : (this.rootItem)? this.timelineModel.store.getValue(this.rootItem, "startDate"): null, //$NON-NLS-1$
				endDate : (this.rootItem)? this.timelineModel.store.getValue(this.rootItem, "endDate"): null, //$NON-NLS-1$
				index: 0
			};

			if (this.rootItem) {
				rootItemObject["projectTimeline"] = this.timelineModel.store.getValue(this.rootItem, "projectTimeline");//$NON-NLS-1$//$NON-NLS-2$
			}
			var previewRootItem = this.previewStore.newItem(rootItemObject);
			
			
			var previewModel = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineModel({
				store: this.previewStore,
				root: previewRootItem
			});
			
			this._previewTree = new com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewTree({
				model: previewModel,
				showArchived: this.showArchived,
				existingChildren: existingRootItemChildren,
				iterationStructureModel: this._iterationStructureModel,
				timelineStore: this.timelineModel.store,
				commonStartDate: this.startDate.get('value'), //$NON-NLS-1$
				scheduledDays: this._scheduledDays
			});
			
			var treeDiv = dojo.create("div", {'class': "previewTreeContainer"}, container); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			treeDiv.appendChild(this._previewTree.domNode);
			
			if (existingRootItemChildren.length > 0) {
				dojo.addClass(treeDiv, "previewTreeContainerShorter"); //$NON-NLS-1$
				
				var toggleDiv = dojo.create("div", {'class': "toggle"}, container); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var label = dojo.create("label", null, toggleDiv); //$NON-NLS-1$
				this._showExistingInPreviewToggle = dojo.create("input", {'type': "checkbox", 'checked': "true"}, label); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$
				label.appendChild(document.createTextNode("\t" + this.messages.CreateIterationsDialog_3));//$NON-NLS-1$
				this.connect(this._showExistingInPreviewToggle, "onclick", dojo.hitch(this, function(event) { //$NON-NLS-1$
					this._previewTree.toggleShowExisting(this._showExistingInPreviewToggle.checked);
				}));
			}
			
			this.connect(this._iterationStructureTable, "selectionChanged", dojo.hitch(this, function(item) { //$NON-NLS-1$
				this._previewTree.iterationStructureItemSelected(item);
			}));
		},
		
		_createInitialIteration: function() {
			var newItem = this._iterationStructureModel.store.newItem({
				name: this.initialName,
				originalName: this.initialName,
				quantity: 1,
				parentId: this._iterationStructureModel.store.getIdentity(this._iterationStructureModel.root),
				hasDeliverable: true,
				iterationTypeItemId: ProcessConstants.ITERATION_TYPE_NONE
			});
			
			var children = this._iterationStructureModel.store.getValues(this._iterationStructureModel.root, "children") || []; //$NON-NLS-1$
			children.push(newItem);
			this._iterationStructureModel.store.setValues(this._iterationStructureModel.root, "children", children); //$NON-NLS-1$
		},
		
		close: function() {
		},
		
		_addExistingPreviewChildren: function(item) {
			var children = [];
			if (item) {
				this.timelineModel.getChildren(item, dojo.hitch(this, function(childItems) {
					dojo.forEach(childItems, dojo.hitch(this, function(childItem) {
						var newChild = this._constructExistingPreviewChild(childItem);
						children.push(newChild);
					}));
				}), function(error) {
					// This should only be the result of a programming error;
					// log to the console w/ static message
					console.error(this, "error in _addExistingPreviewChildren(): ", error); //$NON-NLS-1$
				});
			}
			return children;
		},

		_constructExistingPreviewChild: function(item) {
			var childObject = {
				name: this.timelineModel.store.getValue(item, "name"), //$NON-NLS-1$
				enabledContext: ProcessConstants.PermissionContext.ITERATION,
				existing: true, // mark existing elements
				startDate: this.timelineModel.store.getValue(item, "startDate"), //$NON-NLS-1$
				endDate: this.timelineModel.store.getValue(item, "endDate"), //$NON-NLS-1$
				archived: this.timelineModel.store.getValue(item, "archived"), //$NON-NLS-1$
				current: this.timelineModel.store.getValue(item, "current"), //$NON-NLS-1$
				completed: this.timelineModel.store.getValue(item, "completed"), //$NON-NLS-1$
				iterationTypeItemId: this.timelineModel.store.getValue(item, "iterationTypeItemId") //$NON-NLS-1$
			};
			var children = this._addExistingPreviewChildren(item);
			if (children.length > 0) {
				childObject.items = children;
			}

			var childItem = this.previewStore.newItem(childObject);
			return childItem;
		},
		
		_isSelectedRootItemATimeline: function() {
			if (this.rootItem) {
				return ProcessConstants.PermissionsScopeContext.DEV_LINES === this.timelineModel.store.getValue(this.rootItem, "parent"); //$NON-NLS-1$
			}
			return true;
		},
		
		_getSelectedRootItemContext: function() {
			if (this.rootItem) {
				return this.timelineModel.store.getValue(this.rootItem, "enabledContext"); //$NON-NLS-1$
			}
			return ProcessConstants.PermissionContext.DEV_LINE;
		},
		
		/**
		 * Sizes the various sub-elements in the date/time section based
		 * on the width of the dialog. Should be called on startup and whenever
		 * the dialog's size changes.
		 */
		_adjustDateTimeHeaderLayout: function() {
			//Clear the widths that might have been set from a previous
			//invocation
			dojo.style(this.scheduledDaysAndStartEndTimes, "width", ""); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.style(this.scheduledDays.domNode, "width", ""); //$NON-NLS-1$ //$NON-NLS-2$
			
			//First, calculate and set an explicit width for the right-most
			//section which contains the scheduled days checkboxes and start/
			//end time fields. We want the largest width that keeps the
			//tops of that section and the Start Date field the same.
			
			var scheduledDaysAndStartEndTimesBox = dojo.marginBox(this.scheduledDaysAndStartEndTimes);
			var startDateBox = dojo.marginBox(this.startDate.domNode.parentNode);
			var scheduledDaysAndStartEndTimesWidth = scheduledDaysAndStartEndTimesBox.w;
			var scheduledDaysBox = dojo.marginBox(this.scheduledDays.domNode);
			var scheduledDaysWidth = scheduledDaysBox.w;
			
			while (Math.abs(scheduledDaysAndStartEndTimesBox.t - startDateBox.t) > 5 && scheduledDaysAndStartEndTimesWidth > 0) {
				scheduledDaysAndStartEndTimesWidth -= 5;
				scheduledDaysWidth -= 5;
				dojo.style(this.scheduledDaysAndStartEndTimes, "width", scheduledDaysAndStartEndTimesWidth + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.scheduledDays.domNode, "width", scheduledDaysWidth + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				scheduledDaysAndStartEndTimesBox = dojo.marginBox(this.scheduledDaysAndStartEndTimes);
			}
			
			//If the width is smaller than zero for any reason, (we didn't want
			//the loop to be infinite), clear the width of the right-most
			//section
			if (scheduledDaysAndStartEndTimesWidth <= 0) {
				dojo.style(this.scheduledDaysAndStartEndTimes, "width", ""); //$NON-NLS-1$ //$NON-NLS-2$
			}
			
			//Next, adjust the width of the container of scheduled days
			//checkboxes so that a checkbox and its label are not seperated
			//at the end of the first line/beginning of the second line.
			//This is only done for the first line since trying to account
			//for subsequent lines might cause checkboxes on earlier lines to
			//be split. It's also unlikely that more than two lines would be
			//needed. I tried applying style whitespace: no-wrap to the
			//Scheduled Days elements, which is the technique used to keep
			//the start/end time labels and fields together, but for some
			//reason there is never a line break between the checkboxes, (so
			//they are truncated), when this is done.
			
			scheduledDaysBox = dojo.marginBox(this.scheduledDays.domNode);
			
			var firstSplitLabel = null;
			var checkboxBox = null;
			var labelBox = null;
			
			dojo.some(this.scheduledDays._labels, function(label) {
				checkboxBox = dojo.marginBox(label._checkbox);
				labelBox = dojo.marginBox(label._text);
				if (Math.abs(checkboxBox.t - labelBox.t) > 5) {
					firstSplitLabel = label;
					return true;	
				}
				return false;
			});
			
			if (firstSplitLabel !== null) {
				while (Math.abs(checkboxBox.t - labelBox.t) > 5 && scheduledDaysWidth > 0) {
					scheduledDaysWidth -= 5;
					dojo.style(this.scheduledDays.domNode, "width", scheduledDaysWidth + "px"); //$NON-NLS-1$ //$NON-NLS-2$
					checkboxBox = dojo.marginBox(firstSplitLabel._checkbox);
					labelBox = dojo.marginBox(firstSplitLabel._text);
				}
			}
			
			if (scheduledDaysWidth <= 0) {
				dojo.style(this.scheduledDays.domNode, "width", ""); //$NON-NLS-1$ //$NON-NLS-2$
			}
		},

		getPreviewTree:function(){
			return this._previewTree;
		},
		
		uninitialize: function() {
			this.inherited(arguments);
			
			jazz.util.ResizeNotifier.unregister(this._onResizeEvent);
			delete this._onResizeEvent;
			
			this._scheduledDaysUA.destroyRecursive();
			delete this._scheduledDaysUA;
			
			this.scheduledDays.destroyRecursive();
			delete this.scheduledDays;

			if (this._messageBox) {
				this._messageBox.destroyRecursive();
				delete this._messageBox;
			}
		}
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialog._ScheduledDaysWidget", [dijit._Widget], { //$NON-NLS-1$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-dialog-CreateIterationsDialog_ScheduledDaysWidget"); //$NON-NLS-1$
		},
		
		startup: function() {
            var makeCheckbox = dojo.hitch(this, function(attachPointId) {
            	var label = dojo.create("label", {"class": "scheduledDay"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
            	label._checkbox = this[attachPointId + "-checkbox"] = dojo.create("input", {"type": "checkbox"}, label); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
            	label._text = this[attachPointId + "-text"] = dojo.create("span", {}, label); //$NON-NLS-1$ //$NON-NLS-2$ 
            	this[attachPointId + "-text"].appendChild(document.createTextNode(this.messages["CreateIterationsDialog_" + attachPointId])); //$NON-NLS-1$ //$NON-NLS-2$
            	
            	this.connect(label._checkbox, "onclick", "_onScheduledDayChanged"); //$NON-NLS-1$ //$NON-NLS-2$
            	
            	this._labels.push(label); 
            });
            
            this._labels = [];
            dojo.forEach(["scheduledDaySu", "scheduledDayMo", "scheduledDayTu", "scheduledDayWe", "scheduledDayTh", "scheduledDayFr", "scheduledDaySa"], makeCheckbox); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$ //$NON-NLS-5$ //$NON-NLS-6$ //$NON-NLS-7$
		},
		
		/**
		 * Update the days that are scheduled, which checks/unchecks the
		 * buttons in the UI appropriately.
		 */
		setScheduledDays: function(/*boolean[]*/ scheduledDays) {
			for (var i = 0; i < scheduledDays.length; i++) {
				this._labels[i]._checkbox.checked = scheduledDays[i];
			}
		},
		
		/**
		 * Consumers can dojo.connect to this function to be notified when
		 * the scheduled days are is changed.
		 * 
		 * @param {boolean[]} scheduledDays
		 */
		scheduledDaysChanged: function(scheduledDays) {
		},
		
		_onScheduledDayChanged: function() {
			var scheduledDays = [];
			dojo.forEach(this._labels, function(label) {
				scheduledDays.push(label._checkbox.checked);
			});
			this.scheduledDaysChanged(scheduledDays);
		}
	});
})();
