/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureModel"); //$NON-NLS-1$
	
	dojo.require("dijit.tree.TreeStoreModel"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	/**
	 * Model for a structure of iterations as defined as a tree where each node
	 * is a "pattern" of a certain number of iterations. 
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogIterationStructureModel", dijit.tree.TreeStoreModel, { //$NON-NLS-1$
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._nameGenerators = {};
		},
		
		/**
		 * Returns the store item that is the parent of the specified item.
		 */
		getParent: function(item) {
			var parentId = this.store.getValue(item, "parentId"); //$NON-NLS-1$
			return parentId === null ? null : this.store.fetchItemByIdentity(parentId);
		},
		
		/**
		 * Returns a function that accepts two numeric arguments:
		 * 
		 * index		Zero-based index of real iteration whose name is to
		 *              be generated.
		 * 
		 * batchNum		Zero-based number of "batch" of iterations. Only takes
		 * 				affect when the convention includes numeric counters
		 * 				that span parent nodes, a.k.a. 'continuous' counters.
		 * 
		 * The function returns the name of the real iteration, based on either
		 * the naming convention that has been configured or the default naming
		 * convention.
		 */
		getNameGenerator: function(item) {
			var nameGenerator;
			var nameGeneratorInfo = this._nameGenerators[this.store.getIdentity(item)];
			var quantity = this.store.getValue(item, "quantity"); //$NON-NLS-1$
			if (nameGeneratorInfo === undefined) {
				//default naming convention
				var name = this.store.getValue(item, "name"); //$NON-NLS-1$
				nameGenerator = dojo.hitch(this, function(index) {
					return ((typeof quantity === "undefined") || quantity=== 1) //$NON-NLS-1$
						? name
						: dojo.string.substitute(this.messages.CreateIterationsDialogIterationStructureModel_defaultNamePattern, [name, index+1]);
				});
			} else {
				nameGenerator = function(index, batchNum) {
					return dojo.hitch(nameGeneratorInfo.container, nameGeneratorInfo.generator, index, quantity, batchNum)();
				}
			}
			return nameGenerator;
		},
		
		/**
		 * Update the name generator for an item. If null is passed as the
		 * second argument, getNameGenerator(item) will return an implementation
		 * of the default naming convention.
		 * 
		 * Clients should dojo.connect to namingConventionChanged to be
		 * notified when the naming convention has changed.
		 * 
		 * item						The item whose naming convention is to be
		 * 							set
		 * nameGeneratorContainer	Object that will be 'this' when the name
		 * 							generator function is invoked via
		 * 							dojo.hitch(), or null if the default
		 * 							naming convention is to be used.
		 * nameGeneratorFunction	A function that returns a string and
		 * 							accepts three numeric arguments:
		 * 	- index:	0 based index of item whose name to generate
		 * 	- quantity:	number of items in a "batch"; greater than 1
		 * 	- batchNum:	0 based index of the "batch" whose names to generate
		 */
		setNameGenerator: function(item, nameGeneratorContainer, nameGeneratorFunction) {
			if (nameGeneratorContainer === null) {
				delete this._nameGenerators[this.store.getIdentity(item)];
			} else {
				this._nameGenerators[this.store.getIdentity(item)] = {
					container: nameGeneratorContainer,
					generator: nameGeneratorFunction 
				};
			}
			this.namingConventionChanged(item);
		},
		
		/**
		 * Function that is invoked when the naming convention for an item
		 * has changed; either because the user made a change to the custom
		 * naming convention, or reverted to use the default convention.
		 * 
		 * Consumers may dojo.connect() to this function to be notified. They
		 * can then call getNameGenerator() to access a function that returns
		 * updated names.
		 * 
		 * @param {Object} item
		 */
		namingConventionChanged: function(item) {
		}
	});
})();
