/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights: Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 ******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewTree"); //$NON-NLS-1$

	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogDatesCalculator"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$

	var PROP_HAS_DELIVERABLE = "hasDeliverable"; //$NON-NLS-1$
	var PROP_INDEX = "index"; //$NON-NLS-1$
	var PROP_ITEMS = "items"; //$NON-NLS-1$
	var PROP_ITERATION_TYPE_ITEM_ID = "iterationTypeItemId"; //$NON-NLS-1$
	var PROP_NAME = "name"; //$NON-NLS-1$
	var PROP_PARENT_IDENTITY = "parentIdentity"; //$NON-NLS-1$
	var PROP_QUANTITY = "quantity"; //$NON-NLS-1$
	var PROP_DURATION = "duration"; //$NON-NLS-1$
	var PROP_ITERATION_STRUCTURE_ID = "iterationStructureId"; //$NON-NLS-1$
	var PROP_IS_PROJECT_TIMELINE = "projectTimeline"; //$NON-NLS-1$

	var MAX_ITEMS_TO_EXPAND = 12;

	/**
	 * Subclass of TimelineTree which displays a preview of the sub-iterations
	 * that are to be created in the dialog CreateIterationsDialog.
	 * 
	 * @since 4.0.5
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogPreviewTree", [com.ibm.team.process.web.ui.internal.admin.projects.TimelineTree], { //$NON-NLS-1$

		/**
		 * Required arguments:
		 * 
		 * @param model :
		 *            TimelineModel, required by TimelineTree. This model should
		 *            be set to use CreateIterationdDialogPreviewStore.
		 * @param existingChildren :
		 *            An array of existing items ( in
		 *            CreateIterationsDialogPreviewStore) which may be displayed
		 *            in the tree. The toggleShowExisting() controls if the
		 *            existing items are displayed.
		 * @param iterationStructureModel :
		 *            The iteration structure model, for which this tree
		 *            provides a preview.
		 * @param timelineStore :
		 *            Store that contains scopes defined in the editor, used to
		 *            populate the UI to select iteration types.
		 */
		constructor: function(args) {
			args.showRoot = true;
			args.editorContext = ProcessConstants.EditorContext.PROJECT, this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.showExistingIterations = true;
			this._updateNamesMap = {};
			this._datesCalculator = new com.ibm.team.process.web.ui.internal.admin.projects.CreateIterationsDialogDatesCalculator({
				previewModel: args.model,
				iterationStructureModel: args.iterationStructureModel,
				commonStartDate: args.commonStartDate,
				scheduledDays: args.scheduledDays
			});
		},

		postCreate: function() {
			this.inherited(arguments);

			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-CreateIterationsDialogPreviewTree"); //$NON-NLS-1$

			this.connect(this.iterationStructureModel, "onChildrenChange", "_onIterationStructureChildrenChange"), //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.iterationStructureModel, "onSetItem", "_onIterationStructureSetItem"),//$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this.iterationStructureModel, "namingConventionChanged", "_onIterationStructureNamingConventionChanged");//$NON-NLS-1$ //$NON-NLS-2$
		},

		onLoad: function onLoad() {
			// Unset the default focus. We do not want to have focus on a preview item.
			this.setFocusedNode(undefined);
			this.inherited(arguments);
		},

		getLabel: function(item) {
			var label = this.inherited(arguments);

			// Decorate the label with 'iteration type'
			var iterationTypeId = this.model.store.getValue(item, PROP_ITERATION_TYPE_ITEM_ID);
			return this._decorateLabelWithIterationType(label, iterationTypeId);
		},

		/**
		 * If the timelineStore contains an iteration type with iterationTypeId,
		 * retrieve its display label and append this information to the passed
		 * label.
		 * 
		 * @param label :
		 *            base label
		 * @param iterationTypeId :
		 *            id on the iteration type which label we want to display
		 *            together with the label.
		 * @returns newly constructed label string.
		 */
		_decorateLabelWithIterationType: function(label, iterationTypeId) {
			var decoratedLabel = label;
			if (iterationTypeId && iterationTypeId != ProcessConstants.ITERATION_TYPE_NONE) {
				var fetchIterationTypeItem = dojo.hitch(this, function(itemId) {
					var iterationType = null;
					args = {
						identity: itemId,
						onItem: function(item) {
							iterationType = item;
						}
					};
					this.timelineStore.fetchItemByIdentity(args);
					return iterationType;
				});

				var iterationTypeItem = fetchIterationTypeItem(iterationTypeId);
				if (iterationTypeItem) {
					var iterationTypeLabel = this.timelineStore.getIterationTypeLabel(iterationTypeItem);
					decoratedLabel = dojo.string.substitute(this.messages.CreateIterationsDialogCreateIterationsDialogPreviewTree_1, [label, iterationTypeLabel]);
				}
			}
			return decoratedLabel;
		},

		_onIterationStructureChildrenChange: function(/* dojo.data.Item */structureParent, /* dojo.data.Item[] */structureChildren) {

			var updateStructure = function() {
				var pathsTracker = [];
				var previewNodesAdded = this._updateChildren(structureParent, structureChildren, pathsTracker);

				// If nodes were added, we want them to be highlighted if their
				// iteration structure item is selected. There doesn't seem to be
				// an easy way to be notified when nodes are created, but the node
				// always seems to have been created by this point, so calling
				// iterationStructureItemSelected() again works.
				if (previewNodesAdded) {
					this.iterationStructureItemSelected(this._currentSelectedIterationStructureItem);
				}

				// Expand the tree for newly created children

				this.setPaths(pathsTracker);
				for ( var i in this._updateNamesMap) {
					if (this._updateNamesMap.hasOwnProperty(i)) {
						var structureItem = this._updateNamesMap[i];
						this._updateNames(structureItem);
						delete this._updateNamesMap[i];
					}
				}
				this._updateDates();
			};
			setTimeout(dojo.hitch(this, updateStructure));
		},

		_updateChildren: function(/* dojo.data.Item */structureParent, /* dojo.data.Item[] */structureChildren, pathsTracker) {

			// Find and iterate though all corresponding to the structureParent
			// preview items (preview parents)
			var previewParents = this._getCorrespondingPreviewItemsForStructureItem(structureParent);
			var previewNodesAdded = false;
			dojo.forEach(previewParents, dojo.hitch(this, function(previewParent) {
				var parentIdentity = this.model.getIdentity(previewParent);

				var newPreviewChildren = [];
				// Iterate through structure children.
				dojo.forEach(structureChildren, dojo.hitch(this, function(structureItem) {
					// For each structure child retrieve current corresponding
					// preview items (previewChildren)
					// and the new expected quantity.
					var previewChildren = this._getCorrespondingPreviewItemsForStructureItem(structureItem, parentIdentity);
					var structureItemQuantity = this.iterationStructureModel.store.getValue(structureItem, PROP_QUANTITY);
					if (previewChildren.length <= structureItemQuantity) {
						// Save current children
						this._appendArrayElements(newPreviewChildren, previewChildren);

						// Add new children (if any)
						if (previewChildren.length < structureItemQuantity) {
							var newChildren = this._addNewPreviewItemsForStructureItem(structureItem, previewParent, previewChildren.length, structureItemQuantity);
							this._appendArrayElements(newPreviewChildren, newChildren);

							// Process recursively structureItem children
							var currentStructureItemChildren = this.iterationStructureModel.store.getValues(structureItem, "children") || [];//$NON-NLS-1$
							if (currentStructureItemChildren.length) {
								previewNodesAdded = this._updateChildren(structureItem, currentStructureItemChildren, pathsTracker) | previewNodesAdded;
							} else {
								if (newChildren.length) {
									this._addItemsPathsToTracker(pathsTracker, newChildren[0]);
								}
							}

							previewNodesAdded = true;
						}
					} else { // (previewChildren.length > structureItemQuantity)
						this._appendArrayElements(newPreviewChildren, previewChildren, structureItemQuantity);
						// DELETE UNNEEDED CHILDREN
						for ( var i = structureItemQuantity; i < previewChildren.length; i++) {
							this._deletePreviewItem(previewChildren[i]);
						}
					}
					if (previewChildren.length != structureItemQuantity) {
						var structureItemIdentity = this.iterationStructureModel.getIdentity(structureItem);
						this._updateNamesMap[structureItemIdentity.toString()] = structureItem;
					}
				})); // dojo.forEach(structureChildren

				// Save the new children array for the preview parent
				if (previewParent === this.model.root && this.existingChildren.length > 0 && this.showExistingIterations) {
					this.model.store.setValues(previewParent, PROP_ITEMS, this.existingChildren.concat(newPreviewChildren));
				} else {
					this.model.store.setValues(previewParent, PROP_ITEMS, newPreviewChildren);
				}
			})); // dojo.forEach(previewParents

			return previewNodesAdded;
		},

		_onIterationStructureSetItem: function(/* dojo.data.Item */structureItem, attribute, oldValue, newValue) {
			if (oldValue !== newValue) {
				if (attribute === PROP_QUANTITY) {
					this._updateQuantity(structureItem);
					this._updateDates();
				} else if (attribute === PROP_ITERATION_TYPE_ITEM_ID) {
					this._updatePreviewItemsProperty(structureItem, attribute, newValue);
					this._updateNames(structureItem);
				} else if (attribute === PROP_HAS_DELIVERABLE) {
					this._updatePreviewItemsProperty(structureItem, attribute, newValue);
				} else if (attribute === PROP_DURATION) {
					this._updateDates();
				} else if (attribute === PROP_IS_PROJECT_TIMELINE) {
					this._updatePreviewItemsProperty(structureItem, attribute, newValue);
				}
			}
		},

		_onIterationStructureNamingConventionChanged: function(/* dojo.data.Item */structureItem) {
			this._updateNames(structureItem);
		},

		_updateNames: function(/* dojo.data.Item */structureItem) {
			// Handle name change
			var previewItems = this._getCorrespondingPreviewItemsForStructureItem(structureItem);
			var nameGenerator = this.iterationStructureModel.getNameGenerator(structureItem);

			dojo.forEach(previewItems, dojo.hitch(this, function(previewItem) {
				var parentItem = this._getParent(previewItem);
				var batchNumber = (parentItem) ? this._calculateBatchNumber(parentItem) : 0;
				var index = this.model.store.getValue(previewItem, PROP_INDEX);
				var name = nameGenerator(index, batchNumber);
				this.model.store.setValue(previewItem, PROP_NAME, name);
			}));
		},

		_calculateBatchNumber: function(previewItem) {
			var index = this.model.store.getValue(previewItem, PROP_INDEX);
			var iterationStructureId = this.model.store.getValue(previewItem, PROP_ITERATION_STRUCTURE_ID);
			var structureItem = this.iterationStructureModel.store.fetchItemByIdentity(iterationStructureId);
			var parentItem = this._getParent(previewItem);
			if (parentItem) {
				var parentBatchNumber = this._calculateBatchNumber(parentItem);
				var quantity = this.iterationStructureModel.store.getValue(structureItem, PROP_QUANTITY);
				return parentBatchNumber * quantity + index;
			} else {
				return index;
			}
		},

		_updateQuantity: function(/* dojo.data.Item */structureItem) {
			var parentItem = this.iterationStructureModel.getParent(structureItem);
			var currentChildren = this.iterationStructureModel.store.getValues(parentItem, "children") || [];//$NON-NLS-1$
			this._onIterationStructureChildrenChange(parentItem, currentChildren);
		},

		_updatePreviewItemsProperty: function(/* dojo.data.Item */structureItem, property, newValue) {
			var previewItems = this._getCorrespondingPreviewItemsForStructureItem(structureItem);
			dojo.forEach(previewItems, dojo.hitch(this, function(previewItem) {
				this.model.store.setValue(previewItem, property, newValue);
			}));
		},

		/**
		 * For newly added items compute the tree path and store in pathsTracker
		 * 
		 * @param pathsTracker :
		 *            An array of arrays of items defining a path to the new
		 *            item).
		 * @param item :
		 *            Newly added leaf item added to the tree for which the path
		 *            is computed.
		 */
		_addItemsPathsToTracker: function(pathsTracker, item) {
			if (pathsTracker.length < MAX_ITEMS_TO_EXPAND) {
				var constructPath = dojo.hitch(this, function(item) {
					var path = [];
					var parentItem = this._getParent(item);
					if (parentItem) {
						path = constructPath(parentItem);
					}
					path.push(item);
					return path;
				});

				var path = constructPath(item);
				pathsTracker.push(path);
			}
		},

		setPaths: function(paths) {
			this.inherited(arguments);
			if (paths.length) {
				this.set("selectedItem", undefined); //$NON-NLS-1$
			}
		},

		/**
		 * Retrieves corresponding preview tree items for the specified
		 * structureItem. <br>
		 * 
		 * For every entry in the structure table there is a number (defined by
		 * quantity) of items in the preview tree. If parent's quantity is more
		 * then one, we have a number of groups (matching the number of parents)
		 * of preview items. To find items for a specific parent (group) only,
		 * the parentIdentity should be passed as an argument.
		 * 
		 * @param structureItem :
		 *            An item in the structure table, for which we want to find
		 *            the corresponding preview tree items.
		 * @param parentIdentity :
		 *            if passed to the method, only items belonging to this
		 *            parent will be returned. if undefined, all tree items for
		 *            all parents are returned
		 * @returns {Array} : array of items matching the structureItem and
		 *          parentIdentity if provided
		 */
		_getCorrespondingPreviewItemsForStructureItem: function(/* dojo.data.Item */structureItem, parentIdentity) {
			var structureItemIdentity = this.iterationStructureModel.getIdentity(structureItem);
			var retrievedTreeItems = [];
			var queryObj = {
				iterationStructureId: structureItemIdentity
			};
			if (parentIdentity) {
				queryObj[PROP_PARENT_IDENTITY] = parentIdentity;
			}

			this.model.store.fetch({
				query: queryObj,
				queryOptions: {
					deep: true,
					sort: [{
						attribute: PROP_INDEX
					}]
				},
				onComplete: dojo.hitch(this, function(items) {
					retrievedTreeItems = items;
				})
			});
			return retrievedTreeItems;
		},

		_addNewPreviewItemsForStructureItem: function(structureItem, parentPreviewItem, startIndex, endIndex) {
			var newPreviewItems = [];
			var iterationTypeId = this.iterationStructureModel.store.getValue(structureItem, PROP_ITERATION_TYPE_ITEM_ID);
			var hasDeliverable = this.iterationStructureModel.store.getValue(structureItem, PROP_HAS_DELIVERABLE);
			var parentIdentity = this.model.getIdentity(parentPreviewItem);

			for ( var i = startIndex; i < endIndex; i++) {
				var itemObject = {
					name: "newItem", //A default name should be set to avoid code exceptions. The actuall name will be set by a call to this._updateNames() //$NON-NLS-1$
					enabledContext: ProcessConstants.PermissionContext.ITERATION,
					iterationStructureId: this.iterationStructureModel.getIdentity(structureItem),
					parentIdentity: parentIdentity,
					index: i,
					iterationTypeItemId: iterationTypeId,
					hasDeliverable: hasDeliverable,
					startDate: null,
					endDate: null
				};
				var childItem = this.model.store.newItem(itemObject);
				newPreviewItems.push(childItem);
			}
			return newPreviewItems;
		},

		/**
		 * Recursively deletes a previewItem subtree, starting from its
		 * children.
		 * 
		 * @param previewItem
		 */
		_deletePreviewItem: function(previewItem) {
			// delete descendant items and then the item itself
			var children = this.model.store.getValues(previewItem, PROP_ITEMS);
			dojo.forEach(children, dojo.hitch(this, this._deletePreviewItem));
			this.model.store.deleteItem(previewItem);
		},

		/**
		 * Appends elements from fromArray to toArray.
		 * 
		 * @param toArray
		 * @param fromArray
		 * @param nrOfElements :
		 *            Number of elements to be copied. If undefined, all
		 *            elements in fromArrays are copied.
		 */
		_appendArrayElements: function(toArray, fromArray, nrOfElements) {
			var i, nr = nrOfElements ? nrOfElements : fromArray.length;
			for (i = 0; i < nr; i++) {
				toArray.push(fromArray[i]);
			}
		},

		/**
		 * Returns the preview store item that is the parent of the specified
		 * preview item.
		 */
		_getParent: function(item) {
			var parentId = this.model.store.getValue(item, PROP_PARENT_IDENTITY);
			return parentId === null ? null : this.model.store.fetchItemByIdentity(parentId);
		},

		/**
		 * Change whether or not existing iterations are displayed.
		 */
		toggleShowExisting: function(showExisting) {
			if (this.showExistingIterations != showExisting) {
				this.showExistingIterations = showExisting;
				var children = this.model.store.getValues(this.model.root, PROP_ITEMS);
				if (this.showExistingIterations) {
					this.model.store.setValues(this.model.root, PROP_ITEMS, this.existingChildren.concat(children));
				} else {
					children.splice(0, this.existingChildren.length);
					this.model.store.setValues(this.model.root, PROP_ITEMS, children);
				}
			}
		},

		/**
		 * Reacts to changes to the selected iteration structure row.
		 */
		iterationStructureItemSelected: function(item) {
			this._currentSelectedIterationStructureItem = item;

			dojo.query(".iterationRowSelected", this.domNode).forEach(function(node) { //$NON-NLS-1$
				dojo.removeClass(node, "iterationRowSelected"); //$NON-NLS-1$
			});

			if (item) {
				var pathsTracker = [];
				var firstNode = null;

				dojo.forEach(this._getCorrespondingPreviewItemsForStructureItem(item), dojo.hitch(this, function(previewItem) {
					dojo.forEach(this.getNodesByItem(previewItem), function(node) {
						if (node && node.rowNode) {
							if (firstNode === null) {
								firstNode = node.rowNode;
							}
							dojo.addClass(node.rowNode, "iterationRowSelected"); //$NON-NLS-1$
						}
					});
					this._addItemsPathsToTracker(pathsTracker, previewItem);
				}));

				//Expand the tree to reveal the highlighted items...
				this.setPaths(pathsTracker);

				//...and scroll to the first highlighted node, if neccessary
				if (firstNode !== null) {
					firstNode.scrollIntoView(false);
				}
			}
		},

		/**
		 * Update the dates for the passed array of previewItems.
		 * 
		 */
		_updateDates: function() {
			var warnings = this._datesCalculator.updateDates();

			dojo.query(".warnIcon", this.domNode).forEach(function(node) { //$NON-NLS-1$
				dojo.removeClass(node, "warnIcon"); //$NON-NLS-1$
				dojo.removeAttr(node, "title");//$NON-NLS-1$
			});

			for ( var id in warnings) {
				var item = this.model.store.fetchItemByIdentity(id);
				dojo.forEach(this.getNodesByItem(item), function(node) {
					if (node && node.iconNode) {
						dojo.addClass(node.iconNode, "warnIcon"); //$NON-NLS-1$
						dojo.attr(node.iconNode, "title", warnings[id]);//$NON-NLS-1$
					}
				});
			}
		},

		/**
		 * Processes change in common start date value.
		 * 
		 * @param newCommonStartDate
		 */
		commonStartDateChanged: function(newCommonStartDate) {
			this._datesCalculator.setCommonStartDate(newCommonStartDate);
			this._updateDates();
		},

		/**
		 * Processes change in scheduled days selection.
		 * 
		 * @param scheduledDays :
		 *            An array of 7 boolean elements, each of which represents
		 *            the checked box for selected days [Su, Mo, Tu, We, Th, Fr,
		 *            Sa]
		 */
		scheduledDaysChanged: function(newScheduledDays) {
			this._datesCalculator.setScheduledDays(newScheduledDays);
			this._updateDates();
		},

		uninitialize: function() {
			this.inherited(arguments);
			delete this._datesCalculator;
		}
	});
})();