/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("dojox.xml.parser"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.client.internal.ProcessClient"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.client.projectarea"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.StyledTableItemList"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRToolbar"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.TRLink"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaRecentlyViewed"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper"); //$NON-NLS-1$
	dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button");//$NON-NLS-1$
	
	dojo.require("net.jazz.ajax.contextRoot"); //$NON-NLS-1$
	dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
	var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
	var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var MessageBox = jazz.ui.MessageBox;
	var STATUS_TYPES = MessageBox.STATUS_TYPES;
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var UIHelper = com.ibm.team.process.web.ui.internal.admin.projects.UIHelper;
	var ProjectAreaRecentlyViewed = com.ibm.team.process.web.ui.internal.admin.projects.ProjectAreaRecentlyViewed;
	
	function setText(e, text){
		e.appendChild(document.createTextNode(text));
	}
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ManageArchivedProjectAreas", com.ibm.team.process.web.ui.internal.StyledTableItemList, { //$NON-NLS-1$
		constructor: function(args){
			this._owningApplicationKey = args.owningApplicationKey;
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._resourceLinks = [];
		},
		
		postCreate: function(){
			this.header = new MessageBox();
			this.headerArea.appendChild(this.header.domNode);
			this._actionToolbars = [];
			this._toolbarButtons = [];
			setText(this.heading, this.messages.ManageArchivedProjectAreas_1);
			setText(this.description, this.messages.ManageArchivedProjectAreas_2);
			this.setHelpHooks(ProcessConstants.UAKey.ARCHIVED_PROJECTAREAS_PAGE);
            dojo.addClass(this.tableContents, ProcessConstants.CSSClass.TOOLBAR_TABLE);
			this.createTable({
				columns: [{
            		label: this.messages.ManageArchivedProjectAreas_3,
            		width: "30%", //$NON-NLS-1$
            		sortable: false
            	}, {
            		label: this.messages.ManageArchivedProjectAreas_4,
            		sortable: false
            	}, {
					label: this.messages.ManageArchivedProjectAreas_11,
					className: ProcessConstants.CSSClass.ACTION_COLUMN,
					sortable: false
				}]
			});
			this._refreshLink = this.addLink("#", "", "com.ibm.team.repository.web/ui/internal/images/refresh16.gif", dojo.hitch(this, function() { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				this.refresh();		
			}), this.messages.ManageArchivedProjectAreas_13);	
			
			this.refresh();
		},
		
		refresh: function(){
			this.clearState();
			var tr = document.createElement("tr"); //$NON-NLS-1$
			var td = document.createElement("td"); //$NON-NLS-1$
			td.colSpan = 2;
			td.appendChild(document.createTextNode(this.messages.ManageArchivedProjectAreas_5));
			tr.appendChild(td);
			this.tbody.appendChild(tr);
			var srh = new ServiceResponseHandler(this, "showProjects", "handleError"); //$NON-NLS-1$ //$NON-NLS-2$
			ProcessClient.getAllProjectAreas(srh, {
				onlyArchivedProjects: true,
				owningApplicationKey: this._owningApplicationKey
			});
		},
		
		clearState: function(){
			dojox.xml.parser.removeChildren(this.tbody);
			this.header.hideMessage();
			this._destroyResourceLinks();
			this._destroyActionToolbars();
		},
		
		_createToolbar: function(projectArea) {
			var trToolbar = new com.ibm.team.process.web.ui.internal.TRToolbar();
			
			var unarchiveAction = new jazz.ui.toolbar.Button({
				imgSrc : dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/unarchive_hov.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				title : this.messages.ManageArchivedProjectAreas_7,
				onClick : dojo.hitch(this, function (event) {
					dojo.stopEvent(event);
					if (confirm(dojo.string.substitute(this.messages.ManageArchivedProjectAreas_12, [projectArea.name]))) {
						this.unarchiveProject({
							itemId: projectArea.itemId,
							name: projectArea.name
						});
					}
				})
			});
			this._toolbarButtons.push(unarchiveAction);
			trToolbar.addChild(unarchiveAction);
			
			return trToolbar;
		},
		
		showProjects: function(data){
            UIHelper.runSafe(dojo.hitch(this, function(){
            	dojox.xml.parser.removeChildren(this.tbody);
            	if (data) {
            		var sortedData = data.sort(function(a, b){
            			return a.name.toLowerCase().localeCompare(b.name.toLowerCase());
            		});
            		for (var i = 0; i < sortedData.length; i++) {
            			var actionHref = ProcessConstants.ProjectAreaEditAction.ID_ITEMIDSUFFIX + sortedData[i].itemId;
            			var descNode = this._createDescriptionNode(sortedData[i].summary, sortedData[i].description);
            			var trToolbar = this._createToolbar(sortedData[i]);
						this._actionToolbars.push(trToolbar);						
						
						var row = this.addRow(actionHref, [sortedData[i].name, descNode, trToolbar.domNode]);
            			if (i === 0) {
            				dojo.toggleClass(row.parentNode, "projectTable", true); //$NON-NLS-1$
            			}
						
						var query = "a[href*='" + ProcessConstants.ProjectAreaEditAction.ID + "']";//$NON-NLS-1$ //$NON-NLS-2$
						var link = dojo.query(query, row)[0];
						var linkParent = link.parentNode;
						var presentationUri = link.href;
						var link = com.ibm.team.process.web.ui.internal.TRLink({
							href: this.getProjectAreaUrl(sortedData[i].itemId),
							link: link,
							resourceLink: {
								'presentationUri': presentationUri, //$NON-NLS-1$
								'lazyFetch': true //$NON-NLS-1$ 
							}
						});
						this._resourceLinks.push(link);
						linkParent.appendChild(link.domNode);
            		}
            	} else {
            		if (!this._projectAreaUnarchiving) {
						this.header.showMessage({
            				type: STATUS_TYPES.INFO,
            				summary: this.messages.ManageArchivedProjectAreas_6
            			});
					}
            	}
				if (this._projectAreaUnarchiving) {
					delete this._projectAreaUnarchiving;
				}
            }));
		},
		
		getProjectAreaUrl: function(projectAreaItemId){
			var location = document.location.href;
			var path = location.split("/"); //$NON-NLS-1$
			var serverAddress;
			if (path.length > 0) {
				serverAddress = path.shift() + "//";//$NON-NLS-1$
				path.shift(); //empty from the double //
				serverAddress = serverAddress + path.shift();
			}
			return serverAddress + net.jazz.ajax.contextRoot() + "/process/project-areas/" + projectAreaItemId; //$NON-NLS-1$
		},
		
		unarchiveProject: function(projectArea){
			if (!this._projectAreaUnarchiving) {
				this._projectAreaUnarchiving = projectArea;
				this._disableToolbars(true);
				var srh = new ServiceResponseHandler(this, "unarchiveCallback", "unarchiveErrorCallback"); //$NON-NLS-1$ //$NON-NLS-2$
				this.header.showMessage({
					type: STATUS_TYPES.INFO,
					summary: this.messages.ManageArchivedProjectAreas_8
				});

				ProcessClient.postUnarchiveProcessArea({
					uuid: projectArea.itemId,
					itemType: ProcessConstants.ITEM_TYPE.PROJECT_AREA
				}, srh);
			}
		},
		
		unarchiveCallback: function(){
			dojo.publish(ProcessConstants.ProjectSelectorContext.ADD, [this._projectAreaUnarchiving.name]);
			
			var recentViewedProjectArea = {};
			recentViewedProjectArea.lastVisitTime = new Date().getTime();
			recentViewedProjectArea.projectAreaItemId = this._projectAreaUnarchiving.itemId;
			recentViewedProjectArea.projectAreaName = this._projectAreaUnarchiving.name;
			recentViewedProjectArea.isArchived = false;
			recentViewedProjectArea.owningApplicationKey = this._owningApplicationKey;
			ProjectAreaRecentlyViewed.addRecentlyViewedProjectArea(recentViewedProjectArea);
			
			UIHelper.runSafe(dojo.hitch(this, function(){
				this.refresh();
				this.header.showMessage({
					summary: this.messages.ManageArchivedProjectAreas_9,
					type: STATUS_TYPES.OK,
					closable: true
				});
			}));
		},
		
		unarchiveErrorCallback: function(err){
			delete this._projectAreaUnarchiving;
			this._disableToolbars(false);
			var summary = dojo.string.substitute(this.messages.ManageArchivedProjectAreas_10, [err.message]);
			this.header.showMessage({
				summary: summary,
				errorObject: err
			});
		},
		
		_createDescriptionNode: function(summary, description){
			if (dojo.isIE && summary !== undefined && summary.length === 0 &&
			description !== undefined &&
			description.length === 0) {
				// IE needs a space or it won't paint the TD
				summary = " "; //$NON-NLS-1$
			}
			var descNode = document.createElement("div"); //$NON-NLS-1$
			descNode.appendChild(document.createTextNode(summary));
			if (description) {
				if (summary) {
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
					descNode.appendChild(document.createElement("br")); //$NON-NLS-1$
				}
				descNode.appendChild(document.createTextNode(description));
			}
			return descNode;
		},
		
		_destroyResourceLinks: function(){
			for (var i = 0; i < this._resourceLinks.length; i++) {
				if (this._resourceLinks[i]) {
					this._resourceLinks[i].destroyRecursive();
				}
			}
			this._resourceLinks = [];
		},
		
		_disableToolbars: function(disabled){
			for (var i = 0; i < this._actionToolbars.length; i++) {
				this._actionToolbars[i].disableToolbar(disabled);
			}
		},
		
		_destroyToolbarButtons: function(){
			for (var i = 0; i < this._toolbarButtons.length; i++) {
				if (this._toolbarButtons[i]) {
					this._toolbarButtons[i].destroyRecursive();
				}
			}
			this._toolbarButtons = [];
		},
		
		_destroyActionToolbars: function(){
			this._destroyToolbarButtons();
			for (var i = 0; i < this._actionToolbars.length; i++) {
				if (this._actionToolbars[i]) {
					this._actionToolbars[i].destroyRecursive();
				}
			}
			this._actionToolbars = [];
		},
		
		uninitialize: function(){
			if (this.header) {
				this.header.destroyRecursive();
			}
			this._destroyResourceLinks();
			this._destroyActionToolbars();
			
			this.inherited(arguments);
			
		}
		
	});
	
})();
