/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCopyConfigurationWizard"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.Wizard"); //$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectBehaviorConfigurationWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.RolesStore"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var SelectProcessAreaFilterType = com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget.FilterType;
	
	/**
	 * Invoking the constructor will present a wizard to create a new operation
	 * behavior configuration by copying the behaviors from an existing
	 * configuration. Four pages are presented in sequence, to...
	 * 
	 * 1. Select a project or team area to copy from
	 * 2. Select a configuration from the project or team area
	 * 3. Select a role
	 * 4. Select a time period
	 * 
	 * If the user selects values that match an existing operation behavior
	 * configuration in the process area they are editing, an informational
	 * message is displayed to tell the user that the behaviors from the
	 * configuration they are copying will be added to the one that's already
	 * defined. Clicking Finish will have that affect. If the configuration
	 * being copied does not match an existing one in the process area being
	 * edited, a new configuration is created with the properties and
	 * behaviors of the selected configuration. 
	 * 
	 * When Finish or Cancel is clicked, the wizard is closed and all child
	 * widgets are destroyed recursively.
	 * 
	 * The arguments to the constructor match those of the constructor of type
	 * com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard,
	 * with the addition of the following fields in property "model":
	 * 
	 * owningApplicationKey		string	required
	 * 
	 *    The owning application key of the project or team that configurations
	 *    will be added to
	 * 
	 * processAreaItemId		string	required
	 * 
	 *    The identifier of the project or team that configurations will be
	 *    added to
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorCopyConfigurationWizard", //$NON-NLS-1$
		[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard], {
		
		createSteps: function(model) {
			var onFocusCallback = dojo.hitch(this, function(step) {
				//Some validation problems can be resolved by visiting a
				//specific step after selecting a configuration to copy. For
				//example, if the selected configuration's role is not valid for
				//the project/team being edited, the user will be unable to
				//Finish until they visit the Who step to select a different
				//role or see that Everyone was selected by default. The flags
				//_visited*Step are set to true here and reverted to false when
				//configurations are selected in the selection changed callback
				//for the _whichConfigurationStep below.
				if (this._whoStep === step) {
					this._visitedWhoStep = true;
				} else if (this._whenStep === step) {
					this._visitedWhenStep = true;
				}
				
				//Re-runs the validation logic when a page is navigated to
				this._wizardPageSelectionChanged();
			});
			
			//Callback for pages to determine whether the user is being
			//prevented from finishing, which they will use to change their UI
			var hasValidationProblemCallback = dojo.hitch(this, function() {
				return this._isShowingValidationError;
			});
			
			this._whichContainerStep = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectProcessAreaForCopyStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedProcessAreaContext = selectedValues.processAreaContext;
					if (this._whichConfigurationStep) {
						this._whichConfigurationStep.notifyProcessAreaSelectionChanged(selectedValues.processAreaId, selectedValues.processAreaUrl, selectedValues.processAreaContext, selectedValues.isInParentHierarchy, selectedValues.isProcessProvider);
					}
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback
			});
			
			this._whichConfigurationStep = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectConfigurationForCopyStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues, remoteConfigurationItemStore, remoteContextStore) {
					//Update state based on the configuration that was selected
					this._selectedConfigurationItem = selectedValues.selectedConfigurationItem;
					
					//The step may pass undefined as the value of
					//selectedConfigurationItem, to indicate that a value is
					//being unselected or that the table of configurations is
					//being re-computed for a different project/team
					if (this._selectedConfigurationItem !== undefined) {
						this._remoteConfigurationItemStore = remoteConfigurationItemStore;
						this._selectedOperationId = remoteConfigurationItemStore.getValue(this._selectedConfigurationItem, ProcessConstants.ExtensionDTO.ID);
						var extensionItem = this.extensionStore.fetchItemByIdentity(this._selectedOperationId);
						this._selectedOperationContext = ProcessConstants.ExtensionContext.PROJECT === this.extensionStore.getValue(extensionItem, ProcessConstants.ExtensionDTO.CONTEXT)
							? ProcessConstants.BehaviorContext.PROJECT
							: ProcessConstants.BehaviorContext.TEAM;
							
						//Tell the Who step that a new operation was selected,
						//as it may want to change whether to allow the user to
						//proceed to the When step based on the operation
						this._whoStep.notifyOperationSelectionChanged({
							operationId: this._selectedOperationId,
							operationContext: this._selectedOperationContext
						});
						
						//Clear out the wizard's 'When' and 'Who' values and
						//notify the pages so they can select the same values as
						//the configuration. Record whether they are unable to
						//select those values, as there will be a validation
						//problem to report if they cannot 
						this._selectedRoleId = undefined;
						this._selectedContext = undefined;
						this._selectedContextId = undefined;
						this._remoteConfigurationRoleIsKnown = this._whoStep.notifyRemoteConfigurationChanged(selectedValues.selectedConfigurationItem, remoteConfigurationItemStore);
						this._remoteConfigurationContextIsKnown = this._whenStep.notifyRemoteConfigurationChanged(selectedValues.selectedConfigurationItem, remoteConfigurationItemStore, this._selectedProcessAreaContext, remoteContextStore);
						
						//Clear out state of whether the Who and When pages were
						//visited. We might be displaying an error that can
						//only be resolved by the user visiting those pages
						this._visitedWhoStep = false;
						this._visitedWhenStep = false;
					}
					
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback,
				hasValidationProblemCallback: hasValidationProblemCallback,
				showArchived: this.showArchived
			});
			
			this._whoStep = this._newWhoStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedRoleId = selectedValues.roleId;
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback,
				hasValidationProblemCallback: hasValidationProblemCallback
			});
			
			this._whenStep = this._newWhenStep({
				model: model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues) {
					this._selectedContext = selectedValues.context;
					this._selectedContextId = selectedValues.contextId;
					
					this._wizardPageSelectionChanged();
				}),
				onFocusCallback: onFocusCallback,
				hasValidationProblemCallback: hasValidationProblemCallback
			});
			
			this._whoStep.whenStepCanFinish = dojo.hitch(this._whenStep, this._whenStep.canFinish);
			
			return [this._whichContainerStep, this._whichConfigurationStep, this._whoStep, this._whenStep];
		},
		
		getPrimaryTitle: function() {
			return this.messages.OperationBehaviorCopyConfigurationWizard_12;
		},
		
		/**
		 * Helper function to change the flag that tracks whether an error is
		 * being shown, and notify the current step, so they can change their
		 * presentation if necessary.
		 */
		_changeValidationErrorState: function(showError) {
			this._isShowingValidationError = showError;
			
			var currentStep = this.wizard.contentWidget.getSelectedStep();
			
			currentStep.triggerButtonStateUpdate();
			
			if (currentStep.notifyValidationErrorChanged && dojo.isFunction(currentStep.notifyValidationErrorChanged)) {
				currentStep.notifyValidationErrorChanged(showError);
			}
		},
		
		/**
		 * Validation that is performed when the user makes a change to what's
		 * selected in any wizard page. Show validation warning text and disable
		 * the Finish button based on various criteria.
		 */
		_wizardPageSelectionChanged: function() {
			if (this.wizard === undefined) {
				//A page reported a change (for an initial value) before we've
				//created the wizard. We don't need to validate yet
				return;
			}
			
			var currentStep = this.wizard.contentWidget.getSelectedStep();
			var canFinish = currentStep.canFinish();
			
			var validationErrorMsg = undefined;
			
			if (canFinish || this._isShowingValidationError) {
				//If we can finish or we're showing a message, re-validate to
				//add a message or clear one respectively
				
				var selectedValues = this._getSelectedValues();
				
				if (selectedValues === null || currentStep === this._whichContainerStep) {
					//The user has not entered enough information to be able to
					//validate anything. Clear any errors until more information
					//is entered
					if (this._isShowingValidationError) {
						this._changeValidationErrorState(false);
					}
					this._messageBox.hideMessage();
				} else {
					//Show error if the remote configuration's role or contextId
					//is not known to this process area and the user hasn't been
					//to the appropriate step to select a different one
					if (this._remoteConfigurationRoleIsKnown === false && !this._visitedWhoStep) {
						var roleName = this._remoteConfigurationItemStore.getRoleLabel(this._selectedConfigurationItem);
						validationErrorMsg = dojo.string.substitute(this.messages.OperationBehaviorCopyConfigurationWizard_6, [roleName]);
					} else if (this._remoteConfigurationContextIsKnown === false && !this._visitedWhenStep) {
						var contextName = this._remoteConfigurationItemStore.getContextLabel(this._selectedConfigurationItem);
						validationErrorMsg = dojo.string.substitute(this.messages.OperationBehaviorCopyConfigurationWizard_7, [contextName]);
					}
					
					//Update state of this._isShowingValidationError
					if (this._isShowingValidationError && validationErrorMsg === undefined) {
						this._changeValidationErrorState(false);
					} else if (!this._isShowingValidationError && validationErrorMsg !== undefined) {
						this._changeValidationErrorState(true);
					}
					
					if (validationErrorMsg !== undefined) {
						//We issue a warning instead of an error, because the
						//user is allowed to click through the wizard to resolve
						//the problems we're raising. If there was a
						//show-stopping kind of problem, we would want to use
						//STATUS_TYPES.ERROR for it.
						this._messageBox.showMessage({
							summary: validationErrorMsg,
							type: jazz.ui.MessageBox.STATUS_TYPES.WARNING 
						});
					} else {
						this._messageBox.hideMessage();
					}
				}
			}
		},
		
		/**
		 * Report the selections made by the user to the editor.
		 */
		_onFinish: function() {
			var selectedValues = this._getSelectedValues();
			var copyConfiguration = dojo.hitch(this, function() {
				this.configurationStore.copyConfiguration(selectedValues.operationId, selectedValues.context, selectedValues.contextId, selectedValues.roleId, this._selectedConfigurationItem, this._remoteConfigurationItemStore);
			
				this.wizard.close();
				this._uninitialize();
			});
			var existing = this.configurationStore.getConfiguration(selectedValues.operationId, selectedValues.context, selectedValues.contextId, selectedValues.roleId);
			if (existing === null) {
				copyConfiguration();
			} else {
				this._promptMergeWithExisting(selectedValues, copyConfiguration);
			}
		},
		
		_promptMergeWithExisting: function(selectedValues, onConfirmFunction) {
			var duplicationMessage = this._getConfigurationDuplicationMessage(selectedValues);			
			var mergeMessage = this.messages.OperationBehaviorCopyConfigurationWizard_8;
			
			new com.ibm.team.process.web.ui.internal.admin.projects.PromptDialog({
				message: duplicationMessage + "\n\n" + mergeMessage, //$NON-NLS-1$
				okText: this.messages.OperationBehaviorCopyConfigurationWizard_0,
				okCallback: function() {
					onConfirmFunction();
				}
			});
		},
		
		/**
		 * Returns the values that were selected in the wizard. Some of the
		 * values require calculation, such as the context and contextId which
		 * can vary based on whether the selected operation is a project or team
		 * operation. Since this information is required when processing Finish
		 * and checking for duplicate configurations, it is extracted to a
		 * helper function.
		 */
		_getSelectedValues: function() {
			//Special-case for situation that can arise as user clicks back to
			//the first page and selects a different project/team area after
			//selecting a configuration on the second page, and we try to
			//validate what the user has selected. There is nothing to validate
			//until another configuration is selected, so return null for now.
			if (!this._remoteConfigurationItemStore.isItem(this._selectedConfigurationItem)) {
				return null;
			}
			
			return this.inherited(arguments);
		}
	});
	
	/**
	 * Wizard step where user selects a project/team area to copy a configuration from
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectProcessAreaForCopyStep", //$NON-NLS-1$
			[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep, dijit._Widget], {
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.OperationBehaviorCopyConfigurationWizard_3;
		},
		
		postCreate: function() {
			this.domNode = dojo.create("div", {}); //$NON-NLS-1$
			
			var contents = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(contents, "wizardPageContents"); //$NON-NLS-1$
			
			var onErrorCallback = dojo.hitch(this, function(errorObject) {
				this._errorFetching = true;
				this.triggerButtonStateUpdate();
			});
			
			this._operationBehaviorSelectProcessAreaForCopyWidget = new com.ibm.team.process.web.ui.internal.admin.projects.SelectProcessAreaWidget({
				processAreaItemId: this.model.processAreaItemId,
				editorContext: this.model.editorContext,
				owningApplicationKey: this.model.owningApplicationKey,
				summaryText: this.messages.OperationBehaviorCopyConfigurationWizard_4,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues, doubleClicked) {
					//Capture the process area id that was selected...
					this._selectedProcessAreaId = selectedValues.processAreaId;
					this._selectedProcessAreaUrl = selectedValues.processAreaUrl;
					
					//...and notify the creator of this step that the selection
					//changed 
					this.selectionChangedCallback(selectedValues);
					
					this.triggerButtonStateUpdate();
					
					if (doubleClicked) {
						this.advanceStepIfPossible();
					}
				}),
				onErrorCallback: onErrorCallback,
				filterLabelProvider: dojo.hitch(this, this._provideFilterLabel)
			});
			
			//We want the widget's domNode to be as tall as it can be as
			//contrained by our contents whose height is a fixed value. This way
			//the tree of project/teams in the widget can be as tall as possible
			dojo.style(this._operationBehaviorSelectProcessAreaForCopyWidget.domNode, "height", "100%"); //$NON-NLS-1$ //$NON-NLS-2$
			
			contents.appendChild(this._operationBehaviorSelectProcessAreaForCopyWidget.domNode);
		},
		
		_provideFilterLabel: function(filterType) {
			if (SelectProcessAreaFilterType.PARENT_HIERARCHY === filterType) {
				return this.messages.OperationBehaviorCopyConfigurationWizard_9;
			} else if (SelectProcessAreaFilterType.PROJECT_HIERARCHY === filterType) {
				return this.messages.OperationBehaviorCopyConfigurationWizard_10;
			} else if (SelectProcessAreaFilterType.OTHER_PROJECTS_AND_TEAMS === filterType) {
				return this.messages.OperationBehaviorCopyConfigurationWizard_11;
			}
			return null;
		},
		
		/**
		 * Allow the user to advance if they have selected an operation in the tree
		 */
		canFlipToNextPage: function() {
			return !this._errorFetching && (this._selectedProcessAreaId !== undefined || this._selectedProcessAreaUrl !== undefined);
		},
		
		focusStep: function() {
			this.onFocusCallback();
		},
		
		uninitialize: function(){
			if (this._operationBehaviorSelectProcessAreaForCopyWidget) {
				this._operationBehaviorSelectProcessAreaForCopyWidget.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
	
	/**
	 * Wizard step where user selects a configuration to copy. If necessary,
	 * information about the project or team area that was selected in the
	 * previous step is fetched from the server. This step reports the progress/
	 * errors of this fetching in its own message box, and controls whether the
	 * user can continue based on the results of the fetch.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectConfigurationForCopyStep", //$NON-NLS-1$
			[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep, dijit._Widget], {
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.OperationBehaviorWizard_0;
		},
		
		postCreate: function() {
			this.domNode = dojo.create("div", {}); //$NON-NLS-1$
			
			this._messageBox = new jazz.ui.MessageBox().placeAt(this.domNode);
			
			this._contentsNode = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(this._contentsNode, "wizardPageContents"); //$NON-NLS-1$
		},
		
		canFlipToPreviousPage: function() {
			return !this._initializingRemoteConfigurationStore;
		},
		
		/**
		 * Allow the user to advance if they have selected an operation in the
		 * tree
		 */
		canFlipToNextPage: function() {
			return !this._errorFetching && this._selectedConfiguration !== undefined;
		},
		
		canFinish: function() {
			return this.canFlipToNextPage() && !this.hasValidationProblemCallback();
		},
		
		focusStep: function() {
			//If necessary, load the configurations from the project/team area
			//that was selected in the first page of the wizards, and use the
			//loaded store to create a new widget for the user to select a
			//configuration in
			if (this._remoteConfigurationStore === undefined || this._remoteProcessAreaId !== this._processAreaId || this._remoteProcessAreaUrl !== this._processAreaUrl) {
				this._errorFetching = false;
				this._initializingRemoteConfigurationStore = true;
				this._selectedConfiguration = undefined;
				this.selectionChangedCallback({selectedConfigurationItem: undefined}, null);
				
				//The above assignments should result in Previous, Next, and
				//Finish being disabled while we're initializing
				//the new configuration store
				this.triggerButtonStateUpdate();
				
				//If we've already created a configuration store, uninitialize
				//it and clean up the configuration tree widget that will also
				//be around
				if (this._remoteConfigurationStore) {
					this._cleanupRemoteConfigurationStoreAndWidget();
					
					dojo.empty(this._contentsNode);
				}
				
				//Show Loading message
				this._messageBox.showMessage({
					summary: this.messages.OperationBehaviorCopyConfigurationWizard_1,
					type: jazz.ui.MessageBox.STATUS_TYPES.INFO 
				});
				
				//Fetch the roles and development line id for the selected
				//project asynchronously
				this._fetchProcessData();
				
				//Fetch the contexts for the selected project asynchronously
				this._fetchRemoteContextStore();
				
				//Fetch the behavior configurations. This function will
				//re-schedule itself until the roles are available, or there was
				//an error fetching them. (The configuration store will take
				//care of waiting until the context store is initialized)
				this._fetchRemoteConfigurationStore();
			}
			this.onFocusCallback();
		},
		
		_fetchProcessData: function() {
			if (this._isInParentHierarchy || this._processAreaId === this.model.processAreaItemId) {
				//We're copying from a parent of the process area we're editing,
				//or the process area itself. There's no need to load roles
				this._remoteRolesStore = this.model.rolesStore;
			} else {
				this._loadingProcessData = true;
				
				var srh = new ServiceResponseHandler(this, "_handleProcessDataResponse", "_handleError"); //$NON-NLS-1$ //$NON-NLS-2$
				var funcName = this._processAreaContext === ProcessConstants.EditorContext.TEAM
					? "getTeamAreaByUUIDWithLimitedMembers" //$NON-NLS-1$
					: "getProjectAreaByUUIDWithLimitedMembers"; //$NON-NLS-1$
				ProcessClient[funcName]({
					processAreaItemId: this._processAreaId,
					maxMembers: 0
				}, srh);
			}
		},
		
		_handleProcessDataResponse: function(data) {
			var roles = [];
			if (data.processProviderDTO) {
				roles = data.processProviderDTO.roles;
			} else if (data.processDTO) {
				roles = data.processDTO.roles;
			}
			this._remoteRolesStore = new com.ibm.team.process.web.ui.internal.admin.projects.RolesStore({
				roles: roles
			});
			this._remoteDevelopmentLineId = data.developmentLine
				? data.developmentLine.itemId
				: undefined;
			this._loadingProcessData = false;
		},
		
		_fetchRemoteContextStore: function() {
			if (this._processAreaId === this.model.processAreaItemId
				|| this._isInParentHierarchy && this._processAreaContext === ProcessConstants.EditorContext.TEAM) {
				//We're copying from
				//- the same process area we're editing
				//- a parent team area
				//There's no need to load a context store. If we're copying
				//from a parent project area, we do need to load its store
				//so that items which refer to a development line will be
				//shown
				this._remoteContextStore = this.model.contextStore;
			} else if (this._processAreaUrl !== undefined) {
				// we're copying from a remote process provider, just use the
				// local context store; it will contain all provider iteration
				// types (and providers can't have timeline/iteration behavior)
				this._remoteContextStore = this.model.contextStore;
			} else {
				var args = {
					editorContext: this._processAreaContext,
					processAreaItemId: this._processAreaId,
					errorContext: this,
					errorCallback: this._handleError
				};
				
				if (this._processAreaContext === ProcessConstants.PermissionContext.PROJECT) {
					this._remoteContextStore = new com.ibm.team.process.web.ui.internal.admin.projects.TimelineStore(args);
				} else if (this._processAreaContext === ProcessConstants.PermissionContext.TEAM) {
					if (this._loadingProcessData) {
						// process data needs to be loaded to continue
						setTimeout(dojo.hitch(this, function() {
							this._fetchRemoteContextStore();
						}), 100);
						return;
					}
					args.currentDevelopmentLineItemId = this._remoteDevelopmentLineId;
					this._remoteContextStore = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeStore(args);
				} else {
					throw new Error("Unknown processAreaContext " + this._processAreaContext); //$NON-NLS-1$
				}
			}
		},
		
		_fetchRemoteConfigurationStore: function() {
			if (this._errorFetching) {
				return;
			}
			
			if (this._loadingProcessData) {
				// process data needs to be loaded to continue
				setTimeout(dojo.hitch(this, function() {
					this._fetchRemoteConfigurationStore();
				}), 100);
				return;
			}
			
			var afterConfigurationStoreInitialized = dojo.hitch(this, function() {
				this._messageBox.hideMessage();
				this._initializingRemoteConfigurationStore = false;
				
				this._operationBehaviorSelectConfigurationForCopyWidget = new com.ibm.team.process.web.ui.internal.admin.projects.SelectBehaviorConfigurationWidget({
					model: this.model,
					configurationStore: this._remoteConfigurationStore,
					extensionStore: this.model.extensionStore,
					summaryText: dojo.string.substitute(this.messages.OperationBehaviorCopyConfigurationWizard_5, ["<span class=\"operationBehaviorCopyWhat\">", "</span>"]), //$NON-NLS-1$ //$NON-NLS-2$
					selectionChangedCallback: dojo.hitch(this, function(selectedValues, doubleClicked) {
						this._selectedConfiguration = selectedValues.selectedConfigurationItem;
						this.selectionChangedCallback(selectedValues, this._remoteConfigurationStore, this._remoteContextStore);
						this.triggerButtonStateUpdate();
						
						if (doubleClicked) {
							this.advanceStepIfPossible();
						}
					}),
					showArchived: this.showArchived
				}).placeAt(this._contentsNode, "only"); //$NON-NLS-1$
				
				//Record the process area id/Url that was used to create the
				//store and tree. We will not re-create the store and tree
				//unless a different process area is selected
				this._remoteProcessAreaId = this._processAreaId;
				this._remoteProcessAreaUrl = this._processAreaUrl;
				
				this.triggerButtonStateUpdate();
			});
			
			if (this._processAreaId === this.model.processAreaItemId) {
				//We're copying from the same process area we're editing.
				//There's no need to load a configuration store
				this._remoteConfigurationStore = this.model.configurationStore;
				afterConfigurationStoreInitialized();
			} else {
				var args = {
					processAreaContext: this._processAreaContext,
					owningApplicationKey: this.model.owningApplicationKey,
					extensionStore: this.model.extensionStore,
					rolesStore: this._remoteRolesStore,
					contextStore: this._remoteContextStore
				};
				if (this._processAreaId !== undefined) {
					args.processAreaItemId = this._processAreaId;
				} else {
					args.processProviderUrl = this._processAreaUrl;
				}
				this._remoteConfigurationStore = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorConfigurationStore(args);
				
				this._remoteConfigurationStore.initialize(
					//Success callback
					afterConfigurationStoreInitialized,
					
					//Error callback
					dojo.hitch(this, function(errorObject) {
						this._handleError(errorObject);
					})
				);
			}
		},
		
		_handleError: function(error) {
			this._errorFetching = true;
			this._initializingRemoteConfigurationStore = false;
			
			this._messageBox.showMessage({
				summary: this.messages.OperationBehaviorCopyConfigurationWizard_2,
				errorObject: error
			});
			
			this.triggerButtonStateUpdate();
		},
		
		notifyProcessAreaSelectionChanged: function(processAreaId, processAreaUrl, processAreaContext, isInParentHierarchy, isProcessProvider) {
			this._processAreaId = processAreaId;
			this._processAreaUrl = processAreaUrl;
			this._processAreaContext = processAreaContext;
			this._isInParentHierarchy = isInParentHierarchy;
			this._isProcessProvider = isProcessProvider;
		},
		
		_cleanupRemoteConfigurationStoreAndWidget: function() {
			//If we created the configuration store (if a project/team besides
			//the one we're editing was selected on the first page),
			//uninitialize it
			if (this._remoteConfigurationStore && this._remoteConfigurationStore !== this.model.configurationStore) {
				this._remoteConfigurationStore.uninitialize();
			}
			
			if (this._operationBehaviorSelectConfigurationForCopyWidget) {
				this._operationBehaviorSelectConfigurationForCopyWidget.destroyRecursive();
			}
		},
		
		uninitialize: function() {
			this._cleanupRemoteConfigurationStoreAndWidget();
			if (this._messageBox) {
				this._messageBox.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
})();