/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTreeEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorDialog"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTreeEditor", com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorTree, { //$NON-NLS-1$
		
		/**
		 * Extends OperationBehaviorTree to add a title, an Add button, and a
		 * toolbar to each row in the tree. Supports adding, removing, and
		 * re-ordering behaviors of an operation.
		 * 
		 * @param item {dojo.data.Item} operation item to create a tree for
		 * @param configurationStore {OperationBehaviorConfigurationStore} store
		 *            to make edits in
		 * @param extensionStore {OperationBehaviorExtensionStore} store to read
		 *            from to determine available preconditions/follow-ups, for
		 *            adding
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._actionToolbars = [];
		},
		
		postCreate: function() {
			this.inherited(arguments);
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorTreeEditor"); //$NON-NLS-1$
		},
		
		startup: function() {
			this.inherited(arguments);
			
			this._createHeader();
			
			// keep the title vertically centered next to the header button
			setHeight = dojo.hitch(this, function() {
				dojo.style(this.title, "height", "auto"); //$NON-NLS-1$ //$NON-NLS-2$
				if (this.addButton.domNode.offsetHeight > this.title.offsetHeight) {
					// increase title's height, since its using display:table-
					// cell and vertical-align:middle it will center itself
					dojo.style(this.title, "height", this.addButton.domNode.offsetHeight + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				if (dojo.isIE === 8) {
					// trigger a layout to get title's table-cell to kick in
					dojo.style(this.domNode, "display", "inline-block"); //$NON-NLS-1$ //$NON-NLS-2$
					dojo.style(this.domNode, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
				}
			});
			setHeight();
			this.connect(window, "onresize", setHeight); //$NON-NLS-1$
		},
		
		_createHeader: function() {
			var header = dojo.create("div", null, this.domNode.parentNode, "before"); //$NON-NLS-1$ //$NON-NLS-2$
			dojo.addClass(header, "com-ibm-team-process-web-ui-internal-OperationBehaviorTreeEditorHeader"); //$NON-NLS-1$
			var extension = this.extensionStore.fetchItemByIdentity(this.configurationStore.getValue(this.item, ProcessConstants.ExtensionDTO.ID));
			var preconditionsPossible = this.configurationStore.getPreconditions(this.item).length > 0 || this.extensionStore.getPreconditions(extension).length > 0;
			var followupsPossible = this.configurationStore.getFollowupActions(this.item).length > 0 || this.extensionStore.getFollowupActions(extension).length > 0;
			// title
			this.title = dojo.create("div", { //$NON-NLS-1$
				innerHTML: this._getTitle(preconditionsPossible, followupsPossible),
				"class": "headerTitle j-label" //$NON-NLS-1$ //$NON-NLS-2$
			}, header);
			// add button
			this.addButton = new jazz.ui.toolbar.Button({
				label: this.messages.OperationBehaviorTreeEditor_3,
				title: this._getButtonTooltip(preconditionsPossible, followupsPossible),
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/add_behavior.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				"class": "headerButton", //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, this._add)
			}).placeAt(header, "first"); //$NON-NLS-1$
		},
		
		_getTitle: function(preconditionsPossible, followupsPossible) {
			if (preconditionsPossible && !followupsPossible) {
				return this.messages.OperationBehaviorTreeEditor_0;
			} else if (!preconditionsPossible && followupsPossible) {
				return this.messages.OperationBehaviorTreeEditor_1;
			} else {
				return this.messages.OperationBehaviorTreeEditor_2;
			}
		},
		
		_getButtonTooltip: function(preconditionsPossible, followupsPossible) {
			if (preconditionsPossible && !followupsPossible) {
				return this.messages.OperationBehaviorTreeEditor_4;
			} else if (!preconditionsPossible && followupsPossible) {
				return this.messages.OperationBehaviorTreeEditor_5;
			} else {
				return this.messages.OperationBehaviorTreeEditor_6;
			}
		},
		
		getIconClass: function(item) {
			var cssClass = this.inherited(arguments);
			if (cssClass !== null) {
				cssClass += " toolbarRowIcon"; //$NON-NLS-1$
			}
			return cssClass;
		},
		
		/**
		 * Overridden to create a toolbar for each behavior item node in the
		 * tree, and to keep the up/down buttons disabled as appropriate.
		 */
		updateNode: function(node, previous, next) {
			if (!dojo.isString(node.item) && !node._toolbar) {
				this._createToolbar(node);
			}
			if (node._toolbar) {
				node._toolbar._upButton.set("enabled", previous && !dojo.isString(previous.item)); //$NON-NLS-1$
				node._toolbar._downButton.set("enabled", next && !dojo.isString(next.item)); //$NON-NLS-1$
			}
			this.inherited(arguments);
		},
		
		/**
		 * Creates a toolbar for the given tree node, with buttons to move or
		 * delete its item.
		 */
		_createToolbar: function(node) {
			var toolbar = new jazz.ui.Toolbar();
			dojo.addClass(toolbar.domNode, "treeToolbar"); //$NON-NLS-1$
			node._toolbar = toolbar.placeAt(node.labelNode, "before"); //$NON-NLS-1$
			this._actionToolbars.push(toolbar);
			// up button
			toolbar._upButton = new jazz.ui.toolbar.Button({
				title: this.messages.OperationBehaviorTreeEditor_7,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/up.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(e) {
					dojo.stopEvent(e); // don't select the node
					this._move(node.item, true);
				})
			}).placeAt(toolbar);
			// down button
			toolbar._downButton = new jazz.ui.toolbar.Button({
				title: this.messages.OperationBehaviorTreeEditor_8,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/down.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(e) {
					dojo.stopEvent(e); // don't select the node
					this._move(node.item, false);
				})
			}).placeAt(toolbar);
			// delete button
			toolbar._deleteButton = new jazz.ui.toolbar.Button({
				title: this.messages.OperationBehaviorTreeEditor_9,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/images/delete_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(e) {
					dojo.stopEvent(e); // don't select the node
					this._delete(node.item);
				})
			}).placeAt(toolbar);
			// prevent mouse-down events generated on the toolbar buttons from
			// affecting which tree node appears selected
			dojo.forEach([toolbar._upButton, toolbar._downButton, toolbar._deleteButton], dojo.hitch(this, function(button) {
				this.connect(button.domNode, "onmousedown", function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
				});
			}));
			
			// Intercept TAB key presses and give focus to the toolbar.
			// Otherwise, the toolbar actions are not keyboard accessible.
			// Specifying a tabIndex on the toolbar or its children doesn't
			// work; the user would need to tab through every element on the
			// page which has a non-negative tabIndex before the toolbar would
			// get focus
			this.connect(node.labelNode, "keydown", function(e) { //$NON-NLS-1$
				if (e.keyCode === dojo.keys.TAB) {
					dojo.stopEvent(e);
					
					// Removes the tabIndex from the tree node. Otherwise,
					// typing TAB when the toolbar has focus would give focus
					// back to the tree node, (since the toolbar precedes the
					// label node in the DOM), and users would never be able
					// to tab out of the tree
					node.setFocusable(false);
					
					toolbar.domNode.focus();
				}
			});
		},
		
		_add: function() {
			var id = this.configurationStore.getValue(this.item, ProcessConstants.ExtensionDTO.ID);
			var extension = this.extensionStore.fetchItemByIdentity(id);
			// open a dialog
			new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorDialog({
				item: extension,
				extensionStore: this.extensionStore,
				onOk: dojo.hitch(this, function(behaviorExtension) {
					var behavior = this.configurationStore.newBehavior(this.item, behaviorExtension);
					this.focusNode(this.getNodesByItem(behavior)[0]);
					this._truncateNodes();
				})
			});
		},
		
		_move: function(item, up) {
			var parent = this.configurationStore.getParent(item);
			this.model.getChildren(parent, dojo.hitch(this, function(children) {
				var newIndex, index = 0;
				// find the given items index amongst its parent's children
				for (; index < children.length; index++) {
					if (item === children[index]) {
						break;
					}
				}
				// stepping in the direction we are about to move the item, skip
				// over any items which are not configured, because we need to 
				// move above/below the next configured sibling
				for (newIndex = index + (up ? -1 : 1); 0 <= newIndex < children.length; newIndex += up ? -1 : 1) {
					if (this.model.store.getValue(children[index], ProcessConstants.BehaviorConfigurationDTO.CONFIGURED) === true) {
						break;
					}
				}
				// make the swap
				children.splice(newIndex, 0, children.splice(index, 1)[0]);
				this.configurationStore.setValues(parent, this.model.childrenAttrs[0], children);
			}));
			// "onmouseleave" is not triggered when you move a tree node from
			// under the mouse, so manually remove the hover class
			dojo.removeClass(this.getNodesByItem(item)[0].rowNode, "dijitTreeRowHover"); //$NON-NLS-1$
		},
		
		_delete: function(item) {
			if (this.configurationStore.isNew(item)) {
				this.configurationStore.deleteItem(item);
			} else {
				this.configurationStore.setValue(item, ProcessConstants.BehaviorConfigurationDTO.CONFIGURED, false);
			}
			this._truncateNodes();
		},
		
		/**
		 * Called when a change has been made to the tree that could affect its
		 * height, and thus the existence of a scrollbar, which affects where
		 * labels are truncated.
		 */
		_truncateNodes: function() {
			this.model.getChildren(this.model.root, dojo.hitch(this, function(children) {
				dojo.forEach(children, dojo.hitch(this, function(item) {
					var node = this.getNodesByItem(item)[0];
					if (dojo.isFunction(node.truncate)) {
						node.truncate();
					}
				}));
			}));
		},
		
		uninitialize: function() {
			dojo.forEach(this._actionToolbars, function(toolbar) {
				if (toolbar) {
					toolbar.destroyRecursive();
				}
			});
			delete this._actionToolbars;
			this.inherited(arguments);
		}
		
	});
})();
