/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("dijit._Templated"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.Wizard"); //$NON-NLS-1$
	dojo.require("jazz.ui.wizard.WizardStep"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	
	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var WIZARD_WIDTH = "67em"; //$NON-NLS-1$
	
	/**
	 * Supertype for the wizards to create new and copy existing operation
	 * behavior configurations.
	 * 
	 * The argument to the constructor must be an object with the following
	 * properties:
	 * 
	 * - model	[required]  An object which provides information about the
	 *                      general process model that is not operation-behavior
	 *                      specific. The following fields are required:
	 * 
	 * 		- rolesStore			[required]
	 *          A dojo.data.ItemFileReadStore with roles available to select
	 *          from, with fields id, name, and description
	 * 
	 * 		- contextStore			[required]
	 *          A dojo.data.ItemFileReadStore for iteration types and timelines/
	 *          iterations for the process area
	 * 
	 * 		- configurationStore	[required]
	 *          A dojo.data.ItemFileReadStore which contains operation
	 *          configurations defined
	 * 
	 * 		- extensionStore		[required]
	 *          A dojo.data.ItemFileReadStore which contains categories and
	 *          operations for which operation behavior can be configured
	 * 
	 * 		- editorContext			[required]
	 *          The editor context that the widget is being used in: a value in
	 *          ProcessConstants.EditorContext
	 * 
	 *  	- developmentLineItemId	[optional]
	 *          Required if editorContext is ProcessConstants.EditorContext.TEAM
	 *          A string that is used offer child iterations of the team area's
	 *          selected development line
	 * 
	 *      - showArchived [optional]
	 *      	Whether or not to show archived contexts by default
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizard", null, { //$NON-NLS-1$
		
		constructor: function(args) {
			this.configurationStore = args.model.configurationStore;
			this.extensionStore = args.model.extensionStore;
			this.rolesStore = args.model.rolesStore;
			this.contextStore = args.model.contextStore;
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			
			this._isShowingValidationError = false;
			this.showArchived = args.showArchived || false;
			
			this._messageBox = new jazz.ui.MessageBox();
			dojo.addClass(this._messageBox.domNode, "messageBox"); //$NON-NLS-1$
			
			this.wizard = new jazz.ui.Wizard({
				steps : this.createSteps(args.model),
				width : WIZARD_WIDTH,
				processFinish: dojo.hitch(this, this._onFinish),
				processCancel: dojo.hitch(this, this._onCancel),
				headerNode: this._messageBox.domNode,
				primaryTitle: this.getPrimaryTitle()
			});
			
			dojo.addClass(this.wizard.contentWidget.domNode, "com-ibm-team-process-web-ui-internal-OperationBehaviorWizard"); //$NON-NLS-1$
		},
		
		/**
		 * Subclass-defined function that must return an array of steps,
		 * (instances of jazz.ui.wizard.WizardStep), appropriate for the
		 * wizard.
		 */
		createSteps: function(model) {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		/**
		 * Subclass-defined function that must return a string that will be
		 * used as the title of the wizard.
		 */
		getPrimaryTitle: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		/**
		 * Subclass-defined function that is invoked when the wizard's
		 * Finish button is clicked. It is the responsiblity of the subclass
		 * to call both:
		 * 
		 * this.wizard.close();
		 * this._uninitialize();
		 * 
		 * if desired when their finish handling is complete, to handle
		 * cases where the wizard may not close (the user clicks Cancel in
		 * a prompt, for instance)
		 */
		onFinish: function() {
			throw new Error("subclass must override"); //$NON-NLS-1$
		},
		
		_onFinish: function() {
			this.onFinish();
		},
		
		/**
		 * Returns the values that were selected in the wizard. Some of the
		 * values require calculation, such as the context and contextId which
		 * can vary based on whether the selected operation is a project or team
		 * operation. Since this information is required when processing Finish
		 * and checking for duplicate configurations, it is extacted to a helper
		 * function.
		 */
		_getSelectedValues: function() {
			var operationId = this._selectedOperationId;
			var context;
			var contextId;
			if (ProcessConstants.ExtensionContext.PROJECT === this._selectedOperationContext) {
				//Project operations must apply Always. This will be enforced in
				//the UI by not allowing navigation to the When step when a
				//project operation is selected, but the user could have
				//selected something in the When page and then clicked Previous
				//to select a different operation
				context = this._selectedOperationContext;
				contextId = undefined; //$NON-NLS-1$
			} else {
				context = this._selectedContext || this._selectedOperationContext;
				contextId = this._selectedContextId;
			}
			var roleId = this._selectedRoleId;
			
			return {
				operationId: operationId,
				context: context,
				contextId: contextId,
				roleId: roleId
			};
		},
		
		/**
		 * Just cleanup any created widgets in the wizard or its pages
		 */
		_onCancel: function() {
			this.wizard.close();
			this._uninitialize();
		},
		
		_uninitialize: function() {
			if (this.wizard) {
				this.wizard.destroyRecursive();
				delete this.wizard;
			}
			
			if (this._messageBox) {
				this._messageBox.destroyRecursive();
				delete this._messageBox;
			}
		},
		
		/**
		 * Utility invoked by subclasses. Returns a user-readable string
		 * suitable for error messages which states that a configuration
		 * already exists with the selected operation, role, and context.
		 */
		_getConfigurationDuplicationMessage: function(selectedValues) {
			var configuration = this.configurationStore.getConfiguration(selectedValues.operationId, selectedValues.context, selectedValues.contextId, selectedValues.roleId);
			var operationName = this.configurationStore.getValue(configuration, ProcessConstants.ExtensionDTO.NAME);
			var roleName = this.configurationStore.getRoleLabel(configuration);
			
			var duplicationMessage;
			if (ProcessConstants.BehaviorContext.PROJECT === selectedValues.context
				|| ProcessConstants.BehaviorContext.TEAM === selectedValues.context) {
				duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_4, [operationName, roleName]);
			} else {
				var contextName = this.configurationStore.getContextLabel(configuration);
				if (ProcessConstants.BehaviorContext.ITERATION_TYPE === selectedValues.context) {
					duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_5, [operationName, roleName, contextName]);
				} else {
					duplicationMessage = dojo.string.substitute(this.messages.OperationBehaviorWizard_6, [operationName, roleName, contextName]);	
				}
			}
			return duplicationMessage;
		},
		
		/**
		 * Utility for subclasses that returns an instance of the 'Who' step
		 * where a role is selected, which is common to all operation behavior
		 * wizards.
		 */
		_newWhoStep: function(arguments) {
			return new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhoStep(arguments);
		},
		
		/**
		 * Utility for subclasses that returns an instance of the 'When' step
		 * where a context is selected, which is common to all operation
		 * behavior wizards.
		 */
		_newWhenStep: function(arguments) {
			arguments.showArchived = this.showArchived;
			return new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhenStep(arguments);
		}
	});
	
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep"); //$NON-NLS-1$
	
	/**
	 * Superclass for steps in the operation behavior wizard, to provide
	 * useful utilities and common functions.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep", //$NON-NLS-1$
			[jazz.ui.wizard.WizardStep], {
			
		/**
		 * Either advance to the next page or finish, in that order and if
		 * either action is possible. Useful for handling user actions that
		 * should double as clicking Next or Finish without clicking those
		 * buttons, such as double-clicking on an element in the wizard page.
		 */	
		advanceStepIfPossible: function() {
			if (this.canFlipToNextPage()) {
				this.onNext();
			} else if (this.canFinish()) {
				this.onFinish();
			}
		}
	});
	
	/**
	 * Wizard step where user selects what role this configuration will apply to
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhoStep", //$NON-NLS-1$
			[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep, dijit._Widget], {
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.OperationBehaviorWizard_1;
		},
		
		postCreate: function() {
			this.domNode = dojo.create("div", {}); //$NON-NLS-1$
			
			var contents = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(contents, "wizardPageContents"); //$NON-NLS-1$
			
			this._operationBehaviorSelectRoleWidget = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectRoleWidget({
				model: this.model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues, doubleClicked) {
					//See if a role was selected, so we know whether to enable
					//Next...
					this._selectedRoleId = selectedValues.roleId;
					
					//...and then tell the creator of this step that the
					//selection changed 
					this.selectionChangedCallback(selectedValues);					
					
					this.triggerButtonStateUpdate();
					
					if (doubleClicked) {
						this.advanceStepIfPossible();
					}
				})
			});
			
			contents.appendChild(this._operationBehaviorSelectRoleWidget.domNode);
		},
		
		/**
		 * Allow the user to proceed if a role is selected in the list.
		 */
		canFlipToNextPage: function() {
			return this._selectedRoleId != undefined && !this._projectOperationSelected;
		},
		
		/**
		 * Allow the user to finish if there are no validation problems and
		 * the When step is able to finish.
		 */
		canFinish: function() {
			return !this.hasValidationProblemCallback() && (!this.whenStepCanFinish || this.whenStepCanFinish());
		},
		
		/**
		 * Overriden from jazz.ui.wizard.WizardStep
		 */
		focusStep: function() {
			//When this page is displayed, position the roles list to be as
			//large as it can be, as constrained by the size of the dialog and
			//the height of the content underneath the roles list.
			this._operationBehaviorSelectRoleWidget.resizeRolesList();
			this.onFocusCallback(this);
		},
		
		/**
		 * Called by the wizard controller when an operation is selected in the first page of the wizard.
		 * When a project operation is selected, the user should not be allowed to proceed to configure the
		 * iteration type/iteration for this configuration.
		 */		
		notifyOperationSelectionChanged: function(selectedValues) {
			this._projectOperationSelected = ProcessConstants.ExtensionContext.PROJECT === selectedValues.operationContext;
		},
		
		/**
		 * If possible, select the configuration's role in the 'select role' page as a default.
		 */
		notifyRemoteConfigurationChanged: function(remoteConfigurationItem, remoteConfigurationItemStore) {
			var roleId = remoteConfigurationItemStore.getValue(remoteConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.ROLE_ID);
			return this._operationBehaviorSelectRoleWidget.selectRoleIfAvailable(roleId);
		},
		
		uninitialize: function(){
			if (this._operationBehaviorSelectRoleWidget) {
				this._operationBehaviorSelectRoleWidget.destroyRecursive();
			}
		}
	});
	
	/**
	 * Wizard step where user selects when this configuration will apply, as an
	 * iteration type or specific iteration or timeline.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWhenStep", //$NON-NLS-1$
			[com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorWizardStep, dijit._Widget], {
				
		constructor: function(args) {
			dojo.mixin(this, args);
			
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.stepTitle = this.messages.OperationBehaviorWizard_2;
			
			this._operationBehaviorSelectContextWidget = new com.ibm.team.process.web.ui.internal.admin.projects.OperationBehaviorSelectContextWidget({
				model: this.model,
				selectionChangedCallback: dojo.hitch(this, function(selectedValues, doubleClicked) {
					//See if an iteration type or timeline/iteration was
					//selected, so we know whether to enable Next...
					this._selectedContext = selectedValues.context;
					this._selectedContextId = selectedValues.contextId;
					
					//... and then tell the creator of this step that the
					//selection changed 
					args.selectionChangedCallback(selectedValues);
					
					this.triggerButtonStateUpdate();
					
					if (doubleClicked) {
						this.advanceStepIfPossible();
					}
				}),
				showArchived: this.showArchived
			});
		},
		
		postCreate: function() {
			this.domNode = dojo.create("div", {}); //$NON-NLS-1$
			
			var contents = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(contents, "wizardPageContents"); //$NON-NLS-1$
			
			contents.appendChild(this._operationBehaviorSelectContextWidget.domNode);
		},
		
		focusStep: function() {
			this.onFocusCallback(this);
		},
		
		canFlipToNextPage: function() {
			//Overriden to return false since this is the last page of the
			//wizard. We want the code in the doubleClick condition in
			//selectionChangedCallback above to be activated if this function
			//is changed, (if more pages are added to the wizard)
			return false;
		},
		
		/**
		 * The user may finish if the radio to always apply the configuration is
		 * selected, or if a selection has been made in the iteration type or
		 * iteration section.
		 */
		canFinish: function() {
			return !this.hasValidationProblemCallback() && (
				this._selectedContext === undefined ||
				this._selectedContextId !== undefined
			);
		},
		
		/**
		 * If possible, select the configuration's context in the in the 'select
		 * context' page as a default.
		 */
		notifyRemoteConfigurationChanged: function(remoteConfigurationItem, remoteConfigurationItemStore, remoteProcessAreaContext, remoteContextStore) {
			var context = remoteConfigurationItemStore.getValue(remoteConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT);
			var contextId = remoteConfigurationItemStore.getValue(remoteConfigurationItem, ProcessConstants.BehaviorConfigurationDTO.CONTEXT_ID);
			
			var foundContextId = this._operationBehaviorSelectContextWidget.selectContextIfAvailable(context, contextId);
			
			//Special-case: if we're trying to select a development line and
			//we're editing a team area, we do not want to raise an error if
			//the development line is the one the team is configured to use.
			//The default selection of 'Always' for the configuration is
			//appropriate
			if (remoteContextStore && !foundContextId && ProcessConstants.BehaviorContext.DEV_LINE === context && this.model.developmentLineItemId) {
				remoteContextStore.fetch({
					query: {
						contextId: contextId
					},
					queryOptions: {
						deep: true
					},
					onItem: dojo.hitch(this, function(item) {
						foundContextId = remoteContextStore.getValue(item, "id") === this.model.developmentLineItemId; //$NON-NLS-1$
					})
				});
			}
			
			return foundContextId;
		},
		
		uninitialize: function(){
			if (this._operationBehaviorSelectContextWidget) {
				this._operationBehaviorSelectContextWidget.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
})();
