/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsActionsTree"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel"); //$NON-NLS-1$	
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	
	var MAX_TREE_HEIGHT = 450;
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	var MEMENTO_SELECTED_ACTION = "PermissionsActionsTree#selectedAction"; //$NON-NLS-1$
	
	/**
	 * Tree used in the show by operation presentation of the permissions tab which shows
	 * all categories, actions and operations .
	 */
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsActionsTree", [dijit._Widget], { //$NON-NLS-1$
		
		/**
		 * Constructor is called with the following fields in args
		 * store               a store containing all the extensions
		 * workingCopy         instance of PermissionsWorkingCopy shared among
		 *                     permissions editor widgets		
		 * focusChangedCallout a callout function that takes an item in the store as a parameter
		 * headingText         text to be used in the title area
		 * processAreaStore    the store of permissions scopes for the process
		 *                     area
		 * editorState         Memento object that contains information that
		 *                     was saved before the editor was refreshed. See
		 * 	                   function saveState()
		 * initialTimePeriod   the time period for which to initially show
		 *                     permissions
		 */
		constructor: function(args) {
			var origOnOpen = args.onOpen;
			this.onOpen = dojo.hitch(this, function(item, node) {
				this._shiftChildren(node);
				
				//Add description icons to child nodes that appear on hover
				//Done with setTimeout to ensure that the tree has been
				//created and assigned to this._tree first.
				setTimeout(dojo.hitch(this, function() {
					dojo.forEach(node.getChildren(), dojo.hitch(this, function(childNode) {
						this._addDescriptionIcon(childNode); //$NON-NLS-1$ //$NON-NLS-1$
					}));
				}));
				
				if (origOnOpen) {
					origOnOpen(item, node);
				}
			});
			
			this._headingText = args.headingText || ""; //$NON-NLS-1$
		},
		
		/**
		 * Create the tree
		 */
		postCreate: function() {
			this.inherited(arguments);			
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsActionsTree"); //$NON-NLS-1$			
			this._tree = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
				model: new com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel({
					store: this.store,
					workingCopy: this.workingCopy,
					processAreaStore: this.processAreaStore,
					editorContext: this.editorContext,
					timePeriod: this.initialTimePeriod
				}),
				showRoot: false,
				labelAttr: ProcessConstants.ExtensionDTO.NAME,
				childrenAttr: [ProcessConstants.ExtensionDTO.ITEMS],
				getIconClass: function(item, opened) {
					return null;
				},
				focusChanged: dojo.hitch(this, function(element) {
					this._selectedItem = element.item;
					this.focusChangedCallout(this._selectedItem);
				}),
				persist: false,
				onOpen: dojo.hitch(this, this.onOpen),
				onClick: dojo.hitch(this, this.onClick)
			});
			
			this.connect(this._tree, "onLoad", "_onLoad"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._tree.model, "onChildrenChange", "_onItemChildrenChange"); //$NON-NLS-1$ //$NON-NLS-2$				
			this._headerDiv = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			this._createSectionHeading();
			
			this._noFilterResultsDiv = dojo.create("div", { //$NON-NLS-1$
				'class': "hidden noFilterResultsMessage j-label", //$NON-NLS-1$ //$NON-NLS-2$
				innerHTML: this.messages.PermissionsByActionEditor_8
			}, this.domNode);
			
			this._treeDiv = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			dojo.addClass(this._treeDiv, "actionsTreeContainer"); //$NON-NLS-1$
			
			this._treeDiv.appendChild(this._tree.domNode);
			this.resized();
		},
		
		/**
		 * Called when the container is resized
		 */
		resized: function() {
			setTimeout(dojo.hitch(this, function() {
				//Make sure we haven't been destroyed, (this function may be
				//called when parent containers are destroyed)
				if (this.domNode) {
					this._sectionHeading.layout();
					var parentBox = dojo.marginBox(this.domNode);
					var headingBox = dojo.marginBox(this._sectionHeading.domNode);
					var availableTreeHeight = parentBox.h - headingBox.h;
					if (availableTreeHeight > MAX_TREE_HEIGHT) {
						availableTreeHeight = MAX_TREE_HEIGHT;
					}
					if (availableTreeHeight > 0) {
						dojo.style(this._treeDiv, "height", availableTreeHeight + "px"); //$NON-NLS-2$ //$NON-NLS-1$
					}
				}
			}), 0);
		},
		
		timePeriodSelected: function(scopeItem) {
			if (scopeItem !== null) {
				if (this._tree.model.setTimePeriod(scopeItem)) {
					//If different items are now filtered/not filtered, we need to
					//re-apply the text filter to show appropriate items and
					//display the 'no matching items' message appropriately
					this._refreshTextFiltering(true);
				}
				this._ensureVisibleItemSelected();
			}
		},
		
		/**
		 * Called by Permissions editor when it is being cleared for a refresh
		 * or state. Updates a memento with any state that should be persisted
		 * after the editor is re-loaded, such as the selected action.
		 */
		saveState: function(editorState) {
			if (this._selectedItem) { 
				var id = this._tree.model.store.getValue(this._selectedItem, ProcessConstants.ExtensionDTO.ID);
				if (!this.workingCopy.isCategory(id)) {
					editorState[MEMENTO_SELECTED_ACTION] = id;
				}
			}
		},
		
		/**
		 * If a node was selected that is now filtered out, clear the selection.
		 * Otherwise, if selected node is still visible, ensure that it is still
		 * selected. Filtering and expanding to reveal filtered items could have
		 * caused it to lose the 'selected' treatment. 
         * Returns a deferred.
		 */
		_ensureVisibleItemSelected: function() {
			var paths = [];
			try {
				if (this._selectedItem) {
					var nodeInTree = function(node, treeElement) {
						var parentElement = node.parentElement;
						while (parentElement) {
							if (parentElement === treeElement) {
								return true;
							}
							parentElement = parentElement.parentElement;
						}
						return false;
					};
					var nodes = this._tree.getNodesByItem(this._selectedItem);
					// This can return nodes which have been deleted, filter out any nodes that
					// are in the process of deletion. Defect 272383
					nodes = dojo.filter(nodes, function(node) {
						return (node !== undefined && node.domNode !== null);
					});
					if (nodes.length === 1 && nodeInTree(nodes[0].domNode, this._tree.domNode)) {
						paths = this._getParentPaths([this._selectedItem], true);						
					} else {
						this._selectedItem = undefined;
						this.focusChangedCallout(null);
					}
				}
			} catch(e) {
				if (djConfig.isDebug) {
					console.log("Exception thrown filtering permissions by operation"); //$NON-NLS-1$
				}
			}
			return this._tree.set("paths", paths); //$NON-NLS-1$
		},
		
		_createSectionHeading: function() {
			this._sectionHeading = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading({
				headingText : this._headingText
			});
			this._headerDiv.appendChild(this._sectionHeading.domNode);
			
			this._headingToolbar = new jazz.ui.Toolbar();
			this._sectionHeading.addWidget(this._headingToolbar);
			
		    this._expandAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByActionEditor_9,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					this._expandAll();
				})
			});
			this._headingToolbar.addChild(this._expandAllButton);
			
			this._collapseAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByActionEditor_10,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					this._collapseAll();
				})
			});
			this._headingToolbar.addChild(this._collapseAllButton);
		    
		    this._operationTextFilterBox = new jazz.ui.FilterBox({
				onChange: dojo.hitch(this, function(value) {
					this._refreshTextFiltering(false);
				})
			});
			this._sectionHeading.addWidget(this._operationTextFilterBox);
		},

		_expandAll: function() {
			this._tree.expandAll();
		},
		
		_collapseAll: function() {
			this._tree.collapseAll();
		},
		
		_refreshTextFiltering: function(forceUpdate) {
			//Processing of the filter change is delayed 200 ms to account
			//for fast typing; responding to every key stroke would be extra
			//work and there can be errors if nodes are expanded to reveal
			//matching items at the same time as nodes are shown/hidden when
			//more text is entered
			if (this._refreshTextFilteringTimeout !== undefined) {
				clearTimeout(this._refreshTextFilteringTimeout);
			}
			this._refreshTextFilteringTimeout = setTimeout(dojo.hitch(this, function() {
				delete this._refreshTextFilteringTimeout;
				// If the tree is being expanded from a previous filter operation wait and retry
				// until the expansion is complete - Defect 272383
				if (this._filterInProgress === true) {
					this._refreshTextFiltering(forceUpdate);
					return;
				}
				this._filterInProgress = true;	
				this._tree.set("paths", []).then(dojo.hitch(this, function() { //$NON-NLS-1$   
					this._tree.model.setFilterText(this._operationTextFilterBox.value, forceUpdate);
					var isEmptyQuery = dojo.string.trim(this._operationTextFilterBox.value).length === 0;
					var matchingItems = this._tree.model.getTextFilterMatches();

					//Expand tree nodes so that all matching items are revealed
					//If the filter text is empty expand each category								
					var paths = isEmptyQuery ? this._getInitialExpansionStatePaths()
							: this._getParentPaths(matchingItems, true);

					this._tree.set("paths", paths).then(dojo.hitch(this, function() { //$NON-NLS-1$
						this._ensureVisibleItemSelected().then(dojo.hitch(this, function() { //$NON-NLS-1$	
							delete this._filterInProgress;
							//Show/hide the message that there are no results as appropriate
							var resultsFound = isEmptyQuery || matchingItems.length !== 0;
							dojo.toggleClass(this._noFilterResultsDiv, "hidden", resultsFound);  //$NON-NLS-1$
							dojo.toggleClass(this._treeDiv, "hidden", !resultsFound);  //$NON-NLS-1$

							//If categories are shown that were previously hidden, new
							//tree nodes may be created for them. Just as when the tree
							//was loaded, we need to add a style to each category and
							//ensure that it's expanded
							this._styleCategories();
							this.resized();						
						}));
					}));
				}));
			}), 200);
		},
		
		/*
		 * Return an array of paths suitable for use in tree.set("paths", path)
		 * If includeSelf is true each path includes the item and its ancestors, 
		 * otherwise only the ancestors appear in the path.
		 */
		_getParentPaths: function(items, includeSelf) {
			var addParentsToPath = dojo.hitch(this, function(item, path) {
				if (item !== null) {
					var parent = null;
					var query = {};
					query[ProcessConstants.ExtensionDTO.ID] = this._tree.model.store.getValue(item, com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy.PROP_PARENT_ID);
					this._tree.model.store.fetch({
						query: query,
						queryOptions: {
							deep: true
						},
						onItem: function(item) {
							parent = item;
						}
					});
					addParentsToPath(parent, path);
					if (parent !== null) {
						path.push(parent);
					}
				}
			});
			var paths = [];
			dojo.forEach(items, dojo.hitch(this, function(theItem) {
				var path = [this._tree.model.root];
				addParentsToPath(theItem, path);
				if (includeSelf === true) {
				    path.push(theItem);
			    }
				paths.push(path);
			}));
			return paths;
		},
		
		/*
		 * Expand all categories by setting the paths attribute of the tree
		 * to the first operation of each category.
		 *  
		 * The paths attribute is used to reveal the children of categories 
		 * rather than calling _TreeNode.expand() which does not work 
		 * correctly if the node has never been expanded before.
		 * 
		 * Following the expansion of categories, the action that was
		 * selected before the previous refresh or save is selected, if
		 * applicable and if the action is still in the tree.
		 */
		_setInitialExpansionState:function() {
			var pathsToSelect = this._getInitialExpansionStatePaths();
			this._tree.set("paths", pathsToSelect).then(dojo.hitch(this, function() { //$NON-NLS-1$
				this._tree.set("paths", []); //$NON-NLS-1$
			    
			    //If an action was selected before the editor was refreshed, select it
				//by default
				if (this.editorState && this.editorState[MEMENTO_SELECTED_ACTION] !== undefined) {
					var query = {};
					query[ProcessConstants.ExtensionDTO.ID] = this.editorState[MEMENTO_SELECTED_ACTION];
					this.store.fetch({
						query: query,
						queryOptions: {
							deep: true
						},
						onItem: dojo.hitch(this, function (item) {
							var pathsArray = this._getParentPaths([item], false);
							var path = pathsArray[0];
							path.push(item);
							this._tree.set("path", path).then(dojo.hitch(this, function() { //$NON-NLS-1$
								this._selectedItem = item;
								this.focusChangedCallout(this._selectedItem);
							}));
						})
					});
				}
			}));
		},
		
		_getInitialExpansionStatePaths:function() {
			var pathsToSelect = [];
			this._tree.model.getChildren(this._tree.model.root, dojo.hitch(this, function(categoryItems) {	
				dojo.forEach(categoryItems, dojo.hitch(this, function(categoryItem) {		
					this._addChildOperationToPath(categoryItem, pathsToSelect);
				}));
			}));
			return pathsToSelect;
		},
		
		/*
		 * Add the first operation in the category to the paths
		 */
		_addChildOperationToPath: function(categoryNode, paths) {
			var childOperations = this.store.getValues(categoryNode, ProcessConstants.ExtensionDTO.ITEMS);
			if (childOperations.length > 0) {
			    var path = [this._tree.model.root, categoryNode, childOperations[0]];
			    paths.push(path);
		    }
		},
		
		_onLoad: function() {
		    this._styleCategories();
            this._setInitialExpansionState();
            this.resized();
		},
		
		_styleCategories: function() {
		    this._tree.model.getChildren(this._tree.model.root, dojo.hitch(this, function(categoryItems) {
		        for (var i = 0; i < categoryItems.length; i++) {
                    var item = categoryItems[i];
                    var categoryNode = this._tree.getNodesByItem(item)[0];
                    categoryNode._isCategoryNode = true;
                    dojo.addClass(categoryNode.rowNode, "categoryNode"); //$NON-NLS-1$
                    dojo.style(categoryNode.expandoNode, "display", "none"); //$NON-NLS-1$ //$NON-NLS-2$
                    
                    //Disable the ability to collapse category nodes. The following
                    //results in the node not collapsing via keyboard actions or the
                    //Collapse All button.
                    categoryNode.collapse = function() {};   
                    this._shiftChildren(categoryNode);
                }
            }));  
		},
		
		_onItemChildrenChange: function(/*dojo.data.Item*/ parent, /*dojo.data.Item[]*/ newChildrenList) {
			var node = this._tree.getNodesByItem(parent)[0];
			this._shiftChildren(node);
		},
		
		/**
		 * Indent all operations to level 0, (so that they appear as top-level
		 * nodes in the tree.
		 */
		_shiftChildren: function(node) {
		    if (node._isCategoryNode) {
    			dojo.forEach(node.getChildren(), dojo.hitch(this, function(childNode) {
    				childNode.set("indent", 0); //$NON-NLS-1$
    			}));
    		}
		},
		
		_addDescriptionIcon: function(node) {
			var description = this._tree.model.getDescription(node.item);
			if (null !== description && undefined !== description && undefined === node._descriptionIcon) {
				description = dojo.string.trim(description);
				if (0 !== description.length) {
		    		node._descriptionIcon = dojo.create("a", { //$NON-NLS-1$
		    			title: description,
		    			href: "#" //$NON-NLS-1$
		    		}, node.labelNode);
		    		dojo.addClass(node._descriptionIcon, "descriptionIcon"); //$NON-NLS-1$
		    		
		    		var img = dojo.create("img", { //$NON-NLS-1$
		    			src: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/info_ovr.gif"), //$NON-NLS-1$ //$NON-NLS-2$
		    			title: description
		    		}, node._descriptionIcon);
		    		
		    		this.connect(node._descriptionIcon, "onclick", function(event) { //$NON-NLS-1$
		    			dojo.stopEvent(event);
		    		});
		    	}
	    	}
		},
		
		uninitialize: function() {
			this.inherited(arguments);	
			if (this._sectionHeading) {
				this._sectionHeading.destroyRecursive();
				delete this._sectionHeading;
			}
			if (this._tree) {
				this._tree.destroyRecursive();
				delete this.tree;
			}
		}
	});
})();
