/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleEditor"); //$NON-NLS-1$
	
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsRolesModel"); //$NON-NLS-1$)
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsWorkingCopy"); //$NON-NLS-1$)
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore"); //$NON-NLS-1$)
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForRoleTable"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	dojo.require("dojo.i18n"); //$NON-NLS-1$

	dojo.require("jazz.ui.FilterBox"); //$NON-NLS-1$	
	dojo.require("jazz.ui.Toolbar"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.Button"); //$NON-NLS-1$
	dojo.require("jazz.ui.toolbar.TwoStateButton"); //$NON-NLS-1$

	dojo.requireLocalization("com.ibm.team.process.web.ui.internal", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	var PermittedType = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsStore.PermittedType;
	
	var MAX_ROLES_LIST_HEIGHT = 450;
	
	var FILTERING_DELAY = 400;
	
	/**
	 * Sub-editor in the Permissions tab that is displayed when the option to
	 * show permissions by role is selected. Presents a list of roles and a
	 * tree of operations/actions where the permissions for the selected role
	 * can be edited.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleEditor", [dijit._Widget], { //$NON-NLS-1$
		
		/**
		 * @param {Object} An object with the following fields:
		 * 
		 * processAreaEditor	Instance of AbstractProcessAreaEditor that hosts
		 * 						the Permissions tab
		 * 
		 * processAreaStore		Store of artifacts related to project or team,
		 * 						used for time scopes such as timelines and
		 * 						iteration types
		 * 
		 * workingCopy			Instance of PermissionsWorkingCopy that can be
		 * 						consulted to load permissions and permissions
		 * 						extensions and update permissions
		 * 
		 * editorState			Memento object that contains information that
		 * 						was saved before the editor was refreshed. See
		 * 						function saveState()
		 * 
		 * roleSelectionChangedCallback Function that accepts a role item that
		 * 								will be invoked when the current
		 * 								selected role for the permissions
		 * 								editor changes.
		 */
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		/**
		 * This widget follows the structure required for sub-editors of the
		 * Permissions editor: two fields "firstColumn" and "secondColumn"
		 * that are widgets. Each has a field "initialWidth" which is used
		 * for their initial layout.
		 * 
		 * See function _createSubEditor in PermissionsEditor for how these
		 * fields are incorporated.
		 */
		postCreate: function() {
			var defaultSelectedRole;
			
			this.firstColumn = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleRolesColumn({
				messages: this.messages,
				workingCopy: this.workingCopy,
				rolesStore: this.processAreaEditor.getRolesStore(),
				processAreaEditor: this.processAreaEditor,
				processAreaStore: this.processAreaStore,
				editorState: this.editorState,
				roleSelectionChangedCallback: function(selectedRole) {
					defaultSelectedRole = selectedRole;
				},
				initialWidth: "34%" //$NON-NLS-1$
			});
			
			this.secondColumn = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRolePermissionsColumn({
				messages: this.messages,
				workingCopy: this.workingCopy,
				rolesStore: this.processAreaEditor.getRolesStore(),
				processAreaStore: this.processAreaStore,
				processAreaEditor: this.processAreaEditor,
				selectedRole: defaultSelectedRole,
				initialWidth: "65%" //$NON-NLS-1$
			});
			
			this._selectedRole = defaultSelectedRole;
			this.roleSelectionChangedCallback(this._selectedRole);
			this.firstColumn.roleSelectionChangedCallback = dojo.hitch(this, function(selectedRole) {
				this._selectedRole = selectedRole;
				
				//Notify consumer that selected role changed
				this.roleSelectionChangedCallback(this._selectedRole);
				
				//Notify Permissions table column that selected role changed
				this.secondColumn.roleSelectionChangedCallback(this._selectedRole);
			});
		},
		
		//Read by PermissionsEditor when the section to select a time
		//period is displayed
		containerClassWhenShowingTimeframe: "permissionsByRolePermissionsContainerWhenTimeframesShown", //$NON-NLS-1$
		
		/**
		 * Will be called by Permissions editor when a time period, including
		 * "Always", is selected to edit permissions for. The passed item must
		 * be an item in the store processAreaStore that was passed to this
		 * widget's constructor. 
		 */
		timePeriodSelected: function(scopeItem) {
			this.firstColumn.timePeriodSelected(scopeItem);
			this.secondColumn.timePeriodSelected(scopeItem);
		},
		
		/**
		 * Will be called by Permissions editor when the configured permissions
		 * per role and time period have been loaded, i.e. functions in the
		 * working copy like getConfiguredRoles() are ready to be called.
		 */
		configuredScopesLoaded: function() {
			this.firstColumn.configuredScopesLoaded();
		},
		
		/**
		 * Called by enclosing tab PermissionsEditor when either:
		 * 
		 * 1. The Permissions editor is navigated to and this sub-editor
		 *    is the active one
		 * 2. This sub-editor is made the active one in the Permissions editor
		 * 
		 * The sub-editor will react to changes made outside of itself or
		 * of the Permissions editor, (eg. roles added, removed, ...), in ways
		 * that would not be appropriate immediately when the changes were
		 * made, (eg. because loading from the server could be required).
		 */
		editorShown: function() {
			this.firstColumn.editorShown();
			
			this.roleSelectionChangedCallback(this._selectedRole);
		},
		
		/**
		 * Called by Permissions editor when it is being cleared for a refresh
		 * or state. Updates a memento with any state that should be persisted
		 * after the editor is re-loaded, such as the selected role.
		 */
		saveState: function(editorState) {
			this.firstColumn.saveState(editorState);
		},
		
		startLoading: function() {
			this.firstColumn._startLoading();
		},
		
		stopLoading: function() {
			this.firstColumn._stopLoading();
		}
		
		//There is no need to override uninitialize to destroy the columns.
		//They are added to containers that destroy them on uninitialization
	});
	
	/**
	 * The maximum number of roles displayed in the list, if space permits.
	 * Users will be able to scroll within the list to see any remaining roles.
	 */
	var MAX_ROLES_TO_DISPLAY = 20;
	
	var MEMENTO_SELECTED_ROLE = "PermissionsByRoleRolesColumn#selectedRole"; //$NON-NLS-1$
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRoleRolesColumn", [dijit._Widget], { //$NON-NLS-1$
		
		constructor: function(args) {
			this._selectedTimePeriod = args.processAreaStore.alwaysRootItem;
			this._configuredScopesLoaded = false;
		},
		
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsByRoleRolesColumn"); //$NON-NLS-1$
			
			this._createSectionHeading();
			
			this._rolesList = new com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree({
				model: new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsRolesModel({
					store: this.rolesStore,
					timePeriod: this._selectedTimePeriod,
					workingCopy: this.workingCopy
				}),
				showRoot: false,
				getIconClass: function(item) {
					return null;
				},
				getLabelClass: dojo.hitch(this, function(item) {
					if (this._configuredScopesLoaded && this._rolesList.model.isConfigured(item) && !dojo.hasClass(this._filterConfiguredRolesButton.domNode, "hidden")) { //$NON-NLS-1$
						return "configuredRoleLabel"; //$NON-NLS-1$
					}
				}),
				focusChanged: dojo.hitch(this, function(node) {
					this._selectedRole = node.item;
					this.roleSelectionChangedCallback(this._selectedRole);
				})
			});
			this._rolesListContainer = dojo.create("div", {'class': "rolesTree"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._rolesListContainer.appendChild(this._rolesList.domNode);
			
			//If a role was selected before the editor was refreshed, select it
			//by default
			if (this.editorState && this.editorState[MEMENTO_SELECTED_ROLE] !== undefined) {
				var query = {};
				query[ProcessConstants.ProcessRoleDTO.ID] = this.editorState[MEMENTO_SELECTED_ROLE];
				this._rolesList.model.store.fetch({
					query: query,
					onComplete: dojo.hitch(this, function(items) {
						if (items.length === 1) {
							this._selectedRole = items[0];
							this._rolesList.set("selectedItem", items[0]); //$NON-NLS-1$
							this.roleSelectionChangedCallback(this._selectedRole);
						}
					})
				});
			}
			
			if (this._selectedRole === undefined) {
				this._selectFirstRole();				
			}
			
			this.resized();
			
			this._noCustomizedRolesMessage = dojo.create("div", {'class': "hidden j-label noRolesCustomizedMessage"}, this._rolesListContainer); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._noCustomizedRolesMessage.appendChild(document.createTextNode(this.messages.PermissionsByRoleEditor_14));
			
			this.connect(this.rolesStore, "onSet", this._roleChanged); //$NON-NLS-1$
			this.connect(this._rolesList.model, "onChildrenChange", this._ensureRoleSelected); //$NON-NLS-1$
		},
		
		editorShown: function() {
			this._ensureRoleSelected();
		},
		
		resized: function() {
			setTimeout(dojo.hitch(this, function() {
				//Make sure we haven't been destroyed, (this function may be
				//called when parent containers are destroyed)
				if (this.domNode) {
					this._sectionHeading.layout();
					
					var parentBox = dojo.marginBox(this.domNode);
					var headingBox = dojo.marginBox(this._sectionHeading.domNode);
					var availableRolesListHeight = parentBox.h - headingBox.h;
					if (availableRolesListHeight > MAX_ROLES_LIST_HEIGHT) {
						availableRolesListHeight = MAX_ROLES_LIST_HEIGHT;
					}
					if (availableRolesListHeight > 0) {
						dojo.style(this._rolesListContainer, "height", availableRolesListHeight + "px"); //$NON-NLS-1$ //$NON-NLS-2$
					}
				}
			}), 0);
		},
		
		timePeriodSelected: function(scopeItem) {
			this._selectedTimePeriod = scopeItem;
			var canFilterConfiguredRoles = this.workingCopy.isChildArea() || (scopeItem !== null && scopeItem !== this.processAreaStore.alwaysRootItem);
			dojo.toggleClass(this._filterConfiguredRolesButton.domNode, "hidden", !canFilterConfiguredRoles); //$NON-NLS-1$
			this._rolesList.model.setFilterConfigured(this._filterConfiguredRolesButton.pressed && canFilterConfiguredRoles);
			
			//Let other code that's reacting to the time period selection
			//execute before this; we want the code that reveals/hides roles to
			//be last
			setTimeout(dojo.hitch(this, function() {
				this._rolesList.model.setTimePeriod(this._selectedTimePeriod);
				
				var filterTooltipText;
				if (this._selectedTimePeriod === null || this._selectedTimePeriod === this.processAreaStore.alwaysRootItem) {
					filterTooltipText = this.messages.PermissionsByRoleEditor_11;
				} else {
					var selectedContext = this.processAreaStore.getValue(this._selectedTimePeriod, "enabledContext"); //$NON-NLS-1$
					var scopeLabel = this.processAreaStore.getValue(this._selectedTimePeriod, "name"); //$NON-NLS-1$
					if (ProcessConstants.PermissionContext.ITERATION_TYPE === selectedContext) {
						filterTooltipText = dojo.string.substitute(this.messages.PermissionsByRoleEditor_12, [scopeLabel]);
					} else {
						filterTooltipText = dojo.string.substitute(this.messages.PermissionsByRoleEditor_13, [scopeLabel]);
					}
				}
				this._filterConfiguredRolesButton.set("title", filterTooltipText); //$NON-NLS-1$
			}));
		},
		
		configuredScopesLoaded: function() {
			//Refresh the model, which may result in some roles becoming bold
			this._configuredScopesLoaded = true;
			this._rolesList.model.refresh();
		},
		
		saveState: function(editorState) {
			if (this._selectedRole) {
				editorState[MEMENTO_SELECTED_ROLE] = this._rolesList.model.store.getValue(this._selectedRole, ProcessConstants.ProcessRoleDTO.ID);
			}
		},
		
		_createSectionHeading: function() {
			this._sectionHeading = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading({
				headingText: this.messages.PermissionsByRoleEditor_1
			});
			
			this._toolbar = new jazz.ui.Toolbar();
			this._sectionHeading.addWidget(this._toolbar);
			
			this._filterConfiguredRolesButton = new jazz.ui.toolbar.TwoStateButton({
				title: this.messages.PermissionsByRoleEditor_11,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/filter-role_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function(event) {
					dojo.stopEvent(event);
					this._rolesList.model.setFilterConfigured(this._filterConfiguredRolesButton.pressed);
				})
			});
			dojo.toggleClass(this._filterConfiguredRolesButton.domNode, "hidden", !this.workingCopy.isChildArea()); //$NON-NLS-1$
			this._toolbar.addChild(this._filterConfiguredRolesButton);
			
			this.domNode.appendChild(this._sectionHeading.domNode);
		},
		
		_startLoading: function() {
			this._rolesList.setDisabled(true);
		},
		
		_stopLoading: function() {
			this._rolesList.setDisabled(false);
		},
		
		/**
		 * If the selected role is no longer displayed, set this._selectedRole
		 * to null and notify listeners that no role is selected.
		 * 
		 * If there are no roles to select, the tree is hidden and a message
		 * about there being no customized roles is revealed.
		 */
		_ensureRoleSelected: function() {
			this._rolesList.model.getChildren(this._rolesList.model.root, dojo.hitch(this, function(children) {
				if (children.length === 0) {
					this._rolesList.set("selectedItems", []); //$NON-NLS-1$
					this._selectedRole = null;
					this.roleSelectionChangedCallback(this._selectedRole);
				} else {
					var nodes = this._rolesList.getNodesByItem(this._selectedRole);
					if (nodes.length !== 1 || !nodes[0] || !nodes[0].domNode || !nodes[0].domNode.parentNode) {
						this._rolesList.set("selectedItems", []); //$NON-NLS-1$
						this._selectedRole = null;
						this.roleSelectionChangedCallback(this._selectedRole);
					}
				}
				dojo.toggleClass(this._rolesList.domNode, "hidden", children.length === 0); //$NON-NLS-1$
				dojo.toggleClass(this._noCustomizedRolesMessage, "hidden", children.length !== 0); //$NON-NLS-1$
			}));
		},
		
		_selectFirstRole: function() {
			this._rolesList.model.getChildren(this._rolesList.model.root, dojo.hitch(this, function(children) {
				this._selectedRole = children[0];
				this._rolesList.set("selectedItem", children[0]); //$NON-NLS-1$
				this.roleSelectionChangedCallback(this._selectedRole);
			}));
		},
		
		/**
		 * Callback when role changed in the roles store. If the identifier
		 * changed for a role that was selected, select no role and notify
		 * listeners that no role is selected.
		 */
		_roleChanged: function(roleItem, property) {
			if (ProcessConstants.ProcessRoleDTO.ID === property) {
				if (this._selectedRole === roleItem) {
					this._rolesList.set("selectedItems", []); //$NON-NLS-1$
					this._selectedRole = null;
					this.roleSelectionChangedCallback(this._selectedRole);
				}
			}
		},
		
		uninitialize: function() {
			if (this._editorConnects) {
				dojo.forEach(this._editorConnects, dojo.disconnect);
				delete this._editorConnects;
			}
			
			if (this._sectionHeading) {
				this._sectionHeading.destroyRecursive();
				delete this._sectionHeading;
			}
			
			if (this._rolesList) {
				this._rolesList.model.uninitialize();
				this._rolesList.destroyRecursive();
				delete this._rolesList;
			}
			
			this.inherited(arguments);
		}
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsByRolePermissionsColumn", [dijit._Widget], { //$NON-NLS-1$
		
		constructor: function(args) {
			this._selectedRole = args.selectedRole;
			this._selectedTimePeriod = args.processAreaStore.alwaysRootItem;
		},
		
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsByRolePermissionsColumn"); //$NON-NLS-1$
			
			this._createSectionHeading();
			
			this._updatePermissionsTableHeading();
			
			this.connect(this.rolesStore, "onSet", this._roleChanged); //$NON-NLS-1$
			this.connect(this.processAreaStore, "onSet", this._timePeriodChanged); //$NON-NLS-1$
			
			this._createPermissionsTable();
		},
		
		resized: function() {
			var headingHeight = this._sectionHeading.domNode.clientHeight;
			this._sectionHeading.layout();
			if (this._sectionHeading.domNode.clientHeight !== headingHeight) {
				setTimeout(dojo.hitch(this, function() {
					this.heightChanged();
				}), 0);
			}
		},
		
		/**
		 * The container of this sub-editor listens to invocations of this
		 * function via dojo.connect. heightChanged() should be called whenever
		 * this widget requires more or less vertical space to display all
		 * contents without scrolling.
		 */
		heightChanged: function() {
		},
		
		_createSectionHeading: function() {
			this._contents = dojo.create("div", {}, this.domNode); //$NON-NLS-1$
			
			this._noRoleSelectedMessage = dojo.create("div", {'class': "hidden j-label"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			this._noRoleSelectedMessage.appendChild(document.createTextNode(this.messages.PermissionsByRoleEditor_15));
			
			this._sectionHeading = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading({
			});
			this._contents.appendChild(this._sectionHeading.domNode);
			
			this._operationTableToolbar = new jazz.ui.Toolbar();
			this._sectionHeading.addWidget(this._operationTableToolbar);
			
			this._grantAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByRoleEditor_8,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/grant-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					if (this._permissionsTable) {
						this._changePermissionOfAllOperations(PermittedType.PERMITTED);
					}	
				})
			});
			this._operationTableToolbar.addChild(this._grantAllButton);
			
			this._revokeAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByRoleEditor_9,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/revoke-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					if (this._permissionsTable) {
						this._changePermissionOfAllOperations(PermittedType.NOT_PERMITTED);
					}	
				})
			});
			this._operationTableToolbar.addChild(this._revokeAllButton);
			
			this._expandAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByRoleEditor_5,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/expand-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					if (this._permissionsTable) {
						this._permissionsTable.expandAll();
					}	
				})
			});
			this._operationTableToolbar.addChild(this._expandAllButton);
			
			this._collapseAllButton = new jazz.ui.toolbar.Button({
				title: this.messages.PermissionsByRoleEditor_6,
				imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
				onClick: dojo.hitch(this, function() {
					if (this._permissionsTable) {
						this._permissionsTable.collapseAll();
					}	
				})
			});
			this._operationTableToolbar.addChild(this._collapseAllButton);
			
			//Categories cannot be collapsed by default, unless a query parameter
			//permissionsDebugCollapsableCategories is specified. This is used in
			//automated tests, where we collapse categories so that elements used
			//by the test are in the visible area of the screen.
			if (document.URL.indexOf("permissionsDebugCollapsableCategories") !== -1) { //$NON-NLS-1$
				this._operationTableToolbar.addChild(new jazz.ui.toolbar.Button({
					title: "Collapse All including categories", //$NON-NLS-1$
					imgSrc: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/images/collapse-all_co.gif"), //$NON-NLS-1$ //$NON-NLS-2$
					onClick: dojo.hitch(this, function() {
						if (this._permissionsTable) {
							this._permissionsTable.collapseAll(true);
						}	
					})
				}));
			}
			
			this._operationTextFilterBox = new jazz.ui.FilterBox({
				onChange: dojo.hitch(this, function(value) {
					if (this._permissionsTable) {
						this._refreshTextFiltering(false);
					}
				})
			});
			this._sectionHeading.addWidget(this._operationTextFilterBox);
		},
		
		_refreshTextFiltering: function(forceUpdate) {
			var filterText = this._operationTextFilterBox.value;		
			if (forceUpdate || filterText !== this._currentFilterText) {
				this._currentFilterText = filterText;
				if (this._filteringTimeout) {
					clearTimeout(this._filteringTimeout);
					delete this._filteringTimeout;
				}
				
				// Display "Searching..." message immediately if the filter text is changed.
				this.processAreaEditor.startLoading(this.messages.PermissionsByRoleEditor_16, false);
				
				// Use setTimeout to account for fast typing
				this._filteringTimeout = setTimeout(dojo.hitch(this, function() {
					this._permissionsTable.model.setFilterText(this._operationTextFilterBox.value, forceUpdate);
					var matchingItems = this._permissionsTable.model.getTextFilterMatches();
					this._permissionsTable.expandTo(matchingItems);
					this.processAreaEditor.stopLoading();
					dojo.toggleClass(this._noFilterResultsDiv, "hidden", dojo.string.trim(this._operationTextFilterBox.value).length === 0 || matchingItems.length !== 0);  //$NON-NLS-1$
					this.heightChanged();
				}), FILTERING_DELAY);
			}
		},
		
		_changePermissionOfAllOperations: function(permission) {
			var store = this._permissionsTable.getPermissionsStore();
			this.workingCopy.startBatchUpdate();
			var self = this;
			this._permissionsTable.model.store.fetch({
				onItem: function(categoryItem) {
					self._permissionsTable.model.getChildrenIgnoreTextFilter(categoryItem, function(operationItems) {
						dojo.forEach(operationItems, function(operationItem) {
							var actionId = self._permissionsTable.model.getActionId(operationItem);
							store.setPermission(
								store.getPermissionItem(actionId),
								actionId,
								self._selectedTimePeriod,
								permission
							);
						});
					});
				}
			});
			this.workingCopy.endBatchUpdate();
		},
		
		_getSelectedRole: function() {
			return this._selectedRole;
		},
		
		_getSelectedRoleId: function() {
		    return this.rolesStore.getValue(this._getSelectedRole(), ProcessConstants.ProcessRoleDTO.ID);
		},	
		
		_createPermissionsTable: function() {
			this.workingCopy.loadPermissionExtensions(
				//Success callback
				dojo.hitch(this, function(permissionExtensionsStore) {
					var model = new com.ibm.team.process.web.ui.internal.admin.projects.PermissionExtensionsModel({
						store: permissionExtensionsStore,
						workingCopy: this.workingCopy,
						processAreaStore: this.processAreaStore,
						editorContext: this.processAreaEditor.context,
						timePeriod: this.processAreaStore.alwaysRootItem
					});
					
					this._permissionsTable = com.ibm.team.process.web.ui.internal.admin.projects.PermissionsForRoleTable({
						model: model,
						workingCopy: this.workingCopy,
						heightChanged: dojo.hitch(this, this.heightChanged),
						processAreaEditor: this.processAreaEditor,
						processAreaStore: this.processAreaStore,
						roleId: this._getSelectedRoleId(),
						contextItem: this._selectedTimePeriod
					});
					
					this._contents.appendChild(this._permissionsTable.domNode);
					
					this._permissionsTable.reloadPermissions(dojo.hitch(this, function() {
						//heightChanged() must be called after permissions are
						//loaded, because changes to the rows to show permissions
						//may have impacted their height
						setTimeout(dojo.hitch(this, function() {
							this.heightChanged();
						}), 0);
					}));
					
					this._noFilterResultsDiv = dojo.create("div", { //$NON-NLS-1$
						'class': "hidden noFilterResultsMessage j-label", //$NON-NLS-1$ //$NON-NLS-2$
						innerHTML: this.messages.PermissionsByRoleEditor_10
					}, this._contents);

					//Now that a tree of categories/operations/actions is shown,
					//call heightChanged() so that the container can grow to
					//accommodate our new size
					setTimeout(dojo.hitch(this, function() {
						this.heightChanged();
					}), 0);
					
					//We create the model with the 'always' root item, so that
					//when the permissions table is created, the model returns
					//all permissions extensions, (i.e. none are filtered out),
					//so that all table rows are created. Now that the table is
					//created, set the time period in the model differently if
					//necessary, which may filter out some rows
					if (this._selectedTimePeriod !== this.processAreaStore.alwaysRootItem) {
						setTimeout(dojo.hitch(this, function() {
							model.setTimePeriod(this._selectedTimePeriod);
						}));
					}
				}),
				//Error callback
				dojo.hitch(this, function(error) {
					this.processAreaEditor.handleServerResponseError(error, this.messages.PermissionsByRoleEditor_7);
				})
			);
		},
		
		/**
		 * Callback when role changed in the roles store. If the label of
		 * the selected role changed, update the header above the table.
		 */
		_roleChanged: function(roleItem, property) {
			if (this._selectedRole === roleItem && ProcessConstants.ProcessRoleDTO.LABEL === property) {
				this._updatePermissionsTableHeading();
			}
		},
		
		/**
		 * Callback when role changed in the roles store. If the label of
		 * the selected role changed, update the header above the table.
		 */
		_timePeriodChanged: function(scopeItem, property) {
			if (this._selectedTimePeriod === scopeItem && "name" === property) { //$NON-NLS-1$
				this._updatePermissionsTableHeading();
			}
		},
		
		roleSelectionChangedCallback: function(selectedRole) {
			this._selectedRole  = selectedRole;
			if (this._selectedRole) {
				var selectedRoleId = this.rolesStore.getValue(selectedRole, ProcessConstants.ProcessRoleDTO.ID);
				this._permissionsTable.setRoleId(selectedRoleId);
				
				this._updatePermissionsTableHeading();
	      
				if (this._selectedTimePeriod) {
					this._permissionsTable.reloadPermissions();
				}
			}
			dojo.toggleClass(this._noRoleSelectedMessage, "hidden", selectedRole); //$NON-NLS-1$
			dojo.toggleClass(this._contents, "hidden", !selectedRole); //$NON-NLS-1$
			this.heightChanged();
		},
		
		timePeriodSelected: function(scopeItem) {
			this._selectedTimePeriod = scopeItem;
			
			if (this._permissionsTable) {
				this._permissionsTable.setScope(this._selectedTimePeriod);
				
				if (this._selectedTimePeriod) {
					if (this._permissionsTable.model.setTimePeriod(this._selectedTimePeriod)) {
						//If different items are now filtered/not filtered, we need to
						//re-apply the text filter to show appropriate items and
						//display the 'no matching items' message appropriately
						this._refreshTextFiltering(true);
					}
					
					if (this._selectedRole) {
						this._permissionsTable.reloadPermissions();
					}
				}
			}
			
			this._updatePermissionsTableHeading();
		},
		
		_updatePermissionsTableHeading: function() {
			if (this._selectedRole && this._selectedTimePeriod) {
				var roleLabel = this.rolesStore.getRoleLabel(this._selectedRole);
			
				if (this._selectedTimePeriod === this.processAreaStore.alwaysRootItem) {
					this._sectionHeading.setHeadingText(dojo.string.substitute(this.messages.PermissionsByRoleEditor_2, [roleLabel])); //$NON-NLS-1$
				} else {
					var selectedContext = this.processAreaStore.getValue(this._selectedTimePeriod, "enabledContext"); //$NON-NLS-1$
					var scopeLabel = this.processAreaStore.getValue(this._selectedTimePeriod, "name"); //$NON-NLS-1$
					if (ProcessConstants.PermissionContext.ITERATION_TYPE === selectedContext) {
						this._sectionHeading.setHeadingText(dojo.string.substitute(this.messages.PermissionsByRoleEditor_3, [roleLabel, scopeLabel])); //$NON-NLS-1$
					} else {
						this._sectionHeading.setHeadingText(dojo.string.substitute(this.messages.PermissionsByRoleEditor_4, [roleLabel, scopeLabel])); //$NON-NLS-1$
					}
				}
				
				this.heightChanged();
			}
		},
		
		uninitialize: function() {
			if (this._permissionsTable) {
				this._permissionsTable.destroyRecursive();
				delete this._permissionsTable;
			}
			
			if (this._sectionHeading) {
				this._sectionHeading.destroyRecursive();
				delete this._sectionHeading;
			}
			
			this.inherited(arguments);
		}
	});
})();
