/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

(function(){
	
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel"); //$NON-NLS-1$
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModelMixin"); //$NON-NLS-1$
	
	dojo.require("dijit.tree.ForestStoreModel"); //$NON-NLS-1$
	
	/**
	 * This class is meant to complement a Dojo model (e.g. TreeStoreModel,
	 * ForestStoreModel) as a mixin. It supports archive-able configuration
	 * scope data (like timelines, iterations, iteration types).
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModelMixin", null, { //$NON-NLS-1$		
		/**
		 * Overridden to do various filtering/sorting.
		 */
		getChildren: function(/*dojo.data.Item*/ parentItem, /*function(items)*/ callback, /*function*/ onError) {
			if (parentItem === this.root) {
				var origCallback = callback;
				callback = dojo.hitch(this, function(items) {
					items = this._filterRootItems(items);
					origCallback(items);
				});
			}
			
			if (this.store.isItem(parentItem)) {
				// Hide the development lines item, but show the child iterations 
				// under DEV_LINES root item if the editor context is TEAM.
				if (this.store.getEditorContext() === ProcessConstants.EditorContext.TEAM) {
					var parentItemId = this.store.getValue(parentItem, "id"); //$NON-NLS-1$
					var currentDevLineId = this.store.getCurrentDevelopmentLineItemId();
					if (parentItemId === ProcessConstants.PermissionsScopeContext.DEV_LINES) {
						this.store.fetch({
							query: {
								parent: currentDevLineId
							},
							queryOptions: {
								deep: true
							},
							onComplete: dojo.hitch(this, function(items) {
								items = this._filterAndSortItems(parentItem, items);
								callback(items);
							}),
							onError: onError
						});
						return;
					} 
				}

				// filter and sort the potential child items
				var oldCallback = callback;
				callback = dojo.hitch(this, function(items) {
					items = this._filterAndSortItems(parentItem, items);
					oldCallback(items);
				});
			}
			
			this.inherited(arguments);
		},
		
		/**
		 * Filters the 'Always' item from view if scopes are to be filtered to a
		 * set of specific scopes that does not include Always, (i.e.
		 * permissions in a child area are not configured for the project or
		 * team scopes).
		 */
		_filterRootItems: function(items) {
			dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
			
			var newItems = [];
			dojo.forEach(items, dojo.hitch(this, function(item) {
				if (this.filteredContextIds === undefined || item !== this.store.alwaysRootItem || this.filteredContextIds[""] !== undefined) { //$NON-NLS-1$
					newItems.push(item);
				}
			}));
			return newItems;
		},

		/**
		 * Filter out and sort the items if needed.
		 * 
		 * @param parentItem {Object} 
		 * @param items {Object} 
		 * @return the filtered and sorted items
		 */
		_filterAndSortItems: function(parentItem, items) {
			var needToFilterIterationTypeItems = this.store.getValue(parentItem, "id") === ProcessConstants.PermissionsScopeContext.ITERATION_TYPES; //$NON-NLS-1$
			var filteredContextIds = this.filteredContextIds;
			var self = this;
			if (needToFilterIterationTypeItems || filteredContextIds !== undefined) {
				var store = this.store;
				items = dojo.filter(items, function(item) {
					var isShown = true;
					if (isShown && needToFilterIterationTypeItems) {
						// only show types defined locally and those 
						// from the provider which are not overridden
						isShown = store.isAvailableIterationType(item);
					}
					if (isShown && filteredContextIds !== undefined) {
						isShown = filteredContextIds[self.store.getSavedContextId(item)] || false; //$NON-NLS-1$
					}
					return isShown; 
				});
				if (needToFilterIterationTypeItems) {
					items.sort(dojo.hitch(store, store.compareIterationTypes));
				}
			} 
			
			return items;
		},
		// Overridden to refresh the TimelinesRootItem item instead of the timeline item
		// while a new iteration item was added to the parent timeline item that is the 
		// current selected development line. (NOTES: the timeline items are hidden in the
		// team area editor and the iteratoin items are shown under the TimelinesRootItem 
		// item directly.) 
		onNewItem: function (/* dojo.data.Item */ item, /* Object */ parentInfo){
			// this is only applicable for a team area store.
			if (this.store.getEditorContext() === ProcessConstants.EditorContext.PROJECT) {
				this.inherited(arguments);
				return;
			}
			
			if(!parentInfo){
				return;
			}
			
			if (this.store.getValue(parentInfo.item, "id") === this.store.getCurrentDevelopmentLineItemId()) { //$NON-NLS-1$
				this.store.fetchItemByIdentity({
					identity: ProcessConstants.PermissionsScopeContext.DEV_LINES,
					onItem: dojo.hitch(this, function(item, request) {
						this.getChildren(item, dojo.hitch(this, function(newChildren){
							this.onChildrenChange(item, newChildren);
						}), null);
					})
				});
			} else {
				this.inherited(arguments);
			}
		}, 
		
		// Calls getChildren() on all nodes in the tree
		refreshModel: function() {
			var items = [this.root];
			while (items.length > 0) {
				var item = items.shift();
				this.getChildren(item, dojo.hitch(this, function(newChildren) {
					this.onChildrenChange(item, newChildren);
					items = items.concat(newChildren);
				}));
			}
		},
		
		/**
		 * Overridden to hide archived nodes and show unarchived nodes
		 * immediately after they are changed, and refresh the iteration types
		 * sub-hierarchy when an iteration type id changes.
		 */
		onSetItem: function(item, attribute, oldValue, newValue) {
			this.inherited(arguments);
			if (attribute === "archived" || this.store.getValue(item, "enabledContext") === ProcessConstants.PermissionContext.ITERATION_TYPE && attribute === "contextId") { //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var parent = this.store.getParent(item);
				this.getChildren(parent, dojo.hitch(this, function(newChildren) {
					parent.children = newChildren;
					this.onChildrenChange(parent, newChildren);
				}));
			}
		}
		
	});
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModel", //$NON-NLS-1$
			[dijit.tree.ForestStoreModel, com.ibm.team.process.web.ui.internal.admin.projects.PermissionsScopeModelMixin], {
		
		/*
		 * Constructs the Permissions model. Combines the functionality of
		 * ForestStoreModel and PermissionsScopeModelMixin.
		 * 
		 * @param {Object} args The args are expected to contain the following:
		 *  {
		 *    store: the store the model should use for it's data
		 *    childrenAttrs: list of attribute names that specify that item's
		 *    		children
		 *  }
		 */
		constructor: function(args) {
		}
		
	});
	
})();
