/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	
	/**
	 * Helper widget that displays a left-justified (optional) title and
	 * supports addition of 0..n right-justified widgets that could represent
	 * toolbars, filter boxes, etc...
	 * 
	 * A layout() function is provided which is to be called when the size
	 * of the parent container changes. The title and widgets are rearranged
	 * to make use of the smaller/greater amount of space to line up properly. 
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.PermissionsSectionHeading", [dijit._Widget], { //$NON-NLS-1$
		postCreate: function() {
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-PermissionsSectionHeading"); //$NON-NLS-1$
			
			this._headingTextNode = dojo.create("div", {'class': "headingText"}, this.domNode); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			dojo.addClass(this._headingTextNode, "subHeading j-h2"); //$NON-NLS-1$
			if (this.headingText) {
				this._updateHeadingText();
			}
			
			this._widgets = [];
		},
		
		/**
		 * Updates the text of the heading.
		 * 
		 * @param	text	String to use for heading text.
		 */
		setHeadingText: function(text) {
			this.headingText = text;
			this._updateHeadingText();
		},
		
		/**
		 * Arranges text and widgets such that the widgets are positioned
		 * correctly depending on height of text, (including multiple lines
		 * of wrapped text if the available width causes wrapping).
		 * 
		 * Should be invoked whenever horizontal size of container changes.
		 */
		layout: function() {
			if (this._widgets.length !== 0) {
				var parentBox = dojo.marginBox(this.domNode);
				var headingBox = dojo.marginBox(this._headingTextNode);
				
				var widgetWidthTotal = 0;
				dojo.forEach(this._widgets, function(widget) {
					widgetWidthTotal += dojo.marginBox(widget.domNode).w;
				});
				
				var availableHeadingWidth = parentBox.w - widgetWidthTotal;
				if (availableHeadingWidth > 0) {
					dojo.style(this._headingTextNode, "width", availableHeadingWidth + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				}
				
			 	headingBox = dojo.marginBox(this._headingTextNode);
				dojo.forEach(this._widgets, function(widget) {
					dojo.style(widget.domNode, "top", (headingBox.h - 14) + "px"); //$NON-NLS-1$ //$NON-NLS-2$
				});
			}
		},
		
		
		/**
		 * Place the DOM element for a help widget at the appropriate location,
		 * (to the right of the heading text).
		 * 
		 * @param	helpNode	A DOM element
		 */
		placeHelpNode: function(helpNode) {
			dojo.place(helpNode, this._headingTextNode);
		},
		
		/**
		 * Add a widget to the header. It will be added to the right of
		 * the rest of the content, on the same line.
		 * 
		 * The widget will be destroyed when the section heading is
		 * destroyed.
		 */
		addWidget: function(widget) {
			this._widgets.push(widget);
			dojo.addClass(widget.domNode, "headerWidget"); //$NON-NLS-1$
			this.domNode.appendChild(widget.domNode);
			
			this._deferredLayout((new Date()).getTime());
		},
		
		_deferredLayout: function(startTime) {
			setTimeout(dojo.hitch(this, function() {
				if ((new Date()).getTime() - startTime > 5000) {
					console.error("Timeout while layouting widgets in PermissionsSectionHeading."); //$NON-NLS-1$
					return;
				}
				if (dojo.marginBox(this.domNode).w === 0) {
					this._deferredLayout(startTime);
				}
				//Shift other widgets to the left
				var totalWidth = 0;
				for (var i = this._widgets.length-1; i > -1; i--) {
					dojo.style(this._widgets[i].domNode, !this.isLeftToRight() ? "left" : "right", (10 + totalWidth) + "px"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
					totalWidth += dojo.marginBox(this._widgets[i].domNode).w;
				}
			}));
		},
		
		_updateHeadingText: function() {
			dojo.empty(this._headingTextNode);
			this._headingTextNode.appendChild(document.createTextNode(this.headingText));
		},
		
		uninitialize: function() {
			dojo.forEach(this._widgets, function(widget) {
				if (widget) {
					widget.destroyRecursive();
				}
			});
			delete this.widgets;
		}
	});
})();
