/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessBulkEditRolesDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._Templated"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.web.ui.internal.admin.projects.UIHelper");//$NON-NLS-1$
(function(){

	/**
	 * Gets a list of all the selected indices inside of a list
	 * 
	 * @param {List} The list to check for selected elements
	 */ 
	function getSelectedIndices(list){
		var selected = [];
		for (var i = 0; i < list.options.length; i++) {
			if (list.options[i].selected) {
				selected.push(i);
			}
		}
		return selected;
	}
	
	/**
	 * Creates a dialog for Adding and Removing multiple roles from multiple people 
	 * 
	 * @name com.ibm.team.process.web.ui.internal.admin.projects.ProcessBulkEditRolesDialog
	 * @constructor
	 * @extends dijit._Widget
	 * @extends dijit._Templated
	 */ 
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessBulkEditRolesDialog", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/admin/projects/templates/ProcessBulkEditRolesDialog.html"), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * 
		 * @constructor
		 * @param {Object} The elements which initialize this dialog
		 * allRoles: An array of all the roles to be placed in the dialog
		 * membersCount: The number of members affected by this dialog
		 * isRemove: Is this a Remove Dialog
		 */ 
		constructor: function(args){
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.web.ui.internal.admin.projects", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this.allRoles = args.allRoles;
			this.membersCount = args.membersCount;
			this.isRemove = args.isRemove;
		},
		
		postCreate: function(){
			dojo.empty(this._assignProcessRoles);
			dojo.empty(this._selectedRoles);
			this._configureDialog(this.isRemove);
			this._populateObj();
			
			this.connect(this._selectedRoles, "onchange", "_handleListChange"); //$NON-NLS-1$ //$NON-NLS-2$
			this.connect(this._cancelButton, "onclick", function(e){this.onCancel(this.getChosenRoles());}); //$NON-NLS-1$
			this.connect(this._finishButton, "onclick", function(e){this.onFinish(this.getChosenRoles());}); //$NON-NLS-1$
			
			this._dialog = new jazz.ui.Dialog({
				primaryTitle: !this.isRemove ? this.messages.ProcessBulkAddRolesDialog_Title : this.messages.ProcessBulkRemoveRolesDialog_Title, 
				contentNode: this.domNode,
				width: "30em", //$NON-NLS-1$
				destroyContents: true
			});
		},
		
		/**
		 * This is responsible for actually starting the dialog
		 */
		startup: function(){
			this.inherited(arguments);
			com.ibm.team.process.web.ui.internal.admin.projects.UIHelper.resizeI18NButtons(this._dialogFooter, [this._finishButton, this._cancelButton], 0, 2);
		},
		
		/**
		 * The function configures the dialog for either Remove or ADD
		 * 
		 * @private
		 * @param {Boolean} If true, then make the dialog in the remove state
		 */
		_configureDialog: function(isRemove){
			//Adds text specific to both remove and add
			this._descriptionHeader.innerHTML = this.messages.ProcessBulkEditRolesDialog_DescriptionBoxHeader;
			this._cancelButton.innerHTML = this.messages.ProcessBulkEditRolesDialog_CancelButton;
			
			//Adds text specific to either remove or add
			if(!isRemove){
				this._assignProcessRoles.innerHTML = dojo.string.substitute(this.messages.ProcessBulkAddRolesDialog_HeaderText, [this.membersCount]);
				this._roleSelectorHeader.innerHTML = this.messages.ProcessBulkAddRolesDialog_RolesSelectorHeader;
				this._finishButton.innerHTML = this.messages.ProcessBulkAddRolesDialog_FinishButton;
			} else {
				this._assignProcessRoles.innerHTML = dojo.string.substitute(this.messages.ProcessBulkRemoveRolesDialog_HeaderText, [this.membersCount]);
				this._roleSelectorHeader.innerHTML = this.messages.ProcessBulkRemoveRolesDialog_RolesSelectorHeader;
				this._finishButton.innerHTML = this.messages.ProcessBulkRemoveRolesDialog_FinishButton;
			}
		},
		
		/**
		 * populates the dialog with roles
		 * 
		 * @private
		 */
		_populateObj: function(){			
			var i;
			for (i = 0; i < this.allRoles.length; i++) {
				if(this.allRoles[i].id == "default"){ //$NON-NLS-1$
					continue;
				}
				option = document.createElement("option"); //$NON-NLS-1$
				option.value = this.allRoles[i].id;
				option.appendChild(document.createTextNode(!this.allRoles[i].label ? this.allRoles[i].id : this.allRoles[i].label));
				option.description = this.allRoles[i].description;
				option.setAttribute("aria-describedby", this.id+"_description"); //$NON-NLS-1$ //$NON-NLS-2$
				this._selectedRoles.appendChild(option);
			}
		},
		
		/**
		 * Handles an item being selected and updates the description box
		 * 
		 * @private
		 * @param {Object} Th event object
		 */
		_handleListChange: function(e){
			var selectedItems = getSelectedIndices(e.target);
			dojo.empty(this._description);
			if (selectedItems.length == 1) {
				this._description.value = e.target.options[selectedItems[0]].description;
			}
			if (selectedItems.length > 0) {
				this._finishButton.disabled = false;
			} else {
				this._finishButton.disabled = true;
			}
			
		},
		
		/**
		 * Gets all of the selected roles
		 * 
		 * @private
		 * @returns {Array} All of the selected roles
		 */
		getChosenRoles: function(){
			var roles = [];
			for (var i = 0; i < this._selectedRoles.options.length; i++) {
				var option = this._selectedRoles.options[i];
				if(option.selected){
					roles.push({
						//if the role id is default, use the label we stored (without 
						//the other text we added for display)
						label: option.value === "default" ? this.defaultRoleLabel : option.text,//$NON-NLS-1$
						id: option.value,
						description: option.description
					});
				}
			}
			return roles;
		},
		
		/**
		 * Function is called when the Add/Remove button is pressed
		 * Extend this function in order to capture this event
		 * 
		 * @public
		 * @returns {Array} All of the selected roles
		 */
		onFinish: function(selectedRoles){
			this.close();
		},
		
		/**
		 * Function is called when the Cancel button is pressed
		 * Extend this function in order to capture this event
		 * 
		 * @public
		 * @returns {Array} All of the selected roles
		 */
		onCancel: function(selectedRoles){
			this.close();
		},
		
		/**
		 * This function closes the current dialog
		 * 
		 * @public
		 */
		close: function() {
			this._dialog.close();
		}
	});
	
})();
