/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessStore"); //$NON-NLS-1$
	
	dojo.require("dojo.data.ItemFileWriteStore"); //$NON-NLS-1$
	
	/**
	 * Provides some additional utility functions on top of Dojo's store's 
	 * functionality.
	 */
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessStore", dojo.data.ItemFileWriteStore, { //$NON-NLS-1$
		
		constructor: function(args) {
			this._connects = [];
		},
		
		connect: function(context, method, localMethod) {
			this._connects.push(dojo.connect(context, method, this, localMethod));
		},
		
		/**
		 * Overridden to provide an additional, simpler interface: you can pass
		 * in just an identifier and this method will return the matching item,
		 * or null. Maintains the inherited interface as well.
		 */
		fetchItemByIdentity: function(args) {
			var fetched = null;
			if (!dojo.isObject(args) || args.identity === undefined) {
				// just an identifier was passed in, make args match what the
				// inherited fetchItemByIdentity expects
				args = {
					identity: args,
					onItem: function(item) {
						fetched = item;
					}
				};
			}
			this.inherited(arguments);
			return fetched;
		},
		
		/**
		 * The following methods add support for determining if an item is new
		 * to the store, i.e. added since the last save().
		 */
		isNew: function(item) {
			return this.hasAttribute(item, "_isNew"); //$NON-NLS-1$
		},
		onNew: function(newItem, parentInfo) {
			this.setValue(newItem, "_isNew", true); //$NON-NLS-1$
		},
		save: function() {
			this.fetch({
				query: {
					_isNew: true
				},
				queryOptions: {
					deep: true
				},
				onItem: dojo.hitch(this, function(item) {
					this.unsetAttribute(item, "_isNew"); //$NON-NLS-1$
				})
			});
			this.inherited(arguments);
		},
		
		/**
		 * The following methods add a new event method, "aboutToDelete", that
		 * implementors can connect to to find out when deleteItem is called on
		 * an item, *before* that item is removed from the store.
		 */
		deleteItem: function(item) {
			this.aboutToDelete(item);
			this.inherited(arguments);
		},
		aboutToDelete: function(item) {
			// implementors can connect to this method to find out about a
			// delete event while the item is in the store (so it can be read)
		},
		
		destroy: function() {
			dojo.forEach(this._connects, dojo.disconnect);
		}
		
	});
	
})();
