/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
	dojo.provide("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree"); //$NON-NLS-1$
	dojo.require("dijit.Tree"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.common.internal.ProcessConstants"); //$NON-NLS-1$
	var ProcessConstants = com.ibm.team.process.web.common.internal.ProcessConstants;
	
	dojo.declare("com.ibm.team.process.web.ui.internal.admin.projects.ProcessTree", [dijit.Tree], { //$NON-NLS-1$
	
		disabled : false,
		nodeListById : null,
		
		// summary:
		//		Tree widget for Process component
		// description:
		//		Provides methods for inheriting classes to add follow-up actions 
		//		to various functions in dijit's Tree widget. 
		
		postCreate: function(){
			this.nodeListById = {};
			
			this.inherited(arguments);
			
			dojo.addClass(this.domNode, "com-ibm-team-process-web-ui-internal-ProcessTree"); //$NON-NLS-1$
			
			this.connect(this, "focusNode", "_focusChanged"); //$NON-NLS-1$  //$NON-NLS-2$
			this.connect(this, "_onEnterKey", "enterKeyChanged"); //$NON-NLS-1$  //$NON-NLS-2$
			this.connect(this, "_onClick", "onClicked"); //$NON-NLS-1$  //$NON-NLS-2$
			// Dojo 1.6.1 allows multi-selection by default
			this.dndController.singular = true;
			
			//Workaround for https://bugs.dojotoolkit.org/ticket/15858 by
			//essentially implementing the fix suggested in comment #2 on
			//that ticket. Function _createTreeNode below is another workaround
			//for issues brought up in that ticket.
			this.dndController._origGetSelectedTreeNodes = this.dndController.getSelectedTreeNodes;
			this.dndController.getSelectedTreeNodes = dojo.hitch(this.dndController, function() {
				var nodes = this._origGetSelectedTreeNodes();
				var activeNodes = [];
				dojo.forEach(nodes, dojo.hitch(this, function(node) {
					if (node.domNode === null) {
						if(this.anchor == node){
							delete this.anchor;
						}
						delete this.selection[node.id];
					} else {
						activeNodes.push(node);
					}
				}));
				return activeNodes;
			});
		},
		
		onLoad: function() {
			this.inherited(arguments);
			if (dojo.isIE === 8) {
				// trigger a layout, otherwise there's a gap above the first row
				var originalDisplay = dojo.style(this.domNode, "display"); //$NON-NLS-1$
				dojo.style(this.domNode, "display", "inline-block"); //$NON-NLS-1$ //$NON-NLS-2$
				dojo.style(this.domNode, "display", "block"); //$NON-NLS-1$ //$NON-NLS-2$
				if ("block" !== originalDisplay) { //$NON-NLS-1$
					dojo.style(this.domNode, "display", originalDisplay); //$NON-NLS-1$
				}
			}
		},
		
		_focusChanged: function(/* _tree.Node */node){
			// Dojo 1.6.1 doesn't set the focused item as selected
			// This will handle it automatically for all our trees
			this.dndController.setSelection([node]);
			this.focusChanged(node); 
		},
		
		///////////////////////////////////////////////////////
		// Overridable function to add follow-up actions to dijit.Tree functions  
		focusChanged: function(/* _tree.Node */node){
			// Should be overriden to add follow-up action to Tree's 'focusNode' 
			// action
		},
		
		enterKeyChanged: function(/*Object*/message, /*Event*/ evt){
			// Should be overriden to add follow-up action to Tree's 
			// '_onEnterKey' action 
		},
		
		onClicked: function(/*TreeNode*/nodeWidget, /*Event*/ e){
			// Should be overriden to add follow-up action to Tree's '_onClick'
			// action
		},
		
		setDisabled: function(/*Boolean*/disabled) {
			this.disabled = disabled;
		},
			
		// Workitem 110559...need to override the tree.js private method so 
		// that disable support can be added
		_onKeyPress: function(/*Event*/ e){
			if (!this.disabled) {
				this.inherited(arguments);
			} else {
				dojo.stopEvent(e);
			}
		},
		
		isItemDisplayed: function(/*Object*/ item) {
			return true;
		},
		
		updateItemVisibility: function(item) {
			this.setVisiblity(item, this.isItemDisplayed(item));
		},
		
		updateAllItemsVisibility: function() {
			for (var id in this.nodeListById) {
				var node = this.nodeListById[id];
				this.setVisiblity(node.item, this.isItemDisplayed(node.item));
			}
		},
		
		setVisiblity: function(item, visible) {
			var node = this.nodeListById[item.id];
			if (node.domNode) {
				node.domNode.style.display = visible ? "" : "none"; //$NON-NLS-1$ //$NON-NLS-2$
				node.set("visible", visible); //$NON-NLS-1$
			
				//Set expando node
				var parent = node.getParent();
				if (parent != null && parent._setExpando != null) {
					var children = parent.hasChildren();
					parent.isExpandable = children;
					parent._setExpando(false);
				}
				
				//Deselect the element
				if (visible === false) {
					var selected = this.get("selectedItems"); //$NON-NLS-1$
					if (selected != null && selected.length > 0) {
						for (var i = 0; i < selected.length; i++) {
							if (selected[i].id === item.id) {
								selected.splice(i--, 1);
							}
						}
						this.set("selectedItems", selected); //$NON-NLS-1$
					}
				}
			}
		},
		
		/**
		 * Override function setChildItems in the inner type TreeNode from
		 * dijit.Tree. There is a bug in Dojo 1.8 where setChildItems()
		 * fails due to calling getParent() and other functions on nodes that
		 * had been destroyed, which trips up our trees when we refresh their
		 * contents after filters are toggled.
		 * 
		 * See the following bug and work item:
		 * 
		 * http://bugs.dojotoolkit.org/ticket/15858
		 * 264386: [Dojo 1.8] Exception and missing child nodes in Timelines tab after selecting a child iteration and archiving its parent
		 * 
		 * If a version greater than or equal to Dojo 1.9 is adopted, or the
		 * IBM version of dijit is changed to address the bug, then it should
		 * be safe to remove the following override of _createTreeNode.
		 * 
		 * setChildItems has been modified as follows:
		 * 
		 * - add checks for _isDestroyed in the locations surrounded by
		 *   block comments CHANGE
		 * 
		 * - use pre-1.8 syntax for references to other modules:
		 *    - dojo.forEach and dojo.indexOf instead of array...
		 *    - dijit._Container instead of _Container
		 * 
		 * - add //$NON-NLS-1$ comments where appropriate 
		 */
		_createTreeNode: function(/*Object*/ args) {
		    var node = this.inherited(arguments);
		    this.nodeListById[args.item.id] = node;
		    
		    this.setVisiblity(node.item, this.isItemDisplayed(node.item));
		    
		    //Fixes arrow keying into non-visible element
		    if (node.__oldGetNextSibling == null) {
		    	node.__oldGetNextSibling = node.getNextSibling;
		    }
		    node.getNextSibling = dojo.hitch(node, function() {
		    	var sibling = this.__oldGetNextSibling();
		    	while (sibling && sibling.get("visible") === false) { //$NON-NLS-1$
		    		sibling = sibling.__oldGetNextSibling();
		    	}
		    	return sibling;
		    });
		    if (node.__oldGetPreviousSibling == null) {
		    	node.__oldGetPreviousSibling = node.getPreviousSibling;
		    }
		    node.getPreviousSibling = dojo.hitch(node, function() {
		    	var sibling = this.__oldGetPreviousSibling();
		    	while (sibling && sibling.get("visible") === false) { //$NON-NLS-1$
		    		sibling = sibling.__oldGetPreviousSibling();
		    	}
		    	return sibling;
		    });
		    
		    //Fixes the expando
		    if (node.__oldHasChildren == null) {
		    	node.__oldHasChildren = node.hasChildren;
		    }
		    node.hasChildren = dojo.hitch(node, function() {
		    	if (this.__oldHasChildren() !== false) {
		    		var children = this.getChildren();
			    	for (var i = 0; i < children.length; i++) {
			    		if (children[i].get("visible") !== false) { //$NON-NLS-1$
			    			return true;
			    		}
			    	}
		    	}
		    	return false;
		    });
		    
		    node.setChildItems = dojo.hitch(node, function(/* Object[] */ items) {
		        // summary:
                //      Sets the child items of this node, removing/adding nodes
                //      from current children to match specified items[] array.
                //      Also, if this.persist == true, expands any children that were previously
                //      opened.
                // returns:
                //      Deferred object that fires after all previously opened children
                //      have been expanded again (or fires instantly if there are no such children).
        
                var tree = this.tree,
                    model = tree.model,
                    defs = [];  // list of deferreds that need to fire before I am complete
        
        
                // Orphan all my existing children.
                // If items contains some of the same items as before then we will reattach them.
                // Don't call this.removeChild() because that will collapse the tree etc.
                var oldChildren = this.getChildren();
                dojo.forEach(oldChildren, function(child){
                    dijit._Container.prototype.removeChild.call(this, child);
                }, this);
        
                // All the old children of this TreeNode are subject for destruction if
                //      1) they aren't listed in the new children array (items)
                //      2) they aren't immediately adopted by another node (DnD)
                this.defer(function(){
                    dojo.forEach(oldChildren, function(node){
                        if(!node._destroyed && !node.getParent()){
                            // If node is in selection then remove it.
                            tree.dndController.removeTreeNode(node);
        
                            // Deregister mapping from item id --> this node
                            var id = model.getIdentity(node.item),
                                ary = tree._itemNodesMap[id];
                            if(ary.length == 1){
                                delete tree._itemNodesMap[id];
                            }else{
                                var index = dojo.indexOf(ary, node);
                                if(index != -1){
                                    ary.splice(index, 1);
                                }
                            }
        
                            // And finally we can destroy the node
                            node.destroyRecursive();
                        }
                    });
                });
        
                this.state = "LOADED"; //$NON-NLS-1$
        
                if(items && items.length > 0){
                    // Create _TreeNode widget for each specified tree node, unless one already
                    // exists and isn't being used (presumably it's from a DnD move and was recently
                    // released
                    dojo.forEach(items, function(item){    // MARKER: REUSE NODE
                        var id = model.getIdentity(item),
                            existingNodes = tree._itemNodesMap[id],
                            node;
                        if(existingNodes){
                            for(var i=0;i<existingNodes.length;i++){
                                if(existingNodes[i] && /*CHANGE*/!existingNodes[i]._destroyed/*CHANGE*/ && !existingNodes[i].getParent()){
                                    node = existingNodes[i];
                                    node.set('indent', this.indent+1); //$NON-NLS-1$
                                    break;
                                }
                            }
                        }
                        if(!node){
                            node = this.tree._createTreeNode({
                                item: item,
                                tree: tree,
                                isExpandable: model.mayHaveChildren(item),
                                label: tree.getLabel(item),
                                tooltip: tree.getTooltip(item),
                                ownerDocument: tree.ownerDocument,
                                dir: tree.dir,
                                lang: tree.lang,
                                textDir: tree.textDir,
                                indent: this.indent + 1
                            });
                            if(existingNodes){
                                existingNodes.push(node);
                            }else{
                                tree._itemNodesMap[id] = [node];
                            }
                        }
                        
                        /*CHANGE*/if (!this._destroyed) {/*CHANGE*/
                            this.addChild(node);
                        /*CHANGE*/}/*CHANGE*/
        
                        // If node was previously opened then open it again now (this may trigger
                        // more data store accesses, recursively)
                        if(this.tree.autoExpand || this.tree._state(node)){
                            defs.push(tree._expandNode(node));
                        }
                    }, this);
        
                    // note that updateLayout() needs to be called on each child after
                    // _all_ the children exist
                    dojo.forEach(this.getChildren(), function(child){
                        child._updateLayout();
                    });
                }
                
                this.isExpandable = this.hasChildren();
                
                /*CHANGE*/if (!this._destroyed) {/*CHANGE*/        
                    if(this._setExpando){
                        // change expando to/from dot or + icon, as appropriate
                        this._setExpando(false);
                    }
        
                    // Set leaf icon or folder icon, as appropriate
                    this._updateItemClasses(this.item);
                /*CHANGE*/}/*CHANGE*/
        
                // On initial tree show, make the selected TreeNode as either the root node of the tree,
                // or the first child, if the root node is hidden
                if(this == tree.rootNode){
                    var fc = this.tree.showRoot ? this : this.getChildren()[0];
                    if(fc){
                        fc.setFocusable(true);
                        tree.lastFocused = fc;
                    }else{
                        // fallback: no nodes in tree so focus on Tree <div> itself
                        tree.domNode.setAttribute("tabIndex", "0"); //$NON-NLS-1$ //$NON-NLS-2$
                    }
                }
        
                var def =  new dojo.DeferredList(defs);
                this.tree._startPaint(def);     // to reset TreeNode widths after an item is added/removed from the Tree
                return def;     // dojo/_base/Deferred
		    });
		    
		    return node;
		},
		
		_adjustWidths: function() {
			// See defect 266064:
			// overridden to do nothing, we don't want dojo to set static,
			// explicit widths and we can't just call resize() when the window
			// is resized because that can create an unnecessary scrollbar
		}
	});			
	
})();
