/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameSummaryStep");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.StyledBox");
dojo.require("jazz.ui.wizard.WizardStepHeader");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");

var messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.rename", "ServerRenameStatusMessages");
var TrimStyles = jazz.ui.StyledBox.TrimStyles;
var WizardStepHeader = jazz.ui.wizard.WizardStepHeader;
var WizardStep = jazz.ui.wizard.WizardStep;
var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var LOCAL_STORAGE_KEY = "com-ibm-team-repository-server-rename-verification";
var InfoCenterRecoveryLink = "<a href=\"http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0m6/topic/com.ibm.jazz.install.doc/topics/c_server_rename_recovery.html\" target=\"_blank\">";
var InfoCenterVerifyLink = "<a href=\"http://pic.dhe.ibm.com/infocenter/clmhelp/v4r0m6/topic/com.ibm.jazz.install.doc/topics/c_server_rename_verify.html\" target=\"_blank\">";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.rename.ServerRenameSummaryStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/rename/templates/ServerRenameSummaryStep.html"),
	widgetsInTemplate: true,
	innerBoxTrim:  TrimStyles.EDITOR,
	
	constructor: function(args) {
		this.messages = messages;
		this.stepTitle = this.messages.summaryStepTitle;
	},
	
	postCreate: function() {
		this.header = new WizardStepHeader({stepTitle: this.stepTitle});
		this.headerArea.appendChild(this.header.domNode);
		this.connect(this.unlockButton, "onclick", "triggerButtonStateUpdate");
		this.noneFlaggedDescription.innerHTML = dojo.string.substitute(this.messages.noneFlaggedDescription, [InfoCenterVerifyLink,"</a>"]);
		this.flaggedDescription.innerHTML = dojo.string.substitute(this.messages.flaggedDescription, [InfoCenterRecoveryLink,"</a>"]);
	},
	
	focusStep: function() {
		this.allApplications = dojo.fromJson(localStorage[LOCAL_STORAGE_KEY]);
		this._showFlaggedUrls();	
	},
	
	_showFlaggedUrls: function() {
		this.hasFlaggedUrls = false;
		dojo.empty(this.flaggedUrls);
		for (var a in this.allApplications) {
			if (this.allApplications.hasOwnProperty(a)) {
				var app = this.allApplications[a];
				if(app && app.categories) {
					var categories = app.categories;
					var app_li = dojo.create("li", { innerHTML: "## " + app.title + " ##", className: "appName noDisplay" }, this.flaggedUrls);
					for (var c = 0; c < categories.length; c++) {
						var links = categories[c].links;
						var cat_li = dojo.create("li", { innerHTML: "# " + categories[c].name, className: "catName noDisplay" }, this.flaggedUrls);
						for (var l = 0; l < links.length; l++) {
							var link = links[l];
							if (link.flagged === true) {
								this.hasFlaggedUrls = true;
								dojo.create("li", { innerHTML: "source=" + link.url }, this.flaggedUrls);
								dojo.create("li", { innerHTML: "target=" + link.url }, this.flaggedUrls);
								dojo.removeClass(cat_li,"noDisplay"); //show the category header
								dojo.removeClass(app_li,"noDisplay"); //show the app header
							}
						}						
					}
				}
			}
		}
		if(!this.hasFlaggedUrls) {
			//No Urls flagged, show unlock checkbox!
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.renameComplete
			});
			dojo.removeClass(this.confirmBox, "noDisplay");
			dojo.addClass(this.linksArea, "noDisplay");
			dojo.removeClass(this.noneFlaggedDescription, "noDisplay");
		} else {
			this.header.showMessage({
				type: "WARNING",
				summary: dojo.string.substitute(this.messages.summaryFailed, [InfoCenterRecoveryLink,"</a>"])
			});
			dojo.addClass(this.confirmBox, "noDisplay");
			dojo.removeClass(this.linksArea, "noDisplay");
			dojo.addClass(this.noneFlaggedDescription, "noDisplay");
		}
		this.triggerButtonStateUpdate();
	},

	onFinish: function() {
		if(this.hasFlaggedUrls === false && this.unlockButton.checked === true) {
			this._unlockServer();
		}
		else if(this.hasFlaggedUrls === true) {
			this.wizard.processFinish();
		}
	},
	
	_unlockServer: function() {
		var handler = {
			success: dojo.hitch(this, function(response) {
				this.header.showMessage({
					type: "OK",
					summary: this.messages.serverUnlockedOK
				});
				setTimeout(dojo.hitch(this, function() {
					this.wizard.processFinish();
				}), 2000);
			}),
			error: dojo.hitch(this, function(errObject) {
				this.header.showMessage({
					errorObject: errObject,
					summary: this.messages.unableToUnlock
				});
			})
		};
		var srh = new ServiceResponseHandler(handler, "success", "error");
		ServerStatusClient.postServerRenameOverallStatus(srh);
	},
	
	canFinish: function() {
		return this.hasFlaggedUrls === true || (this.hasFlaggedUrls === false && this.unlockButton.checked === true);
	},

	uninitialize: function () {
		this.header.destroyRecursive();
		this.inherited(arguments);
    }

});

})();