/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");

    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportContributor");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType");

    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("dojo.i18n");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
    
    var ReportDateAndType = com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType;
    var LicenseReportContributor = com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportContributor;
    
    
    var REPO_BUNDLE = "com.ibm.team.repository.service";
    var USAGE_PATH = "licenseReports/LicenseUsage.rptdesign";
    var DENIALS_PATH = "licenseReports/LicenseDenials.rptdesign";
    var COMPARISON_PATH = "licenseReports/LicenseUsageComparison.rptdesign";
    
    var USAGE_CALLBACK = "processUsageReport";
    var DENIALS_CALLBACK = "processDenialsReport";
    var COMPARISON_CALLBACK = "processComparisonReport";
    
    var FROM_TIME = "FROM_TIME";
    var TO_TIME = "TO_TIME";
    var START_KEY = 'startDate';
    var END_KEY = 'endDate';
    
    var REPORTS = [{
        bundle: REPO_BUNDLE,
        path: USAGE_PATH,
        callback: USAGE_CALLBACK,
        params: [{
            name: 'Max Licenses',
            calValue: 'total'
        }, {
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    }, {
        bundle: REPO_BUNDLE,
        path: DENIALS_PATH,
        callback: DENIALS_CALLBACK,
        params: [{
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    }, {
        bundle: REPO_BUNDLE,
        path: COMPARISON_PATH,
        callback: COMPARISON_CALLBACK,
        params: [{
            name: FROM_TIME,
            formValue: START_KEY
        }, {
            name: TO_TIME,
            formValue: END_KEY
        }]
    }];
    
    var REPORT_VIEW_STYLE = "reportView";
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor", LicenseReportContributor, {
    
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/FloatingLicenseReportContributor.html"),
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
            
        },
        postCreate: function(){
            com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseReportContributor.superclass.postCreate.apply(this);
           
            this.dateAndLicenseWidget = new ReportDateAndType(null, this.dateAndLicenseArea);
			this.connect(this.dateAndLicenseWidget, "handleDateSelected", "onChange");
			this.connect(this.dateAndLicenseWidget, "onLicenseLoad", "handleLicenseLoad");
			this.connect(this.dateAndLicenseWidget, "onError", "handleServiceError");
        },
        
        _createReportViews: function(container){
            this.usageReportView = document.createElement("div");
            this.usageReportView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.usageReportView);
            
            this.denialReportView = document.createElement("div");
            this.denialReportView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.denialReportView);
            
            this.comparisonReportView = document.createElement("div");
            this.comparisonReportView.className = REPORT_VIEW_STYLE;
            container.appendChild(this.comparisonReportView);
            
        },
        
        clearReports: function(){
            dojox.xml.parser.removeChildren(this.usageReportView);
            dojox.xml.parser.removeChildren(this.denialReportView);
            dojox.xml.parser.removeChildren(this.comparisonReportView);
        },
        
        _fetchReports: function(){
            this.reportResponses = 0;
            var self = this;
            var licenseId = this.dateAndLicenseWidget.getLicenseId();
            var licenseInfo = this.dateAndLicenseWidget.getLicenseInfo(licenseId);
            this.startDate = this.dateAndLicenseWidget.getStartDate();
            this.endDate = this.dateAndLicenseWidget.getEndDate();
            
            dojo.forEach(REPORTS, function(report){
                var names = ["NAME"];
                var values = [licenseId];
                if (report.params) {
                    dojo.forEach(report.params, function(param){
                        if (param.name && param.calValue && licenseInfo[param.calValue]) {
                            names.push(param.name);
                            values.push(licenseInfo[param.calValue]);
                        }
                        if (param.name && param.formValue && self[param.formValue]) {
                            names.push(param.name);
                            values.push(self[param.formValue]);
                        }
                    });
                }				
				var args = this._prepareReportArgs(report.bundle,report.path,names,values,"false");
                self._invokeReport(args, report.callback);
            }, this);
            
        },
                                           
        
        VIEWS: {
            processUsageReport: "usageReportView",
            processDenialsReport: "denialReportView",
            processComparisonReport: "comparisonReportView"
        },
                								        
        processUsageReport: function(htmlReport){
            this.handleResponseCount();
            this._displayReportInfo(htmlReport, this[this.VIEWS["processUsageReport"]]);
        },
        
        processDenialsReport: function(htmlReport){
            this.handleResponseCount();
            this._displayReportInfo(htmlReport, this[this.VIEWS["processDenialsReport"]]);
        },
        
        processComparisonReport: function(htmlReport){
            this.handleResponseCount();
            this._displayReportInfo(htmlReport, this[this.VIEWS["processComparisonReport"]]);
        },
                        
        handleResponseCount: function(){
            this.reportResponses++;
            if (this.reportResponses == REPORTS.length) {
                this.onFetchComplete();
            }
        },    
		
		isValid : function(){
			return this.dateAndLicenseWidget.isValid();
		},
		
		uninitialize: function () {
			if(this.dateAndLicenseWidget) {
				this.dateAndLicenseWidget.destroyRecursive();
			}
			this.inherited(arguments);
	    } 

    });
    
    
    
})();
