/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("net.jazz.ajax.core.internal.util");
dojo.require("com.ibm.team.repository.web.client.internal.licensing");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");

var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
var getLicenseInfo = LIC_P.getLicenseInfo;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo", [dijit._Widget, dijit._Templated], {
	
	templateString: "<span class='com-ibm-team-repository-admin-floatingLicenseTypeCombo' dojoAttachPoint='licenseSelectionDiv'></span>",
	
	//callback functions
	onError: null,
	
	constructor: function(args){
	  this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");	
	},
	
	postCreate: function() {
		this._fetchAndDisplayTypeSelect();
	}, 
	
	_fetchAndDisplayTypeSelect: function () {
		var handler = {
			success: dojo.hitch(this, function(licenseInfo) {				
				this._displayCalTypesCombo(licenseInfo);
			}),
			error: dojo.hitch(this, function(errorObj) {				
				if (this.onError && typeof this.onError == "function") {
					this.onError(errorObj);
				}
			})
		};
		getLicenseInfo(new SRH(handler, "success", "error"));
	},
	
	_displayCalTypesCombo: function(info) {
		if(!info || !this.licenseSelectionDiv) return;
		
		this.floatingLicenses = info.floatingLicenseTypes || [];
		if (info.floatingLicenseTypes && info.floatingLicenseTypes.length > 0) {
			if (!this.licenseId) {
				this.licenseId = info.floatingLicenseTypes[0].id;				
			}
			this.licenseSelectionDiv.innerHTML = "";
			var licenseSelect = dojo.create("select", {}, this.licenseSelectionDiv);
			var self = this;
			var pos = 0;
			dojo.forEach(info.floatingLicenseTypes, function(cal) {
				dojo.create("option", { value: cal.id, innerHTML: cal.productName + " - " + cal.name }, licenseSelect);	
				if (cal.id === self.licenseId) {
					licenseSelect.selectedIndex = pos;
					self.licenseName = self.getLicenseName(self.licenseId);
				}
				pos++;
			});
			// inserts an 'All Licenses' choice into the selection if the parent has asked for it			
			if (this.allowAllLicensesChoice) {
				dojo.create("option", { value: this.messages.allLicenses, innerHTML: this.messages.allLicenses }, licenseSelect);
			}
			
			var handler = {
				changed: function() {
					self.licenseId = licenseSelect.value == self.messages.allLicenses ? null : licenseSelect.value;
					self.licenseName = self.getLicenseName(self.licenseId); 					
					if (self.onChange) {
						self.onChange(self.licenseId, self.getLicenseById(self.licenseId));	
					}
				}
			}
			this.connect(licenseSelect, "onchange", handler.changed);
			if (this.onLoad) {
				this.onLoad(this.licenseId, this.getLicenseById(this.licenseId));
			}
		} else {
			if (this.onLoad) {
				this.onLoad(null);
			}
		}
	}, 
	
	getLicenseById: function (calId){
		var result;
		if (!calId || !this.floatingLicenses) return null;
		dojo.forEach(this.floatingLicenses, function(cal) {
			if (cal.id === calId) {
				result = cal;
			}
		});
		return result;
	},
	 
    getLicenseName: function(calId){
		var result;
		if (calId) {
			dojo.forEach(this.floatingLicenses, function(cal) {
				if (cal.id === calId) {
					result = cal.name;
				}
			});
		}
		return result;
	},
	
	uninitialize: function () {
		this.inherited(arguments);
    }

});

})();
