/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dijit._Templated");
    dojo.require("jazz.ui.HoverPopup");
	dojo.require("jazz.ui.Wizard");
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep");
    dojo.require("dojo.i18n");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");

    var HoverPopup = jazz.ui.HoverPopup;
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var LicenseAgreementStep = com.ibm.team.repository.web.ui.internal.admin.server.LicenseAgreementStep;
	var LicenseAdminClient = com.ibm.team.repository.web.client.internal.licensing;
   	var messages;
	
	/*
	 *  LicenseHelper contains shared code used for UI components across various licensing pages
	 *  in the Admin UI. It is different from licensing.js which contains mostly utility functions
	 *  that do not have direct UI impact.  
	 * 
	 */
	
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseHelper", [dijit._Widget, dijit._Templated], {
        
		_hoverHelpLinks: [],
		constructor: function(){
            this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "LicenseEditorMessages");
        },
        
        postCreate: function(){
			return;
        },

		_getInfopopLink: function(label, info, className){
            var hoverLink = dojo.create("a", { href: "#", innerHTML: label, className: className || "infopop" });
            hoverLink.onclick = function(e){
                dojo.stopEvent(e);
				return false;
            };
            var tooltip = new HoverPopup({
                around: hoverLink,
                html: info,
                openDelay: 300,
                orientation: "R"
            }, dojo.create("span"));
			this._hoverHelpLinks.push(tooltip);
            return hoverLink;
        },
        
        _setInfopop: function(headerCell, label, info, className){
            dojo.empty(headerCell);
            var hoverLink = this._getInfopopLink(label, info, className);
            headerCell.appendChild(hoverLink);
        },
        
        _setInfopopHeading: function(headerCell, labelKey, infoKey){
            this._setInfopop(headerCell, this.messages[labelKey], this.messages[infoKey], "infopop infopop-faded");
        },
		
		_setPurchasesOrTrialTableHeader: function(headerCell){
            var isTrial = this.licenseInfo.serverLicense.trial;
            var labelKey = isTrial ? "trialHeader" : "purchasedHeader";
            var infoKey = isTrial ? "trialHeaderInfo" : "purchasedHeaderInfo";
            this._setInfopopHeading(headerCell, labelKey, infoKey);
        },
		
		// Manual Activation methods
		// Note: you must implement your own postActivateTrial() method
		
		_createActivateLink: function(type) {
			var aLink = dojo.create("a", { title: messages.activeLinkTitle, href: "#", className: "connectLink", innerHTML: messages.activeLinkTitle });
	        this.connect(aLink, "onclick", dojo.hitch(this, function(e) {
				dojo.stopEvent(e);
				if(!aLink._disabled) {
					aLink._disabled = true;
					//Prevent multiple clicks (#167172)
					//Re-enable the link again in case the user cancels the dialog or an error occurs
					setTimeout(function() {
						if(aLink) {
							aLink._disabled = false;
						}
					}, 1000);
					this._activeTrialLicense(type);
				}
			}));
			return aLink;
		},
		
		_activeTrialLicense: function(type) {
			var key = type;
			//Some properties are stored inside the key only. If available, using the key is preferred.			
			if(type.keys && type.keys.length == 1) {
				key = type.keys[0];
			}

			if(key.hasAgreement) {
				this._showAgreement(key, /* readOnly */ false);
			} else {
				var isExpiredTrial = LicenseAdminClient.hasExpiredTrial(key);
				if(!isExpiredTrial) {
					var durationOfTrial = LicenseAdminClient.getExpirationInfo(key.unactivatedTrialExpirationTime).daysRemaining;
					var msg = dojo.string.substitute(messages.confirmActivate, [ durationOfTrial ]);
					if(confirm(msg)) {
						this.postActivateTrial(key.id, /* hasAgreed */ true);
					}
				} else {
					//Trial is expired. Developer should have checked this before showing the activate link...
					alert(this.messages.expiredTrial);
				}
			}
		},
		
		//Helper methods for License Agreement dialogs
		
		_showAgreement: function(key, readonly) {  		  
		  var agreementHandler = {
			success: dojo.hitch(this, function(agreement){
			  	this._launchAgreementDialog(key, agreement, readonly);
			}),
			failure: dojo.hitch(this, function(err) {
				this._handleServiceError(err);
			})
		  }
		  var srh = new SRH(agreementHandler, "success", "failure");
		  LicenseAdminClient.getLicenseAgreement(srh, {
			"licenseId": key.id
		  });
		},
		
		_launchAgreementDialog: function(key, agreement , readOnly) {
			if(!key) return;
			this._destroyWizards();
			var self = this;
			var agreementCallback = dojo.hitch(this, "postActivateTrial");
			var sTitle = messages.licenseAgreementHeader;
			this.agreementStep = new LicenseAgreementStep({
				licenseKey: key,
				agreementText: agreement, 
				readOnly: readOnly,
				stepTitle: sTitle,
				onAgreement: agreementCallback
			});
			var wizardArgs = {
				steps: [this.agreementStep],
				width: "870px",
				primaryTitle: sTitle,
				customPrefix: "stepDOM_agreement_",
				fullPageContainer: false,
				showStepSummary: false,
				includeCancel: !readOnly,
				processFinish: function() {
					self.agreementWizard.contentArgs.container.close();
				},
				processCancel: function() {
					self.agreementWizard.contentArgs.container.close();
				}
			};
			this.agreementWizard = new jazz.ui.Wizard(wizardArgs);
		},
		
		_destroyInfoHovers: function() {
			// Info decorator hover popups
			if (this._hoverHelpLinks) {
				for (textID in this._hoverHelpLinks) {
					if (this._hoverHelpLinks[textID]) {
						this._hoverHelpLinks[textID].destroy();
						delete this._hoverHelpLinks[textID];
					}
				}
				delete this._hoverHelpLinks;
			}
		},
		
		_destroyWizards: function() {
			if (this.agreementWizard) {
				this.agreementWizard.destroyRecursive();
				delete this.agreementWizard;
			}
		},
		
		uninitialize: function() {
			this._destroyInfoHovers();
			this._destroyWizards();
			this.inherited(arguments);
		}
		
    });
    
})();
