/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType");
    
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
    dojo.require("dojox.xml.parser");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
    dojo.require("com.ibm.team.repository.web.ui.internal.common.TimestampWidget");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
    dojo.require("dojo.i18n");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
    
    var FloatingLicenseTypeCombo = com.ibm.team.repository.web.ui.internal.admin.server.FloatingLicenseTypeCombo;
	var util = com.ibm.team.repository.web.ui.internal.admin.util;
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.LicenseReportDateAndType", [dijit._Widget, dijit._Templated], {
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/LicenseReportDateAndType.html"),
        
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "FloatingLicenseReportViewMessages");
        },
        
        postCreate: function(){
            this.validDates = false;
            this._createForm();
        },
        
        _createForm: function(){
            var self = this;
            var args = {				
                onChangeFunction: "handleDateSelected",
                parent: self
            };
            this.dateWidget1 = new com.ibm.team.repository.web.ui.internal.common.TimestampWidget(args);
            this.startDatePicker.appendChild(this.dateWidget1.domNode);
            
            var args = {
				useCurrentDateIfDateIsNull: true,
                onChangeFunction: "handleDateSelected",
                parent: self
            };
            this.dateWidget2 = new com.ibm.team.repository.web.ui.internal.common.TimestampWidget(args);
            this.endDatePicker.appendChild(this.dateWidget2.domNode);
            
            
            var params = {
                licenseId: self.licenseId,
                allowAllLicensesChoice: this.allowAllLicensesChoice				
            }
            this.licenseWidget = new FloatingLicenseTypeCombo(params, this.licensePicker);
            this.connect(this.licenseWidget, "onLoad", "onLicenseLoad");
            this.connect(this.licenseWidget, "onError", "onError");
                        
            this._validateDates();
        },
        
		//
		// Validates the values of the 2 date widgets.  Also looks to see that the dates are not null
		//
        _validateDates: function(){
            var error = null;
            // Fix for 100822.  If the dates are null , then the reports run forever.            
            if (!this.dateWidget1.getDateValue()) 
                error = this.messages.startDateCannotBeNull;
            if (!error) {				
				var startDate = this.dateWidget1.getDateValue();
				// The end date can be null , it defaults to current date
				var endDate = this.dateWidget2.getDateValue();
                if (startDate && endDate && startDate > endDate) {
                    error = this.messages.dateError;
                }
            }
            this._setDateMessage(error);
        },
        		
		//
		// Receives notification that a date has changed
		// 
        handleDateSelected: function(date){
            this._validateDates();
        },
        
		//
		// Sets the date message area 			
		//
        _setDateMessage: function(msg){
			dojox.xml.parser.removeChildren(this.dateErrorMsg);            
            if (msg) {
                this.dateErrorMsg.appendChild(document.createTextNode(msg));
            }
            dojo.toggleClass(this.dateErrorMsg, "noDisplay", !msg);
        },
        
        //
        // Called when the license type combo has been filled
        // 
        onLicenseLoad: function(licenseId, licenseInfo){
        
        },
        
        //
        // Called when an error has occured fetching the floating license types
        //
        onError: function(errorObj){
        
        },
        
        getStartDate: function(){
            return this.dateWidget1.getBirtValue();
        },
        
        getEndDate: function(){
 			var endDate = this.dateWidget2.getDateValue();
			// add one to the end date so the query includes data up to the end of that day
			endDate.setDate(endDate.getDate() + 1);
			return util.getBirtDateValue(endDate);
        },
        
        getLicenseId: function(){
            return this.licenseWidget.licenseId;
        },
        
        getLicenseName: function(){
            return this.licenseWidget.licenseName;
        },
        
        getLicenseInfo: function(licenseId){
            return this.licenseWidget.getLicenseById(licenseId);
        },
        
        //
        // Checks the validity of this widget.  Simply checks to see if the 
        // error section is displayed.
        //										
        isValid: function(){
            return dojo.hasClass(this.dateErrorMsg, "noDisplay");
        },
		
		uninitialize: function () {
			if(this.dateWidget1) {
				this.dateWidget1.destroyRecursive();
			}
			if(this.dateWidget2) {
				this.dateWidget2.destroyRecursive();
			}
			if(this.licenseWidget) {
				this.licenseWidget.destroyRecursive();
			}

			this.inherited(arguments);
	    } 
    });            
})();
