/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.RegisteredApplications");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("jazz.ui.MessageBox");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("jazz.app.internal.util.Rootservices");
dojo.require("jazz.app.internal.applicationAbout");
dojo.require("jazz.ui.Dialog");
dojo.require("jazz.ui.wizard.WizardStep");
dojo.require("dojo.i18n");
dojo.require("com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog");
dojo.require("com.ibm.team.repository.web.ui.internal.common.Toolbar");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService");
dojo.require("jazz.util.binding");
dojo.require("jazz.app.ua.TextHoverHelp");
dojo.require("jazz.app.ua.PageHoverHelp");
dojo.require("dojox.xml.parser");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");

(function() {
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.admin.server.";
var messages;
var MessageBox = jazz.ui.MessageBox;
var STATUS_TYPES = MessageBox.STATUS_TYPES;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WizardStep = jazz.ui.wizard.WizardStep;
var TeamContributorSelectionDialog = com.ibm.team.repository.web.ui.internal.TeamContributorSelectionDialog;
var Toolbar = com.ibm.team.repository.web.ui.internal.common.Toolbar;
var Rootservices = jazz.app.internal.util.Rootservices;
var applicationAbout = jazz.app.internal.applicationAbout;
var AboutServiceSchema = com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService;
var binding = jazz.util.binding;
var TextHoverHelp = jazz.app.ua.TextHoverHelp;
var PageHoverHelp = jazz.app.ua.PageHoverHelp;

var JFS_NS = "http://jazz.net/xmlns/prod/jazz/jfs/1.0/";
var ADMIN_WEB_UI = "adminWebUI";
var SETUP_WIZARD_DESCRIPTOR = "setupWizardDescriptor";

dojo.declare(PACKAGE_PREFIX + "RegisteredApplications", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/RegisteredApplications.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
		this.stepTitle = messages.pageTitle;
		this.oauthFormData = {};
	},
	
	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		//Append the context sensitive help
		this._pageHelpHover = new PageHoverHelp({ uaKey: "com.ibm.jazz.repository.web.admin.ui.regApps" });
		this.helpHover.appendChild(this._pageHelpHover.domNode);
		
		//So we can destroy them later
		this._actionToolbars = new Array();
			
		this.reloadFriends();
	}, 
	
	reload: function () {
		this.header.hideMessage();
		this.reloadFriends();
	},
	
	reloadFriends: function () {
		this.getFriends();
	},
		
	errorCallback: function (errorObj, contextMsg) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: contextMsg
		});
	},

	setFormStatus: function (status) {
		if (status === -1) {
			this.header.hideMessage();
		}
		else {
			this.header.showMessage({
				summary: status.message,
				type: status.type
			});
		}
	}, 

	getFriends: function (callback) {
		var srh = new SRH(this, callback || "displayFriends", "errorFetchingFriends");
		OAuthAdminClient.getFriends(srh, { includeInternal: true });
	},
	
	displayFriends: function (friends) {
		if(!this.domNode) return;
		dojo.empty(this.applicationsTableBody);
		this.showLoadingRow();
		if (friends){
			this.friendList = friends;
			if(this.friendList.length > 1) {
				this.sortFriends();
			}	
			var separatedFriends = this.separateFriendsByInternal(this.friendList);
			var internalFriends = separatedFriends.internalFriends;
		} 
		
		//Get the Application Abouts so we can get Application Types
		var abouts = this.latestAbouts ? this.latestAbouts : applicationAbout.get();
		
		//Add "internal friends". i.e. applications
		if(!internalFriends || internalFriends.length <= 0) {
			this.showNoApplicationsRow();
			this.header.hideMessage();
		} else {
			dojo.forEach(internalFriends, function(friend){
				for(i in abouts) {
					if(friend && friend.scrURL) {
						var pos = friend.scrURL.indexOf(abouts[i].contextRoot + "/"); //add trailing slash for matching same-prefix roots like jazz jazz02
						if(pos !== -1) {
							friend.applicationName = abouts[i].title;
							friend.description = abouts[i].description;
							friend.version = abouts[i].version;
							friend.identifier = abouts[i].identifier;
							break;
						}
					}
				}
				this.addApplicationListRow(friend, /*allowDelete*/ true);
            }, this);
		}
	},
	
	separateFriendsByInternal: function (friends) {
		var internalFriends = [];
		var publicFriends = [];
		dojo.forEach(friends, function(friend){
        	if (friend.internal && friend.internal == true){
				internalFriends.push(friend);
			} else {
				publicFriends.push(friend);
			}
       	});
		return {internalFriends: internalFriends, publicFriends: publicFriends}	
	},
	
	errorFetchingFriends: function (errorObj) {
		this.errorCallback(errorObj, this.messages.fetchFriendsError);
	},
	
	sortFriends: function () {	
		return this.friendList.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	addApplicationListRow: function(friend, allowDelete) {	
		if(!this.applicationsTableBody || !friend) return;
		
		var row = dojo.create("tr", { id: "row-" + friend.oAuthConsumerKey, className: "hasAction" });
		var self = this;
		var cell1 = dojo.create("td", {}, row);
		cell1.appendChild(document.createTextNode(friend.title));
		
		//Add the Applicatin Type and a hoverPopup for this with the friend.description
		var cell2 = dojo.create("td", { }, row);
		var appTypeCell = dojo.create("span", { innerHTML: friend.applicationName || this.messages.unknown }, cell2);
		if(friend.applicationName && friend.description) {
			new TextHoverHelp({
				helpContentString: friend.description,
				contentNode: appTypeCell
			});	
		}
		
		var versionTd = dojo.create("td", { }, row);
		versionTd.appendChild(document.createTextNode(friend.version || " "));

		friend.scrURL = friend.scrURL || ""; //if it's empty the value comes back as undefined. Causes issues.
		var displaySRC = friend.scrURL == "" ? "-" : friend.scrURL;
		dojo.create("td", { innerHTML: displaySRC }, row);
		dojo.create("td", {innerHTML: friend.oAuthConsumerKey }, row);

		//TODO: Actually implement a status call to get App status. 
		var icon_td = dojo.create("td");
		var icon = dojo.create("div", { className: "icon-ok"}, icon_td); 
		row.appendChild(icon_td);
		dojo.create("td", { innerHTML: this.messages.installed, className: "no-wrap" }, row);

		//Use the RootServices document for the AdminWebUI service
		var settings_td = dojo.create("td", { className: "no-wrap" }, row);
		var settingsLink = dojo.create("a", { className: "disabledLink", innerHTML: this.messages.settings}, settings_td);
		if(friend.rootServicesURL && friend.rootServicesURL != "") {
			var deferred = this._getAdminLink(friend.rootServicesURL);	
			deferred.addCallback(function(url) {
				if(url) {
					settingsLink.href = url;
					settingsLink.title = self.messages.manageLink;
					dojo.removeClass(settingsLink,"disabledLink");
				}
			});
		}
		
		var actions = dojo.create("td", {}, row);
		var toolbar = new Toolbar();
		this._actionToolbars.push(toolbar);
		if (allowDelete) {
			var deleteAction = {};
			deleteAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/delete_hov.gif");
			deleteAction.label = this.messages.removeApplication;
			deleteAction.callbackData = {
				key: friend.oAuthConsumerKey
			};
			var callback = function(e){
				dojo.stopEvent(e);
				this._deleteApplication(e.currentTarget.callbackData.key);
			};
			toolbar.addButton(this, callback, deleteAction);
		}
		var editAction = {};
		editAction.imgSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/images/edit-ellipses.gif");
		editAction.label = this.messages.editApplication;
		editAction.callbackData = {
			friend: friend
		};
		var callback = function(e){
			dojo.stopEvent(e);
			this._launchEditFriend(e.currentTarget.callbackData.friend);
		};
		toolbar.addButton(this, callback, editAction);
		actions.appendChild(toolbar.domNode);
		
		if(this.loadingRow) {
			this.applicationsTableBody.removeChild(this.loadingRow);
			this.loadingRow = null;
		}
		this.applicationsTableBody.appendChild(row);		
	}, 
	
	//Makes a *passive* call to get the RootServices doc, so it fails quitely. 
	_getAdminLink: function(rootServicesURL) {		
		var deferred = new dojo.Deferred();
		var rootDeferred = Rootservices.load(rootServicesURL, /* passive */ true);
		rootDeferred.addCallback(function(root){
			var adminURL = root.getServiceUrl(JFS_NS, ADMIN_WEB_UI);
			deferred.callback(adminURL);
		});
		return deferred;
	},
	
	_deleteApplication: function(key) {
		var verifyDelete = confirm(this.messages.confirmApplicationDelete);
        if (verifyDelete) {
			var app = this._getFriendForKey(key);
			if(app) {
				this._keyPendingDeletion = key;
				OAuthAdminClient.postUnregisterApp(new SRH(this, "removeAppRow", "errorDeletingFriend"), {title: app.title});
			}
		}
	},
	
	_getFriendForKey: function(key) {
		if (this.friendList) {
			for (friend in this.friendList) {
				if (this.friendList[friend].oAuthConsumerKey === key) {
					return this.friendList[friend];
				}
			}
		}
		return null;
	},
	
	removeAppRow: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.applicationsTableBody.removeChild(elem);
		if (this.applicationsTableBody.childNodes.length == 0) {
			this.showNoApplicationsRow();
		}
		//Update the Application About
		var deferred = applicationAbout.refresh();
		deferred.addCallback(dojo.hitch(this, function(abouts) {			
			this.latestAbouts = abouts;
		}));
	}, 
	
	showNoApplicationsRow: function() {
		var url = "<a id=\"_addAppLink\" class=\"editLink\">" + this.messages.addApplicationLink + "</a>";
		var msg = dojo.string.substitute(this.messages.noAppsYet, [url]);
		var row = dojo.create("tr");
		dojo.create("td", {className: "noRowsYet", colSpan: 9, innerHTML: msg} , row);
		dojo.empty(this.applicationsTableBody);
		this.applicationsTableBody.appendChild(row);
		this.connect(dojo.byId("_addAppLink"), "onclick", "_launchApplicationWizard");
	},
	
	showLoadingRow: function() {
		this.loadingRow = dojo.create("tr");
		dojo.create("td", {className: "noRowsYet", colSpan: 9, innerHTML: this.messages.loadingApplications} , this.loadingRow);
		dojo.empty(this.applicationsTableBody);
		this.applicationsTableBody.appendChild(this.loadingRow);
	},
	
	errorDeletingFriend: function (errorObj) {
		var contextMsg = dojo.string.substitute(this.messages.deleteFriendError, [this._keyPendingDeletion]);
		this.errorCallback(errorObj, contextMsg);
	}, 
	
	_launchEditFriend: function (friend) {
		this._destroyWizards();
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.EditApplicationStep({
			stepTitle: this.messages.editApplicationDialogTitle,
			friend: friend, 
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			width: "745px",
			primaryTitle: this.messages.editApplicationHeading,
			showStepSummary: false,
			processFinish: function() {
				this.wizardFinishButton.disabled = true;
				this._steps[0].saveUpdates();
			},
			processCancel: function() {	
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}				
			}
		}
		this.wizard = new jazz.ui.Wizard(wizardArgs);
					
	},	
		
	_launchApplicationWizard: function() {	
		this._destroyWizards();
		var caller = this;
		var step1 = new com.ibm.team.repository.web.ui.internal.admin.server.AddApplicationStep({
			stepTitle: this.messages.addApplicationStepHeader,
			caller: caller
		});

		var wizardArgs = {
			steps: [step1],
			primaryTitle: this.messages.addApplicationStepHeader,
			width: "745px",
			showStepSummary: false,
			processFinish: function() {
				this.wizardFinishButton.disabled = true;
				this._steps[0].saveUpdates();
			},
			processCancel: function() {		
				if (this._steps[0].hasUpdates()) {
					var verifyCancel = confirm(caller.messages.confirmEditCancel)
					if (verifyCancel) {
						this.container.viewBox.close();
					}
				} else {
					this.container.viewBox.close();
				}	
			}
		}
		this.addApplicationWizard = new jazz.ui.Wizard(wizardArgs);
	},
	
	_handleRegistrationSuccess: function(friends) {
		var rootServicesURL;
		dojo.forEach(friends, function(friend) {
			if(friend.id === this.newAppId) {
				rootServicesURL = friend.rootServicesURL;
				return;
			}
		}, this);
		
		//Look for a wizard descriptor and see if we can link to the app's setup wizard
		if(rootServicesURL) {
			var setupURL;
			var showCompleted = dojo.hitch(this, this.showCompletedMessage);
			var deferred = this._getWizardDescriptor(rootServicesURL);
			deferred.addCallback(function(response) {
				if(response && response.baseSetupURL) {
					var setupUrlLink = "<a href=\"" + response.baseSetupURL + "\">";
					var msg = dojo.string.substitute(messages.registrationCompleteWithSetup, [ setupUrlLink, "</a>" ]);
					showCompleted(msg);
				} else {
					//Show message w/o Setup link
					showCompleted();
				}
			});	
			deferred.addErrback(function(err) {
				showCompleted();
			});
		} else {
			this.showCompletedMessage();
		}
		this.displayFriends(friends);
	},
	
	_getWizardDescriptor: function(url) {
		var self = this;
		var errback = dojo.hitch(this, this.showCompletedMessage);
		var deferred = new dojo.Deferred();
		var rootDeferred = Rootservices.load(url, /* passive */ true);
		rootDeferred.addCallback(function(root){
			var wizardDescriptorURL = root.getServiceUrl(JFS_NS, SETUP_WIZARD_DESCRIPTOR);
			if(wizardDescriptorURL) {
				jazz.client.xhrGet({
					url: wizardDescriptorURL,
					handleAs: "json",
					headers: self.headers,
					load: function(response) {
						deferred.callback(response);
					},
					error: errback
				});	
			} else {
				deferred.callback(false);
			}
		});		
		rootDeferred.addErrback(function(arguments) {
			//Can't find the Rootservices document for that app.
			deferred.errback(arguments);
		});
		return deferred;
	},
	
	showCompletedMessage: function(msg) {
		this.header.showMessage({
			type: "OK",
			summary: msg || this.messages.registrationComplete,
			closable: true
		});	
	},
	
	_destroyWizards: function() {
		if (this.wizard) {
			this.wizard.destroyRecursive();
			delete this.wizard;
		}
		if (this.addApplicationWizard) {
			this.addApplicationWizard.destroyRecursive();
			delete this.addApplicationWizard;
		}
	},
	
	destroyActionToolbars: function(){
		for (var i = 0; i < this._actionToolbars.length; i++) {
			this._actionToolbars[i].destroyRecursive();
		}
		this._actionToolbars = [];
	},
	
	uninitialize: function() {
		this.header.destroyRecursive();	
		this._pageHelpHover.destroyRecursive();
		this._destroyWizards();	
		this.destroyActionToolbars();
		this.inherited(arguments);
	}
	
});


dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.EditApplicationStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/EditApplication.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		// Not using handleKeyUp with timeout because the validate is a simple check that there is content.
		// This way the user gets the Finish button much faster, but there are no errors to worry about.
		this.connect(this._editAppTable,"onkeyup","validateChange");
		this.requiredInputs = [
			this._titleInput
		]	
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});
		this._titleInput.value = this.friend.title;
	}, 
	
	validate: function() {
		//Calls out to the generic validate
		return validate(this);
	},

	validateChange: function () {
		this.triggerButtonStateUpdate();
	},
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var args = {
			id: this.friend.id
		};
		
		if (this.friend.scrURL != this._scrInput.value) {
			args.rootServices = this._scrInput.value;
		}
		if (this.friend.title != this._titleInput.value) {
			args.title = this._titleInput.value;
		}
		
		this.header.showMessage({
			summary: this.messages.savingUpdates,
			type: "INFO"
		});
		
		OAuthAdminClient.updateFriend(new SRH(this, "handleFriendUpdateSuccess", "serviceError"), args);
	},
		
	handleFriendUpdateSuccess: function () {
		this.caller.header.hideMessage();
		this.wizard.container.viewBox.close();

		//Refresh the applicationAbout service to get the latest data
		var deferred = applicationAbout.refresh();
		deferred.addCallback(dojo.hitch(this, function(abouts) {
			this.latestAbouts = abouts;
			this.caller.reloadFriends();
		}));	
		deferred.addErrback(dojo.hitch(this, function(err) {
			this.caller.reloadFriends();
		}));
	},
	
	serviceError: function(errorObj) {
		this.header.showMessage({
			summary: this.messages.errorUpdating,
			errorObject: errorObj
		});
		this.wizard.wizardFinishButton.disabled = false;
	},
	
	canFinish: function(){
		return this.validate() && this.hasUpdates();
	}, 
	
	hasUpdates: function () {
		if (this.friend.scrURL != this._scrInput.value) {
			return true;
		}
		if (this.friend.title != this._titleInput.value) {
			return true;
		}
		return false;
	},
	
	uninitialize: function() {
		this.header.destroyRecursive();
		this.inherited(arguments);
	}

});

/*
 * Wizard Step for adding new "Internal" Applications to the Application Group
 */

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.AddApplicationStep", [WizardStep, dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/AddApplication.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	

	postCreate: function () {
		this.header = new MessageBox();
		this.stepHeaderArea.appendChild(this.header.domNode);
		this.connect(this._addTableBody,"onkeyup","handleKeyUp");
		this._hoverNodes = new Array();
		this.requiredInputs = [
			this._title,
			this._secretAgain
		]
		var self = this;
		dojo.forEach(this.requiredInputs, function(elem) {
			if(elem) {
				self.connect(elem,"onchange","validateChange");
			}
		});
		
		this.connect(this._selectUserLink,"onclick", function (e) {
			dojo.stopEvent(e);
			this._userPicker = new TeamContributorSelectionDialog({ allowMultiple : false, hideAdminGuest: true });
			this.connect(this._userPicker,"onAdd","associateUser");
		});
		
		this.connect(this._selectedConsumerName, "onkeyup", "_handleFunctionalUserChange");
		this.connect(this._selectedConsumerName, "onchange", "_handleFunctionalUserChange");
		this.connect(this._srcURI, "onblur", "_handleScrChange");
		
		//Add text hover help to the form
		
		this._hoverNodes.push(new TextHoverHelp({
			helpContentString: this.messages.help_consumerSecret,
			contentNode: this.help_consumerSecret
		}));
		this._hoverNodes.push(new TextHoverHelp({
			helpContentString: this.messages.help_functionalId,
			contentNode: this.help_functionalId
		}));
	},

	associateUser: function (newUsers) {
		if (newUsers != null && newUsers[0] != null) {
			this._selectedConsumerName.value = newUsers[0].userId;
			this._handleFunctionalUserChange();
		}
	},
	
	_handleScrChange: function() {
		if (this._srcURI.value) {
			this._getPublisherUrl(this._srcURI.value);
		} else {
			this._appType.innerHTML = this.messages.unknown;
		}
	},
	
	_getPublisherUrl: function(scrUrl) {
		if(!scrUrl) return;
		jazz.client.xhrGet({
			url: scrUrl,
			handleAs: "xml",
			load: dojo.hitch(this, function(doc) {
				var parsed = binding.read(doc.documentElement, AboutServiceSchema.application);
				if(parsed && parsed.Application) {
					this._getApplicationAbout(parsed.Application.publisher);
				} else {
					this._appType.innerHTML = this.messages.unknown;
				}
			}),
			error: dojo.hitch(this, function(err) {
				if(err.status == 403) {
					this._getAboutDetails(scrUrl);
				} else {
					this._appType.innerHTML = this.messages.unknown;
				}
			})
		});
	},
	
	_getApplicationAbout: function(publisherUrl) {
		if(!publisherUrl) {
			this._appType.innerHTML = this.messages.unknown;
			return;
		}
		this._setPendingAboutRequest(true);
		jazz.client.xhrGet({
			url: publisherUrl,
			handleAs: "xml",
			headers: this.headers,
			load: dojo.hitch(this, function(doc) {
				var parsed = binding.read(doc.documentElement, AboutServiceSchema.aboutService);
				this._updateAppInfo(parsed);
			}),
			error: dojo.hitch(this, function(error) {
				this._handleAboutFailure(error);
			})
		});
	},
	
	//Get additional details about the application if the Web UI proxy fails with a 403. (For distributed setups)
	_getAboutDetails: function(scrUrl) {
		this._setPendingAboutRequest(true);
		var handler = {
			success: dojo.hitch(this, function(response) {
				if(response) {
					var doc = dojox.xml.parser.parse(response);
					var parsed = binding.read(doc.documentElement, AboutServiceSchema.aboutService);
					this._updateAppInfo(parsed);
				} else {
					//No About document specified in the SCR
					this._updateAppInfo(null);
				}
			}),
			failure: dojo.hitch(this, function(error) {
				this._handleAboutFailure(error);
			})
		};
		var srh = new SRH(handler, "success", "failure");
		OAuthAdminClient.getAboutServiceContent(srh, { url: scrUrl });
	},
	
	_setPendingAboutRequest: function(value) {
		this.pendingAboutRequest = value;
		if(value) {
			this.header.showMessage({
				type: "INFO",
				summary: this.messages.loadingAbout
			});	
		} else {
			this.header.hideMessage();
		}
		this.triggerButtonStateUpdate();
	},
	
	_handleAboutFailure: function(errorObj) {
		this._appType.innerHTML = this.messages.unknown;
		this._appType.hasError = true;
		this._setPendingAboutRequest(false);
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.errorFetchingAbout
		});
	},
	
	_updateAppInfo: function(parsed) {
		if(parsed && parsed.About) {
			if(parsed.About.identifier) {
				var parts = parsed.About.identifier.split("application/");
				if(parts && parts[1]) {
					this._selectedConsumerName.value = parts[1] + "_user";
					this._handleFunctionalUserChange();
				}
			}
			if (parsed.About.title) {
				this._appType.innerHTML = parsed.About.title;
				if (parsed.About.description) {
					this._hoverNodes.push(new TextHoverHelp({
						helpContentString: parsed.About.description,
						contentNode: this._appType
					}));
				}
			} else {
				this._appType.innerHTML = this.messages.unknown;
			}
		} else {
			this._appType.innerHTML = this.messages.unknown;
		}
		this._appType.hasError = false;
		this._setPendingAboutRequest(false);
	},
	
	_handleFunctionalUserChange: function() {
		if (this._selectedConsumerName.value) {
			if (this._selectedConsumerName.value === "ADMIN") {
				this.header.showMessage({
					type: STATUS_TYPES.ERROR,
					summary: this.messages.errorFunctionalAdminNotAllowed
				});
				this._invalidFunctionalUser = true;
			} else {
				this._invalidFunctionalUser = false;
			}
		} else {
			this._invalidFunctionalUser = false;
		}
		this.validateChange();
	},
	
	saveUpdates: function () {
		var isValid = this.validate();
		if(!isValid) return;
		
		var newUser = this._selectedConsumerName.value;
		var app = {
			title: this._title.value,
			oauthConsumerSecret: this._secret.value,
			rootServices: this._srcURI.value
		}	
		if(newUser) {
			app.oauthUserId = newUser;
		}
		
		this.header.showMessage({
			summary: this.messages.registeringApplication,
			type: "INFO"
		});
		
		var srh = new SRH(this, "handleCreateAppSuccess", "handleCreateAppError");
		OAuthAdminClient.postRegisterApp(srh, app);
	},
	
	handleCreateAppSuccess: function(newAppId) {
		this.wizard.container.viewBox.close();
		this.caller.newAppId = newAppId;
		
		//Refresh the applicationAbout service to get the latest data
		var deferred = applicationAbout.refresh();
		deferred.addCallback(dojo.hitch(this, function(abouts) {
			this.latestAbouts = abouts;
			this.caller.getFriends("_handleRegistrationSuccess");
		}));	
		deferred.addErrback(dojo.hitch(this, function(err) {
			this.caller.getFriends("_handleRegistrationSuccess");
		}));
	},
	
	handleCreateAppError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj,
			summary: this.messages.postNewApplicationError
		});
		this.wizard.wizardFinishButton.disabled = false;
	},
	
	hasUpdates: function() {
		var fieldsNotEmpty = false;
		dojo.forEach(this.requiredInputs, function(input) {
			if(isInputEmpty(input) == false) {
				fieldsNotEmpty = true;
				return;
			}
		});
		return fieldsNotEmpty;
	},
	
	validateChange: function () {
		this.triggerButtonStateUpdate();
	},
	
	validate: function() {
		var secretsMatch = this._secret.value == this._secretAgain.value ? true : false;
		dojo.toggleClass(this.mismatchErr, "noDisplay", secretsMatch);
		
		//Two-stage validation
		if(!this._invalidFunctionalUser && secretsMatch && !this._appType.hasError && !this.pendingAboutRequest) {
			return validate(this);
		} else {
			return false;
		}
	},
	
	serviceError: function(errorObj) {
		this.header.showMessage({
			errorObject: errorObj
		});
	},
	
	canFinish: function() {
		return this.validate() && this.hasUpdates();
	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validateChange);
			var timeout = 1000;
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	},
	
	uninitialize: function() {
		this.header.destroyRecursive();
		// TextHoverHelp Nodes
		if (this._hoverNodes) {
			for (textID in this._hoverNodes) {
				if (this._hoverNodes[textID]) {
					this._hoverNodes[textID].destroy();
					delete this._hoverNodes[textID];
				}
			}
			delete this._hoverNodes;
		}
		if(this._userPicker) {
			this._userPicker.destroyRecursive();
		}
		this.inherited(arguments);
	}

});

function isInputEmpty(input) {
	return input.value === "";
}

function validate(widget) {
	//Make sure none of the required input fields is blank
	var isValid = true;
	dojo.forEach(widget.requiredInputs, function(elem) {
		if(elem && elem.value ==  "") {
			isValid = false;
			return; //doesn't return out of the function, only the forEach
		} 
	});
	if(isValid) {
		widget.header.hideMessage();			
		return true;
	} else {
		return false;
	}		
}

})();