/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.SendTestEmailDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.form.Button");
dojo.require("dijit.form.TextBox");
dojo.require("dojo.i18n");
dojo.require("dojo.string");
dojo.require("jazz.ui.MessageBox");
dojo.require("jazz.util.ui");
dojo.require("com.ibm.team.repository.web.ui.internal.Util");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SendTestEmailDialogMessages");


(function() {

var MIN_BUTTON_WIDTH = 52;
var Util = com.ibm.team.repository.web.ui.internal.Util;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.SendTestEmailDialog", [dijit._Widget, dijit._Templated], {
		
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/SendTestEmailDialog.html"),
	widgetsInTemplate: true,
	
	constructor: function (args) {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "SendTestEmailDialogMessages");
	},
	
	postCreate: function () {
		this.header = new jazz.ui.MessageBox();
		this.headerArea.appendChild(this.header.domNode);
		this._updateButtonWidth();
		this.dialog = new jazz.ui.Dialog({
			contentNode: this.contentNode,
			primaryTitle: this.messages.sendTestEmail,
			// FF can autosize the dialog but IE can't
			width: dojo.isIE ? "25em" : "" //$NON-NLS-1$
		});
		this.connect(this.cancelButton, "onclick", "_cancelClicked");
		this.connect(this.sendButton, "onclick", "_sendClicked");
	},

	_cancelClicked: function(event) {
		dojo.stopEvent(event);
		this.close();
	},

	_sendClicked: function(event) {
		dojo.stopEvent(event);
		var regex = new RegExp(Util.emailAddressRegex());
		var validEmail = regex.exec(this.emailAddress.value);
		if (validEmail) {
			this.clearErrorMessage();
			if (this.onSend && typeof this.onSend == "function") {
				this.onSend()
			}
			this.sendButton.disabled = true;
			this.showInfoMessage(this.messages.sendingTestEmail);
		}
		else {
			this.showErrorMessage(this.messages.invalidEmailAddress);
			this.emailAddress.focus();
		}
	},

	showErrorMessage: function (msg) {
		this.header.showMessage({
			summary: msg,
			type: jazz.ui.MessageBox.STATUS_TYPES.ERROR
		});
	},

	showInfoMessage: function (msg) {
		this.header.showMessage({
			summary: msg,
			type: jazz.ui.MessageBox.STATUS_TYPES.INFO
		});
	},

	clearErrorMessage: function() {
		this.header.hideMessage();
	},
	
	close: function () {
		this.dialog.close();
	},
	
	_updateButtonWidth: function() {
		var tester = dojo.create("div", { style: { fontSize: "1em" }}, document.body);
		var newWidth = jazz.util.ui.findMaxStringDisplayWidth([this.messages.cancel, this.messages.send], tester, MIN_BUTTON_WIDTH);
		// add 23 pixels to accommodate button borders and margin
		var newWidthStyle = (newWidth + 23) + "px";
		this.sendButton.style.width = newWidthStyle;
		this.cancelButton.style.width = newWidthStyle;
		document.body.removeChild(tester);
	},
	
	uninitialize: function() {
		if(this.dialog) {
			this.dialog.destroyRecursive();
		}
	}
});

})();