/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function(){
    dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus");
    dojo.require("dijit._Widget");
    dojo.require("dijit._Templated");
	dojo.require("dijit.TitlePane");
    dojo.require("dojo.i18n");
    dojo.require("jazz.ui.MessageBox");
	dojo.require("jazz.util.date");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
    dojo.require("com.ibm.team.repository.web.client.internal.ServerStatusClient");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.DataTableContainer");
    dojo.require("com.ibm.team.repository.web.ui.internal.admin.util");
    dojo.require("com.ibm.team.repository.web.client.internal.licensing");
    dojo.require("net.jazz.ajax.contextRoot");
    dojo.require("net.jazz.ajax.ui.PlatformUI");
	dojo.require("net.jazz.ajax.BootstrapProperties");
    dojo.require("jazz.ui.HoverPopup");
    dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
	dojo.require("com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil");
    
    dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerStatusMessages");
    
    var MessageBox = jazz.ui.MessageBox;
    var STATUS_TYPES = MessageBox.STATUS_TYPES;
    var util = com.ibm.team.repository.web.ui.internal.admin.util;
    var DataTableContainer = com.ibm.team.repository.web.ui.internal.admin.DataTableContainer;
    var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
    var ServerStatusClient = com.ibm.team.repository.web.client.internal.ServerStatusClient;
	var STATUS_OK_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/okay_obj.gif");
	var STATUS_ERR_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/error_obj.gif");
	var STATUS_WARN_IMG_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "graphics/icons/warn_obj.gif");
    var LIC_P = com.ibm.team.repository.web.client.internal.licensing;
    var getLicenseInfo = LIC_P.getLicenseInfo;
    var getExpirationInfo = LIC_P.getExpirationInfo;
    var PlatformUI = net.jazz.ajax.ui.PlatformUI;
    var HoverPopup = jazz.ui.HoverPopup;
    var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;
	var PUBLIC_URL_SERVICE_NAME = "com.ibm.team.repository.service.internal.BaseServerDescriptionService";
	var PUBLIC_URL_NAME = "server.webapp.url";
	var JTS_PRODUCT_ID = "com.ibm.team.jazz.foundation.server";
	var BootstrapProperties = net.jazz.ajax.BootstrapProperties;
	var DiagnosticsUtil = com.ibm.team.repository.web.ui.internal.admin.server.DiagnosticsUtil;
	var LIMIT_SERVICE_ERRORS = 10; //default number of service errors to show
    
    dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.ServerStatus", [dijit._Widget, dijit._Templated], {
    
        templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/ServerStatus.html"),
		_dataTables: [],
		_tooltips: [],
        
        constructor: function(){
            this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "ServerStatusMessages");
            this.rssFeedTitle = this.messages.rssFeedTitle;
            this.JAZZ_ROOT = net.jazz.ajax.contextRoot();
        },
        
        postCreate: function(){
            this.header = new MessageBox();
            this.headerArea.appendChild(this.header.domNode);
			this._pendingResponseCount = 0;
			
			this.diagUtil = new DiagnosticsUtil();
			
			this.connect(this.showAllLink, "onclick", "_showAllErrors");
			
			//See if we are running as an Application, or on the JTS/Standalone
			this.serverMode = dojo.getObject("com.ibm.team.repository.server.mode");
			this.isJTS = (this.serverMode == "APPLICATION") ? false : true;
			
			this.checkClusterState();
        },
		
		checkClusterState: function() {
			var state = dojo.getObject("com.ibm.team.repository.isInCluster");
			if (state === true) {
				this.isCluster = true;
				this.getClusterInformation();
			} else if (state === false) {
				this.isCluster = false;
				this.reload();
			} else { //not set, usually happens when user refreshes this page directly
				this._getClusterStatus();
			}
		},
		
		_getClusterStatus: function() {
			var handler = {
				success: dojo.hitch(this, function(response) {
					dojo.setObject("com.ibm.team.repository.isInCluster", response === "false" ? false : true);
					this.checkClusterState();
				}),
				error: dojo.hitch(this, function() {
					dojo.setObject("com.ibm.team.repository.isInCluster", false);
					console.error("Unable to determine if the server is running in a cluster. Defaulting to standalone mode.");
					this.checkClusterState();	
				})
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
			ServerStatusClient.getIsInCluster(srh);
		},
			
		getClusterInformation: function() {
			var srh = new ServiceResponseHandler(this, "handleClusterInfo", "handleClusterError");
            ServerStatusClient.getClusterMembers(srh);
		},
		
		handleClusterInfo: function(nodelist) {
			this.clusterNodes = nodelist || [];
			this.reload();
		},
		
		handleClusterError: function(error) {
			console.error("Unable to get Cluster status. Assuming no cluster.");
			this.reload();
		},
        
        reload: function(){
            this.header.hideMessage();           
			if(this._pendingResponseCount == 0) {
				if(!this.isJTS) {
					var srh= new ServiceResponseHandler(this, "displayJTSInfo", "handleCantGetJTSURL");
					ServerStatusClient.getConfiguredJazzTeamServerURI(srh);
				} else {
					this._fetchAndDisplayServerHealthInfo();
				}
			}
        },

        _fetchAndDisplayServerHealthInfo: function() {
			this._emptyIfDomExists(this.serverStatusArea);
            this.handleLoadingData();
            var srh = new ServiceResponseHandler(this, "displayServerHealthInfo", "handleServerHealthErrorReponse");
            ServerStatusClient.getServerInfo(srh);
        },
        
        _fetchAndDisplayLicenseInfo: function(){
			this._emptyIfDomExists(this.licenseSummaryArea);
            this.handleLoadingData();
            var srh = new ServiceResponseHandler(this, "displayLicenseInfo", "handleLicenseErrorReponse");
            getLicenseInfo(srh);
        },
        
        _fetchAndDisplayProductInfo: function(){
			this._emptyIfDomExists(this.productInfoArea);
            this.handleLoadingData();
            var srh = new ServiceResponseHandler(this, "displayProductInfo", "handleProductErrorResponse");
            ServerStatusClient.getAllProductInfo(srh);
        },
        
        _fetchAndDisplayDatabaseInfo: function(){
			this._emptyIfDomExists(this.additionalDBInfoArea);
            this.handleLoadingData();
            var srh = new ServiceResponseHandler(this, "displayDatabaseStatusInfo", "handleDatabaseErrorReponse");
            ServerStatusClient.getDatabaseStatus(srh);
        },
        
        _fetchAndDisplayServiceSummaryInfo: function(){
			if(!this.domNode) return;
			this.allComponentErrors = {};
			dojo.removeClass(this.serviceStatusSummary, "noDisplay");
			if(this.clusterNodes && this.isCluster) {
				this.pendingComponentRequests = this.clusterNodes.length;
				dojo.forEach(this.clusterNodes, function(node) {
					this._getComponentDetails(node.humanReadableNodeId);
				}, this);
			} else {
				this._getComponentDetails();
			}
        },
		
		_getMigrationStatus: function() {
			var handler = {
				success: dojo.hitch(this, function(isMigration) {
					if(isMigration === "true") {
						this.header.showMessage({
		                    summary: this.messages.migrationMode,
							type: "INFO"
		                });
					}
				}),
				error: function(err) {
					//no need to do anything, not critical
				}
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
            ServerStatusClient.getIsMigrationInProgress(srh);	
		},
		
		_getComponentDetails: function(nodeId) {
			var handler = {
				success: dojo.hitch(this, function(response) {
					if(!nodeId) {
						//No Cluster, go the "Simple" route
						this.failingServiceList = [];
						this.handleComponentResponse(response);	
					} else {
						response.nodeId = nodeId;
						this.handleComponentResponse(response);
						this.pendingComponentRequests--;
						if(this.pendingComponentRequests === 0) {
							this._sortErrorsForDisplay();
						}	
					}
					
				}),
				error: dojo.hitch(this, "handleServiceStatusFetchError")
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
			ServerStatusClient.getComponentsAndStatus(srh, nodeId);
		},
        
        _fetchAndDisplayFailingServiceInfo: function(services){
            var srh = new ServiceResponseHandler(this, "handleServiceStatusResponse", "handleServiceStatusFetchError");
            ServerStatusClient.postStatusForServiceList(srh, services);
        },
		
		_fetchAndDisplayPublicURI: function() {
			var srh = new ServiceResponseHandler(this, "displayPublicUrl", "handlePublicUriFetchError");
			ServerStatusClient.getServiceConfiguration(srh, PUBLIC_URL_SERVICE_NAME);
		},
		
		_fetchAndDisplayDiagnosticStatus: function() {
			var srh = new ServiceResponseHandler(this, "displayDiagnosticStatus", "handleDiagnosticsFetchError");
			ServerStatusClient.getAllDiagnosticResults(srh);
		},
		
		_fetchAndDisplayJTSStatus: function() {
            var srh = new ServiceResponseHandler(this, "checkForJTSErrors", "handleServiceStatusFetchError");
            ServerStatusClient.getJtsConfigurationStatus(srh);
        },
		
		fetchAndDisplayActiveServices: function() {
			this.totalActiveServices = 0;
			if(this.clusterNodes && this.isCluster) {
				this.pendingCalls = this.clusterNodes.length;
				dojo.forEach(this.clusterNodes, function(node) {
					this._getServices(node.humanReadableNodeId);
				}, this);
			}
        },
		
		_getServices: function(nodeId) {
			var handler = {
				success: dojo.hitch(this, function(response) {
					this.totalActiveServices += response.services.length;
					this.showActiveServicesCount();
				}),
				error: dojo.hitch(this, function(err) {
					console.error("Unable to get the total Active Services count for node: " + nodeId);
					this.showActiveServicesCount();
				})
			};
			var srh = new ServiceResponseHandler(handler, "success", "error");
			ServerStatusClient.getActiveServiceInfo(srh, nodeId);
		},
		
		showActiveServicesCount: function() {
			if(!this.domNode) return;
			this.pendingCalls--;
			if(this.pendingCalls === 0) {
				var aLink = dojo.create("a", { href: "#action=com.ibm.team.repository.admin.activeServices", innerHTML: this.totalActiveServices });
				var node = dojo.byId("totalActiveServices");
				if(node) dojo.place(aLink, node, "only");
			}
		},
			
		_emptyIfDomExists: function(node) {
			if(node) {
				node.innerHTML = "";
				return true;
			} else {
				return false;
			}
		},
        
        handleLoadingData: function(){
            this._pendingResponseCount++;
            getWorkbench().showMessage(this.messages.loadingData);
        },
        
        handleDataLoaded: function(){
            this._pendingResponseCount--;
            if (this._pendingResponseCount === 0) {
                getWorkbench().hideMessageArea();
            }
        },
        
        displayServerHealthInfo: function(data){
			if(!this.domNode) return;
			dojo.empty(this.serverStatusArea);
			
			this.serverStatusExtensionArea.innerHTML = "";
            var summaryDiv = document.createElement("div");
            
            var dbStyle = "";
            var dbStatus = this.messages.unavailable;
            if (data.databaseAvailable) {
                dbStatus = this.messages.connected;
            }
            else {
                dbStyle = "error";
            }
            var heading = {
                label: this.isJTS ? this.messages.serverStatus : this.messages.applicationStatus
            };
            var dataBlock = [{
                label: this.messages.databaseStatus,
                value: dbStatus,
                style: dbStyle
            }];
			if(!this.isCluster) {
				var uptimeDisplay = jazz.util.date.formatDuration(data.uptime, {
					style: "full",
					approximate: true
				});
				dataBlock.push({
	                label: this.messages.serverUptime,
	                value: uptimeDisplay
	            });
			}
			dataBlock.push({
                label: this.messages.publicURL,
                value: dojo.create("span", { id: "publicURINode", innerHTML: this.messages.loading, className: "loading" }),
            });
			
			dataBlock.push({
                label: this.messages.diagnosticStatus,
                value: dojo.create("div", { id: "diagNode", innerHTML: this.messages.loading, className: "loading" }),
            });
					
            this.serverHealthTable = this._appendInfoBlock(summaryDiv, heading, dataBlock);
            
			//Show VM Memory Usage or Cluster Status
			if(!this.isCluster) {
				heading = {  label: this.messages.memoryUsage };
	            var percentFree = Math.floor(data.freeMemory / data.maxMemory * 100) + " %";
	            dataBlock = [{
	                label: this.messages.maxMemAlloc,
	                value: util.formatSizeFromBytes(data.maxMemory),
	                style: "numeric"
	            }, {
	                label: this.messages.currMemAlloc,
	                value: util.formatSizeFromBytes(data.totalMemory),
	                style: "numeric"
	            }, {
	                label: this.messages.freeMem,
	                value: percentFree,
	                style: "numeric"
	            }];
			} 
			else {
				heading = { label: this.messages.clusterStatus };
	            dataBlock = [{
	                label: this.messages.totalClusterNodes,
	                value: this.clusterNodes.length
	 
	            }, {
	                label: this.messages.totalActiveServices,
	                value: dojo.create("span", { id: "totalActiveServices", innerHTML: this.messages.loading, className: "loading" })
	                //style: "numeric"
	            }, {
	                label: this.messages.moreInformation,
	                value: dojo.create("a", { innerHTML: this.messages.goToClusterPage, href: "#action=com.ibm.team.repository.admin.clusterNodes" })
	            }];
			}
			this._appendInfoBlock(summaryDiv, heading, dataBlock);
            
            this.serverStatusArea.appendChild(summaryDiv);
            
            var vmDetails = util.replaceNewLineWithBreak(data.vmInfo, true);
            var vmDetailsDiv = document.createElement("div");
            vmDetailsDiv.innerHTML = vmDetails;
            dataBlock = [{
                label: this.messages.javaVersion,
                value: data.javaVersion
            }, {
                label: this.messages.vmVendor,
                value: data.vmVendor
            }, {
                label: this.messages.vmName,
                value: data.vmName
            }, {
                label: this.messages.vmDetails,
                value: vmDetailsDiv
            }];
            if (data.vmErrorMessage) {
                dataBlock.unshift({
                    label: this.messages.vmUnsupported,
                    value: data.vmErrorMessage,
                    style: "error"
                });
                this._displayErrorMessage(data.vmErrorMessage);
            }
            var vmDiv = document.createElement("div");
            heading = {
                label: this.isJTS ? this.messages.serverVM : this.messages.applicationVM
            };
            this._appendInfoBlock(vmDiv, heading, dataBlock);
            this.serverStatusExtensionArea.appendChild(vmDiv);
            
            this.handleDataLoaded();
            
			this._fetchAndDisplayDatabaseInfo();
			this._fetchAndDisplayProductInfo();

			if(this.isJTS){
				this._fetchAndDisplayLicenseInfo();
			} else {
				//If applicationInfo is undefined, the DOM has been destroyed already (page switched).
				var exists = this._emptyIfDomExists(this.applicationInfo);
				if(exists) {
					if(this.jtsURL) {
						this._jtsURLNode = dojo.create("span",{ innerHTML: this.messages.connectedTo + " " });
						dojo.create("a",{ innerHTML: this.jtsURL, href: this.jtsURL }, this._jtsURLNode);
						//Check the connected JTS version & Friend configured
						this._fetchAndDisplayJTSStatus();
					} else {
						var span = dojo.create("span",{ innerHTML: this.messages.noJTSURL, className: "errorIcon" });
					}
					var appDiv = dojo.create("div", {}, this.applicationInfo);
					var dataBlock = [{
		                label: this.messages.jazzTeamServer,
		                value: this._jtsURLNode || span
		            }];
		            var heading = {
		                label: this.messages.applicationInfo
		            };
								
					//Show Application version if available
					if(BootstrapProperties.APP_VERSION) {
						dataBlock.push({
			                label: this.messages.version,
			                value: BootstrapProperties.APP_VERSION
			            });
					}
					
		            this._appendInfoBlock(appDiv, heading, dataBlock);
				}
			}
			this._fetchAndDisplayPublicURI();
        },

		displayJTSInfo: function(response) {		
			if(response) {
				this.jtsURL = response;
			} else {
				//Instruct the user to visit the JTS Setup Wizard to register this application.
				this.header.showMessage({
                    summary: this.messages.appNotRegistered,
					type: "ERROR"
                });
			}
			this._fetchAndDisplayServerHealthInfo();
		},
		
		handleCantGetJTSURL: function(err) {
			this._fetchAndDisplayServerHealthInfo();
		},
		
		checkForJTSErrors: function(data) {
			if(data && this._jtsURLNode) {
				dojo.create("div", { innerHTML: data, className: "errorIcon", style: { color: "#F00" } }, this._jtsURLNode);
			}
		},
		
		displayPublicUrl: function(serviceInfo) {
			this._fetchAndDisplayDiagnosticStatus();
			this._fetchAndDisplayServiceSummaryInfo();
			if(serviceInfo && this.serverHealthTable) {
				var label;
				for (var i=0; i<serviceInfo.properties.length; i++) {
					if (serviceInfo.properties[i].name === PUBLIC_URL_NAME) {
						var prop = serviceInfo.properties[i]
						label = prop.overrideValue != null ? prop.overrideValue : prop.defaultValue;
					}	
				};
				var node;
				if(!label || label == "") {
					var setupURL;
					if(this.isJTS) {
						setupURL = this._buildSetupURL(this.JAZZ_ROOT);
					} else if(this.jtsURL) {
						setupURL = this._buildSetupURL(this.jtsURL);
					}
					node = dojo.create("div");
					dojo.create("span",{ innerHTML: this.messages.notSpecified, className: "errorIcon" }, node);
					if(setupURL) {
						dojo.create("a", { innerHTML: this.messages.goToSetup, className: "setupLink", href: setupURL , title: this.messages.publicUrlHelp }, node);	
					}
				} else {
					node = dojo.create("a", { innerHTML: label, href: label });
				}
				var publicURINode = dojo.byId("publicURINode");
				if(publicURINode) dojo.place(node, publicURINode, "only");
			}
		},
		
		displayDiagnosticStatus: function(response) {
			if(response && this.serverHealthTable) {
				var diagNode = dojo.byId("diagNode");
				var display = this.diagUtil.displayDiagnosticStatus(response);
				if(diagNode && display) {
					dojo.place(display, diagNode, "only");
				} else {
					this.unableToDisplayDiagnostics();
				}
			} else {
				this.unableToDisplayDiagnostics();
			}
			this.fetchAndDisplayActiveServices();
			this._getMigrationStatus();
		},
		
		unableToDisplayDiagnostics: function() {
			var diagNode = dojo.byId("diagNode");
			if(diagNode) {
				diagNode.innerHTML = this.messages.lowerUnavailable;
				var aLink = dojo.create("a", { href:"#", innerHTML: this.messages.tryAgain, className: "aLink" }, diagNode);
				this.connect(aLink, "onclick", dojo.hitch(this, function(e) {
					dojo.stopEvent(e);
					this._fetchAndDisplayDiagnosticStatus();
					diagNode.innerHTML = this.messages.loading;
				}));
			}
		},
		
		_buildSetupURL: function(jts) {	
			var lastChar = jts.charAt(jts.length-1);
			if(lastChar != '/') jts += "/";
			return jts + "setup";
		},
        
        handleComponentResponse: function(componentAndStatuses) {
			if(!this.domNode) return;
			
			var nodeId = componentAndStatuses.nodeId;
			
            if (componentAndStatuses && componentAndStatuses.length > 0) {
                componentAndStatuses.sort(function(a, b){
                    return a.component.id.toLowerCase() > b.component.id.toLowerCase() ? 1 : -1;
                });
				//Check each service status for failing services
                dojo.forEach(componentAndStatuses, function(componentAndStatus){
					var statuses = componentAndStatus.serviceStatuses;
                    if (statuses && statuses.length > 0) {
						dojo.forEach(statuses, function(serviceObj) {
							var statusObj = serviceObj.status;
							if (statusObj && statusObj.code != "OK") {
								if(nodeId) {
									statusObj.nodeId = nodeId;
		                        	this.addOrUpdateComponentError(statusObj);	
								} else {
									//No Cluster, Simple case
									this.failingServiceList.push(statusObj);
								}
		                    }
						}, this);
                    }
                }, this);
				
				//No cluster, skip the crazy priority sorting and just display the list
				if(!nodeId) {
					//Sort by interface name
	                this.failingServiceList.sort(function(a, b){
	                    return a.interfaceName.toLowerCase() > b.interfaceName.toLowerCase() ? 1 : -1;
	                });
					this._displayServiceStatusSummary();
				}
            }
        },
		
		addOrUpdateComponentError: function(statusObject) {
			var key = statusObject.interfaceName;
			if(this.allComponentErrors[key] != null) {
				//We already have this error from another node. Increment it
				this.allComponentErrors[key].nodesEffected++;
			} else {
				//This is the first time we've seen this error. Save it.
				this.allComponentErrors[key] = {
					nodesEffected: 1,
					statusObject: statusObject,
					nodes: []
				};
			}
			this.allComponentErrors[key].nodes.push(statusObject.nodeId);
		},
        
        displayDatabaseStatusInfo: function(dbStatus){
			if(!this.domNode) return;
			dojo.empty(this.additionalDBInfoArea);
			
            var errorClass = "";
            
            if (dbStatus.stacktrace) {
                var statusDetails = new dijit.TitlePane({
                    title: dbStatus.message
                });
                var stacktrace = document.createElement("div");
                stacktrace.innerHTML = util.replaceNewLineWithPaddedBreaks(dbStatus.stacktrace, false);
                statusDetails.set("content", stacktrace);
                statusDetails.toggle();
                statusDetails.arrowNode.title = dbStatus.message;
                var statusInfo = statusDetails.domNode;
                
                this._displayErrorMessage(-1);
            }
            else {
                if (dbStatus.code && dbStatus.code === "ERROR") {
                    errorClass = "error";
                }
                var statusInfo = dbStatus.message
            }
            var dbInfo = this._createDisplayBlock(this.messages.databaseStatus, statusInfo, errorClass);
            
            this.additionalDBInfoArea.appendChild(dbInfo);
            this.handleDataLoaded();
        },
        
        displayProductInfo: function(prodInfo){
			//If productInfoArea is undefined, the DOM has been destroyed already (page switched).
			if(!this.productInfoArea) return;
			
            if (prodInfo) {
				var jtsBlock = [];
                var dataBlock = [];
                this.prodInfo = prodInfo;
                this.prodInfo = this.sortProducts();
                dojo.forEach(this.prodInfo, function(product){
					var block = {
                        label: product.name,
                        value: product.version + " (" + product.buildId + ")"
                    };
					//Always show JTS libraries last (#154782)
					if(product.id == JTS_PRODUCT_ID) {
						jtsBlock.push(block);
					} else {
						dataBlock.push(block);
					}
                    dojo.forEach(product.patches, function(patch){
						var patchBlock = {
                        	label: patch.name,
                        	value: patch.version + " (" + patch.buildId + ")",
                        	labelStyle: "patch patchName",
                        	style: "patch"
                    	};
						if(product.id == JTS_PRODUCT_ID) {
							jtsBlock.push(patchBlock);
						} else {
							dataBlock.push(patchBlock);
						}
                    });
                    
                });
				
				var allBlocks = dataBlock.concat(jtsBlock);
                
                var pDiv = dojo.create("div", {}, this.productInfoArea);
                var heading = {
                    label: this.messages.buildInformation
                };
                this._appendInfoBlock(pDiv, heading, allBlocks);
            }
            this.handleDataLoaded();
        },
        
        displayLicenseInfo: function(licenseInfo){
			if(!this.domNode) return;
			dojo.empty(this.licenseSummaryArea);
			
			var dataBlock = [];
            
            //Only show if some CALs can expire. 
            var calStatus = this._getLicenseStatusCALS(licenseInfo);
            if (calStatus) {
                dataBlock.push({
                    label: this.messages.clientLicenseStatus,
                    value: calStatus
                });
            }
            
            if (licenseInfo.serverLicense.disabledReason && licenseInfo.serverLicense.disabledReason.length > 0) {
                dataBlock.push({
                    label: this.messages.deactivated,
                    value: licenseInfo.serverLicense.disabledReason,
                    style: "error"
                });
            }
            
            if (licenseInfo.floatingSetupStatus) {
                var status;
                var description;
                if (licenseInfo.floatingSetupStatus.server === true) {
                    status = this.messages.configured;
                    description = this.messages.configuredFloatingServer;
                }
                else {
                    status = this.messages.notConfigured;
                    var floatTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=floating'>";
                    description = dojo.string.substitute(this.messages.notConfiguredFloatingServer, [floatTabLink, "</a>"]);
                }
                
                var licenseServerHover = this._getInfopopLink(status, description);
                
                dataBlock.push({
                    label: this.messages.floatingServer,
                    value: licenseServerHover
                });
                
                var extraDetails;
                if (licenseInfo.floatingSetupStatus.client === true) {
                    status = this.messages.configured;
                    var licenseServerLink = "<a href=\"" + licenseInfo.floatingSetupStatus.serverURL + "\">" + licenseInfo.floatingSetupStatus.serverURL + "</a>"
                    extraDetails = dojo.string.substitute(this.messages.usingLicneseServer, [licenseServerLink]);
                    description = dojo.string.substitute(this.messages.usingLicenseServerDescription, [licenseServerLink]);
                    
                    if (licenseInfo.floatingSetupStatus.connectionError) {
                        extraDetails = extraDetails + "<br/><span class=\"error\">" + this.messages.licenseServerError + licenseInfo.floatingSetupStatus.connectionError + "</span>";
                    }
                }
                else {
                    status = this.messages.notConfigured;
                    var serverTabLink = "<a href='#action=com.ibm.team.repository.admin.manageLicenses&focusTab=server'>";
                    description = dojo.string.substitute(this.messages.notConfiguredFloatingClient, [serverTabLink, "</a>"]);
                }
                
                var fullStatus = document.createElement("span");
                var licenseClientHover = this._getInfopopLink(status, description);
                var hoverSpan = document.createElement("span");
                hoverSpan.appendChild(licenseClientHover);
                fullStatus.appendChild(hoverSpan);
                
                if (extraDetails) {
                    var detailsSpan = document.createElement("span");
                    detailsSpan.innerHTML = extraDetails;
                    fullStatus.appendChild(detailsSpan);
                }
                
                dataBlock.push({
                    label: this.messages.floatingClient,
                    value: fullStatus
                });
            }
            
            var licDiv = document.createElement("div");
            var heading = {
                label: this.messages.licenseStatus
            };
            this.licenseStatusTable = this._appendInfoBlock(licDiv, heading, dataBlock);
            this.licenseSummaryArea.appendChild(licDiv);
            this.handleDataLoaded();
        },
        
        _getLicenseStatus: function(serverLicense){
            if (serverLicense.trial) {
                var expirationDate = dojo.date.stamp.fromISOString(serverLicense.expirationTime);
                var now = new Date();
            }
            var statusString = !serverLicense.trial ? this.messages.permanent : expirationDate > now ? this.messages.trial : this.messages.expiredTrial;
            
            if (expirationDate) {
                var expirationInfo = getExpirationInfo(serverLicense.expirationTime);
                statusString += " - ";
                if (expirationInfo.isExpired) {
                    statusString += dojo.string.substitute(this.messages.expiredDate, [expirationInfo.expirationDateRelative]);
                }
                else {
                    statusString += dojo.string.substitute(this.messages.expiresDate, [expirationInfo.expirationDateRelative]);
                }
            }
            
            return statusString;
        },
        
        _getLicenseStatusCALS: function(licenseInfo){
        
            var minDaysRemaining;
            var expInfo;
            var types = licenseInfo.types || [];
			var combined = types;
            if (licenseInfo && licenseInfo.floatingSetupStatus) {
                var floatingTypes = licenseInfo.floatingLicenseTypes || [];
                combined = types.concat(floatingTypes);
            }
            
            dojo.some(combined, function(type){
                if (type.expirationTime) {
                    //Find out how many days till it expires
                    var expirationInfo = getExpirationInfo(type.expirationTime);
                    if (expirationInfo.isExpired) {
                        minDaysRemaining = -1;
                        expInfo = expirationInfo;
                        return;
                    }
                    else {
                        var days = expirationInfo.daysRemaining;
                        if (days < minDaysRemaining || !minDaysRemaining) {
                            minDaysRemaining = days;
                            expInfo = expirationInfo;
                        }
                    }
                }
            });
			
            if (!minDaysRemaining) {
                return false;
            } else {
				var msg = expInfo.isExpired ? this.messages.oneOfTheCALsExpired : this.messages.oneOfTheCALsExpires;
				return dojo.string.substitute(msg, [expInfo.expirationDateRelative]);
            }
        },

		_sortErrorsForDisplay: function() {
			this.failingServiceList = [];
			
			//First, loop the errors looking for "Cluster-wide" errors
			for (var key in this.allComponentErrors) {
				var nodesEffected = this.allComponentErrors[key].nodesEffected;
				if(nodesEffected == this.clusterNodes.length) {
					//This error happens on every node. Display it!
					var statusObject = this.allComponentErrors[key].statusObject;
					statusObject.nodes = "ALL";
					this.failingServiceList.push(statusObject);
				}
			}
			//Second, loop the errors looking for "Node-specific" errors
			for (var key in this.allComponentErrors) {
				var nodesEffected = this.allComponentErrors[key].nodesEffected;
				if(nodesEffected < this.clusterNodes.length) {
					//This error happens on one(some) nodes
					var statusObject = this.allComponentErrors[key].statusObject;
					statusObject.nodes = this.allComponentErrors[key].nodes;
					this.failingServiceList.push(statusObject);
				}
			}
			this._displayServiceStatusSummary();
		},
        
        _displayServiceStatusSummary: function(showAll) {
			if(!this.domNode) return;
			dojo.empty(this.servicesDiv);
			dojo.removeClass(this.servicesDiv, "loading");
            if (this.failingServiceList.length > 0) {
				var showNum = (this.failingServiceList.length <= 10 || showAll) ? this.failingServiceList.length : LIMIT_SERVICE_ERRORS;
				for(var i = 0; i < showNum; i++) {
					var status = this.failingServiceList[i];
					var title = status.interfaceName;
					if(status.nodes) {
						var nodePrefix = status.nodes == "ALL" ? this.messages.allNodes : status.nodes[0]; //TODO, what if it shows up on more than one node but not all???
						title = "[" + nodePrefix + "] " + title;
					}
					var statusDetails = new dijit.TitlePane({title: title});
					var errorContent = dojo.create("div");
					dojo.create("div", { className: "status-summary", innerHTML: status.message }, errorContent);
					if(status.stacktrace) {
						dojo.create("div", { innerHTML: util.replaceNewLineWithPaddedBreaks(status.stacktrace, false) }, errorContent);
					}
					statusDetails.set("content",errorContent);
					statusDetails.toggle();
					statusDetails.arrowNode.title = status.message;
					this.servicesDiv.appendChild(statusDetails.domNode);
				}
				
				if(showNum < this.failingServiceList.length || showAll) {
					var showingMessage = dojo.string.substitute(this.messages.currentlyShowing, [showNum, this.failingServiceList.length]);
					this.servicesCount.innerHTML = showingMessage;	
				}
				dojo.toggleClass(this.showAllLink, "noDisplay", showNum >= this.failingServiceList.length);	
            }
            else {
				dojo.create("div",{ className: "failed-summary", innerHTML: this.messages.noServiceErrors }, this.servicesDiv);
            }
        },
		
		_showAllErrors: function(e) {
			dojo.stopEvent(e);
			this._displayServiceStatusSummary(true);
		},

        _createDisplayBlock: function(heading, data, dataClass){
			var status = dojo.create("div", { className: "com-ibm-team-repository-admin-table" });
            var statusDiv = dojo.create("div", { className: "wrapper2" }, status);
			dojo.create("div", { className: "heading", innerHTML: heading }, statusDiv);

            if (isNode(data)) {
				statusDiv.appendChild(data);
            }
            else {
                var services = dojo.create("div", { className: "statusRow" }, statusDiv);
				var failedService = dojo.create("div", { innerHTML: data}, services);
				failedService.className = dataClass ? "failed-summary " + dataClass : "failed-summary";
            }
            return status;
        },
        
        _appendInfoBlock: function(node, heading, infoList){
            var infoTable = new DataTableContainer({
                heading: heading
            });
			this._dataTables.push(infoTable);
			dojo.forEach(infoList, function(item) {
				this._insertRow(infoTable, item);
			}, this);
			var infoBlock = dojo.create("div", { className: "stats-block" }, node);
            infoBlock.appendChild(infoTable.domNode);
            return infoTable;
        },
        
        _insertRow: function(table, item){
            var rowData = {
                0: {
                    content: item.label,
                    // Allows the label to have additional styles other than default
                    style: "rank tight double" + (item.labelStyle ? " " + item.labelStyle : "")
                },
                1: {
                    content: item.value,
                    style: item.style
                }
            };
            table.addRow(rowData, 2);
        },
        
        handleServerHealthErrorReponse: function(errorObj){
            this._handleErrorResponse(errorObj, this.serverStatusArea, this.messages.serverStatus);
        },
        
        handleDatabaseErrorReponse: function(errorObj){
            this._handleErrorResponse(errorObj, this.additionalDBInfoArea, this.messages.databaseStatus);
        },
        
        handleProductErrorResponse: function(errorObj){
            this._handleErrorResponse(errorObj, this.serverStatusArea, this.messages.serverStatus);
        },
        
        handleLicenseErrorReponse: function(errorObj){
            this._handleErrorResponse(errorObj, this.licenseSummaryArea, this.messages.serverLicense);
        },
        
        handleServiceStatusFetchError: function(errorObj){
            this._displayErrorMessage(this.messages.errorFetchingServerStatus, errorObj);
        },
		
		handlePublicUriFetchError: function(errorObj) {
			if(!this.domNode) return;
			var publicURINode = dojo.byId("publicURINode");
			if (publicURINode) publicURINode.innerHTML = this.messages.lowerUnavailable;
			this._fetchAndDisplayDiagnosticStatus();
			this._fetchAndDisplayServiceSummaryInfo();
			this.handleServiceStatusFetchError(errorObj);
		},
		
		handleDiagnosticsFetchError: function(errorObj) {
			if(!this.domNode) return;
			var diagNode = dojo.byId("diagNode");
			if (diagNode) diagNode.innerHTML = this.messages.lowerUnavailable;
			this.fetchAndDisplayActiveServices();
			if(this.isJTS || this.jtsURL) { // For apps w/o a JTS, suppress this message so we can show 'Visit Setup Wizard' message instead.
				this.handleServiceStatusFetchError(errorObj);
			}
		},
        
        _handleErrorResponse: function(errorObj, sectionArea, sectionHeading){
            this._displayErrorMessage(this.messages.errorFetchingServerStatus, errorObj);
            
            if (errorObj && errorObj.status === 404) {
				this.provisionServerMsg.innerHTML = "";
                var provisionLink = "<a href='" + this.JAZZ_ROOT + "/admin?internal=true#action=com.ibm.team.repository.admin.provisionStatus'>";
                var provisionMsg = dojo.string.substitute(this.messages.provision, [provisionLink, "</a>"]);
                var provision = this._createDisplayBlock(this.messages.serverStatus, provisionMsg);
                this.provisionServerMsg.appendChild(provision);
            }
            if (sectionArea && errorObj && errorObj.status === 503) {
				sectionArea.innerHTML = "";
                var errorBlock = this._createDisplayBlock(sectionHeading, this.messages.errorFetchingServerStatus, "error");
                sectionArea.appendChild(errorBlock);
            }
            this.handleDataLoaded();
        },
        
        _displayErrorMessage: function(message, errorObj){
            if (!this._dbMessageSet) {
                if (message === -1) {
                    this._dbMessageSet = true;
                    message = this.messages.expandDBStatusInfo;
                }
                
                if(errorObj){
					errorObj.message = errorObj.message ? errorObj.message: message;
				}else{
					var errorObj={
						message: message,
						status: STATUS_TYPES.ERROR
					}
				}
				
                this.header.showMessage({
                    errorObject: errorObj,
					summary: message
                });
            }
        },
        
        sortProducts: function(){
            return this.prodInfo.sort(function(a, b){
                var aName = a.name.toLowerCase();
                var bName = b.name.toLowerCase();
                return aName > bName ? 1 : -1;
            });
        },
        
        getVersionInfo: function(data){
            var version = "";
            if (data.version) {
                version = data.version;
            }
            if (data.buildId) {
                var build = data.buildId;
                if (data.version) {
                    build = " (" + build + ") ";
                }
                version += build;
            }
            if (!version) {
                version = this.messages.notAvailable;
            }
            return version;
        },
        
        _getInfopopLink: function(label, info, className){
            var hoverLink = document.createElement("a");
            hoverLink.href = "#";
            hoverLink.onclick = function(){
                return false;
            };
            hoverLink.innerHTML = label;
            if (className) {
                hoverLink.className = className;
            }
            else {
                hoverLink.className = "infopop";
            }
            var tooltip = new HoverPopup({
                around: hoverLink,
                html: info,
                openDelay: 300,
                orientation: "R"
            }, document.createElement("span"));
			this._tooltips.push(tooltip);
            
            return hoverLink;
        },
		
		_destroyDataTables: function() {
			if (this.serverHealthTable) {
				delete this.serverHealthTable;
			}
			
			if (this.licenseStatusTable) {
				delete this.licenseStatusTable;
			}
			
			for (var i = 0; i < this._dataTables.length; i++) {
				this._dataTables[i].destroyRecursive();
			}
			this._dataTables = [];
			
		},
		
		_destroyTooltips: function() {			
			for (var i = 0; i < this._tooltips.length; i++) {
				this._tooltips[i].destroyRecursive();
			}
			this._tooltips = [];
			
		},
		
		uninitialize: function () {
			this._destroyDataTables();
			this._destroyTooltips();
			
			// Destroy the message box
			this.header.destroyRecursive();
				
			this.inherited(arguments);
        }
		
    });
    
    function getWorkbench(){
        return net.jazz.ajax.ui.PlatformUI.getWorkbench();
    }
    
    function isNode(o){
        return o && o.nodeType && o.cloneNode;
    }
    
})();
