/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab");

(function() {

var SimpleTab = com.ibm.team.repository.web.ui.internal.admin.users.SimpleTab;

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.users.SimpleTabContainer",
	[dijit._Widget, dijit._Templated], {

	// Properties //----------------------------------------------------------->

	callbackTabSwitch: null,
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/templates/SimpleTabContainer.html"),
	_static: function() {}, 	

	// Public Methods //------------------------------------------------------->

	constructor: function(args) {
	},

	postCreate: function() {
		this._initializeStaticVariables();
		this._initializeInstanceVariables();
		this._initializeWidget();
	},
	
	destroy: function() {		
		// Tabs
		if (this._tabHolder) {
			for (var tabID in this._tabHolder) {
				if (this._tabHolder[tabID]) {
					this._tabHolder[tabID].destroy();
					delete this._tabHolder[tabID];
				}
			}
			delete this._tabHolder;
		}
		this.inherited(arguments);
	},

	/*
 	 * @param String tabID
 	 * @param String title
 	 * @param Function callbackGetNode
 	 * 
 	 * @return void
 	 */		
	addTab: function(tabID, title, callbackGetNode) {
		// Create new tab
		var tabWidget = new SimpleTab({
			tabID: tabID,
			tabTitle: title,
			callbackGetNode: callbackGetNode,
			callbackOnClick: dojo.hitch(this, this._handleEventOnClickTab)
		});
		// Add tab to container
		this._tabArea.appendChild(tabWidget.domNode);
		// Store tab
		this._tabHolder[tabID] = tabWidget;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return void
 	 */		
	selectTab: function(tabID) {
		
		if ((tabID in this._tabHolder) === false) {
			return;
		}
		
		var previousTab = this._selectedTab;
		
		if (previousTab && 
			(previousTab.tabID === tabID)) {
			// Same tab
			// Nothing to do
			return;
		} else if (previousTab) {
			// Deselect previous tab
			previousTab.deselect();
		}

		var currentTab = this._tabHolder[tabID];
		
		// Select current tab
		currentTab.select();
		
		// Update selection
		this._selectedTab = currentTab;
	},

	/*
 	 * @param String tabID
 	 * 
 	 * @return Element
 	 */		
	getContentNode: function(tabID) {
		if ((tabID in this._tabHolder) === false) {
			return null;
		}
		return this._tabHolder[tabID].contentNode;
	},

	// Private Methods //------------------------------------------------------>
	
	_initializeStaticVariables: function() {
		if (this._static._instanceCount) {
			this._static._instanceCount++;
			return;
		}
		this._static._instanceCount = 1;
	},
		
	_initializeInstanceVariables: function() {
		this._tabHolder = new Object();
		this._selectedTab = null;
	},
	
	_initializeWidget: function() {
	},
		
	_handleEventOnClickTab: function(tab) {
		var newTabID = tab.tabID;
		var oldTabID = null;
		if (this._selectedTab) {
			oldTabID = this._selectedTab.tabID;
		}
		
		if (oldTabID === newTabID) {
			return;
		}
		
		// Update tab selection
		this.selectTab(newTabID);
		// Fire event
		if (this.callbackTabSwitch) {
			this.callbackTabSwitch(newTabID, oldTabID);
		}		

	}
	
});

})();
