/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.dialogs.ExportDialog"); //$NON-NLS-1$
dojo.require("dijit._Widget"); //$NON-NLS-1$
dojo.require("dijit._TemplatedMixin"); //$NON-NLS-1$
dojo.require("dijit._WidgetsInTemplateMixin"); //$NON-NLS-1$
dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("jazz.ui.Dialog");//$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$ 
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$	
(function() {

	var _currentRequest;
	var _WIDTH_IN_EM = 50;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var SERVICE_URL = ProcessAuthoringClient.getJtsFileServiceUrl();
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.dialogs.ExportDialog", [dijit._Widget, dijit._TemplatedMixin, dijit._WidgetsInTemplateMixin], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/dialogs/templates/ExportDialog.html"),//$NON-NLS-1$ //$NON-NLS-2$ 
		constructor: function(args) {
			if (!args) {
				args = {};
			}
			this.width = _WIDTH_IN_EM + "em"; //$NON-NLS-1$
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		postCreate: function() {
			var self = this;
			this.dialog = new jazz.ui.Dialog({
				contentNode: this.contentNode,
				primaryTitle: this.messages.ExportDialog_1,
				width: this.width,
				destroyContents: true,
				onClose: function() {
					if (self.exportId) {
						self._invokeService(SERVICE_URL + "/cancelExport?exportId=" + self.exportId, null, null); //$NON-NLS-1$
					}
				}
			});
			this._topText.innerHTML = this.message;
			
			this.okButton.disabled = true;
			//TODO: disable Cancel button for now until cancel support is implemented
			this.cancelButton.disabled = true;
			
			this._showProgressBar(true);
			
			var successCallback = function(response) {
				var json = null;
				try {
					json = response instanceof Object ? response : dojo.fromJson(response);
				} catch (e) {
					//
				}
				if (json) {
					self.exportId = json.exportId;
				}
			};
			
			var errorCallback = function(error) {
				if (this.intervalId) {
					clearInterval(this.intervalId);
				}

				var responseText = "";//$NON-NLS-1$
				if (error.responseText) {
					responseText = error.responseText;
				} else if (error.message) {
					responseText = error.message;
				} else {
					responseText = error;
				}
				
				self._setStatusMessage(responseText, true);
				
			};
			
			this._invokeService(SERVICE_URL + "/export?nowait=true&uri=" + this.resourceUrl, successCallback, errorCallback); //$NON-NLS-1$
			
			// check progress
			this.intervalId = setInterval(dojo.hitch(this, "_pollProgress"), 1000); //$NON-NLS-1$

			try {
				com.ibm.team.process.authoring.web.common.common.internal.UIHelper.resizeI18NButtons(this.dialogFooter, [this.okButton, this.cancelButton]);
			} catch (e) {
				//failed to resize on IE for tooltipDialog, have to catch it. This is logged in workitem 111790
			}
			
			this.connect(this.cancelButton, "onclick", function(e) { //$NON-NLS-1$
				dojo.stopEvent(e);
				if (this.intervalId) {
					clearInterval(this.intervalId);
				}
				if (_currentRequest) {
					_currentRequest.cancel();
				}
				if (window.stop) {
					window.stop();
				}
				
				this.dialog.close();
			});
			this.connect(this.okButton, "onclick", "_onOk"); //$NON-NLS-1$ //$NON-NLS-2$ 
		},
		
		_showProgressBar: function(indeterminate) {
			this._progress.update({
				indeterminate: indeterminate
			});
			this._progress.domNode.style.display = "block";//$NON-NLS-1$
		},
		
		_pollProgress: function() {
			if (!this.exportId) {
				return;
			}
			var checkProgressUrl = SERVICE_URL + "/checkProgress?exportId=" + this.exportId; //$NON-NLS-1$
			jazz.client.xhrGet({
				url: checkProgressUrl,
				handleAs: "json", //$NON-NLS-1$
				preventCache: true,
				load: dojo.hitch(this, function(response, ioArgs) {
					var json = response instanceof Object ? response : dojo.fromJson(response);
					//TODO: show real progress 
//					this._progress.update({
//						maximum: 100,
//						progress: json.percentage
//					});
					if (json.percentage == 100) {
						clearInterval(this.intervalId);
						this.dialog.close();
						window.location = SERVICE_URL + "/export/" + this.exportId; //$NON-NLS-1$
					} 
//					else {
//						this._setStatusMessage(json.message, false);
//					}
					return response;
				}),
				error: function(response, ioArgs) {
					return response;
				}
			});
		},
		
		exportCallback: function(serviceUrl) {
			window.location = serviceUrl;
		},
		
		_onOk: function(e) {
			this.okButton.disabled = true;
			if (e) {
				dojo.stopEvent(e);
			}
			
			this.exportId = null;
			this.dialog.close();
			
		},
		
		_clearStatusMessage: function() {
			this.status.style.visibility = "hidden";//$NON-NLS-1$
		},
		
		_setStatusMessage: function(msg, isError) {
			this.status.style.visibility = "visible";//$NON-NLS-1$
			dojox.xml.parser.replaceChildren(this.status, document.createTextNode(msg));
			this.status.className = isError ? "ErrorStatus" : "NormalStatus";//$NON-NLS-1$ //$NON-NLS-2$
		},
		
		_invokeService: function(url, successCallback, errorCallback) {
			var args = {
				url: url,
				headers: {
					Accept: "*" //$NON-NLS-1$
				}
			};
			var deferred = jazz.client.xhrGet(args);
			
			deferred.addCallback(successCallback);
			deferred.addErrback(errorCallback);
			
			this._currentRequest = deferred;
		}
		
	});
	
})();
