/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2009, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
	dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreePane"); //$NON-NLS-1$
	dojo.require("dojo.i18n"); //$NON-NLS-1$
	dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.editors.PracticeTreePane", [dijit._Widget, dijit._Templated], { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/editors/templates/PracticeTreePane.html"), //$NON-NLS-1$ //$NON-NLS-2$
		trees: null,
		editPractice: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		viewOnly: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		isOutlinePane: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		isPlainTree: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		rootOnly: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		isAssociated: false, //TODO: this variable is not used anywhere in this widget, it should be removed
		selectedNode: null, // reference to the currently selected node in a practice tree
		selectedNodeItemIndex: null, // index of the currently selected node in a practice tree
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.editors", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
			this.trees = [];
		},
		
		addPracticeTree: function(practiceTree) {
		    var needplus = false;
			for (var i = 0; i < this.trees.length; i++) {
				if (practiceTree.indexInParent == this.trees[i].indexInParent) {
					needplus = true;
				}
				if (needplus) {
					this.trees[i].indexInParent = this.trees[i].indexInParent + 1;
				}
			}
			this.trees.push(practiceTree);
			//sort the trees alphabetically
			if (this.trees.length === 1) {
				this.treeSection.appendChild(practiceTree.domNode);
			} else {
				this.trees.sort(function(a, b) {
					return a.indexInParent - b.indexInParent;
				});
				
				var index = dojo.indexOf(this.trees, practiceTree);
				var next = this.trees[index + 1];
				if (next) {
					dojo.place(practiceTree.domNode, next.domNode, "before"); //$NON-NLS-1$	
				} else {
					this.treeSection.appendChild(practiceTree.domNode);
				}
			}
		},
		
		removePracticeTrees: function() {
			dojo.empty(this.treeSection);
			this.trees = [];
		},
		
		removePracticeTree: function(practiceResource) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var treePracticeId = null;
			
			//find element to delete
			var i = 0;
			for (; i < this.trees.length; i++) {
				treePracticeId = this.trees[i].practiceResource.substring(this.trees[i].practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
				if (this.trees[i].indexInParent === practiceResource || treePracticeId === practiceId) {
					break;
				}
			}
			
			//remove from tree
			var treeNode = this.trees.splice(i, 1);
			treeNode[0].destroyRecursive();
			
			//reset remaining order of the tree
			for (; i < this.trees.length; i++) {
				this.trees[i].indexInParent = this.trees[i].indexInParent - 1;
			}
		},
		
		getItem: function(practiceResource, node, callback) {
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var treePracticeId = null;
			dojo.some(this.trees, function(tree) {
				treePracticeId = tree.practiceResource.substring(tree.practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
				if ((tree.practiceResource === practiceResource) || (treePracticeId === practiceId)) {
					if (node) {
						tree.model.store.fetchItemByIdentity({
							identity: node.id,
							onItem: function(item) {
								callback(item, tree);
							}
						});
					} else {
						// return root item
						callback(tree.rootNode.item, tree);
					}
					return true;
				}
				return false;
			}, this);
			
		},
		
		/**
         * Check if the a tree is existing in the tree pane
         *
         * @param {String} practiceResource the URL of the practice which indicates unique tree
         */
		exist: function(practiceResource){
			var practiceId = practiceResource.substring(practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
			var treePracticeId = null;
			return dojo.some(this.trees, function(tree) {
				treePracticeId = tree.practiceResource.substring(tree.practiceResource.lastIndexOf("/") + 1); //$NON-NLS-1$
				if ((tree.practiceResource === practiceResource) || (treePracticeId === practiceId)) {
					return true;
				}
				return false;
			}, this);
		},
		
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		//  Make sure selectTree is called in the following conditions
		//  1. Use PracticeTree.set("selectedItem", path) to select a ROOT node in the practice tree.
		//  2. Use PracticeTree.set("path", path) to select a non-ROOT node in the practice tree.
		//  3. Perform a click event in PracticeTree.onClick.
		//  4. Add a practice/practice element/category in left navigator.
		//////////////////////////////////////////////////////////////////////////////////////////////////////
		selectTree: function(practiceTree) {
			dojo.forEach(this.trees,function(tree) {
				if (tree === practiceTree) {
					dojo.toggleClass(tree.domNode, "selectedTree", true); //$NON-NLS-1$
				} else {
					dojo.toggleClass(tree.domNode, "selectedTree", false); //$NON-NLS-1$
				}
			});
		},
		
		uninitialize: function() {
			dojo.forEach(this.trees, function(tree) {
				if (tree) {
					tree.destroyRecursive();
				}
			});
			
			this.inherited(arguments);
		}
	});
	
})();
