/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.dashboard.web.ui.internal.util.JPIMappingRegistry");

dojo.require("com.ibm.team.dashboard.web.util.internal.Util");

var Util = com.ibm.team.dashboard.web.util.internal.Util;

var _jpiRequestScheduled = [false]; // for pass by reference
var _urlRequestScheduled = [false]; // for pass by reference
var _pendingJPIRequests = [];
var _pendingURLRequests = [];

var SERVICE_PATH = "/dashboard-jpi-mappings";
var JPI_SERVICE = SERVICE_PATH + "/jpis";
var URL_SERVICE = SERVICE_PATH + "/urls";

var _requestJPIs = dojo.partial(_requestURIs, _pendingJPIRequests, _jpiRequestScheduled, JPI_SERVICE);
var _requestURLs = dojo.partial(_requestURIs, _pendingURLRequests, _urlRequestScheduled, URL_SERVICE);

/*
 * Provides functions to convert between URLs and JPIs.
 */
com.ibm.team.dashboard.web.ui.internal.util.JPIMappingRegistry = {
	
	/*
	 * Converts the given array of JPI strings into URL strings.
	 * 
	 * @param {string[]} jpis The JPIs to convert to URLs
	 * @return {dojo.Deferred} An array of matching URLs (same indexes)
	 */
	/* dojo.Deferred */ jpisToURLs: dojo.partial(_convertURIs, _pendingURLRequests, _urlRequestScheduled, _requestURLs),
	
	/*
	 * Converts the given array of URL strings into JPI strings.
	 * 
	 * @param {string[]} urls The URLs to convert to JPIs
	 * @return {dojo.Deferred} An array of matching JPIs (same indexes)
	 */
	/* dojo.Deferred */ urlsToJPIs: dojo.partial(_convertURIs, _pendingJPIRequests, _jpiRequestScheduled, _requestJPIs)
};

/*
 * A generalized function for converting batches of JPIs to URLs and vice versa. Both
 * operations are very similar so we differentiate via the parameters.
 */
function _convertURIs(/* string[] */ pendingQueue, /* boolean[1] */ requestScheduled, /* function */ requestFunc, /* string[] */ uris) {
	// validate input
	var len = uris.length;
	for (var i=0;i<len;++i) {
		if (typeof(uris[i]) !== "string") {
			throw new Error("Invalid argument for JPI/URL conversion: must be of type string (but was " + typeof(uris[i]) + ")");
		}
	}
	
	// queue uris for next batch request
	var dfd = new dojo.Deferred();
	pendingQueue.push({dfd: dfd, uris: uris});
	if (!requestScheduled[0]) {
		// yield to allow requests to accumulate
		setTimeout(requestFunc, 0);
		requestScheduled[0] = true;
	}
	return dfd;
}

/*
 * Fire a batch request with the given URIs (JPIs or URLs) to the given service.
 */
function _requestURIs(/* string[] */ pendingQueue, /* boolean[] */ requestScheduled, /* string */ servicePath) {
	// consume the pending requests
	var requests = pendingQueue.splice(0, pendingQueue.length);
	requestScheduled[0] = false;
	
	// compose a batch request
	var uris = [];
	var len = requests.length;
	for (var i=0;i<len;++i) {
		uris.push.apply(uris, requests[i].uris);
	}
	
	// fire the batch request
	jazz.client.xhrPost({
		url: Util.getServiceRoot() + servicePath,
		handleAs: "json",
		headers: { "Content-Type": "application/json" },
		postData: dojo.toJson(uris),
		load: function(/* string[] */ uris) {
			// slice the result and return each chunk to its corresponding deferred
			var len = requests.length;
			var index = 0;
			for (var i=0;i<len;++i) {
				var request = requests[i];
				var num = request.uris.length;
				request.dfd.callback(uris.slice(index, index + num));
				index += num;
			}
		},
		error: function(/* Error */ e) {
			// call each dererred's errback
			var len = requests.length;
			for (var i=0;i<len;++i) {
				requests[i].dfd.errback(e);
			}
		}
	});
}

})();
