/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.PracticePreferenceProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.dashboard.web.ui.CustomPreferenceProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ViewletSelectionDialog"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper");//$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("dojo.string"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$//$NON-NLS-2$

(function() {

	var messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
	var commonMessages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.dialogs", "Messages");//$NON-NLS-1$ //$NON-NLS-2$
	var ViewletSelectionDialog = com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ViewletSelectionDialog;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.PracticePreferenceProvider", //$NON-NLS-1$
        com.ibm.team.dashboard.web.ui.CustomPreferenceProvider, {
	// summary:
	//		PracticePreferenceProvider provides a custom preference for practice. It will load the process description 
	//      and create ViewletSelectionDialog (pass the loaded process description data into the ViewletSelectionDialog). 
	//      
	//      PracticePreferenceProvider will also get the user selections in the picker dialog and put them into the
	//      settings dialog in the viewlet.
	//
	// parameters:
	//		viewlet: (String) the viewlet instance that the custom preference belongs to.
	//
	// methods:
	//		createChooser: fetch the process description data and create ViewletSelectionDialog.
	//      dialogOk: get the user selections in the picker dialog and put them into the settings dialog of the viewlet.
	//
	// usage:
	//      this._practicePrefProvider = new PracticePreferenceProvider({
	//					viewlet: ${viewlet}
	//				});
	
		viewlet: null,
		
		constructor: function(args) {
			this.viewlet = args.viewlet;
		},
		
		getLabel: function(value) {
			if (value) {
				return this._buildLabel(value);
			}
			return this._buildLabel(messages.NoValue);
		},
		
		getLoadingLabel: function() {
			return this._buildLabel(messages.loading);
		},
		
		_buildLabel: function(text) {
			return "<input type='text' readonly='readonly' value='" + text + "'/>"; //$NON-NLS-1$ //$NON-NLS-2$
		},
		
		createChooser: function(callback, settings) {
			if (!settings.scopeItemId) {
				alert(messages.PracticePreferenceProvider_1);
				this.viewlet.getSite().doneRefresh();
				return;
			}
			
			this._scope = settings.scope;
			this._callback = callback;
			this._practiceUrl = settings.preferences.practiceUrl;
			this._fragmentId = settings.preferences.fragmentId;
			
			var projectAreaItemId, teamAreaItemId;
			if (settings.scope == ProcessAuthoringConstants.PROJECT_AREA) {
				projectAreaItemId = settings.scopeItemId;
				var projectAreaUrl = UIHelper.getProcessAreaUrl(projectAreaItemId, "");  //$NON-NLS-1$
				this._createSelectionDialog(projectAreaUrl);
			} else if (settings.scope == ProcessAuthoringConstants.TEAM_AREA) {
				teamAreaItemId = settings.scopeItemId;
				
                ProcessAuthoringClient.getTeamAreaUrl({
                	teamAreaUUID: teamAreaItemId,
                	load: dojo.hitch(this, function(data, ioArgs) {
                		var teamAreaUrl = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
                		this._createSelectionDialog(teamAreaUrl);
                	}),
                	error: dojo.hitch(this, function(error, ioArgs) {
                		this._handleError(error);
                	})
                });
			} else {
				// clear contents and show msg
				alert(messages.PracticePreferenceProvider_1);
				this.viewlet.getSite().doneRefresh();
				return;
			}
		},
		
		_createSelectionDialog: function(processAreaUrl) {
			this.processAreaUrl = processAreaUrl;
			// this.viewlet.getScopeItem().name is the persistent name, we need the changing name in settingsPanel.
            var scopeItemName = this.viewlet.getSite().viewletAdapter._settingsPanel._getPreferenceProvider("_scope").getTextLabel({ //$NON-NLS-1$
            	type: this.viewlet.changingScope ? this.viewlet.changingScope : this.viewlet.getScope(),
            	itemId: this.viewlet.changingScopeItem ? this.viewlet.changingScopeItem : this.viewlet.getScopeItem().itemId
            })
            this.selectionDialog = new ViewletSelectionDialog({
				scope: this.processAreaUrl,
            	scopeItemName: scopeItemName,
            	selectedUrl: this._practiceUrl, //url of practice or practice element
            	selectedFragmentId: this._fragmentId, //section or field Id
            	onOk: dojo.hitch(this, this.dialogOk, this._callback)
            });
		},
		
		dialogOk: function(callback) {
			// set label and id in the settings dialog for the selected field, section, practice, practice element
			var practice = this.selectionDialog.getSelectedPracticeOrPracticeElement();
			var fragment = this.selectionDialog.getSelectedFragment();
			callback(practice.url, "practiceUrl"); //$NON-NLS-1$
			callback(fragment.id, "fragmentId"); //$NON-NLS-1$
			
			var practiceLabel = "";  //$NON-NLS-1$
			if(practice.label){
				practiceLabel = fragment.label ? practice.label + "/" + fragment.label : practice.label; //$NON-NLS-1$
			}
			callback(practiceLabel, "practiceLabel"); //$NON-NLS-1$
			
			var viewletTitle;
			if (fragment.label) {
				if (fragment.label === commonMessages.NoSectionLabel || fragment.label === commonMessages.NoFieldLabel) {
					viewletTitle = messages.ProcessDescriptionFragmentViewlet_2;
				} else {
					viewletTitle = fragment.label;
				}
			} else if (practice.label) {
				viewletTitle = practice.label;
			}
			if (viewletTitle) {
				this.viewlet.setTitle(viewletTitle);
			}
			this.selectionDialog.close();
		},
		
		_handleError: function(error) {
			if (error.status && error.status == ProcessAuthoringConstants.SC_FORBIDDEN) {
				alert(messages.permissionDenied);
			} else if (error.status && error.status == ProcessAuthoringConstants.SC_NOT_FOUND) {
				if (this._scope == ProcessAuthoringConstants.TEAM_AREA) {
					alert(messages.noDescTeam);
				} else {
					alert(messages.noDescProject);
				}
			} else {
				this.viewlet.getSite().doneRefresh();
				this.viewlet.getSite().error(error);
			}
		}
	});
	
})();
