/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012, 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ProcessDescriptionFragmentViewlet"); //$NON-NLS-1$
dojo.require("com.ibm.team.dashboard.web.ui.Viewlet"); //$NON-NLS-1$
dojo.require("jazz.client.xhr"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.UIHelper"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants"); //$NON-NLS-1$
dojo.require("dojo.html"); //$NON-NLS-1$
dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require("jazz.ui.ResourceLink");//$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.common.internal.PracticeElement"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.PracticePreferenceProvider"); //$NON-NLS-1$
dojo.require("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer");  //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
(function() {
	var Viewlet = com.ibm.team.dashboard.web.ui.Viewlet;
	var UIHelper = com.ibm.team.process.authoring.web.common.common.internal.UIHelper;
	var ProcessAuthoringConstants = com.ibm.team.process.authoring.web.common.common.internal.ProcessAuthoringConstants;
	var PracticePreferenceProvider =com.ibm.team.process.authoring.web.common.ui.internal.viewlets.PracticePreferenceProvider;
	var ResourceLink = jazz.ui.ResourceLink;
	var FieldViewer = com.ibm.team.process.authoring.web.common.ui.internal.viewlets.FieldViewer;
	var ProcessAuthoringClient = com.ibm.team.process.authoring.web.common.client.internal.ProcessAuthoringClient;
	var PracticeElement = com.ibm.team.process.authoring.web.common.common.internal.PracticeElement;
	
	dojo.declare("com.ibm.team.process.authoring.web.common.ui.internal.viewlets.ProcessDescriptionFragmentViewlet", Viewlet, { //$NON-NLS-1$
		templatePath: dojo.moduleUrl("com.ibm.team.process.authoring.web.common", "ui/internal/viewlets/templates/ProcessDescriptionFragmentViewlet.html"), //$NON-NLS-1$ //$NON-NLS-2$
		showFragmentTitle: true,
		
	// summary:
	//		ProcessDescriptionFragmentViewlet provides a process description viewlet. This viewlet allows us to display
	//      the process description/practice/practice element/section/field data in it.
	//      
	//      Case 1: Choose nothing in the practice picker dialog, the viewlet will only show process description 
	//              title and description.
	//      Case 2: Choose a practice or practice element, the viewlet will show practice/practice element title 
	//              and description.
	//      Case 3: Choose section or field, the viewlet will show the content of the section or field and a 
	//              resource link of the owning practice/practice element.
	//
	//
	// parameters (preferences):
	//      scope (predefined preference): project area or team area
	//      (optional) practiceLabel (custom preference): the name of the practice
	//      (optional) practiceUrl (hidden preference|String): the url of the practice
	//      (optional) fragmentId (hidden preference|String): the id of the section or field
	//      (optional) showFragmentTitle (boolean): indicates whether or not display the title of section or field. 
	//      
		
		/*
	     * Static Variable Container
	     */
		_static: function() {
		},
		
		constructor: function(args) {
			this.messages = dojo.i18n.getLocalization("com.ibm.team.process.authoring.web.common.ui.internal.viewlets", "Messages"); //$NON-NLS-1$ //$NON-NLS-2$
			this._resourceLinks = [];
			this.showFragmentTitle = this.getPreference(this._static._PREFERENCE_ID_SHOW_FRAGMENT_NAME);
		},
		
		init: function() {
			this._static._PREFERENCE_ID_PRACTICE_NAME = "practiceLabel"; //$NON-NLS-1$
			this._static._PREFERENCE_ID_PRACTICE_URL = "practiceUrl"; //$NON-NLS-1$
			this._static._PREFERENCE_ID_FRAGMENT_ID = "fragmentId"; //$NON-NLS-1$
			this._static._PREFERENCE_ID_SHOW_FRAGMENT_NAME = "showFragmentTitle"; //$NON-NLS-1$
			this._static._PREFERENCE_ID_UNCONFIGURED = "unconfigured"; //$NON-NLS-1$
		},
		
		refresh: function() {
			this._clearAllContent();
			this.getSite().setLoading(true);
			this._messagesArea.style.display = "none" //$NON-NLS-1$
			this._displayContentArea();
		},
		
		/*
		 * This method is used to reset the hidden fields when the selection is changed
		 */
		settingsChanged: function(oldSettings, newSettings) {
			if (this.practiceUrlResetRequired) {
				this.setPreference(this._static._PREFERENCE_ID_PRACTICE_URL, ""); //$NON-NLS-1$ 
				this.practiceUrlResetRequired = false;
			}
			
			if (this.fragmentIdResetRequired) {
				this.setPreference(this._static._PREFERENCE_ID_FRAGMENT_ID, ""); //$NON-NLS-1$
				this.fragmentIdResetRequired = false;
			}
			this.showFragmentTitle = this.getPreference(this._static._PREFERENCE_ID_SHOW_FRAGMENT_NAME);
			
			var unconfigured = this.getPreference(this._static._PREFERENCE_ID_UNCONFIGURED);
			if (unconfigured == "true" && newSettings.scope && newSettings.scopeItemId) { //$NON-NLS-1$
				this.setPreference(this._static._PREFERENCE_ID_UNCONFIGURED, "false"); //$NON-NLS-1$
			}
			this.getSite().refresh();
		},
		
		/*
		 * This method is used to reset the custom fields when the selection is changing
		 */
		settingChanging: function(/* string */settingId, workingCopy) {
			var result = new Array();
			if (settingId == "scope" || settingId == "scopeItem") { //$NON-NLS-1$ //$NON-NLS-2$
			    /* if scope is changing (e.g. project area was changed to team area) 
			     * or scope item (e.g. one project area was changed to another) is changing,
			     * then reset the practice name and fragment name. 
			     */ 
				result.push(this._static._PREFERENCE_ID_PRACTICE_NAME); //$NON-NLS-1$
				// practice url and fragment id also need to be reset
				this.practiceUrlResetRequired = true;
				this.fragmentIdResetRequired = true;
				this.changingScope = workingCopy.scope;
				this.changingScopeItem = workingCopy.scopeItemId;
			} else if (settingId == this._static._PREFERENCE_ID_PRACTICE_URL) {
				// fragment id needs to be reset
				this.fragmentIdResetRequired = true;
				// no need to reset practice url since it has value now
				this.practiceUrlResetRequired = false;
			} else if (settingId == this._static._PREFERENCE_ID_FRAGMENT_ID) {
				// no need to reset fragment id since it has value now
				this.fragmentIdResetRequired = false;
			}
			return result;
		},
		
		_isValidUrl: function(url) {
			// Regexp taken from RFC 3986 Appendix A
			return url.match(new RegExp('^(http(s)?:)(//([^/?#]*))([^?#]*)(\\?([^#]*))?(#(.*))?')); //$NON-NLS-1$
		},
		
		prepareUrl: function(url) {
			if (dojo.indexOf(url, "?") === -1) { //$NON-NLS-1$
				return url + "?view=description"; //$NON-NLS-1$
			} else {
				return url + "&view=description"; //$NON-NLS-1$
			}
		},
		
		/*
		 * Display content in viewlet
		 */
		_displayContentArea: function() {
			var unconfigured = this.getPreference(this._static._PREFERENCE_ID_UNCONFIGURED);
 			// Newly added viewlet
			if (unconfigured == "true") { //$NON-NLS-1$
				// Reset the default value of the viewlets added into Project or Team Dashboard.
				if (this.getScope() && this.getScopeItem()) {
					this.setPreference("_scope", ""); //$NON-NLS-1$ //$NON-NLS-2$
					this.setPreference("_scopeItem", ""); //$NON-NLS-1$ //$NON-NLS-2$
				}
				this._showMessage(this.messages.ProcessDescriptionFragmentViewlet_1);
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
				return;
			}
			
			if (this.getPreference(this._static._PREFERENCE_ID_PRACTICE_URL) && this._isValidUrl(this.getPreference(this._static._PREFERENCE_ID_PRACTICE_URL))) { //$NON-NLS-1$ //$NON-NLS-2$
			    // display practice, practice element, section or field 
				this._getPracticeOrElement();
			} else if (this.getScope() && this.getScopeItem()) {
				// display process description
				var projectAreaItemId, teamAreaItemId;
				if (this.getScope() == ProcessAuthoringConstants.PROJECT_AREA) {
					projectAreaItemId = this.getScopeItem().itemId;
					var projectAreaUrl = UIHelper.getProcessAreaUrl(projectAreaItemId, ""); //$NON-NLS-1$
					this._getPrivateProcessDescription(projectAreaUrl);
				} else if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
					teamAreaItemId = this.getScopeItem().itemId;
                    ProcessAuthoringClient.getTeamAreaUrl({
						teamAreaUUID: teamAreaItemId,
						load: dojo.hitch(this, function(data, ioArgs) {
							var teamAreaUrl = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
							this._getPrivateProcessDescription(teamAreaUrl); 
						}),
						error: dojo.hitch(this, function(error, ioArgs) {
							this._handleError(error);
						})
					});
				} else {
					// clear contents and show msg
					this._showMessage(this.messages.ProcessDescriptionFragmentViewlet_1);
					this.getSite().setLoading(false);
					this.getSite().doneRefresh();
				}
			} else {
				this._showMessage(this.messages.ProcessDescriptionFragmentViewlet_1);
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			}
		},
		
		/**
		 * Get private process description from a project area or team area.
		 * 
		 * @param {String} processAreaUrl the url of project area or team area that may have a private process description.
		 */
		_getPrivateProcessDescription: function(processAreaUrl) {
			var self = this;
			ProcessAuthoringClient.getProcessDescriptionsForOwner({
				ownerUrl: processAreaUrl,
				load: function(data, ioArgs) {
					var descriptionsNode = UIHelper.getNode(data.childNodes, ProcessAuthoringConstants.XMLELEMENT_RDF, ProcessAuthoringConstants.NAMESPACE_RDF);
					if (descriptionsNode) {
						var allProcessDescriptionNodes = UIHelper.getNodes(descriptionsNode.childNodes, ProcessAuthoringConstants.XMLELEMENT_PROCESSDESCRIPTION_ROOT, ProcessAuthoringConstants.NAMESPACE_JTP);
						if (allProcessDescriptionNodes && allProcessDescriptionNodes.length > 0) {
							var processDescriptionNode = allProcessDescriptionNodes[0];
							var description = new PracticeElement({
								docElement: processDescriptionNode
							});
							self._update(description, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION);
							self.getSite().setLoading(false);
							self.getSite().doneRefresh();
						} else {
							// no private process description, check associated process description
							if (self.getScope() == ProcessAuthoringConstants.PROJECT_AREA || self.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
								self._getAssociatedProcessDescription(processAreaUrl);
							} else {
								// clear contents and show msg
								this._showMessage(this.messages.ProcessDescriptionFragmentViewlet_1);
								self.getSite().doneRefresh();
							}
						}
					}
				},
				error: function(error) {
					self._handleError(error);
				}
			});
		},
		
		/**
		 * Get associated process description from a project area or team area.
		 * 
		 * @param {String} processAreaUrl the url of project area or team area that may be associated with a shared process description.
		 */
		_getAssociatedProcessDescription: function(processAreaUrl) {
			var self = this;
			ProcessAuthoringClient.checkAssociateSharedProcessDescriptionForProcessArea({
				targetUrl: processAreaUrl,
				load: function(data, ioArgs) {
					var associationURL = ioArgs.xhr.getResponseHeader("Location"); //$NON-NLS-1$
					ProcessAuthoringClient.getAssociation({
						url: associationURL,
						load: function(data, ioArgs) {
							var descriptionURL = UIHelper.getProcessDescriptionUrlFromAssociation(data);
							if (descriptionURL) {
								ProcessAuthoringClient.getProcessDescription({
									resourceURL: descriptionURL,
									load: function(data, ioArgs) {
										var description = new PracticeElement({
											xml: data
										});
										self._update(description, ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION);
										self.getSite().setLoading(false);
										self.getSite().doneRefresh();
									},
									error: function(error) {
										self._handleError(error);
									}
								});
							}
						},
						
						error: function(error) {
							self._handleError(error);
						}
					});
				},
				error: function(error) { //no associate shared process description
					self._handleError(error);
				}
			});
		},
		
		/**
		 * Update the content of the viewlet with the latest choice.
		 * 
		 * @param {Object} practice the process description, practice or practice element.
		 * @param {String} type {e.g. ProcessDescription|Practice}
		 */
		_update: function(practice, type) {
			this._clearAllContent();
			this._summary.className = "summary"; //$NON-NLS-1$
			this._details.className = ""; //$NON-NLS-1$
			if (practice && (practice.title || practice.description)) {
				// set viewlet title
				if (practice.title) {
					this.setTitle(practice.title);
				} else if (type === ProcessAuthoringConstants.XMLELEMENT_PRACTICE){
					this.setTitle(this.messages.ProcessDescriptionFragmentViewlet_3);
				} else if (type === ProcessAuthoringConstants.XMLELEMENT_DESCRIPTION){
					this.setTitle(this.messages.ProcessDescriptionFragmentViewlet_4);
				}
				
				// set description field
				if (practice.description) {
					dojo.html.set(this._details, practice.description);
				}
				
				// set name link
				if (practice.title) {
					var link = document.createElement("a"); //$NON-NLS-1$
					link.href = practice.url;
					link.appendChild(document.createTextNode(practice.title));
					var rl = new ResourceLink({
						'presentationUri': practice.url, //$NON-NLS-1$
						'lazyFetch': true //$NON-NLS-1$
					}, link);
					this._summary.appendChild(link);
					this._resourceLinks.push(rl);
				}
			} else {
				if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
					this._details.appendChild(document.createTextNode(this.messages.noDescTeam));
				} else {
					this._details.appendChild(document.createTextNode(this.messages.noDescProject));
				}
				
			}
			if (!this._summary.firstChild) {
				dojo.addClass(this._summary, "hidden"); //$NON-NLS-1$
			}
			if (!this._details.firstChild) {
				dojo.addClass(this._details, "hidden"); //$NON-NLS-1$
			}
		},
		
		_clearAllContent: function() {
			dojo.empty(this._content);
			dojo.empty(this._details);
			dojo.empty(this._summary);
		},
		/**
	    * Load practice or practice element and display the initial content of the selected field or section.
	    */
		_getPracticeOrElement: function() {
			jazz.client.xhrGet({
				preventCache: true,
				url: this.prepareUrl(this.getPreference(this._static._PREFERENCE_ID_PRACTICE_URL)), //$NON-NLS-1$
				headers: {
					"Accept": "application/xml" //$NON-NLS-1$ //$NON-NLS-2$
				},
				handleAs: "xml", //$NON-NLS-1$
				error: dojo.hitch(this, function(error) {
					var message = error.responseText ? error.responseText : error.message;
					if (error.status === ProcessAuthoringConstants.SC_FORBIDDEN) {
						this._showMessage(this.messages.permissionDenied);
					} else {
						this._showMessage(message);
					}
					this.getSite().setLoading(false);
					this.getSite().doneRefresh();
				}),
				load: dojo.hitch(this, function(data) {
					this._clearAllContent();
					
					var practice = new PracticeElement({
						xml: data
					});
					
					var fragmentId = this.getPreference(this._static._PREFERENCE_ID_FRAGMENT_ID); //$NON-NLS-1$
					if (fragmentId) {
						var sections = UIHelper.getAllSections(data);
						var found = false;
						var label;
						dojo.some(sections, function(section) {
							var sectionId = UIHelper.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
							if (sectionId && sectionId.value == fragmentId) {
								label = UIHelper.getAttributeNode(section, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
								var fields = UIHelper.getAllFields(section);
								
								// add section title
								if (this.showFragmentTitle) {
									var div = document.createElement("div"); //$NON-NLS-1$
									div.appendChild(document.createTextNode(label));
									dojo.addClass(div, "sectionTitle"); //$NON-NLS-1$
									this._content.appendChild(div);
								}
			
								dojo.forEach(fields, function(field) {
									var fieldTitle = UIHelper.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
									var fieldContent = UIHelper.getFieldText(field);
									if (fieldContent) {
										fieldContent = UIHelper.unescapeXML(fieldContent);
									}
									this._addFieldContent(fieldTitle, fieldContent);
								}, this);
								this._addNameResouceLink(practice.title, practice.url);
								found = true;
								return true;
							} else {
								var fields = UIHelper.getAllFields(section);
								return dojo.some(fields, function(field) {
									var idAttr = UIHelper.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_ID, ProcessAuthoringConstants.NAMESPACE_RDF);
									if (idAttr && idAttr.value == fragmentId) {
										label = UIHelper.getAttributeNode(field, ProcessAuthoringConstants.XMLATTRIBUTE_LABEL, "").value; //$NON-NLS-1$
										var fieldContent = UIHelper.getFieldText(field);
										if (fieldContent) {
											fieldContent = UIHelper.unescapeXML(fieldContent);
										}
										if (this.showFragmentTitle) {
											this._addFieldContent(label, fieldContent); //$NON-NLS-1$
										} else {
											this._addFieldContent("", fieldContent); //$NON-NLS-1$
										}
										this._addNameResouceLink(practice.title, practice.url);
										found = true;
										return true
									}
									
									return false;
								}, this);
							}
						}, this);
						
						if (!found) {
							this._showMessage(this.messages.ProcessDescriptionFragmentViewlet_0);
						}
						if (label) {
							this.setTitle(label);
						} else {
							this.setTitle(this.messages.ProcessDescriptionFragmentViewlet_2);
						}
					} else {
						// display practice overview
						this._update(practice, ProcessAuthoringConstants.XMLELEMENT_PRACTICE);
					}
					this.getSite().setLoading(false);
					this.getSite().doneRefresh();
				})
			});
		},
		
		/**
		 *  Set viewlet title
		 *  
		 * @param {String} title
		 */
		setTitle: function(title) {
			this.getSite().setTitle(title);
		},
		
		/**
		 * Add resource link of practice or practice element in the viewlet.
		 * 
		 * @param {String} title the name of the practice or practice element.
		 * @param {String} url the url of the practice or practice element.
		 */
		_addNameResouceLink: function(title, url) {
			// add practice or practice element name resource link
			if (title) {
				var link = document.createElement("a"); //$NON-NLS-1$
				link.href = url;
				link.appendChild(document.createTextNode(title));
				var rl = new ResourceLink({
					'presentationUri': url, //$NON-NLS-1$
					'lazyFetch': true //$NON-NLS-1$
				}, link);
				this._summary.appendChild(link);
				this._resourceLinks.push(rl);
			}
		},
		
		/**
		 * Add field content in the viewlet.
		 * 
		 * @param {String} fieldTitle
		 * @param {String} fieldContent
		 */
		_addFieldContent: function(fieldTitle, fieldContent) {
			if (this.fieldViewer) {
				this.fieldViewer = null;
			}
			// add fields content
			var div = document.createElement("div"); //$NON-NLS-1$
			this._content.appendChild(div);
			this.fieldViewer = new FieldViewer({
				fieldTitle: fieldTitle,
				fieldContent: fieldContent
			}, div);
		},
		
		_showMessage: function(message) {
			var span = document.createElement("span"); //$NON-NLS-1$
			span.appendChild(document.createTextNode(message));
			dojo.empty(this._messagesArea);
			this._messagesArea.appendChild(span);
			this._messagesArea.style.display = "" //$NON-NLS-1$
		},
		
		/*
		 * return preference provider according to the preference id 
		 */
		getPreferenceProvider: function(preferenceId) {
			if (preferenceId === this._static._PREFERENCE_ID_PRACTICE_NAME) {
				if (!this._practicePrefProvider) {
					this._practicePrefProvider = new PracticePreferenceProvider({
						viewlet: this
					});
				}
				return this._practicePrefProvider;
			}
		},
		
		_handleError: function(error) {
			if (error.status && error.status == ProcessAuthoringConstants.SC_FORBIDDEN) {
				var span = document.createElement("span"); //$NON-NLS-1$
				span.className = "errorText"; //$NON-NLS-1$
				span.appendChild(document.createTextNode(this.messages.permissionDenied));
				this._showMessage(span);
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			} else if (error.status && error.status == ProcessAuthoringConstants.SC_NOT_FOUND) {
				this._summary.innerHTML = ""; //$NON-NLS-1$
				this._summary.className = "summary"; //$NON-NLS-1$
				this._details.innerHTML = ""; //$NON-NLS-1$
				this._details.className = ""; //$NON-NLS-1$
				if (this.getScope() == ProcessAuthoringConstants.TEAM_AREA) {
					this._details.appendChild(document.createTextNode(this.messages.noDescTeam));
				} else {
					this._details.appendChild(document.createTextNode(this.messages.noDescProject));
				}
				this.getSite().setLoading(false);
				this.getSite().doneRefresh();
			} else {
				this.getSite().doneRefresh();
				this.getSite().error(error);
			}
		},
		
		_destroyResourceLinks: function(){
			for (var i = 0; i < this._resourceLinks.length; i++) {
				if (this._resourceLinks[i]) {
					this._resourceLinks[i].destroyRecursive();
				}
			}
			this._resourceLinks = [];
		},
		
		uninitialize: function() {
			this._destroyResourceLinks();
			if (this.fieldViewer) {
				this.fieldViewer.destroyRecursive();
			}
			this.inherited(arguments);
		}
	});
	
})();
