/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010, 2011. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
(function() {
	
dojo.provide("com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("jazz.ui.wizard.ExternalWizardStep");
dojo.require("jazz.app.internal.util.Rootservices");
dojo.require("com.ibm.team.repository.web.client.internal.HttpHeaders");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService");
dojo.require("jazz.util.binding");
dojo.require("jazz.client.xhr");
dojo.require("dojo.i18n");

dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ContributedStepsControllerMessages");
	
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var ExternalWizardStep = jazz.ui.wizard.ExternalWizardStep;
var Rootservices = jazz.app.internal.util.Rootservices;
var AboutServiceSchema = com.ibm.team.repository.web.ui.internal.wizard.schema.AboutService;
var binding = jazz.util.binding;
var STEPS_URI_PREFIX = "/steps/";
var JFS_NS = "http://jazz.net/xmlns/prod/jazz/jfs/1.0/";
var LPA_NS = "http://jazz.net/xmlns/foundation/lpa/1.0/";
var OSLC_NS = "http://open-services.net/ns/core#";
var SETUP_WIZARD_DESCRIPTOR = "setupWizardDescriptor";
var LPA_REGISTRATION = "Registration";
var LPA_IDENTIFIER = "http://jazz.net/application/lpa";
var messages;

// ContributedStepsController provides non-UI support for inspecting registered applications for
// Wizard step contributions and dynamically adding them to the current wizard. 

dojo.declare("com.ibm.team.repository.web.ui.internal.wizard.ContributedStepsController", dijit._Widget, {
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "ContributedStepsControllerMessages");
		this.headers = com.ibm.team.repository.web.client.internal.HttpHeaders._getHeaders();
	},
	
	initParams: function(callbackObj, onSuccess, onFailure, args){		
		this.callbackObj = callbackObj;
		this.successCallback = onSuccess;
		this.failureCallback = onFailure;
		
		this.wizard = this.callbackObj.wizard;

		if(args && args.existingApps) {
			this.existingApps = args.existingApps;
		}
		if(args && args.existingLPA) {
			this.lpaInstalled = args.existingLPA;
		}
		//See if there is a MessageBox we can use to push error messages more accurately.
		if(callbackObj && callbackObj.header) {
			this.messageBox = callbackObj.header;
		}
		
		//We eventually need to return whether or not steps were added.
		this.stepsAddedToWizard = false;
		
		this._getRegisteredApps();
	},

	_getRegisteredApps: function() {
		var srh = new ServiceResponseHandler(this, "_handleListOfApps", "_errorFetchingApps");
		var args = { includeInternal: true };
		OAuthAdminClient.getFriends(srh,args);	
	},
	
	_handleListOfApps: function(apps) {
		var newApps = new Array();
		dojo.forEach(apps, function(app) {
			//Only want the "applications" not all friends
			if(app.internal && app.internal == true){
				newApps.push(app);
			}
		});
		if(newApps && this.existingApps) {
			//We only want to add the step contributions from the new apps we just added
			newApps = this._compareAppsObjects(newApps, this.existingApps);
		}
		this._getStepContributions(newApps);
	},
	
	/* Determines the apps that are actually "new".
	 * Effectively: total apps ("After") - existing apps ("Before")
	 * Returns: an array of "newly added" applications
	 */
	_compareAppsObjects: function(appsAfter, appsBefore) {
		var newapps = [];
		dojo.forEach(appsAfter, function(appA) {
			var appExisted = false;
			dojo.forEach(appsBefore, function(appB) {
				if(appA.id === appB.id) {
					appExisted = true;
					return;
				}
			});
			if(!appExisted) {
				newapps.push(appA);
			}
		});
		return newapps;
	},

	_getStepContributions: function(apps) {
		var self = this;
		if (apps){
			this.registeredApps = apps;
			if(this.registeredApps.length > 1) {
				this._sort(this.registeredApps);
			}	
			var separatedFriends = this._separateFriendsByInternal(this.registeredApps);
			var applications = separatedFriends.internalFriends;
		} 
		if(applications && applications.length > 0) {
			this.callBacksRemaining = applications.length;
			this.appStepsToAdd = new Array();
			
			dojo.forEach(applications, function(app){
				var deferred = self._getStepsFromApplication(app);
				var instanceName = app.title;
				deferred.addCallback(function(appSteps) {
					self.callBacksRemaining--;
					if(appSteps) {
						//Check to see if we were able to get more information about this application from it's About service
						if(appSteps.title) {
							//Great, we have an "official" application name. Overwrite the one from the wizard descriptor
							appSteps.application = appSteps.title + "<span class=\"instanceName\">(" + instanceName + ")</span>";
						}
						self.appStepsToAdd.push(appSteps);
					}
					self._appendAllSteps();
				});
				deferred.addErrback(function(errObj) {
					self.callBacksRemaining--;
					self._appendAllSteps();
					//Failed to get steps from the application
					if(self.messageBox) {
						self.messageBox.showMessage({
							errorObject: errObj,
							summary: self.messages.getRootServicesFailed
						});
					}
				});
            });
			
		} else {
			//No applications, go back to add the summary step.
			this._handleCallback();
		}
	},
	
	_appendAllSteps: function() {
		if(this.callBacksRemaining !== 0) return;
		//OK, we are ready to append the steps
		this.appStepsToAdd = this._sort(this.appStepsToAdd);
		
		//Ensure that LPA gets placed at the end of the steps
		this.appStepsToAdd.sort(function(a, b){
			if(a.lpa) return 1;
			if(b.lpa) return -1;
			return 0;
		});

		dojo.forEach(this.appStepsToAdd, function(app){
			this._addNewStepGroup(app);
		}, this);
		
		//If LPA had been *previously* installed, we need to move it.
		if(this.lpaInstalled) {
			var groupNodes = dojo.query(".stepGroup");
			dojo.forEach(groupNodes, function(node) {
				if(node._lpa) {
					var appSteps = node._appSteps;
					this.wizard.removeGroup(node);
					this._addNewStepGroup(appSteps);
				}
			}, this);
		}
		//All finished, go back to add the summary step.
		this._handleCallback();
	},
	
	_errorFetchingApps: function(errorObj) {
		if(errorObj.status == 503) {
			//Service is Unavailable (no DB?)
			this._handleListOfApps([]);
		} else {
			if(this.messageBox) {
				this.messageBox.showMessage({
					errorObject: errorObj,
					summary: this.messages.errorFetchingApps
				});
			} else {
				console.error(this.messages.errorFetchingApps);
			}
		}
	},

	_addNewStepGroup: function(appSteps) {
		var self = this;
		var baseURL = appSteps.baseSetupURL;
		var steps = new Array();
		dojo.forEach(appSteps.steps, function(step) {
			//Append the new step to the wizard
			if(!step.displayName) {
				console.error(self.messages.missingDisplayName);
				return;
			}
			if(!step.stepNumber) {
				console.error(dojo.string.substitute(self.messages.missingStepNumber, [ step.displayName ]));
				return;
			}
			if(!step.stepURL) {
				step.stepURL = baseURL + "?embed=true#" + STEPS_URI_PREFIX + step.stepNumber;
			}
			steps.push(new ExternalWizardStep(step));
		});
		if(steps.length > 0) {
			this.stepsAddedToWizard = true;
		} 
		var group = { name: appSteps.application, steps: steps};
		var node = this.wizard.addStepGroup(group);
		if(appSteps.lpa) {
			node._lpa = true;
			node._appSteps = appSteps;
		}
	},
	
	_handleMissingWizardDescriptor: function(err) {
		if(this.messageBox) {
			this.messageBox.showMessage({
				errorObject: err,
				summary: this.messages.errorFetchingWizardDescriptor
			});
		}
		console.error(this.messages.errorFetchingWizardDescriptor);
	},
	
	_getStepsFromApplication: function(app) {
		var self = this;
		var deferred = new dojo.Deferred();
		var rootDeferred = Rootservices.load(app.rootServicesURL, /* passive */ false);
		rootDeferred.addCallback(function(root){
			var wizardDescriptorURL = root.getServiceUrl(JFS_NS, SETUP_WIZARD_DESCRIPTOR);
			if(wizardDescriptorURL) {
				jazz.client.xhrGet({
					url: wizardDescriptorURL,
					handleAs: "json",
					headers: self.headers,
					load: function(response) {
						//Try to get more information about this application from it's about service
						var applicationAboutURL = root.getPublisher();
						if(applicationAboutURL) {
							self._getApplicationAbout(deferred, response, applicationAboutURL);
						} else {
							//We need to find out if this application is the LPA.
							//If it is, we should display it's steps last.
							var isThisLPA = root.getServiceUrl(LPA_NS, LPA_REGISTRATION);
							response.lpa = isThisLPA ? true : false;
							deferred.callback(response);
						}
					},
					error: function(err) {
						self._handleMissingWizardDescriptor(err);
						deferred.callback(false);
					}
				});	
			} else {
				deferred.callback(false);
			}
		});		
		rootDeferred.addErrback(function(arguments) {
			//Can't find the Rootservices document for that app.
			deferred.errback(arguments);
		});
		return deferred;
	},
	
	//Parses the application about information from the app's RootServices document
	_getApplicationAbout: function(deferred, response, url) {
		var self = this;
		jazz.client.xhrGet({
			url: url,
			handleAs: "xml",
			headers: self.headers,
			load: function(doc) {
				var parsed = binding.read(doc.documentElement, AboutServiceSchema.aboutService);
				if(parsed && parsed.About) {
					response.lpa = parsed.About.identifier == LPA_IDENTIFIER;
					response.title = parsed.About.title;
					response.description = parsed.About.description;		
				}
				deferred.callback(response);
			},
			error: function(error) {
				//Just abandon this and return the original response object from the wizard descriptor call
				deferred.callback(response);
			}
		});
	},

	_separateFriendsByInternal: function (friends) {
		var internalFriends = [];
		var publicFriends = [];
		dojo.forEach(friends, function(friend){
        	if (friend.internal && friend.internal == true){
				internalFriends.push(friend);
			} else {
				publicFriends.push(friend);
			}
       	});
		return {internalFriends: internalFriends, publicFriends: publicFriends}	
	},
	
	_sort: function (list) {	
		return list.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	_handleCallback: function () {
		if(this.callbackObj && dojo.isFunction(this.callbackObj[this.successCallback])) {
			var obj= this.callbackObj;
			var returnVal = { stepsAdded: this.stepsAddedToWizard };
			obj[this.successCallback].call(obj, returnVal);
		} 
	},

	_handleServiceError: function(errorObj) {
		if(this.callbackObj && dojo.isFunction(this.callbackObj[this.failureCallback])) {
			var obj= this.callbackObj;
			obj[this.failureCallback].call(obj, errorObj);
		}  
	}

});


})();