/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
 
(function() {
	dojo.provide("com.ibm.team.process.web.ui.operationbehavior.AbstractOperationBehaviorCustomEditor"); //$NON-NLS-1$
	
	dojo.require("dijit._Widget"); //$NON-NLS-1$
	dojo.require("com.ibm.team.process.web.ui.operationbehavior.ProcessInfo"); //$NON-NLS-1$
	
	/**
	 * This abstract class should be used for any editors which add more
	 * specific configuration for a particular precondition or follow-up action.
	 * Such editors are declared in the plugin.xml with the
	 * 'com.ibm.team.process.web.operationBehaviorEditors' extension point.
	 * 
	 * NOTE: please see RequireDescriptionAdvisorEditor.js in the Process
	 * Examples component for an example of such an editor.
	 */
	dojo.declare("com.ibm.team.process.web.ui.operationbehavior.AbstractOperationBehaviorCustomEditor", dijit._Widget, { //$NON-NLS-1$
		
		constructor: function(args) {
			// internal fields/methods
			this._changesPendingCallback = args.changesPendingCallback;
			this._processInfo = args.processInfo;
			// prevent public references to private fields/methods
			delete args.changesPendingCallback;
			delete this.changesPendingCallback;
			delete args.processInfo;
			delete this.processInfo;
		},
		
		/**
		 * This method will be called by the containing editor to initialize
		 * this editor with its starting state.
		 * 
		 * @param content {String} if an IXMLConverter is registered for this
		 * 		  operation behavior, then this will be whatever is returned
		 * 		  from that, or null if this is a newly added behavior.
		 * 		  Otherwise it will be the XML content of this precondition or
		 * 		  follow-up action
		 * @param operationBehaviorId {String} identifier of the precondition/
		 * 		  follow-up action this editor is editing
		 */
		initialize: function(content, operationBehaviorId) {
			// do nothing by default
		},
		
		/**
		 * This method will be called by the containing editor when this editor
		 * is about to be displayed. Subclasses can override to update data
		 * before the editor shows. Short operations are recommended over long
		 * operations.
		 */
		aboutToShowEditor: function() {
			// do nothing by default
		},
		
		/**
		 * Subclasses must implement this method. It should return a string with
		 * content representing the current state of the editor's configuration,
		 * so that it can be saved into the process specification.
		 * 
		 * @returns {String} if an IXMLConverter is registered for this
		 * 		  	operation behavior, then this should return something which
		 * 			that can interpret, otherwise the XML content of this
		 * 			operation behavior. The outer node of this XML will be a
		 * 			&lt;precondition/> or &lt;followup-action/> node, this outer
		 * 			node will be ignored by the Process framework, only the
		 * 			inner XML is of interest
		 */
		getContent: function() {
			alert("Subclasses of com.ibm.team.process.web.ui.AbstractOperationBehaviorCustomEditor must implement the 'getContent' function"); //$NON-NLS-1$
			return ""; //$NON-NLS-1$
		},
		
		/**
		 * Subclasses may override this to indicate whether changes have been
		 * made to the editor. The containing editor will call this method when
		 * notifyContentChanged() is called.
		 * 
		 * @returns {Boolean} whether or not changes have been made
		 */
		changesPending: function() {
			return false;
		},
		
		/**
		 * Subclasses may override this to indicate whether the containing
		 * editor should be allowed to save or not. The containing editor will
		 * call this method when notifyContentChanged() is called.
		 * 
		 * @returns {Boolean} whether the save button should be enabled
		 */
		canSave: function() {
			return true;
		},
		
		/**
		 * Subclasses may override this to return an error message if the
		 * current configuration of the editor is not valid. The containing
		 * editor will call this method when notifyContentChanged() is called.
		 * 
		 * @return {String} an error message if not valid, null|""|undefined
		 *         otherwise
		 */
		validate: function() {
			return null;
		},
		
		/**
		 * Call this method to notify the containing editor that they have
		 * pending changes. This allows the containing editor to update its
		 * dirty status (changesPending()), whether or not the save button is
		 * enabled (canSave()), and whether or not an error message is displayed 
		 * (validate()), based on feedback from this editor.
		 * 
		 * NOTE: This is a convenience method, it is not meant to be overridden
		 * by subclasses.
		 */
		notifyContentChanged: function() {
			this._changesPendingCallback();
		},
		
		/**
		 * Returns an object containing information about the process area this
		 * precondition or follow-up action is being configured for. The object
		 * will be of type
		 * 'com.ibm.team.process.web.ui.operationbehavior.ProcessInfo'.
		 * 
		 * NOTE: This is a convenience method, it is not meant to be overridden
		 * by subclasses.
		 */
		getProcessInformation: function() {
			return this._processInfo;
		}
		
	});
})();
